<?php
/**
 * WholesaleX BulkOrder
 *
 * @package WHOLESALEX_PRO
 * @since 1.0.7
 */

namespace WHOLESALEX_PRO;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}
/**
 * WholesaleX BulkOrder
 */
class BulkOrder {


	/**
	 * Constructor
	 */
	public function __construct() {
		add_filter( 'wholesalex_bulkorder_addon_my_account_menu_title', array( $this, 'bulkorder_menu_title' ) );
		add_filter( 'wholesalex_bulkorder_addon_my_account_end_point', array( $this, 'bulkorder_menu_endpoint' ) );

		$menu_endpoint = apply_filters( 'wholesalex_bulkorder_addon_my_account_end_point', __( 'wholesalex-bulkorder', 'wholesalex-pro' ) );

		if ( 'yes' === wholesalex()->get_setting( 'wsx_addon_whitelabel' ) ) {
			add_shortcode( wholesalex()->get_setting( 'bulkorder_form_shortcode', 'wholesalex_bulkorder_form' ), array( $this, 'wholesalex_bulkorder_form' ), 1 );
			if ( 'wholesalex_bulkorder_form' !== wholesalex()->get_setting( 'bulkorder_form_shortcode', 'wholesalex_bulkorder_form' ) ) {
				add_shortcode( 'wholesalex_bulkorder_form', array( $this, 'wholesalex_bulkorder_form' ), 1 );
			}
		} else {
			add_shortcode( 'wholesalex_bulkorder_form', array( $this, 'wholesalex_bulkorder_form' ), 1 );
		}

		add_action( 'woocommerce_account_' . $menu_endpoint . '_endpoint', array( $this, 'bulkorder_form' ) );

		add_filter( 'woocommerce_account_menu_items', array( $this, 'add_bulkorder_form_on_myaccount_menu' ) );

		add_action( 'init', array( $this, 'bulkorder_form_add_end_point' ) );

		add_filter( 'woocommerce_get_query_vars', array( $this, 'add_bulkorder_to_woocommerce_query_vars' ) );

		add_action( 'rest_api_init', array( $this, 'bulkorder_shortcode_rest_api_init' ), 99999999 );

		add_action( 'template_redirect', array( $this, 'bulkorder_add_to_cart_success_message' ), 1 );

		add_action( 'wp_ajax_bulkorder_add_to_cart', array( $this, 'bulkorder_add_to_cart' ) );

		add_filter( 'wholesalex_setting_fields', array( $this, 'bulkorder_settings' ) );
	}

	/**
	 * Add WholesaleX BulkOrder in Account Menu Items
	 *
	 * @param array $menu_links Array of menu links slug.
	 * @since 1.0.7
	 */
	public function add_bulkorder_form_on_myaccount_menu( $menu_links ) {
		$menu_title    = apply_filters( 'wholesalex_bulkorder_addon_my_account_menu_title', __( 'Bulk Order', 'wholesalex-pro' ) );
		$menu_endpoint = apply_filters( 'wholesalex_bulkorder_addon_my_account_end_point', __( 'wholesalex-bulkorder', 'wholesalex-pro' ) );

		$menu_links = array_slice( $menu_links, 0, 5, true )
		+ array( $menu_endpoint => $menu_title )
		+ array_slice( $menu_links, 5, null, true );
		return $menu_links;
	}

	/**
	 * WholesaleX Bulkorder Add End Point
	 *
	 * @since 1.0.7
	 */
	public function bulkorder_form_add_end_point() {
		$menu_endpoint = apply_filters( 'wholesalex_bulkorder_addon_my_account_end_point', __( 'wholesalex-bulkorder', 'wholesalex-pro' ) );
		add_rewrite_endpoint( $menu_endpoint, EP_PAGES );
	}
	/**
	 * WholesaleX Bulkorder Page
	 *
	 * @since 1.0.7
	 */
	public function bulkorder_form() {
		echo $this->wholesalex_bulkorder_form(); //phpcs:ignore
	}

	/**
	 * Add Custom Query Vars in WooCommerce
	 *
	 * @param array $query Query.
	 * @since 1.0.7
	 */
	public function add_bulkorder_to_woocommerce_query_vars( $query ) {
		$menu_endpoint           = apply_filters( 'wholesalex_bulkorder_addon_my_account_end_point', __( 'wholesalex-bulkorder', 'wholesalex-pro' ) );
		$query[ $menu_endpoint ] = $menu_endpoint;
		return $query;
	}

	/**
	 * Bulk Order Data Add To Cart Ajax Action.
	 *
	 * @return void
	 */
	public function bulkorder_add_to_cart() {
		if ( ! ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'order_form' ) ) ) {
			wp_send_json_error( 'Nonce Verification Failed', 403 );
			return;
		}
		$data = isset( $_POST['data'] ) ? json_decode( sanitize_text_field( wp_unslash( $_POST['data'] ) ), true ) : array();
		WC()->cart->empty_cart();
		foreach ( $data as $value ) {
			if ( isset( $value['_product_select'][0]['value'], $value['_product_quantity'] ) ) {
				$__product_id = $value['_product_select'][0]['value'];
				$__quantity   = $value['_product_quantity'];
				WC()->cart->add_to_cart( $__product_id, $__quantity );
			}
		}
		set_transient( 'bulkorder_data_' . get_current_user_id(), true );

		$clear_bulkorder_session = wholesalex()->get_setting( '_bulkorder_clear_session_after_save' );
		wp_send_json_success(
			array(
				'redirect_url'     => get_permalink( get_option( 'woocommerce_cart_page_id' ) ),
				'is_clear_session' => $clear_bulkorder_session,
			)
		);
	}

	/**
	 * After Adding bulk order items to cart, display a success notice.
	 *
	 * @return void
	 */
	public function bulkorder_add_to_cart_success_message() {
		$status = get_transient( 'bulkorder_data_' . get_current_user_id() );

		if ( is_cart() && $status ) {
			wc_add_notice( 'Bulk Order items added to cart.', 'success' );
		}
		delete_transient( 'bulkorder_data_' . get_current_user_id() );
	}

	/**
	 * Bulk Order Form Shortcode Render
	 *
	 * @param array $atts    Shortcode attributes. Default empty.
	 * @since 1.0.0
	 */
	public function wholesalex_bulkorder_form( $atts = array() ) {
		wp_enqueue_script( 'wholesalex_bulkorder' );
		wp_enqueue_style( 'dashicons' );
		wp_localize_script(
			'wholesalex_bulkorder',
			'wholesalex_bulkorder',
			array(
				'fields'         => $this->get_form_fields(),
				'nonce'          => wp_create_nonce( 'order_form' ),
				'ajax'           => admin_url( 'admin-ajax.php' ),
				// 'tags'           => $this->get_product_tags(),
				// 'cat'            => $this->get_product_cats(),
				// 'all_products'   => $this->get_all_products(),
				'disable_filter' => wholesalex()->get_setting( '_bulkorder_filter_on_order_form' ),
				'user_id'        => get_current_user_id(),
				'i18n' => array(
					'please_fill_all_fields' => __('Please Fill All Fields','wholesalex-pro'),
					'quick_bulkorder_form' => __('Quick / Bulk Order Form','wholesalex-pro'),
					'products' => __('Products','wholesalex-pro'),
					'quantity' => __('Quantity','wholesalex-pro'),
					'subtotal' => __('Subtotal','wholesalex-pro'),
					'add_products' => __('Add Products','wholesalex-pro'),
					'update_list' => __('Update List','wholesalex-pro'),
					'save_order_list' => __('Save Order List','wholesalex-pro'),
					'add_to_cart' => __('Add to Cart','wholesalex-pro'),
					'calculating' => __('Calculating...','wholesalex-pro'),
					'total' => __('Total:','wholesalex-pro'),
					'order_list_name' => __('Order List Name:','wholesalex-pro'),
					'save' => __('Save','wholesalex-pro'),
					'cancel' => __('Cancel','wholesalex-pro'),
					'all_products' => __('All Products','wholesalex-pro'),
					'category' => __('Category','wholesalex-pro'),
					'tags' => __('Tags','wholesalex-pro'),
					'no_data_found' => __('No Data Found! Please try with another keyword.','wholesalex-pro'),
					'enter_more_character' => __('Enter 2 or more character to search.','wholesalex-pro'),
					'searching' => __('Searching','wholesalex-pro'),
					'search_for_products' => __('Search for Products...','wholesalex-pro'),
				),
			)
		);
		ob_start();
		echo '<div id="wholesalex_bulk_order_form"></div>';
		return ob_get_clean();
	}

	/**
	 * Bulk order rest api init
	 *
	 * @since 1.0.0
	 */
	public function bulkorder_shortcode_rest_api_init() {
		register_rest_route(
			'wholesalex/v1',
			'/order_form/',
			array(
				array(
					'methods'             => 'POST',
					'callback'            => array( $this, 'bulkorder_shortcode_action_callback' ),
					'permission_callback' => function () {
						$__user_id = get_current_user_id();
						$__account_status = get_user_meta( $__user_id, '__wholesalex_status', true );
						return ( 'active' === $__account_status || current_user_can( 'manage_options' ) );
					},
					'args'                => array(),
				),
			)
		);
	}

	/**
	 * Bulk Order Form Actions.
	 *
	 * @param object $server Server.
	 * @return void
	 * @since 1.0.0
	 */
	public function bulkorder_shortcode_action_callback( $server ) {
		$post = $server->get_params();
		if ( ! ( isset( $post['nonce'] ) && wp_verify_nonce( sanitize_key( $post['nonce'] ), 'order_form' ) ) ) {
			return;
		}
		$type = isset( $post['type'] ) ? sanitize_text_field( $post['type'] ) : '';

		switch ( $type ) {
			case 'get_all_categories':
				wp_send_json_success( $this->get_product_cats() );
				break;
			case 'get_all_tags':
				wp_send_json_success( $this->get_product_tags() );
				break;
			case 'search_product':
				$filter = isset( $post['filter'] ) ? sanitize_text_field( $post['filter'] ) : '';
				$query  = isset( $post['query'] ) ? sanitize_text_field( $post['query'] ) : '';

				wp_send_json( $this->get_products( $query, $filter, false ) );

				break;
			case 'get_price':
				$product_id = isset( $post['id'] ) ? $post['id'] : 0;
				$quantity   = isset( $post['qty'] ) ? $post['qty'] : 0;
				if ( 0 == $product_id || $quantity < 1 ) {
					wp_send_json_success(
						array(
							'_price'           => 0,
							'_formatted_price' => wc_price( 0 ),
						)
					);
				}
				$product = wc_get_product( $product_id );
				$__list  = json_decode( $post['list'], true );
				if ( 'product_variation' === $product->post_type ) {
					$product_id = $product->get_parent_id();
				}

				add_filter(
					'wholesalex_dynamic_rules_cart_product_count',
					function ( $count, $__product_id ) use ( $product_id, $quantity ) {
						if ( $__product_id == $product_id ) {
							return $quantity;
						} else {
							return $count;
						}
					},
					15,
					2
				);

				add_filter(
					'wholesalex_dynamic_rules_category_cart_count',
					function ( $count, $cat_id, $__product_id ) use ( $__list, $product_id, $quantity ) {
						if ( $__product_id == $product_id ) {
							$count = $this->bulk_order_category_count( $cat_id, $__list, $product_id, $quantity );
						}
						return $count;
					},
					15,
					3
				);
				add_filter(
					'wholesalex_category_discounts_cart_count',
					function ( $count, $cat_id, $__product_id ) use ( $__list, $product_id, $quantity ) {
						if ( $__product_id == $product_id ) {
							$count = $this->bulk_order_category_count( $cat_id, $__list, $product_id, $quantity );
						}
						return $count;
					},
					10,
					3
				);

				add_filter(
					'wholesalex_profile_discounts_category_cart_count',
					function ( $count, $cat_id, $__product_id ) use ( $__list, $product_id, $quantity ) {
						if ( $__product_id == $product_id ) {
							$count = $this->bulk_order_category_count( $cat_id, $__list, $product_id, $quantity );
						}
						return $count;
					},
					10,
					3
				);

				add_filter(
					'wholesalex_profile_discounts_product_count',
					function ( $count, $__product_id ) use ( $product_id, $quantity ) {
						if ( $__product_id == $product_id ) {
							return $quantity;
						} else {
							return $count;
						}
					},
					15,
					2
				);

				add_filter(
					'wholesalex_single_product_discounts_data',
					function( $data ) use ( $product_id, $quantity ) {
						if ( $data['product']->get_id() === $product_id ) {
							return array(
								'sale_price' => $data['sale_price'],
								'product'    => $data['product'],
								'quantity'   => $quantity,
							);
						}
						return $data;

					}
				);

				$price = 0;
				if ( $product->get_sale_price() ) {
					$price = $quantity * $product->get_sale_price();
				} else {
					$price = $quantity * $product->get_regular_price();
				}

				wp_send_json_success(
					array(
						'_price'           => $price,
						'_formatted_price' => wc_price( $price ),
					)
				);
				break;
			case 'format_price':
				return wp_send_json_success( wc_price( isset( $post['value'] ) ? $post['value'] : 0.00 ) );
			case 'add_to_cart':
				$data = json_decode( $post['data'], true );

				set_transient( 'bulkorder_data_' . get_current_user_id(), $data );

				wp_send_json_success( array( 'redirect_url' => get_permalink( get_option( 'woocommerce_cart_page_id' ) ) ) );

				break;
			case 'save_list':
				$__data  = $post['data'];
				$__data  = wholesalex()->sanitize( $__data );
				$__lists = get_user_meta( get_current_user_id(), '__bulkorder_purchase_lists', true );
				if ( ! $__lists ) {
					$__lists = array();
				}
				$__id             = $__data['id'];
				$__lists[ $__id ] = $__data;
				update_user_meta( get_current_user_id(), '__bulkorder_purchase_lists', $__lists );
				$clear_bulkorder_session = wholesalex()->get_setting( '_bulkorder_clear_session_after_save' );

				wp_send_json_success(
					array(
						'id'               => $__id,
						'message'          => __(
							'Successfully Saved.',
							'wholesalex-pro'
						),
						'is_clear_session' => $clear_bulkorder_session,
					)
				);

				break;
			default:
				// code...
				break;
		}

		if ( 'get' === $type ) {
			$data = array();
			if ( isset( $post['id'] ) && ! empty( $post['id'] ) ) {
					$__id = $post['id'];
				$__lists  = get_user_meta( get_current_user_id(), '__bulkorder_purchase_lists', true );
				if ( isset( $__lists[ $__id ] ) ) {
					$data = $__lists[ $__id ];
				}
			}

			wp_send_json_success(
				array(
					'data' => $data,
				)
			);
		}
	}

	/**
	 * Get Products
	 *
	 * @param string $query Search Query
	 * @return array
	 * @since 1.2.4
	 */
	public function get_products( $query = '', $filter = 'all_products', $without_child = false ) {

		$args = array(
			'status' => 'publish',
			'limit'  => 20,
			'search' => $query,
			'type'   => apply_filters( 'wholesalex_bulkorder_form_product_types', array( 'simple', 'variable', 'variation', 'external' ) ),
		);
		if ( 'all_products' !== $filter ) {
	
			$filter_array = explode( '_', $filter );
			if ( isset( $filter_array[0] ) && 'cat' === $filter_array[0] ) {
				$term             = get_term_by( 'id', $filter_array[1], 'product_cat' );
				$args['category'] = array( $term->slug );
			}
	
			if ( isset( $filter_array[0] ) && 'tag' === $filter_array[0] ) {
				$term        = get_term_by( 'id', $filter_array[1], 'product_tag' );
				$args['tag'] = array( $term->slug );
			}
		}
	
		if ( $query ) {
			$args['s'] = $query;
		}
		$product_objs = wc_get_products( 
			$args
		 );
	
		$products = array();
		foreach ( $product_objs as $__product ) {
			$product_id = $__product->get_id();
	
			if ( $__product->is_type( 'variable' ) ) {
				// If the product is a variable product, only add its variations
				$children_ids = $__product->get_children();
				foreach ( $children_ids as $child_id ) {
					$child_product = wc_get_product( $child_id );
					$products[] = array(
						'value' => strval( $child_product->get_id() ),
						'name'  => $child_product->get_name(),
						'sku'   => $child_product->get_sku(),
					);
				}
			} else {
				// If not a variable product, add the product itself
				$product = array(
					'value' => strval( $product_id ),
					'name'  => $__product->get_name(),
					'sku'   => $__product->get_sku()
				);
				if ( $without_child ) {
					$__childrens = $__product->get_children();
					if ( empty( $__childrens ) ) {
						$products[] = $product;
					}
				} else {
					$products[] = $product;
				}
			}
		}
	
		return array(
			'status' => true,
			'data'   => $products,
		);
	}

	/**
	 * Bulk order category count.
	 *
	 * @param int|string $cat_id Category ID.
	 * @param array      $list Bulk Order Form Data.
	 * @param int|string $product_id Product ID.
	 * @param int|string $product_qty Product Quantity.
	 * @return int $cat_count
	 * @since 1.0.0
	 */
	public function bulk_order_category_count( $cat_id, $list, $product_id, $product_qty ) {
		$cat_count = 0;
		if ( is_array( $list ) ) {
			foreach ( $list as $key => $item ) {
				$__product_id = isset( $item['_product_select'][0]['value'] ) ? $item['_product_select'][0]['value'] : '0';
				$__product    = wc_get_product( $__product_id );

				if ( 'product_variation' === $__product->post_type ) {
					$__product_id = $__product->get_parent_id();
				}

				if ( has_term( $cat_id, 'product_cat', $__product_id ) ) {
					if ( $product_id == $__product_id ) {
						$cat_count += $product_qty;
					} else {
						$cat_count += isset( $item['_product_quantity'] ) ? $item['_product_quantity'] : 0;
					}
				}
			}
		}
		return $cat_count;

	}

	/**
	 * Bulk Order Form Fields
	 *
	 * @since 1.0.0
	 */
	public function get_form_fields() {

		return apply_filters(
			'bulk_order_form_fields',
			array(
				'_form' => array(
					'attr' => array(
						'_product_select'   => array(
							'type'    => 'selectwithsearch',
							'label'   => '',
							'options' => array(),
							'default' => '',
						),
						'_product_quantity' => array(
							'type'        => 'number',
							'label'       => '',
							'default'     => '1',
							'placeholder' => '',
							'help'        => '',
						),
						'_price'            => array(
							'type'        => 'price',
							'label'       => '',
							'default'     => '100',
							'placeholder' => '',
							'help'        => '',
						),
					),
				),
			),
		);
	}

	/**
	 * Grt All Tags
	 *
	 * @return array $term_array
	 * @since 1.0.0
	 */
	public function get_product_tags() {
		$terms      = get_terms( 'product_tag' );
		$term_array = array();
		if ( ! empty( $terms ) && ! is_wp_error( $terms ) ) {
			foreach ( $terms as $term ) {
				$term_array[ $term->term_id ] = $term->name;
			}
		}
		return $term_array;
	}

	/**
	 * Get All Product Categories
	 *
	 * @return array $categories_options
	 * @since 1.0.0
	 */
	public function get_product_cats() {
		// Category Options.
		$categories = get_terms(
			array(
				'taxonomy'   => 'product_cat',
				'hide_empty' => false,
			)
		);
		$categories_options = array();
	
		foreach ($categories as $category) {
			$product_count = $category->count;
	
			// Check if the category has any products.
			if ($product_count > 0) {
				$categories_options[$category->term_id] = $category->name;
			}
		}
		return $categories_options;
	}

	/**
	 * Get All Products
	 *
	 * @return array $products
	 * @since 1.0.0
	 */
	public function get_all_products() {
		// Product Options.

		// @TODO exclude hidden product ids and cat ids
		$product_ids = get_posts(
			array(
				'post_type'   => 'product',
				'post_status' => 'publish',
				'numberposts' => -1,
				'fields'      => 'ids',
			)
		);

		$products = array();
		foreach ( $product_ids as $product_id ) {
			$__product = wc_get_product( $product_id );
			if ( ! $__product ) {
				continue;
			}

			$__childrens = $__product->get_children();
			$__cats      = wc_get_product_term_ids( $product_id, 'product_cat' );
			$__tags      = $__product->get_tag_ids();

			if ( empty( $__childrens ) ) {
				$product    = array(
					'value'      => $product_id,
					'name'       => $__product->get_name(),
					'tags'       => $__tags,
					'categories' => $__cats,
					'sku'        => $__product->get_sku(),
				);
				$products[] = $product;
			} else {
				foreach ( $__childrens as $child_id ) {
					$__children = wc_get_product( $child_id );
					if ( ! $__children ) {
						continue;
					}
					$products[] = array(
						'value'      => $child_id,
						'name'       => $__children->get_name(),
						'tags'       => $__tags,
						'categories' => $__cats,
						'sku'        => $__children->get_sku(),
					);
				}
			}
		}

		return $products;
	}

	/**
	 * Bulkorder setting on Global Settings
	 *
	 * @param array $sections Settings Section.
	 * @return array
	 * @since 1.0.7
	 */
	public function bulkorder_settings( $sections ) {
		$bulkorder_shortcode = apply_filters( 'wholesalex_bulkorder_addon_shortcode', 'wholesalex_bulkorder_form' );

		$settings                       = array(
			'_bulkorder_clear_session_after_save'      => array(
				'type'    => 'switch',
				'label'   => __( 'Clear session after saving purchase list ', 'wholesalex-pro' ),
				'desc'    => __( 'Clear Order Form Session', 'wholesalex-pro' ),
				'help'    => '',
				'default' => 'no',
			),
			'_bulkorder_clear_cart_after_save'         => array(
				'type'    => 'switch',
				'label'   => __( 'Clear cart page after saving purchase list ', 'wholesalex-pro' ),
				'desc'    => __( 'Clear Cart Page Session ', 'wholesalex-pro' ),
				'help'    => '',
				'default' => 'no',
			),
			'_bulkorder_filter_on_order_form'          => array(
				'type'    => 'switch',
				'label'   => __( 'Disable Filter on Bulkorder Form', 'wholesalex-pro' ),
				'desc'    => __( 'Disable ', 'wholesalex-pro' ),
				'help'    => '',
				'default' => 'no',
			),
			'_bulkorder_filter_on_order_form'          => array(
				'type'    => 'switch',
				'label'   => __( 'Disable Filter on Bulkorder Form', 'wholesalex-pro' ),
				'desc'    => __( 'Disable ', 'wholesalex-pro' ),
				'help'    => '',
				'default' => 'no',
			),
			'_bulkorder_copy_shortcode'                => array(
				'type'    => 'shortcode',
				'label'   => __( 'Bulkorder Form Shortcode', 'wholesalex-pro' ),
				'desc'    => __( 'Get Shortcode ', 'wholesalex-pro' ),
				'help'    => '',
				'default' => sprintf( '[%s]', $bulkorder_shortcode ),
				'tooltip' => __( 'Click on the button to directly copy the shortcode.', 'wholesalex-pro' ),
			),
			'_bulkorder_my_account_endpoint'           => array(
				'type'        => 'text',
				'label'       => __( 'Bulkorder My Account Page Endpoint', 'wholesalex-pro' ),
				'placeholder' => '',
				'help'        => '',
				'default'     => 'wholesalex-bulkorder',
			),
			'_bulkorder_my_account_endpoint_title'     => array(
				'type'        => 'text',
				'label'       => __( 'Bulkorder My Account Endpoint Title', 'wholesalex-pro' ),
				'placeholder' => '',
				'help'        => '',
				'default'     => __( 'Bulk Order', 'wholesalex-pro' ),
			),
			'_purchase_lists_myaccount_endpoint'       => array(
				'type'        => 'text',
				'label'       => __( 'Purchase Lists My Account Page Endpoint', 'wholesalex-pro' ),
				'placeholder' => '',
				'help'        => '',
				'default'     => 'wholesalex-purchase-list',
			),
			'_purchase_lists_myaccount_endpoint_title' => array(
				'type'        => 'text',
				'label'       => __( 'Purchase Lists My Account Endpoint Title', 'wholesalex-pro' ),
				'placeholder' => '',
				'help'        => '',
				'default'     => __( 'Purchase Lists', 'wholesalex-pro' ),
			),
		);
		$sections['bulkorder']['label'] = __( 'Bulk Order', 'wholesalex-pro' );
		$sections['bulkorder']['attr']  = $settings;
		return $sections;
	}

	public function bulkorder_menu_title( $title ) {
		if ( '' != wholesalex()->get_setting( '_bulkorder_my_account_endpoint_title', 'Bulk Order' ) ) {
			$title = wholesalex()->get_setting( '_bulkorder_my_account_endpoint_title', 'Bulk Order' );
		}
		return $title;
	}

	public function bulkorder_menu_endpoint( $endpoint ) {
		if ( '' != wholesalex()->get_setting( '_bulkorder_my_account_endpoint', 'wholesalex-bulkorder' ) ) {
			$endpoint = wholesalex()->get_setting( '_bulkorder_my_account_endpoint', 'wholesalex-bulkorder' );
		}
		return $endpoint;
	}

}
