<?php
/**
 * WholesaleX Purchase List
 *
 * @package WHOLESALEX_PRO
 * @since 1.0.7
 */

namespace WHOLESALEX_PRO;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}
/**
 * WholesaleX Purchase Lists
 */
class PurchaseLists {

	/**
	 * Construtor
	 */
	public function __construct() {
		add_filter( 'wholesalex_bulkorder_addon_purchase_list_my_account_title', array( $this, 'purchase_lists_menu_title' ) );
		add_filter( 'wholesalex_bulkorder_addon_purchase_list_my_account_end_point', array( $this, 'purchase_lists_menu_endpoint' ) );
		add_filter( 'woocommerce_account_menu_items', array( $this, 'add_purchase_list_on_myaccount_menu' ) );

		add_action( 'init', array( $this, 'purchase_list_add_end_point' ) );

		add_filter( 'woocommerce_get_query_vars', array( $this, 'add_purchase_list_to_woocommerce_query_vars' ) );

		add_action( 'woocommerce_cart_actions', array( $this, 'add_save_order_list_button' ) );

		add_action( 'rest_api_init', array( $this, 'purchase_list_shortcode_rest_api_init' ) );

		add_action( 'wp_ajax_save_bulk_order_purchase_list', array( $this, 'save_bulk_order_purchase_list' ) );

		add_action( 'woocommerce_after_cart', array( $this, 'save_cart_to_purchase_list_modal' ) );

		$menu_endpoint = apply_filters( 'wholesalex_bulkorder_addon_purchase_list_my_account_end_point', __( 'wholesalex-purchase-list', 'wholesalex-pro' ) );

		add_action( 'woocommerce_account_' . $menu_endpoint . '_endpoint', array( $this, 'purchase_list' ) );

	}

	/**
	 * Purchase List Shortcode Render
	 *
	 * @param array $atts    Shortcode attributes. Default empty.
	 * @since 1.0.0
	 */
	public function purchase_list_shortcode_render( $atts = array() ) {
		$menu_endpoint = apply_filters( 'wholesalex_bulkorder_addon_my_account_end_point', __( 'wholesalex-bulkorder', 'wholesalex-pro' ) );
		wp_enqueue_script( 'wholesalex_bulkorder' );
		wp_enqueue_style( 'dashicons' );
		$user_id = get_current_user_id();
		$__lists = get_user_meta( $user_id, '__bulkorder_purchase_lists', true );
		$__lists = apply_filters( 'wholesalex_addon_bulkorder_purchase_lists', $__lists, $user_id );
		if ( ! is_array( $__lists ) ) {
			$__lists = array();
		}
		foreach ( $__lists as $key => $list ) {
			$__lists[ $key ]['price_html'] = wc_price( $list['total_price'] );
		}
		wp_localize_script(
			'wholesalex_bulkorder',
			'wholesalex_bulkorder',
			array(
				'nonce'              => wp_create_nonce( 'order_form' ),
				'bulkorder_endpoint' => wc_get_account_endpoint_url( $menu_endpoint ),
				'lists'              => $__lists,
				'i18n'=> array(
					'view_list' => __('View List','wholesalex-pro'),
					'download' => __('Download','wholesalex-pro'),
					'showing_x_purchase_lists' => sprintf(__('Showing %s purchase lists','wholesalex-pro'),'##'),
					'previous' => __('Previous','wholesalex-pro'),
					'next' => __('Next','wholesalex-pro'),
					'previous' => __('Previous','wholesalex-pro'),
					'purchase_list' => __('Purchase list','wholesalex-pro'),
					'search' => __('Search...','wholesalex-pro'),
					'add_new_list' => __('Add New List','wholesalex-pro'),
					'name' => __('Name','wholesalex-pro'),
					'items' => __('Items','wholesalex-pro'),
					'quantity' => __('Quantity','wholesalex-pro'),
					'total_price' => __('Total Price','wholesalex-pro'),
					'actions' => __('Actions','wholesalex-pro'),
					'no_purchase_lists_found' => __('No Purchase Lists Found!','wholesalex-pro'),
				),
				'currency_format_num_decimals'                    => wc_get_price_decimals(),
				'currency_format_symbol'                          => get_woocommerce_currency_symbol( ),
				'currency_format_decimal_sep'                     => esc_attr( wc_get_price_decimal_separator() ),
				'currency_format_thousand_sep'                    => esc_attr( wc_get_price_thousand_separator() ),
				'currency_format'                                 => esc_attr( str_replace( array( '%1$s', '%2$s' ), array( '%s', '%v' ), get_woocommerce_price_format() ) ),
				'rounding_precision'                              => wc_get_rounding_precision(), 
			)
		);
		echo '<div id="wholesalex_bulkorder_purchase_lists"></div>';
	}
	/**
	 * Purchase List rest api init
	 *
	 * @since 1.0.0
	 */
	public function purchase_list_shortcode_rest_api_init() {
		register_rest_route(
			'bulkorder/v1',
			'/purchase_list/',
			array(
				array(
					'methods'             => 'POST',
					'callback'            => array( $this, 'purchase_list_action_callback' ),
					'permission_callback' => function () {
						$__user_id = get_current_user_id();
						$__account_status = get_user_meta( $__user_id, '__wholesalex_status', true );
						return ( 'active' === $__account_status || current_user_can( 'manage_options' ) );
					},
					'args'                => array(),
				),
			)
		);
	}

	/**
	 * Purchase Lists Actions.
	 *
	 * @param object $server Server.
	 * @return void
	 * @since 1.0.0
	 */
	public function purchase_list_action_callback( $server ) {
		$post = $server->get_params();
		if ( ! ( isset( $post['nonce'] ) && wp_verify_nonce( sanitize_key( $post['nonce'] ), 'order_form' ) ) ) {
			return;
		}

		$type = isset( $post['type'] ) ? sanitize_text_field( $post['type'] ) : '';

		if ( 'get' === $type ) {
			$user_id = get_current_user_id();
			$__lists = get_user_meta( $user_id, '__bulkorder_purchase_lists', true );
			$__lists = apply_filters( 'wholesalex_addon_bulkorder_purchase_lists', $__lists, $user_id );
			if ( ! is_array( $__lists ) ) {
				$__lists = array();
			}
			foreach ( $__lists as $key => $list ) {
				$__lists[ $key ]['price_html'] = wc_price( $list['total_price'] );
			}
			wp_send_json_success(
				array(
					'bulkorder_endpoint' => wc_get_account_endpoint_url( 'wholesalex-bulkorder' ),
					'lists'              => $__lists,
				)
			);

		} elseif ( 'set' === $type ) {
			$data = wholesalex()->sanitize( $post['data'] );
			update_user_meta( get_current_user_id(), '__bulkorder_purchase_lists', $data );
			wp_send_json_success( array( 'message' => __( 'Successfully Deleted.', 'wholesalex-pro' ) ) );

		}
	}


	/**
	 * Save Cart Items as purchase list
	 *
	 * @return void
	 */
	public function save_bulk_order_purchase_list() {
		if ( ! ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'order_form' ) ) ) {
			wp_send_json_error( 'Nonce Verification Failed!', 403 );
		}
		$list_name = isset( $_POST['name'] ) ? sanitize_text_field( wp_unslash( $_POST['name'] ) ) : '';
		$list_id   = isset( $_POST['id'] ) ? sanitize_text_field( wp_unslash( $_POST['id'] ) ) : '';

		$cart_contents = WC()->cart->get_cart_contents();

		$list                   = array();
		$list['items']          = array();
		$list['quantity_count'] = 0;
		$list['item_count']     = 0;
		$list['name']           = $list_name;
		$list['id']             = $list_id;
		$list['total_price']    = 0;

		foreach ( $cart_contents as $key => $cart_item ) {

			$item['id']           = $key;
			$__product_id         = $cart_item['product_id'];
			$__product            = wc_get_product( $__product_id );
			$__quantity           = $cart_item['quantity'];
			$__price              = $cart_item['line_subtotal'];
			$list['total_price'] += $__price;
			$__product_name       = $__product->get_title();

			$item['_product_select'] = array(
				array(
					'name'  => $__product_name,
					'value' => $__product_id,
				),
			);

			$item['_product_quantity'] = $__quantity;
			$item['_price']            = $__price;
			$item['_formatted_price']  = wc_price( $__price );
			$item['_product_sku']      = $__product->get_sku();

			$list['quantity_count'] += $__quantity;
			$list['item_count']++;

			$list['items'][] = $item;
		}

		$list['price_html'] = wc_price( $list['total_price'] );

		$__lists = get_user_meta( get_current_user_id(), '__bulkorder_purchase_lists', true );

		if ( ! $__lists ) {
			$__lists = array();
		}
		$__lists[ $list_id ] = $list;

		update_user_meta( get_current_user_id(), '__bulkorder_purchase_lists', $__lists );

		$clear_cart = wholesalex()->get_setting( '_bulkorder_clear_cart_after_save' );
		if ( 'yes' === $clear_cart ) {
			WC()->cart->empty_cart();
		}

		wp_send_json_success(
			array(
				'message'    => __( 'Successfully Saved as purchase list.', 'wholesalex-pro' ),
				'clear_cart' => $clear_cart,
			)
		);

		wp_die();

	}

	/**
	 * Add WholesaleX BulkOrder in Account Menu Items
	 *
	 * @param array $menu_links Array of menu links slug.
	 * @since 1.0.7
	 */
	public function add_purchase_list_on_myaccount_menu( $menu_links ) {
		$menu_title = apply_filters( 'wholesalex_bulkorder_addon_purchase_list_my_account_title', __( 'Purchase Lists', 'wholesalex-pro' ) );

		$menu_endpoint = apply_filters( 'wholesalex_bulkorder_addon_purchase_list_my_account_end_point', __( 'wholesalex-purchase-list', 'wholesalex-pro' ) );

		$menu_links = array_slice( $menu_links, 0, 5, true )
		+ array( $menu_endpoint => $menu_title )
		+ array_slice( $menu_links, 5, null, true );
		return $menu_links;
	}

	/**
	 * WholesaleX Bulkorder Add End Point
	 *
	 * @since 1.0.7
	 */
	public function purchase_list_add_end_point() {
		$menu_endpoint = apply_filters( 'wholesalex_bulkorder_addon_purchase_list_my_account_end_point', __( 'wholesalex-purchase-list', 'wholesalex-pro' ) );

		add_rewrite_endpoint( $menu_endpoint, EP_PAGES );

	}
	/**
	 * WholesaleX Bulkorder Page
	 *
	 * @since 1.0.7
	 */
	public function purchase_list() {
		$this->purchase_list_shortcode_render();
	}

	/**
	 * Add Custom Query Vars in WooCommerce
	 *
	 * @param array $query Query.
	 * @since 1.0.7
	 */
	public function add_purchase_list_to_woocommerce_query_vars( $query ) {
		$menu_endpoint           = apply_filters( 'wholesalex_bulkorder_addon_purchase_list_my_account_end_point', __( 'wholesalex-purchase-list', 'wholesalex-pro' ) );
		$query[ $menu_endpoint ] = $menu_endpoint;
		return $query;
	}

	/**
	 * Add Save Order List Button on Cart Page
	 *
	 * @return void
	 * @since 1.0.7
	 */
	public function add_save_order_list_button() {

		wp_enqueue_script( 'wholesalex_bulkorder_cart', WHOLESALEX_PRO_URL . 'addons/bulkorder/js/save_purchase_list.js', array( 'wp-api-fetch', 'jquery' ), WHOLESALEX_PRO_VER, true );
		wp_localize_script( 'wholesalex_bulkorder_cart', 'bulkorder', array( 'ajax' => admin_url( 'admin-ajax.php' ) ) );

		$purchase_list_btn_text = wholesalex()->get_setting( '_language_bulkorder_cart_page_purchase_list_save_text', __( 'Save as List', 'wholesalex-pro' ) );

		?>
		<div class="wholesalex_loader"></div>
		<button class="button bulkorder_save_order_btn" ><?php echo esc_html( $purchase_list_btn_text ); ?></button>

		<?php
	}


	/**
	 * Save Cart Data to Purchase List Modal
	 *
	 * @return void
	 * @since 1.0.7
	 */
	public function save_cart_to_purchase_list_modal() {
		?>
		<div id="wholesalex_save_cart_modal">
				<div class="modal_content">
					<div class="modal_header">
						<div class="modal-close-btn">
							<span class="close-modal-icon dashicons dashicons-no-alt" ></span>
						</div>
					</div>
					<form id="wholesalex_save_cart_modal_form">
					<label htmlFor="list_name" class="input_label"><?php echo esc_html__( 'Order List Name:', 'wholesalex-pro' ); ?></label>
						<input name='name' type='text' value='' />
						<input type='hidden' name="nonce" value="<?php echo esc_attr( wp_create_nonce( 'order_form' ) ); ?>"/>
						<div class="modal_buttons">
							<button class="button save-btn" > <?php echo esc_html__( 'Save', 'wholesalex-pro' ); ?></button>
							<button class="button cancel-btn cancel_modal"><?php echo esc_html__( 'Cancel', 'wholesalex-pro' ); ?></button>
						</div>
					</form>
				</div>
			</div>
		<?php
	}

	public function purchase_lists_menu_title( $title ) {
		if ( '' != wholesalex()->get_setting( '_purchase_lists_myaccount_endpoint_title', 'Purchase Lists' ) ) {
			$title = wholesalex()->get_setting( '_purchase_lists_myaccount_endpoint_title', 'Purchase Lists' );
		}
		return $title;
	}

	public function purchase_lists_menu_endpoint( $endpoint ) {
		if ( '' != wholesalex()->get_setting( '_purchase_lists_myaccount_endpoint', 'wholesalex-purchase-list' ) ) {
			$endpoint = wholesalex()->get_setting( '_purchase_lists_myaccount_endpoint', 'wholesalex-purchase-list' );
		}
		return $endpoint;
	}

}
