<?php
/**
 * WholesaleX Conversation Account Page
 *
 * @package WHOLESALEX_PRO
 * @since 1.0.0
 */

namespace WHOLESALEX_PRO;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Account Page Class
 *
 * @since 1.0.0
 */
class AccountPage {

	public $emails = array();


	/**
	 * Constructor
	 */
	public function __construct() {
		$__show_conversation_settings = wholesalex()->get_setting( '_settings_show_conversation_my_account_page', 'yes' );

		if ( 'yes' === $__show_conversation_settings ) {
			add_filter( 'wholesalex_conversation_addon_my_account_end_point', array( $this, 'conversation_end_point' ) );
			$this->conversation_endpoint();
			add_filter( 'woocommerce_account_menu_items', array( $this, 'conversation_link_my_account' ) );
			add_action( 'woocommerce_account_' . $this->get_conversation_endpoint() . '_endpoint', array( $this, 'conversation_content' ), 9999999999 );
		}

		add_action( 'wp_ajax_wholesalex_new_conversation', array( $this, 'create_conversation' ) );
		add_action( 'wp_ajax_wholesalex_conversation_reply', array( $this, 'reply_conversation' ) );
		add_filter('woocommerce_email_actions', array($this,'wholesalex_add_email_actions'));
		add_action('woocommerce_email',array($this,'handle_conversation_email'));

	}

	public function wholesalex_add_email_actions($actions) {
		$actions[]='wholesalex_conversation_create_front_end';
		$actions[]='wholesalex_conversation_reply_front_end';
		$actions[]='wholesalex_conversation_create_back_end';
		$actions[]='wholesalex_conversation_reply_back_end';
		return $actions;
	}
	

	public function handle_conversation_email($wc_emails){
		$this->emails = $wc_emails->get_emails();
		add_action( 'wholesalex_conversation_create_front_end_notification', array( $this, 'conversation_email_sent' ), 10, 3 );
		add_action( 'wholesalex_conversation_create_back_end_notification', array( $this, 'conversation_email_sent' ), 10, 3 );
		add_action( 'wholesalex_conversation_reply_front_end_notification', array( $this, 'conversation_email_reply' ), 10, 3 );
		add_action( 'wholesalex_conversation_reply_back_end_notification', array( $this, 'conversation_email_reply' ), 10, 3 );
	}

	/**
	 * Send An Email to Conversation admin user.
	 *
	 * @param string|int Conversation Conversation User ID.
	 * @param string|int $parent_user_id Parent User ID.
	 * @return void
	 * @since 1.0.9
	 */
	public function conversation_email_sent($id, $user_id, $user_emails ) {
		$email = $this->emails['WholesaleX_New_Conversation_Email'];
		$email->trigger($id, $user_id, $user_emails );
	}

	/**
	 * Send An Email to Conversation Reply user.
	 *
	 * @param string|int Conversation Conversation User ID.
	 * @param string|int $parent_user_id Parent User ID.
	 * @return void
	 * @since 1.0.9
	 */
	public function conversation_email_reply( $post_id, $user_id, $user_emails ) {
		$email = $this->emails['WholesaleX_New_Conversation_Reply_Email'];
		$email->trigger( $post_id, $user_id, $user_emails );
	}

	/**
	 * Rewrite Conversation Endpoint and Save Conversation Content.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function conversation_endpoint() {
		add_rewrite_endpoint( $this->get_conversation_endpoint(), EP_ROOT | EP_PAGES );
	}

	public function conversation_end_point() {
		return wholesalex()->get_setting('_conversation_my_account_endpoint','wsx-conversation');
	}

	/**
	 * Add Conversation Link On My Account Page
	 *
	 * @param array $items array of all links.
	 * @return array updated links.
	 * @since 1.0.0
	 * @since 1.0.1 Move Conversation Position before Logout.
	 * @since 1.0.5 wholesalex_addon_conversation_my_account_menu_title Filter Hook Added.
	 */
	public function conversation_link_my_account( $items ) {
		$items = array_slice( $items, 0, 5, true )
		+ array( $this->get_conversation_endpoint() => $this->get_conversation_endpoint_title() )
		+ array_slice( $items, 5, null, true );
		return $items;
	}

	public static function view_conversation( $view_id, $back_url = '' ) {
		if ( '' == $back_url ) {
			$menu_endpoint    = apply_filters( 'wholesalex_conversation_addon_my_account_end_point', __( 'wsx-conversation', 'wholesalex-pro' ) );
			$conversation_url = wc_get_account_endpoint_url( $menu_endpoint );

		} else {
			$conversation_url = $back_url;
		}
		$user_id = get_current_user_id();
		$uri     = isset( $_SERVER['REQUEST_URI'] ) ? esc_url_raw( wp_unslash( $_SERVER['REQUEST_URI'] ) ) : '#';

		$post_author_id                       = get_post_field( 'post_author', $view_id );
		$has_eligibility_to_view_conversation = apply_filters( 'wholesalex_addon_conversation_has_eligibility_to_view_conversation', ( $post_author_id == $user_id ), $post_author_id, $view_id );
        if ($has_eligibility_to_view_conversation) { //phpcs:ignore
			$authors   = apply_filters( 'wholesalex_addon_conversation_view_author_ids', array( $user_id, $post_author_id ), $user_id );
			$args      = array(
				'p'              => $view_id,
				'post_type'      => 'wsx_conversation',
				'post_status'    => 'publish',
				'posts_per_page' => -1,
				// 'author'         => $user_id,
				'author__in'     => array( $user_id, $post_author_id ),
			);
			$the_query = new \WP_Query( $args );

			if ( $the_query->have_posts() ) {
				while ( $the_query->have_posts() ) {
					$the_query->the_post();
					echo '<div class="wsx-conversation-back">';
					echo '<div class="wsx-conversation-header">' . esc_html( get_the_title() ) . '</div>';
					echo '<a href="' . esc_url( $conversation_url ) . '"><div class="wholesalex-btn wholesalex-btn-md wholesalex-btn-primary wsx-go-back"> <span class="dashicons dashicons-arrow-left-alt"></span><span>' . esc_html( wholesalex()->get_language_n_text( '_language_conversation_go_back', __( 'Go back.', 'wholesalex-pro' ) ) ) . '</span></div></a>';
					echo '</div>';
					$content = get_the_content();
					if ( has_blocks( $content ) ) {
						$reply_lists = parse_blocks( $content );
						foreach ( $reply_lists as $val ) {
							$user          = get_userdata( $val['attrs']['author'] );
							$display_name  = $user ? $user->display_name : __( 'Deleted User', 'wholesalex-pro' );
							$display_name  = apply_filters( 'wholesalex_addon_conversation_reply_display_name', $display_name, $user ? $user->ID : 0, $post_author_id );
							$display_class = apply_filters( 'wholesalex_addon_conversation_reply_class', in_array( $val['attrs']['author'], $authors ) ? 'wsx-reply-right' : 'wsx-reply-left', $val['attrs']['author'], $view_id );
							?>
                            <div class="wsx-reply-lists <?php echo $display_class; //phpcs:ignore ?> "> 
								<div class="wsx-reply-meta">
                                    <span class="wsx-name"><?php echo  $display_name; //phpcs:ignore ?></span> |
									<span class="wsx-time"><?php echo esc_html( human_time_diff( strtotime( $val['attrs']['time'] ), time() ) ); ?> <?php echo esc_html( wholesalex()->get_language_n_text( '_language_conversation_x_time_ago', __( 'ago', 'wholesalex-pro' ) ) ); ?></span> |
									<span class="wsx-date"><?php echo esc_html( get_date_from_gmt( $val['attrs']['time'], 'Y-m-d h:i:s A' ) ); ?></span>

								</div>
								<div class="wsx-content">
								<?php
								do_action('wholesalex_addon_conversation_before_message_content',$view_id,$val);
								$output = $val['innerHTML'];
								echo wp_kses(
									$output,
									array(
										'br'     => array(),
										'a'      => array( 'href' => array() ),
										'div'    => array(
											'class' => array(),
											'id'    => array(),
										),
										'span'   => array(
											'class' => array(),
											'id'    => array(),
										),
										'button' => array(
											'class' => array(),
											'id'    => array(),
										),
										'table'  => array(
											'class' => array(),
											'id'    => array(),
										),
										'thead'  => array(),
										'tbody'  => array(),
										'tr'     => array(),
										'td'     => array(),
										'th'     => array(),
									)
								);
								do_action( 'wholesalex_addon_conversation_my_account_after_reply_message', $view_id );
								?>
								</div>
							</div>
							<?php
						}

						do_action( 'wholesalex_addon_conversation_my_account_after_messages', $view_id );
					}
				}
			}
			wp_reset_postdata();
			self::reply_conversation_form( $view_id );
			do_action( 'wholesalex_addon_conversation_my_account_after_reply_form' );
		} else {
			echo '<h3>' . esc_html( wholesalex()->get_language_n_text( '_language_conversation_error', __( 'Something Went Wrong!', 'wholesalex-pro' ) ) ) . '</h3>';
		}
	}



	/**
	 * Conversation Content
	 *
	 * @since 1.0.0
	 * @since 1.0.1 Preserve Conversation New Line, Fix Pagination Bug and Rewrite Conversations.
	 * @since 1.0.5 Conversation Back Button Issue Fixed.
	 * @since 1.2.3 Add wholesalex-pro-public script and localize nonce
	 */
	public function conversation_content() {
		wp_enqueue_style( 'dashicons' );
		wp_enqueue_script( 'wholesalex-pro-public' );
		wp_localize_script(
			'wholesalex-pro-public',
			'wholesalex_conversation',
			array(
				'create_nonce'       => wp_create_nonce( 'wholesalex-new-conversation' ),
				'recaptcha_site_key' => wholesalex()->get_setting( '_settings_google_recaptcha_v3_site_key' ),
				'recaptcha_status'   => wholesalex()->get_setting( 'wsx_addon_recaptcha' ),
			)
		);

		do_action( 'wholesalex_conversation_metabox_content_account_page' );
		$conversation_url = wc_get_account_endpoint_url( 'wsx-conversation' );
		$user_id          = get_current_user_id();

		if (isset($_GET['wsxview'])) { //phpcs:ignore
			self::view_conversation( sanitize_key( $_GET['wsxview'] ) );
		} else {

			?>
			<div class="wsx-conversation-wrapper">
				<div class="wsx-conversation-title">
					<div class="wsx-conversation-header"><?php echo esc_html( wholesalex()->get_language_n_text( '_language_conversation_heading', __( 'Conversations', 'wholesalex-pro' ) ) ); ?></div>
					<button class="wsx-title wholesalex-btn wholesalex-btn-primary"><?php echo esc_html( wholesalex()->get_language_n_text( '_language_new_conversation', __( 'New Conversation', 'wholesalex-pro' ) ) ); ?></button>
				</div>
				<div class="wsx-content" style="display:none;">
					<?php echo $this->new_conversation(); //phpcs:ignore ?>
				</div>
			</div>

			<?php

			/**
			 * Pagination and Table Updated.
			 *
			 * @since 1.0.2
			 */
			$paged = ( isset( $_GET['conv'] ) && is_numeric( $_GET['conv'] ) ) ? $_GET['conv'] : 1; //phpcs:ignore

			/**
			 * Conversation Type Meta Compare Added to filter out quote request.
			 *
			 * @since 1.0.5
			 */
			$args      = array(
				'post_type'      => 'wsx_conversation',
				'post_status'    => 'publish',
				'posts_per_page' => apply_filters( 'wholesalex_conversation_list_per_page_my_account_page', 5 ),
				'paged'          => $paged,
				// 'author'         =>  get_current_user_id(),
				'author__in'     => apply_filters( 'wholesalex_addon_conversation_view_author_ids', array( $user_id ), $user_id ),
				'orderby'        => 'modified',
				'order'          => 'DESC',
				'meta_key'       => '__conversation_type',
				'meta_value'     => ( 'yes' !== wholesalex()->get_setting( 'wsx_addon_raq' ) ) ? 'quote' : '',
				'meta_compare'   => '!=',
			);
			$new_query = new \WP_Query( $args );

			$conversation_columns = apply_filters(
				'wholesalex_conversation_my_account_columns',
				array(
					'title'  => wholesalex()->get_language_n_text( '_language_conversations_list_title', __( 'Conversation', 'wholesalex-pro' ) ),
					'type'   => wholesalex()->get_language_n_text( '_language_conversations_list_type', __( 'Type', 'wholesalex-pro' ) ),
					'time'   => wholesalex()->get_language_n_text( '_language_conversations_last_reply_time', __( 'Last Reply Time', 'wholesalex-pro' ) ),
					'action' => wholesalex()->get_language_n_text( '_language_conversations_action', __( 'Action', 'wholesalex-pro' ) ),
				)
			);

			if ( $new_query->have_posts() ) {
				?>
				<div class="wsx-conversation-table-wrapper">
				<table class="wsx-conversation-list">
					<thead class="wsx-conversation-list-head">
						<tr>
							<?php
							foreach ( $conversation_columns as $column_id => $column_name ) {
								?>
									<th class="wsx-conversation-list-<?php echo esc_html( $column_id ); ?>">
										<?php echo esc_html( $column_name ); ?>
									</th>
								<?php
							}
							?>
						</tr>
					</thead>

					<?php
					/**
					 * Last Reply Time Issue Fixed
					 *
					 * @since 1.0.5
					 */
					while ( $new_query->have_posts() ) :
						$new_query->the_post();
						$post_id           = get_the_ID();
						$conversation_type = get_post_meta( $post_id, '__conversation_type', true );
						$status            = get_post_meta( $post_id, '__conversation_status', true );

						?>
						<tbody class="wsx-conversation-list-body">
							<tr>
								<?php
								foreach ( $conversation_columns as $column_id => $column_name ) {
									switch ( $column_id ) {
										case 'title':
											?>
												<td class="wsx-conversation-list-item">
													<div class="wsx-conversation-list-open conversation-title">
														<span class=" <?php echo 'new' === $status ? 'new_conversation' : 'open_conversation'; ?>"></span>
														<a href="<?php echo esc_url( add_query_arg( 'wsxview', $post_id ) ); ?>" <?php echo 'new' === $status ? 'onclick="conversationOpen()"' : ''; ?> data-security="<?php echo wp_create_nonce( 'view_conversation_' . esc_attr( $post_id ) ); //phpcs:ignore ?>" data-conv-id="<?php echo esc_attr( $post_id ); ?>"> <?php echo esc_html( get_the_title() ); ?></a>
													</div>
												</td>
												<?php
											break;
										case 'type':
											?>
												<td class="wsx-conversation-list-item conversation-type">
												<?php echo esc_html( $conversation_type ); ?>
												</td>
												<?php
											break;
										case 'time':
											?>
													<td class="wsx-conversation-list-item">
														<span class="wsx-conversation-time"><?php echo esc_html( human_time_diff( get_post_modified_time( 'U', true ), time() ) ) . ' ' . esc_html( wholesalex()->get_language_n_text( '_language_conversation_x_time_ago', __( 'ago', 'wholesalex-pro' ) ) ); ?> </span>
														<span class="wsx-conversation-date"><?php echo esc_html( get_the_modified_date() ); ?></span>
													</td>
												<?php
											break;
										case 'action':
											?>
												<td class="wsx-conversation-list-item conversation-action">
													<a class="button wholesalex-btn wholesalex-btn-black wholesalex-btn-sm" href="<?php echo esc_url( add_query_arg( 'wsxview', $post_id ) ); ?>" <?php echo 'new' === $status ? 'onclick="conversationOpen()"' : ''; ?> data-security="<?php echo wp_create_nonce( 'view_conversation_' . $post_id );//phpcs:ignore ?>"  data-conv-id="<?php echo esc_attr($post_id); ?>"><?php echo esc_html( wholesalex()->get_language_n_text( '_language_conversations_view', __( 'View', 'wholesalex-pro' ) ) ); ?></a>
												</td>
												<?php
											break;

										default:
											do_action( 'wholesalex_conversation_my_account_default_column_values', $column_id, $post_id );
											break;
									}
								}

								?>
							</tr>
						</tbody>
						<?php endwhile; ?>
				</table>
				<div>

					<div>
						<?php

						//phpcs:ignore
						echo paginate_links(
							array(
								'total'   => $new_query->max_num_pages,
								'format'  => '?conv=%#%',
								'current' => $paged,
							)
						);
						?>
					</div>
				<?php

			}
			wp_reset_postdata();
		}
	}


	/**
	 * Create Conversation
	 *
	 * @since 1.2.4 Rewrite using wp ajax to create conversation
	 */
	public function create_conversation() {
		if ( isset( $_POST['wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['wpnonce'] ), 'wholesalex-new-conversation' ) ) {
			if ( isset( $_POST['wsx-frontend-editor'] ) && isset( $_POST['type'] ) && isset( $_POST['title'] ) ) {

				do_action( 'wholesalex_before_conversation_create', $_POST );

				$user_id = get_current_user_id();
				$content = sanitize_textarea_field( wp_unslash( $_POST['wsx-frontend-editor'] ) );
				$reply_id = 1;
				$data    = array(
					'post_title'   => sanitize_text_field( wp_unslash( $_POST['title'] ) ),
					'post_type'    => 'wsx_conversation',
					'post_author'  => $user_id,
					'post_status'  => 'publish',
					'post_content' => '<!-- wp:wsx_reply {"id":"' . $reply_id . '","author":"' . $user_id . '", "time":"' . gmdate( 'Y-m-d h:i:s A' ) . '"} -->' . $content . '<!-- /wp:wsx_reply -->',
				);
				$id      = wp_insert_post( $data );
				if ( $id && isset( $_POST['conversation_type'] ) && ! empty( $_POST['conversation_type'] ) ) {
					update_post_meta( $id, '__conversation_type', sanitize_text_field( wp_unslash( $_POST['conversation_type'] ) ) );
					// When new conversation created, it status should be new.
					/**
					 * Conversation Status Changed open to new
					 *
					 * @since 1.0.5
					 */
					update_post_meta( $id, '__conversation_status', 'new' );
					update_post_meta($id,'wholesalex_conversation_reply_id',$reply_id);
				}
				do_action( 'wholesalex_conversation_created', $id , $user_id);
				$admin_email = wholesalex()->get_setting('_conversation_admin_emails','okokok');
				do_action( 'wholesalex_conversation_create_front_end', $id , $user_id, $admin_email);				
			}
			wp_send_json_success( 'Success' );
		}
		die();

	}

	/**
	 * Reply Conversation
	 *
	 * @since 1.2.4 Rewrite using wp ajax to create conversation
	 */
	public function reply_conversation() {
		if ( isset( $_POST['wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['wpnonce'] ), 'wholesalex-conversation' ) ) {
			if ( isset( $_POST['wsx-frontend-editor'] ) && isset( $_POST['post_id'] ) ) {

				do_action( 'wholesalex_before_conversation_reply', $_POST );

				$post_id     = sanitize_text_field( wp_unslash( $_POST['post_id'] ) );
				$new_content = sanitize_textarea_field( wp_unslash( $_POST['wsx-frontend-editor'] ) );
				if ( $post_id ) {
					$reply_id = (int) get_post_meta($post_id,'wholesalex_conversation_reply_id',true);
					$reply_id = $reply_id+1;
					$user_id = get_current_user_id();
					$content = get_post_field( 'post_content', $post_id );
					$data    = array(
						'ID'           => $post_id,
						'post_author'  => get_post_field( 'post_author', $post_id ),
						'post_content' => $content . '<!-- wp:wsx_reply {"id":"' . $reply_id . '","author":"' . $user_id . '", "time":"' . gmdate( 'Y-m-d h:i:s A' ) . '"} -->' . nl2br( $new_content ) . '<!-- /wp:wsx_reply -->',
					);
					wp_update_post( $data );
					update_post_meta($post_id,'wholesalex_conversation_reply_id',$reply_id);
				}

				do_action( 'wholesalex_conversation_update', $post_id, $user_id );
				$admin_email = wholesalex()->get_setting('_conversation_admin_emails','okokok');
				do_action( 'wholesalex_conversation_reply_front_end', $post_id , $user_id, $admin_email);
			}
			wp_send_json_success( 'Success' );
		}
		die();
	}


	/**
	 * New Conversation
	 *
	 * @param string $type Conversation Type.
	 * @return void
	 * @since 1.0.0
	 * @since 1.2.4 wholesalex_new_conversation_form_before_type, wholesalex_new_conversation_form_after_title Action Hook Added
	 */
	public function new_conversation( $type = 'message' ) {
		$conversation_types = apply_filters(
			'wholesalex_addon_conversation_types',
			array(
				'inquiry' => wholesalex()->get_language_n_text( '_language_conversations_list_type_inquiry', __( 'Inquiry', 'wholesalex-pro' ) ),
				'message' => wholesalex()->get_language_n_text( '_language_conversations_list_type_message', __( 'Message', 'wholesalex-pro' ) ),
			)
		);
		?>
		<form name="wholesalex_new_conversation" id="wholesalex_new_conversation" method="post"  class="clearfix wsx-conversation-form">
			<input type="hidden" name="wpnonce" value="<?php echo esc_attr( wp_create_nonce( 'wholesalex-new-conversation' ) ); ?>" />
			<input type="hidden" name="type" value="<?php echo esc_attr( $type ); ?>" />
			<input type="hidden" name="user-id" value="<?php echo esc_attr( get_current_user_id() ); ?>" />
			<?php do_action( 'wholesalex_new_conversation_form_before_type' ); ?>

			<div class="wsx-conversation-element wsx-conversation-form-type">
				<label for="text"><?php echo esc_html( wholesalex()->get_language_n_text( '_language_conversations_list_type', __( 'Type', 'wholesalex-pro' ) ) ); ?></label>
				<select name="conversation_type" id="conversation_type">
				<?php
				foreach ( $conversation_types as $key => $value ) {
					?>
						<option value="<?php echo esc_attr( $key ); ?>"><?php echo esc_html( $value ); ?></option>
					<?php
				}
				?>
				</select>
			</div>
			<div class="wsx-conversation-element wsx-conversation-form-title">
				<label for="text"><?php echo esc_html( wholesalex()->get_language_n_text( '_language_conversations_list_title', __( 'Title', 'wholesalex-pro' ) ) ); ?></label>
				<input type="text" name="title" />
			</div>
			<?php do_action( 'wholesalex_new_conversation_form_after_title' ); ?>
			<div class="wsx-conversation-element wsx-conversation-form-message">
				<label for="wsx-frontend-editor"><?php echo esc_html( wholesalex()->get_language_n_text( '_language_conversations_list_type_message', __( 'Message', 'wholesalex-pro' ) ) ); ?></label>
				<textarea name="wsx-frontend-editor"></textarea>
			</div>
			<input id="wholesalex_conversation_submit" type="submit" class="wsx-send-reply wholesalex-btn wholesalex-btn-black" value="<?php echo esc_html( wholesalex()->get_language_n_text( '_language_start_conversation', __( 'Start Conversation', 'wholesalex-pro' ) ) ); ?>" />
		</form>
		<?php

	}

	/**
	 * Reply Conversation Form
	 *
	 * @param string $conv_id Conversation ID.
	 * @since 1.2.4 Rewrite using wp ajax to create conversation
	 */
	public static function reply_conversation_form( $conv_id ) {
		?>
		<form name="wholesalex_conversatio_reply" id="wholesalex_conversatio_reply" method="post" class="clearfix">
			<input type="hidden" name="wpnonce" value=" <?php echo esc_attr( wp_create_nonce( 'wholesalex-conversation' ) ); ?> "/>
			<input type="hidden" name="post_id" value="<?php echo esc_attr( $conv_id ); ?>"/>
			<div class="wsx-conversation-element wsx-conversation-form-reply">
				<textarea name="wsx-frontend-editor"></textarea>
				<button id="wholesalex_conversation_submit" type="submit" class="wsx-send-reply wholesalex-btn wholesalex-btn-primary"> <?php echo esc_html( wholesalex()->get_language_n_text( '_language_conversation_reply', __( 'Reply', 'wholesalex-pro' ) ) ); ?></button>
			</div>
		</form>
		<?php
	}

	public function get_conversation_endpoint() {
		$menu_endpoint = apply_filters( 'wholesalex_conversation_addon_my_account_end_point', __( 'wsx-conversation', 'wholesalex-pro' ) );
		return $menu_endpoint;
	}

	public function get_conversation_endpoint_title() {
		$menu_title = apply_filters( 'wholesalex_addon_conversation_my_account_menu_title', __( 'Conversation', 'wholesalex-pro' ) );
		return $menu_title;
	}


}
