<?php
/**
 * Table Column
 *
 * @package WHOLESALEX_PRO
 * @since 1.0.0
 */

namespace WHOLESALEX_PRO;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

/**
 * TableColumn Class
 */
class TableColumn extends \WP_List_Table {

	/**
	 * Long Description.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_filter( 'manage_wsx_conversation_posts_columns', array( $this, 'conversation_column_add' ) );
		add_action( 'manage_wsx_conversation_posts_custom_column', array( $this, 'conversation_column_data' ), 10, 2 );
	}

	/**
	 * Add Conversation Column
	 *
	 * @param array $columns Columns.
	 * @return array Columns.
	 * @since 1.0.0
	 */
	public function conversation_column_add( $columns ) {
		unset( $columns['date'] );
		$columns['user']       = __( 'User', 'wholesalex-pro' );
		$columns['title']      = __( 'Conversation', 'wholesalex-pro' );
		$columns['type']       = __( 'Type', 'wholesalex-pro' );
		$columns['status']     = __( 'Status', 'wholesalex-pro' );
		$columns['last_reply'] = __( 'Last Reply Time', 'wholesalex-pro' );
		return $columns;
	}

	/**
	 * Conversation Column Data
	 *
	 * @param array $columns Column.
	 * @param mixed $post_id Post ID.
	 * @return array
	 * @since 1.0.0
	 * @since 1.0.1 Deleted User Text Added.
	 * @since 1.0.5 Guest User Quote Check Added.
	 */
	public function conversation_column_data( $columns, $post_id ) {
		switch ( $columns ) {
			case 'user':
				$status    = get_post_meta( $post_id, '__conversation_type', true );
				$is_quote  = ( 'quote' === $status );
				$username  = '';
				$email     = '';
				$author_id = get_post_field( 'post_author', $post_id );
				$user      = get_userdata( $author_id );

				if ( $is_quote && '0' === $author_id ) {
					$quote_data = get_post_meta( $post_id, '__wholesalex_quote_data', true );
					$quote_data = json_decode( $quote_data, true );
					$username   = isset( $quote_data['name'] ) ? $quote_data['name'] . '(Guest)' : 'Guest';
					$email      = isset( $quote_data['user_email'] ) ? $quote_data['user_email'] : '';
				} else {
					$username = $user ? $user->display_name : __( 'Deleted User', 'wholesalex-pro' );
					$email    = $user ? $user->user_email : __( 'Deleted User', 'wholesalex-pro' );
				}

				if ( ! $user && ! $is_quote ) {
					echo esc_html( __( 'Deleted User', 'wholesalex-pro' ) );
				} else {
					echo '#' . esc_html( $author_id ) . ' ' . esc_html( $username ) . '<br>' . esc_html( $email );
				}
				break;
			case 'type':
				echo esc_html( get_post_meta( $post_id, '__conversation_type', true ) );
				break;
			case 'status':
				echo esc_html( get_post_meta( $post_id, '__conversation_status', true ) );
				break;
			case 'last_reply':
				$content     = get_the_content( $post_id );
				$reply_lists = parse_blocks( $content );
				$last_item   = isset( $reply_lists[ count( $reply_lists ) - 1 ] ) ? $reply_lists[ count( $reply_lists ) - 1 ] : array();
				if ( isset( $last_item['attrs']['time'] ) ) {
					echo count( $reply_lists ) . ' ' . esc_html__( 'Replays', 'wholesalex-pro' ) . '<br/>';
					echo esc_html( human_time_diff( strtotime( $last_item['attrs']['time'] ), time() ) ) . ' ' . esc_html__( 'ago', 'wholesalex-pro' ) . '<br/>';
					echo esc_html( get_date_from_gmt( $last_item['attrs']['time'], 'Y-m-d h:i:s A' ) );
				}
				break;
		}
		return $columns;
	}

}
