<?php
/**
 * WholesaleX Request A Quote
 *
 * @package WHOLESALEX_PRO
 * @since 1.0.5
 */

namespace WHOLESALEX_PRO;

use WHOLESALEX\WHOLESALEX_Email_Manager;
use WC_Order_Item_Product;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}
/**
 * WholesaleX Request Quote
 */
class RequestQuote {

	/**
	 * Request A Quote Custom Order Status.
	 *
	 * @var array
	 */
	public $quote_statuses = array( 'whx-quote-pending', 'whx-quote-expired', 'whx-quote-reject' );

	public $emails;
	public $is_hidden_request_quote = false;
	public $hidden_dynamic_rules = array();

	/**
	 * Constructor
	 */
	public function __construct() {

		add_action( 'wholesalex_addon_conversation_my_account_menu_title', array( $this, 'change_my_account_menu_title' ) );

		// Support Minimog Child Theme
		if('minimog-child' == get_option('stylesheet') ) {
			add_action( 'woocommerce_before_cart_collaterals', array( $this, 'request_custom_quote_button' ) );
		} else {
			add_action( 'woocommerce_cart_actions', array( $this, 'request_custom_quote_button' ) );
		}
		// add_action( 'woocommerce_before_cart_collaterals', array( $this, 'custom_quote_request_form' ) );
		add_action( 'woocommerce_after_cart', array( $this, 'custom_quote_request_form' ) );
		add_action( 'wp_footer', array( $this, 'custom_quote_request_form' ) );

		add_action( 'wp_ajax_send_quote_request_action', array( $this, 'send_quote_request_action' ) );

		add_action( 'wp_ajax_nopriv_send_quote_request_action', array( $this, 'send_quote_request_action' ) );

		add_action( 'wp_ajax_quote_request_make_offer', array( $this, 'quote_request_make_offer' ) );

		add_action( 'wp_ajax_send_quote_to_user_action', array( $this, 'send_quote_to_user_action' ) );

		add_action( 'wholesalex_edit_conversation_before_send_message_button', array( $this, 'add_make_offer_button' ) );

		add_filter( 'wc_order_statuses', array( $this, 'quote_order_status_in_order_page' ) );

		add_filter( 'wc_order_is_editable', array( $this, 'make_pending_quote_request_order_editable' ), 10, 2 );

		add_action( 'woocommerce_order_item_add_action_buttons', array( $this, 'send_quote_request_to_customer_button' ) );

		add_action( 'wp_loaded', array( $this, 'change_quote_status' ) );

		add_filter( 'woocommerce_create_order', array( $this, 'set_cart_hash' ), 1 );

		add_action( 'save_post_shop_order', array( $this, 'save_quote_option_metabox_data' ), 10, 3 );

		add_action( 'admin_init', array( $this, 'quote_option_metabox_register' ) );

		add_action( 'woocommerce_cart_calculate_fees', array( $this, 'add_quote_discount' ) );

		add_action( 'wp_loaded', array( $this, 'register_request_a_quote_cron_jobs' ) );

		add_action( 'wholesalex_raq_expire_date_check', array( $this, 'raq_orders_expire_date_check' ) );

		add_action( 'wholesalex_after_update_quote_expire_date', array( $this, 'change_order_status_after_update_expire_date' ), 10, 2 );

		// add_filter( 'wholesalex_raq_send_email', array( $this, 'send_email_to_user' ), 10, 5 );

		add_filter( 'wholesalex_email_smart_tags', array( $this, 'request_a_quote_email_smart_tags' ), 10, 2 );

		add_filter( 'wholesalex_setting_fields', array( $this, 'raq_settings' ) );

		add_filter( 'wholesalex_role_fields', array( $this, 'raq_role_settings' ) );

		// add_action( 'wholesalex_quote_reminder_email', array( $this, 'send_expiring_email' ), 10, 3 );

		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

		add_filter( 'woocommerce_product_add_to_cart_text', array( $this, 'change_add_to_cart_text' ) );

		add_filter( 'woocommerce_product_single_add_to_cart_text', array( $this, 'change_single_product_add_to_cart_text' ) );

		add_filter( 'wholesalex_addon_conversation_delete_user_notice', array( $this, 'guest_user_conversation_notice' ), 10, 2 );

		add_action( 'wholesalex_addon_conversation_my_account_after_messages', array( $this, 'remove_accept_reject_button_script' ) );

		add_action( 'wholesalex_addon_conversation_my_account_after_reply_form', array( $this, 'reject_modal' ) );

		add_action( 'wp_loaded', array( $this, 'quote_submission_success_message' ) );

		add_filter( 'woocommerce_email_classes', array( $this, 'wholesalex_add_email_classes' ) );

		add_action( 'woocommerce_email', array( $this, 'handle_quote_email' ) );

		add_filter( 'woocommerce_email_actions', array( $this, 'wholesalex_add_email_actions' ) );

		add_filter( 'wholesalex_role_exporter_default_columns', array( $this, 'add_raq_fields_on_role_exporter' ) );

		add_filter( 'wholesalex_role_export_role_column__raq_replace_add_to_cart_with_quote', array( $this, 'get_column_value__raq_replace_add_to_cart_with_quote' ), 10, 2 );

		add_action('wholesalex_addon_conversation_before_message_content',array($this,'add_quote_table_before_message_content'),10,2);
		
		add_filter( 'wholesalex_dynamic_rules_field', array( $this, 'dynamic_rule_fields' ), 1 );
        //Hide Request A Quote Form For Logged in users
		add_action( 'wholesalex_after_hidden_price_quote', array($this, 'wsx_check_hidden_price_info'), 10, 1 );

	}

	/**
	 * Hidden Price Dynamic Rules Valid Data
	 *
	 * @param [array] $valid_rules
	 * @return void
	 */
	public function wsx_check_hidden_price_info( $valid_rules) {
		$request_quote_btn_text =  wholesalex()->get_language_n_text( '_language_hidden_price_request_quote_btn_text', __( 'Request Quote', 'wholesalex-pro' ) );
		$this->is_hidden_request_quote = $valid_rules['request_quote_enable'];
		//Add New Button For Shop Page
		add_action(
			'woocommerce_loop_add_to_cart_link',
			function ( $button ) use( $valid_rules, $request_quote_btn_text ) {
				global $product;
				if ( $product->is_type( 'simple' ) ){
					$product_id = $product->get_id();
					if ( $this->is_hidden_price_rule_apply( $product_id, $valid_rules ) ) {
						$custom_class = 'wholesalex_request_custom_quote_btn';
						$product_link = '#';
						$button_text = $request_quote_btn_text;
						$button .= sprintf(
							'<a href="%s" data-quantity="1" class="button %s" data-product_id="%s">%s</a>',
							esc_url( $product_link ),
							esc_attr( $custom_class ),
							esc_attr( $product_id ),
							esc_html( $button_text )
						);
					}
					return $button;
				}
			},
			10,
			1
		);
		
		//Add New Button In Single Page For Request Quote
		add_action(
			'woocommerce_product_meta_start',
			function ( ) use( $valid_rules, $request_quote_btn_text ) {
				global $product;
				if ( $product->is_type( 'simple' ) ){
					$product_id = $product->get_id();
					if ( $this->is_hidden_price_rule_apply( $product_id, $valid_rules ) ) {
						$custom_class = 'wholesalex_request_custom_quote_btn wsx-single-product button ' . esc_attr( wc_wp_theme_get_element_class_name( 'button' ) ? ' ' . wc_wp_theme_get_element_class_name( 'button' ) : '' );
						$button_text = $request_quote_btn_text;
						$button = sprintf(
							'<div class="ct-cart-actions"> <button data-quantity="1" class="button %s" data-product_id="%s">%s</button> </div>',
							esc_attr( $custom_class ),
							esc_attr( $product_id ),
							esc_html( $button_text )
						);
						echo $button;
						
					}
				}
			}
		);
		// Add New Button Variation Product
		add_action(
			'woocommerce_after_add_to_cart_button',
			function ( ) use( $valid_rules, $request_quote_btn_text ) {
				$is_all_products	= ($valid_rules['is_all_products'] ? $valid_rules['is_all_products'] : 0);
				$include_product 	= implode( ',', $valid_rules['include_product'] );
				$exclude_product 	= implode( ',', $valid_rules['exclude_product'] );
				$include_variations = implode( ',', $valid_rules['include_variations'] );
				$exclude_variations = implode( ',', $valid_rules['exclude_variations'] );
				$include_cat 		= implode( ',', $valid_rules['include_cat'] );
				$exclude_cat 		= implode( ',', $valid_rules['exclude_cat'] );
				//$include_variations[] = '12';
				global $product;
				if ( $product->is_type( 'variable' ) ){
					$product_id = $product->get_id();
					$include_variation_with_parent_ids[] = $product_id;
					
					$category_ids = $product->get_category_ids();
						$custom_class = 'wholesalex_request_custom_quote_btn wsx_single_quote_button button ' . esc_attr( wc_wp_theme_get_element_class_name( 'button' ) ? ' ' . wc_wp_theme_get_element_class_name( 'button' ) : '' );
						$custom_id = 'wholesalex_quote_btn';
						$button_text = $request_quote_btn_text;
						$button = sprintf(
							'<button data-parent-id="%s"
							  data-is-all-product="%s"
							  data-include-id="%s"
							  data-exclude-id="%s" 
							  data-in-variation-id="%s" 
							  data-ex-variations-id="%s" 
							  data-include-cat="%s" 
							  data-exclude-cat="%s" 
							  data-categories="%s" 
							  data-quantity="1" 
							  class="button %s" 
							  id="%s">%s</button>',
							esc_attr( $product_id ),
							esc_attr( $is_all_products ),
							esc_attr( $include_product ),
							esc_attr( $exclude_product ),
							esc_attr( $include_variations ),
							esc_attr( $exclude_variations ),
							esc_attr( $include_cat ),
							esc_attr( $exclude_cat ),
							esc_attr( $category_ids[0] ),
							esc_attr( $custom_class ),
							esc_attr( $custom_id ),
							esc_html( $button_text )
						);
						echo $button;
				}
			}
		);
	}
	public function is_hidden_price_rule_apply( $product_id, $valid_rules ) {
			$cats = wc_get_product_term_ids( $product_id, 'product_cat' );
			if($valid_rules['hidden_is_all_products']  ) {
				return true;
			}
			if(!empty($valid_rules['hidden_request_product_list']) && in_array($product_id,$valid_rules['hidden_request_product_list'])) {
				return true;
			} else if(!empty($valid_rules['hidden_exclude_products']) && !in_array($product_id,$valid_rules['hidden_exclude_products'])) {
				return true;
			} else if(!empty($valid_rules['hidden_include_cats']) && !empty(array_intersect($cats,$valid_rules['hidden_include_cats']))) {
				return true;
			} else if(!empty($valid_rules['hidden_exclude_cats']) && empty(array_intersect($cats,$valid_rules['hidden_exclude_cats']))) {
				return true;
			} else if(!empty($valid_rules['hidden_include_variations']) && in_array($product_id,$valid_rules['hidden_include_variations'])) {
				return true;
			} else if(!empty($valid_rules['hidden_exclude_variations']) && !in_array($product_id,$valid_rules['hidden_exclude_variations'])) {
				return true;
		}
		return false;
	}
	/**
	 * Add New Setting For Hidden Price
	 *
	 * @param [type] $sections
	 * @return void
	 */
	function dynamic_rule_fields( $sections ) {
		$__sections = array(
			'hidden_price'   => array(
				'label'      => __( 'Hide Product Pricing with "Request Quote"', 'wholesalex-pro' ),
				'type'       => 'hidden_price',
				'depends_on' => array(
					array(
						'key'   => '_rule_type',
						'value' => 'hidden_price',
					),
				),
				'attr'       => array(
					'_hidden_price_checkbox' => array(
						'type'        	=> 'switch',
						'label'		  	=> __('Enable "Request Quote" for Product','wholesalex'),
						'desc'       	=> __( 'Hide Product Pricing with "Request Quote"', 'wholesalex-pro' ),
						'default'     	=> 'no',
						'placeholder' 	=> '',
						'descTooltip'	=> __('Show "Request Quote" without revealing the price to the customer.','wholesalex'),
					),
				),
			),
		);
		if ( isset( $sections['_new_rule']['attr'] ) && is_array( $sections['_new_rule']['attr'] ) ) {
			$sections['_new_rule']['attr'] = wholesalex()->insert_into_array( $sections['_new_rule']['attr'], $__sections, 3 );
		}
		return $sections;
	}

	public function wholesalex_add_email_classes( $email_classes ) {
		$email_classes['WholesaleX_Quote_Request_Recieved_Admin']     = include WHOLESALEX_PRO_PATH . '/addons/raq/emails/class-wholesalex-raq-received-admin.php';
		$email_classes['WholesaleX_Quote_Make_Offer_Email']     = include WHOLESALEX_PRO_PATH . '/addons/raq/emails/class-wholesalex-raq-make-offer.php';
		$email_classes['WholesaleX_Quote_Expiring_Offer_Email'] = include WHOLESALEX_PRO_PATH . '/addons/raq/emails/class-wholesalex-raq-offer-expiring.php';

		return $email_classes;
	}

	public function wholesalex_add_email_actions( $actions ) {
		$actions[] = 'wholesalex_raq_send_email';
		$actions[] = 'wholesalex_quote_reminder_email';
		$actions[] = 'wholesalex_after_new_quote_request';
		return $actions;
	}


	public function handle_quote_email( $wc_emails ) {
		$this->emails = $wc_emails->get_emails();

		add_action( 'wholesalex_raq_send_email_notification', array( $this, 'send_email_to_user' ), 10, 5 );
		add_action( 'wholesalex_quote_reminder_email_notification', array( $this, 'send_expiring_email' ), 10, 3 );
		add_action( 'wholesalex_after_new_quote_request_notification', array( $this, 'send_quote_request_received_email' ), 10, 3 );
	}
	/**
	 * Account Page Reject Offer Modal
	 *
	 * @return void
	 */
	public function reject_modal() {
		?>
		<div id="wholesalex_reject_modal">
				<div class="modal_content">
					<div class="modal_header">
						<div class="modal-close-btn">
							<span class="close-modal-icon dashicons dashicons-no-alt" ></span>
						</div>
					</div>
					<div class="wholesalex-modal__body">
					<?php echo esc_html_e( 'Do you realy want to reject this offer?','wholesalex-pro' ); ?>
				</div>

				<div class="wholesalex-quote-modal__footer">
					<button class="btn-modal-cancel"><?php echo esc_html_e( 'Cancel', 'wholesalex-pro' ); ?></button>
					<button id="reject_link" class="btn-modal-delete"><?php echo esc_html_e( 'Reject', 'wholesalex-pro' ); ?></button>
				</div>
				</div>
			</div>
		<?php
	}

	/**
	 * Remove Quote Buttons.
	 *
	 * @param int|string $post_id Conversation ID.
	 * @return void
	 */
	public function remove_accept_reject_button_script( $post_id, $is_admin = false ) {
		$__order_id = get_post_meta( $post_id, '__wholesalex_quote_order_id', true );
		$order      = wc_get_order( $__order_id );
		if ( ! $order ) {
			return;
		}

		$status = $order->get_status();
		if ( 'whx-quote-expired' === $status ) {
			?>
			<script>
				let elements = document.querySelectorAll('.quote-buttons');
				if(elements.length) {
					let lastElement = '';
					if(elements.length>1){
						for (let index = 0; index < elements.length-1; index++) {
							const element = array[index];
							element.remove();
						}
					}
					lastElement = elements[elements.length-1];
					lastElement.innerHTML = '<span class="expired"><?php echo esc_html__('Expired','wholesalex-pro'); ?></span>';
				}

			</script>
			<?php
		} elseif ( 'whx-quote-reject' === $status ) {
			?>
			<script>
				elements = document.querySelectorAll('.quote-buttons');
				if(elements.length) {
					let lastElement = '';
					if(elements.length>1){
						for (let index = 0; index < elements.length-1; index++) {
							const element = elements[index];
							element.remove();
						}
					}
					lastElement = elements[elements.length-1];
					lastElement.innerHTML = '<span class="rejected"><?php echo esc_html__('Rejected','wholesalex-pro'); ?></span>';

				}

			</script>
			<?php
		} elseif ( 'whx-quote-pending' === $status ) {
			?>
			<script>
				elements = document.querySelectorAll('.quote-buttons');
				if(elements.length) {
					let lastElement = '';
					if(elements.length>1){
						for (let index = 0; index < elements.length-1; index++) {
							const element = elements[index];
							element.remove();
						}
					}
					lastElement = elements[elements.length-1];
					let rejectButton = document.getElementById('quote_reject_button');
					let rejectLink = rejectButton.firstChild;
					rejectButton.removeChild(rejectLink);
					document.getElementById('quote_reject_button').innerText="<?php echo esc_html__('Reject','wholesalex-pro'); ?>";
					document.getElementById('quote_reject_button').addEventListener('click',(e)=>{
						e.preventDefault();
						document.getElementById('wholesalex_reject_modal').style.display = 'block';
						document.getElementById('reject_link').innerHTML="<a href='"+rejectLink+"'><?php echo esc_html__('Reject','wholesalex-pro'); ?></a>";
					})



					document.addEventListener(
						"click",
						function (event) {
							if ( event.target.matches(".close-modal-icon") || event.target.matches(".btn-modal-cancel")  ) {
									document.getElementById('wholesalex_reject_modal').style.display='none';
							}
						},
						false
					);
				}


			</script>
			<?php
		} else {
			?>
			<script>
				elements = document.querySelectorAll('.quote-buttons');
				if(elements.length) {
					let lastElement = '';
					if(elements.length>1){
						for (let index = 0; index < elements.length-1; index++) {
							const element = elements[index];
							element.remove();
						}
					}
					lastElement = elements[elements.length-1];
					lastElement.innerHTML = '<span class="accepted"><?php echo esc_html__('Accepted','wholesalex-pro'); ?></span>';
				}

			</script>
			<?php
		}

	}

	/**
	 * Guest User Conversation Notice
	 *
	 * @param string $message Conversation Notice.
	 * @param string $status Conversation Status.
	 * @return string Conversation Notice.
	 */
	public function guest_user_conversation_notice( $message, $status ) {
		if ( 'quote' === $status ) {
			$message = __( 'The quote request was sent by guest user.', 'wholesalex-pro' );
		}
		return $message;
	}

	/**
	 * Custom Quote Request Button on Cart Page.
	 *
	 * @return void
	 * @since 1.0.5
	 */
	public function request_custom_quote_button() {
		$raq_btn_text = wholesalex()->get_setting( '_language_raq_cart_page_request_text', __( 'Request Quote', 'wholesalex-pro' ) );
		?>
			<button id="wholesalex_request_btn" class="wholesalex_request_custom_quote_btn button"><?php echo esc_html( $raq_btn_text ); ?></button>
		<?php
	}

	/**
	 * Custom Quote Request Form
	 *
	 * @return void
	 */
	public function custom_quote_request_form() {
		
		$is_hidden_apply = false;
		if ( $this->is_hidden_request_quote && (is_shop() || is_product() ) ) {
			$is_hidden_apply = true;
		}
		if ( is_cart() ) {
			$is_hidden_apply = true;
		}
		if ( ! $is_hidden_apply ) {
			return;
		}
        $is_hide_for_logged_in_user = wholesalex()->get_setting('raq_form_hide_from_logged_in_user','no');
        $is_hide_for_logged_in_user = apply_filters( 'wholesalex_request_a_quote_hide_form_for_logged_in_user',  $is_hide_for_logged_in_user);
		wp_enqueue_script('wholesalex_request_a_quote');
		wp_localize_script('wholesalex_request_a_quote','wsx_raq', array(
			'i18n'=> array(
				'please_enter_a_valid_email' => __('Please Enter a valid email','wholesalex-pro')
			),
			'hide_quote_form_logged_in_user' => $is_hide_for_logged_in_user,
			'is_user_logged_in'=>is_user_logged_in()
		));
        wp_nonce_field( 'wholesalex-new-conversation', 'wholesalex_request_quote_nonce' );
      
		$user_id = get_current_user_id();

		$fields = apply_filters(
			'wholesalex_raq_fields',
			array(
				'wholesalex_quote_title'     => array(
					'type'     => 'text',
					'name'     => 'wholesalex_quote_title',
					'label'    => __(
						'Quote Title',
						'wholesalex-pro'
					),
					'class'    => 'quote_title',
					'value'    => '',
					'required' => true,
				),
				'wholesalex_quote_product_id'     => array(
					'type'     => 'hidden',
					'name'     => 'wholesalex_quote_product_id',
					'label'    => '',
					'class'    => 'quote_hidden_price',
					'value'    => '',
					'required' => false,
				),
				'wholesalex_quote_product_quantity'     => array(
					'type'     => 'hidden',
					'name'     => 'wholesalex_quote_product_quantity',
					'label'    => '',
					'class'    => 'quote_hidden_quantity',
					'value'    => '1',
					'required' => false,
				),
				'wholesalex_quote_user_name' => array(
					'type'     => 'text',
					'name'     => 'wholesalex_quote_user_name',
					'label'    => __(
						'Name',
						'wholesalex-pro'
					),
					'class'    => 'quote_user_name',
					'value'    => '',
					'required' => true,
				),
				'wholesalex_quote_email'     => array(
					'type'     => 'email',
					'name'     => 'wholesalex_quote_email',
					'label'    => __(
						'Email',
						'wholesalex-pro'
					),
					'class'    => 'quote_email',
					'value'    => '',
					'required' => true,
				),
				'wholesalex_quote_phone'     => array(
					'type'     => 'number',
					'name'     => 'wholesalex_quote_phone',
					'label'    => __(
						'Phone',
						'wholesalex-pro'
					),
					'class'    => 'quote_phone',
					'value'    => '',
					'required' => true,
				),
				'wholesalex_quote_message'   => array(
					'type'     => 'textarea',
					'name'     => 'wholesalex_quote_message',
					'label'    => __(
						'Message',
						'wholesalex-pro'
					),
					'class'    => 'quote_message',
					'value'    => '',
					'required' => true,
				),
			),
		);
		$email  = '';
		if ( $user_id ) {
			$user_data                                 = get_userdata( $user_id );
			$email                                     = $user_data->user_email;
			$fields['wholesalex_quote_email']['value'] = $email;
			unset( $fields['wholesalex_quote_user_name'] );
		}
		?>
			<div class="wholesalex_loader"></div>
			<div class="wholesalex_add_to_quote_modal">
				<div class="modal_content">
					<div class="modal_header">
						<div class="modal-close-btn">
							<span class="close-modal-icon" > <span class="wsx-raq-close-icon"></span></span>
						</div>
					</div>
					<form id="wholesalex_request_custom_quote_form">
					<?php
					foreach ( $fields as $key => $field ) {
						switch ( $field['type'] ) {
							case 'text':
							case 'number':
							case 'email':
							case 'hidden':
								?>
									<div class="<?php echo esc_attr( $field['class'] ); ?> wholesalex-field">
										<label for="<?php echo esc_attr( $key ); ?>"> 
										<?php echo esc_html( $field['label'] ); ?>
										</label>
										<input type="<?php echo esc_attr( $field['type'] ); ?>" name="<?php echo esc_attr( $key ); ?>" id="<?php echo esc_attr( $key ); ?>" value="<?php echo esc_attr( $field['value'] ); ?>" class="" <?php echo $field['required'] ? 'required' : ''; ?>/>
									</div>
								<?php
								break;
							case 'textarea':
								?>
									<div class="<?php echo esc_attr( $field['class'] ); ?> wholesalex-field">
										<label for="<?php echo esc_attr( $key ); ?>"> 
										<?php echo esc_html( $field['label'] ); ?>
										</label>
										<textarea name="<?php echo esc_attr( $key ); ?>" id="<?php echo esc_attr( $key ); ?>" class="" <?php echo $field['required'] ? 'required' : ''; ?>> </textarea>
									</div>
								<?php
								break;

							default:
								// code...
								break;
						}
					}
					?>
					<button id="wholesalex-quote-submit" class="button"><?php echo esc_html_e( 'Submit', 'wholesalex-pro' ); ?></button>
					</form>
				</div>
			</div>
		<?php
	}



	/**
	 * Make Offer Quote Request Action
	 *
	 * @return void
	 */
	public function quote_request_make_offer() {
		//phpcs:ignore
		$post_id = isset( $_POST['postID'] ) ? sanitize_text_field( $_POST['postID'] ) : '';
		if ( $post_id ) {
			$customer_id = isset( $_POST['customer_id'] ) ? sanitize_text_field( wp_unslash( $_POST['customer_id'] ) ) : ''; //phpcs:ignore
			if ( '' === $customer_id ) {
				wp_send_json_error( __( 'Customer ID is Empty!', 'wholesalex-pro' ) );
			}
			$quote_data  = get_post_meta( $post_id, '__wholesalex_quote_data', true );
			$quote_data  = json_decode( $quote_data, true );
			$fee         = isset( $quote_data['fee'] ) ? $quote_data['fee'] : '';
			$quote_items = isset( $quote_data['items'] ) ? $quote_data['items'] : '';

			$__order_id = get_post_meta( $post_id, '__wholesalex_quote_order_id', true );
			if ( $__order_id ) {
				$order_url = admin_url( 'post.php?post=' . $__order_id ) . '&action=edit';
				wp_send_json_success( array( 'redirect' => $order_url ) );
			}
			$order    = wc_create_order(
				array(
					'status'      => 'whx-quote-pending',
					'customer_id' => $customer_id,
				)
			);
			$order_id = '';

			if ( ! is_array( $quote_items ) ) {
				$quote_items = array();
			}

			$cart_contents = get_post_meta( $post_id, 'wholesalex_quote_cart_contents', true );
			if ( $cart_contents && ! empty( $cart_contents ) && '2.0' == get_post_meta( $post_id, 'wholesalex_quote_version', true ) ) {
				foreach ( $cart_contents as $cart_item_key => $values ) {

					$item                       = apply_filters( 'woocommerce_checkout_create_order_line_item_object', new WC_Order_Item_Product(), $cart_item_key, $values, $order );
					$product                    = $values['data'];
					$item->legacy_values        = $values; // @deprecated 4.4.0 For legacy actions.
					$item->legacy_cart_item_key = $cart_item_key; // @deprecated 4.4.0 For legacy actions.
					$item->set_props(
						array(
							'quantity'     => $values['quantity'],
							'variation'    => $values['variation'],
							'subtotal'     => $values['line_subtotal'],
							'total'        => $values['line_total'],
							'subtotal_tax' => $values['line_subtotal_tax'],
							'total_tax'    => $values['line_tax'],
							'taxes'        => $values['line_tax_data'],
						)
					);

					if ( $product ) {
						$item->set_props(
							array(
								'name'         => $product->get_name(),
								'tax_class'    => $product->get_tax_class(),
								'product_id'   => $product->is_type( 'variation' ) ? $product->get_parent_id() : $product->get_id(),
								'variation_id' => $product->is_type( 'variation' ) ? $product->get_id() : 0,
							)
						);
					}

					$item->set_backorder_meta();

					do_action( 'woocommerce_checkout_create_order_line_item', $item, $cart_item_key, $values, $order );

					// Add item to order and save.
					$order->add_item( $item );

				}
			} else {

				foreach ( $quote_items as $item ) {
					$product_id = $item['variation_id'] ? $item['variation_id'] : $item['product_id'];
					$quantity   = $item['quantity'];
					if ( $product_id ) {
						$order->add_product(
							wc_get_product( $product_id ),
							$quantity,
							array(
								'subtotal' => $item['subtotal'],
								'total'    => $item['total'],
							)
						);
					}
				}
			}

			$order->update_meta_data( '__wholesalex_quote_status', 'pending' );

			$order->calculate_totals();
			$order_id = $order->get_id();
			if ( $order_id ) {
				update_post_meta( $order_id, '__wholesalex_quote_id', $post_id );
				$order_url = admin_url( 'post.php?post=' . $order_id ) . '&action=edit';
				update_post_meta( $post_id, '__wholesalex_quote_order_id', $order_id );
				wp_send_json_success( array( 'redirect' => $order_url ) );
			}
		}

	}

	/**
	 * Send Quote Request Action
	 *
	 * @return void
	 */
	public function send_quote_request_action() {
		if ( isset( $_POST['wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['wpnonce'] ), 'wholesalex-new-conversation' ) ) {
			$post_title 				= isset( $_POST['wholesalex_quote_title'] ) ? sanitize_text_field( wp_unslash( $_POST['wholesalex_quote_title'] ) ) : __( 'Quote Request', 'wholesalex-pro' );
			$message    				= isset( $_POST['wholesalex_quote_message'] ) ? sanitize_textarea_field( wp_unslash( $_POST['wholesalex_quote_message'] ) ) : '';
			$email      				= isset( $_POST['wholesalex_quote_email'] ) ? sanitize_email( wp_unslash( $_POST['wholesalex_quote_email'] ) ) : '';
			$phone      				= isset( $_POST['wholesalex_quote_phone'] ) ? sanitize_text_field( wp_unslash( $_POST['wholesalex_quote_phone'] ) ) : '';
			$hidden_product_id  	  	= isset( $_POST['wholesalex_quote_product_id'] ) ? sanitize_text_field( wp_unslash( $_POST['wholesalex_quote_product_id'] ) ) : '';
			$hidden_product_quantity  	= isset( $_POST['wholesalex_quote_product_quantity'] ) ? sanitize_text_field( wp_unslash( $_POST['wholesalex_quote_product_quantity'] ) ) : '';
			$name       = '';
			if ( ! get_current_user_id() ) {
				$name = isset( $_POST['wholesalex_quote_user_name'] ) ? sanitize_text_field( wp_unslash( $_POST['wholesalex_quote_user_name'] ) ) : '';
			} else{
                $user_data = get_userdata(get_current_user_id(  ));
                $name = $user_data->display_name;
            }

			$user_id       = get_current_user_id();
            if(is_user_logged_in() && empty($email))  {
				$user_data = get_userdata($user_id);
				$email = $user_data->user_email;
			}
			$row           = '';
			function process_product_item($product, $quantity = 1) {
					$product_id    = $product->get_id();
					$variation_id  = ($product->is_type('variation')) ? $product->get_id() : 0;
					$unit_price    = $product->get_price();
					$product_title = $product->get_name();
					$sku           = $product->get_sku();
					$subtotal      = $unit_price * $quantity;
					$total         = $subtotal;
					$subtotal_tax  = 0;
					$total_tax     = 0;
					$tax_data      = array();

					return array(
						'product_id'    => $product_id,
						'variation_id'  => $variation_id,
						'quantity'      => $quantity,
						'unit_price'    => $unit_price,
						'product_title' => $product_title,
						'sku'           => $sku,
						'subtotal'      => $subtotal,
						'total'         => $total,
						'total_tax'     => $total_tax,
						'subtotal_tax'  => $subtotal_tax,
						'tax_data'      => $tax_data,
					);
				}

				$quote_data = array();

				if (isset($hidden_product_id) && !empty($hidden_product_id)) {
					$product = wc_get_product($hidden_product_id);
					
					if ($product) {
						$quote_data[] = process_product_item( $product, $hidden_product_quantity );
					} else {
						echo "Product not found.";
					}
				} else {
					$cart_contents = WC()->cart->get_cart();
					foreach ($cart_contents as $cart_item) {
					$product = $cart_item['data'];
					$quantity = $cart_item['quantity'];
					$subtotal_tax = $cart_item['line_subtotal_tax'];
					$total_tax = $cart_item['line_tax'];
					$tax_data = $cart_item['line_tax_data'];
					$processed_item = process_product_item($product, $quantity);
					$processed_item['subtotal_tax'] = $subtotal_tax;
					$processed_item['total_tax'] = $total_tax;
					$processed_item['tax_data'] = $tax_data;
					$quote_data[] = $processed_item;
				}
			}
			$reply_id = 1;

			$data     = array(
				'post_title'   => $post_title,
				'post_type'    => 'wsx_conversation',
				'post_author'  => $user_id,
				'post_status'  => 'publish',
				'post_content' => '<!-- wp:wsx_reply {"id":"' . $reply_id. '","author":"' . $user_id . '", "time":"' . gmdate( 'Y-m-d h:i:s A' ) . '","is_requested_quote":true} -->'.nl2br( $message ).'<!-- /wp:wsx_reply -->',
			);
			do_action( 'wholesalex_before_quote_create', $_POST );
			$id = wp_insert_post( $data );
			if ( $id ) {
				update_post_meta( $id, '__conversation_type', 'quote' );
				update_post_meta( $id, '__conversation_status', 'open' );
				update_post_meta( $id, 'wholesalex_quote_version', '2.0' );
				update_post_meta( $id, 'wholesalex_quote_cart_contents', $cart_contents );
				update_post_meta(
					$id,
					'__wholesalex_quote_data',
					wp_json_encode(
						array(
							'fee'        => WC()->cart->get_fee_total(),
							'items'      => $quote_data,
							'user_email' => $email,
							'name'       => $name,
							'phone'      => $phone,
						)
					)
				);
				update_post_meta($id,'wholesalex_conversation_reply_id',$reply_id);	
			}

			do_action( 'wholesalex_quote_created', $id );

            do_action( 'wholesalex_after_new_quote_request', $id, $name, $quote_data );

			$clear_cart = wholesalex()->get_setting( '_raq_clear_cart_after_request' );
			if ( 'yes' === $clear_cart ) {
				WC()->cart->empty_cart( true );
			}
			if ( $user_id ) {
				// Logged in user.
				wp_send_json_success(
					array(
						'type'      => 'user',
						'message'   => __( 'To continue with quotations we will redirect you to conversation page.', 'wholesalex-pro' ),
						'redirect'  =>
							add_query_arg(
								array(
									'wsxview' => $id,
									'status'  => 'success',
									'nonce'   => wp_create_nonce( 'quote_request_success' ),
								),
								wc_get_account_endpoint_url( apply_filters( 'wholesalex_conversation_addon_my_account_end_point', __( 'wsx-conversation', 'wholesalex-pro' ) ) )
							),
						'shop_page' =>
							add_query_arg(
								array(
									'status' => 'success',
									'nonce'  => wp_create_nonce( 'quote_request_success' ),
								),
								get_permalink( get_option( 'woocommerce_shop_page_id' ) )
							),
					)
				);
			} else {
				wp_send_json_success(
					array(
						'type'     => 'guest',
						'redirect' => get_permalink( get_option( 'woocommerce_shop_page_id' ) ),
						'message'  => __(
							'Your quote request has been received. We will review it, and let you know the update via mail.',
							'wholesalex-pro'
						),
					)
				);
			}
		}

	}

	/**
	 * Add Make Offer Button on Quote
	 *
	 * @param string|int $conversation_id Conversation/Quote ID.
	 * @return void
	 */
	public function add_make_offer_button( $conversation_id ) {
		$__post_type = get_post_meta( $conversation_id, '__conversation_type', true );
		if ( 'quote' === $__post_type ) {
			$__quote_data = get_post_meta( $conversation_id, '__wholesalex_quote_data', true );
			if ( $__quote_data ) {
				echo '<div class="wholesalex_loader"></div>';
				echo '<button id="wholesalex-quote-make-offer" class="wholesalex-btn wholesalex-btn-make-offer">' . esc_html__( 'Make Offer', 'wholesalex-pro' ) . '</button>';
				$this->make_offer_script();
				$this->remove_accept_reject_button_script( $conversation_id );
			}
		}
	}

	/**
	 * Make offer script
	 *
	 * @return void
	 */
	public function make_offer_script() {
		?>
			<script>
				document.getElementById('wholesalex-quote-make-offer').addEventListener('click',(e)=>{
					e.preventDefault();
					const formData = new FormData();
					formData.append('action','quote_request_make_offer');
					formData.append('postID',document.getElementById('post_ID').value);
					formData.append('customer_id',document.getElementById('post_author').value);
					// formData.append('wpnonce', document.getElementById('wholesalex_request_quote_nonce').value);
					jQuery(".wholesalex_loader").show();
					fetch(
						wholesalex.ajax, {
						method: 'POST',
						body:formData ,
					})
					.then(res => res.json())
					.then(res => {
						if(res.data) {
							window.location.href = res.data.redirect;
						}
					})
				});

			</script>
		<?php
	}

	/**
	 * Add Quote Pending Order Status on Edit Order Page.
	 *
	 * @param array $order_statuses Order Statuses.
	 * @return array
	 */
	public function quote_order_status_in_order_page( $order_statuses ) {
		$order_statuses['wc-whx-quote-pending'] = _x( 'Pending Quote Request', 'Order status', 'wholesalex-pro' );
		$order_statuses['wc-whx-quote-expired'] = _x( 'Expired Quote Request', 'Order status', 'wholesalex-pro' );
		$order_statuses['wc-whx-quote-reject']  = _x( 'Rejected Quote Request', 'Order status', 'wholesalex-pro' );
		return $order_statuses;
	}

	/**
	 * Make quote pending order statuses order editable.
	 *
	 * @param bool   $status Previous Order Status.
	 * @param object $order Order Object.
	 * @return bool
	 */
	public function make_pending_quote_request_order_editable( $status, $order ) {
		$__order_status = $order->get_status();
		if ( in_array( $__order_status, $this->quote_statuses, true ) ) {
			$status = true;
		}
		return $status;
	}

	/**
	 * Send Quote Request to Customer Button on Order Page.
	 *
	 * @param object $order Order Object.
	 * @return void
	 */
	public function send_quote_request_to_customer_button( $order ) {
		$customer_id    = $order->get_user_id();
		$is_guest       = $customer_id ? false : true;
		$__order_status = $order->get_status();
		if ( in_array( $__order_status, $this->quote_statuses, true ) ) {
			if ( ! $is_guest ) {
				?>
					<div class="wholesalex_loader"></div>
					<button onclick="sendQuoteToUser()" data-type="quote_request_to_message" class="raq-to-message"><?php echo esc_html_e( 'Send to Email & Conversation', 'wholesalex-pro' ); ?></button>
					<button onclick="sendQuoteToUser()" data-type="quote_request_to_email" class="raq-to-email"><?php echo esc_html_e( 'Send to Email', 'wholesalex-pro' ); ?></button>
				<?php
			} else {
				?>
					<button onclick="sendQuoteToUser()" data-type="quote_request_to_email" class="raq-to-email"><?php echo esc_html_e( 'Send to Email', 'wholesalex-pro' ); ?></button>
				<?php
			}
			?>
				<?php
				$this->send_quote_to_customer_scripts();
		}
	}


	/**
	 * Send Quote request to customer.
	 *
	 * @return void
	 */
	public function send_quote_to_customer_scripts() {
		?>
			<script>
				const sendQuoteToUser = () => {
					event.preventDefault();
					const type = event.target.getAttribute('data-type');
					const formData = new FormData();
					formData.append('action','send_quote_to_user_action');
					formData.append('postID',document.getElementById('post_ID').value);
					formData.append('nonce',"<?php echo esc_attr( wp_create_nonce( 'wholesalex_quote' ) ); ?>");
					const hasExpireData = document.getElementById('wholesalex_quote_expire_date');
					const hasReminderDate = document.getElementById('wholesalex_reminder_email_date');
					if(hasExpireData) {
						formData.append('expireOn',document.getElementById('wholesalex_quote_expire_date').value);
					}
					if(hasReminderDate) {
						formData.append('reminderOn',document.getElementById('wholesalex_reminder_email_date').value);
					}
					formData.append('type',type);
					jQuery(".wholesalex_loader").show();
					// formData.append('wpnonce', document.getElementById('wholesalex_request_quote_nonce').value);
					fetch(
						wholesalex.ajax, {
						method: 'POST',
						body:formData ,
					})
					.then(res => res.json())
					.then(res => {
						jQuery(".wholesalex_loader").hide();
						if(res.success) {
							if(res.data) {
								if(res.data.redirect) {
									if(res.data.message && window.confirm(res.data.message)) {
										window.location.href = res.data.redirect;
									}
								}
							}
						} else if(res.type) {
							window.alert(res.message);
						}
					})
				}
				jQuery('button.save-action').on('click', (e) => { jQuery('button.calculate-action').trigger('click'); });
			</script>
		<?php
	}

	/**
	 * Send Quote to user action.
	 *
	 * @return void
	 */
	public function send_quote_to_user_action() {
		if ( ! ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'wholesalex_quote' ) ) ) {
			return;
		}
		$order_id = isset( $_POST['postID'] ) ? sanitize_text_field( $_POST['postID'] ) : ''; //phpcs:ignore
		$type     = isset( $_POST['type'] ) ? sanitize_text_field( wp_unslash( $_POST['type'] ) ) : '';
		if ( $order_id ) {
			$conversation_id = get_post_meta( $order_id, '__wholesalex_quote_id', true );
			if ( $conversation_id ) {
				$conversation         = get_post( $conversation_id );
				$conversation_content = $conversation->post_content;
				$expire_date          = isset( $_POST['expireOn'] ) ? sanitize_text_field( $_POST['expireOn'] ) : ''; //phpcs:ignore
				$reminder_date		  = isset( $_POST['expireOn'] ) ? sanitize_text_field( $_POST['expireOn'] ) : ''; //phpcs:ignore		

				$order = wc_get_order( $order_id );

				if ( $expire_date ) {
					update_post_meta( $order_id, '__wholesalex_quote_expire_on', $expire_date );
					do_action( 'wholesalex_after_update_quote_expire_date', $order_id, $expire_date, $order->get_status() );
				}

				if ( $reminder_date ) {
					update_post_meta( $order_id, '__wholesalex_quote_reminder_on', $reminder_date );
					do_action( 'wholesalex_after_update_quote_reminder_date', $order_id, $reminder_date, $order->get_status() );
				}

				$order->update_status( 'whx-quote-pending' );

				$order->calculate_totals();

				$order->save();
				switch ( $type ) {
					case 'quote_request_to_message':
						$current_user_id  = apply_filters( 'wholesalex_set_current_user', $conversation->post_author );
						$accepted_url     = $this->get_quote_link( $conversation_id, $order_id, 'accept_quote', $current_user_id );
						$rejected_url     = $this->get_quote_link( $conversation_id, $order_id, 'reject_quote', $current_user_id );
						$order_table_data = $this->get_order_content( $order_id );
						$items            = $order_table_data['items'];
						$reply_id = (int) get_post_meta($conversation_id,'wholesalex_conversation_reply_id',true);
						$reply_id = $reply_id+1;
						$data = array(
							'ID'           => $conversation_id,
							'post_content' => $conversation_content . '<!-- wp:wsx_reply {"id":"' . $reply_id . '","author":"' . get_current_user_id() . '", "time":"' . gmdate( 'Y-m-d h:i:s A' ) . '","is_offered_quote":true} --><!-- /wp:wsx_reply -->',
						);
						wp_update_post( $data );

						update_post_meta( $conversation_id, '__quote_accept_url', $accepted_url );
						update_post_meta( $conversation_id, '__quote_reject_url', $rejected_url );
						update_post_meta($conversation_id,'wholesalex_conversation_reply_id',$reply_id);
						update_post_meta($conversation_id,'wholesalex_quote_offer_'.$reply_id,json_encode(array('items'=>$items)));

						$message = $this->send_quote_request_to_email( $order, $conversation_id, $order_id, false );
						if ( isset( $message['type'] ) ) {
							switch ( $message['type'] ) {
								case 'success':
									$message['message'] = __( "Your quote is successfully sent to the user's email and conversation.", 'wholesalex-pro' );
									break;
								case 'failed':
									$message['message'] = __( 'Yout quote is succesfully send to the user conversation', 'wholesalex-pro' );
									break;
								case 'disabled_template':
									$message['message'] = __( 'Yout quote is succesfully send to the user conversation', 'wholesalex-pro' );
									break;
								case 'not_exist':
									$message['message'] = __( 'Yout quote is succesfully send to the user conversation', 'wholesalex-pro' );
									break;
								default:
									// code...
									break;
							}
						}

						$conversation_url = admin_url( 'post.php?post=' . $conversation_id ) . '&action=edit';

						wp_send_json_success(
							array(
								'message'  => isset( $message['message'] ) ? $message['message'] : '',
								'redirect' => $conversation_url,
							)
						);
						break;
					case 'quote_request_to_email':
						$this->send_quote_request_to_email( $order, $conversation_id, $order_id );
						break;

					default:
						// code...
						break;
				}
			}
		}
	}

	/**
	 * Get Customer Email
	 *
	 * @param object     $order Order.
	 * @param string|int $conversation_id Conversation ID.
	 * @return string
	 * @since 1.0.5
	 */
	public function get_customer_email( $order, $conversation_id ) {
		$__user_email = '';
		if ( ! $order->get_user_id() ) {
			// Send Email.
			$quote_data   = get_post_meta( $conversation_id, '__wholesalex_quote_data', true );
			$quote_data   = json_decode( $quote_data, true );
			$__user_email = $quote_data['user_email'];
		} else {
			$userdata     = get_userdata( $order->get_user_id() );
			$__user_email = $userdata->user_email;
			$__user_email = isset( $quote_data['user_email'] ) ? $quote_data['user_email'] : $__user_email;
		}

		return $__user_email;
	}

	/**
	 * Send Quote Request to email
	 *
	 * @param object     $order Order Object.
	 * @param int|string $conversation_id Conversation ID.
	 * @param int|string $order_id Conversation ID.
	 * @return void
	 */
	public function send_quote_request_to_email( $order, $conversation_id, $order_id, $return_json = true ) {
		$__user_email = $this->get_customer_email( $order, $conversation_id );
		$quote_data   = $this->get_quote_data( $conversation_id );
		$message      = array(
			'type'    => 'error',
			'message' => __( 'Something is wrong!', 'wholesalex-pro' ),
		);
		if ( '' == $__user_email || ! $__user_email ) {
			$message = array(
				'type'    => 'not_exist',
				'message' => __( 'User Email Does not exist!', 'wholesalex-pro' ),
			);
		} else {
			$message = apply_filters( 'wholesalex_raq_send_email', array(), $__user_email, $order_id, $conversation_id, $quote_data );
			if ( $return_json ) {
				wp_send_json( $message );
			} else {
				return $message;
			}
		}
		if ( $return_json ) {
			wp_send_json( $message );
		} else {
			return $message;
		}
	}

	/**
	 * Change Quote Request Status
	 *
	 * @return void
	 */
	public function change_quote_status() {
		$request_data = $_REQUEST; //phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( ! ( isset( $request_data['quote_nonce'] ) && isset( $request_data['wsxview'] ) && isset( $request_data['status'] ) ) ) {
			return;
		}

		$__user_id = get_current_user_id();
		$order_id  = get_post_meta( $request_data['wsxview'], '__wholesalex_quote_order_id', true );

		if ( current_user_can( 'manage_options' ) ) {
			switch ( $request_data['status'] ) {
				case 'accept_quote':
					$order = wc_get_order( $order_id );
					if ( $order->get_customer_id() === get_current_user_id() ) {
						wc_add_notice( __('You cannot accept your own quote request','wholesalex-pro'), 'error' );
						return;
					}
					break;
				case 'reject_quote':
					$order  = wc_get_order( $order_id );
					$status = $order->get_status();
					if ( 'whx-quote-pending' === $status ) {
						$order->update_status( 'whx-quote-reject' );
						$order->save();
					} else {
						wc_print_notice( __('Quote is already accepted/rejected.','wholesalex-pro'), 'error' );
						return;
					}

					$order_url = admin_url( 'post.php?post=' . $order_id ) . '&action=edit';
					wp_safe_redirect( $order_url );
					exit;
					break;

				default:
					break;
			}
		}

		$__user_id = apply_filters( 'wholesalex_set_current_user', $__user_id );
		switch ( $request_data['status'] ) {
			case 'accept_quote':
				if ( $this->verify_unique_quote_token( $request_data['quote_nonce'], $request_data['wsxview'], $order_id, $request_data['status'], $__user_id ) ) {
					$order  = wc_get_order( $order_id );
					$status = $order->get_status();

					$expire_date = get_post_meta( $order_id, '__wholesalex_quote_expire_on', true );

					$has_expire_date = $expire_date ? true : false;

					$expire_date = strtotime( $expire_date );

					$expire_date += ( 24 * 60 * 60 ) - 1;

					$current_time = time();

					if ( $has_expire_date && $current_time > $expire_date ) {
						// Expired.
						do_action( 'wholesalex_raq_before_expire', $order );
						$order->update_status( 'whx-quote-expired' );
						$order->save();
						$__user_email = $this->get_customer_email( $order, $request_data['wsxview'] );
						do_action( 'wholesalex_raq_after_expire', $__user_email, $order_id, $request_data['wsxview'] );

						wc_print_notice( __('Quote is already expired.','wholesalex-pro'), 'error' );
						return;

					} else {
						if ( 'whx-quote-pending' === $status ) {
							$order->update_meta_data( '__wholesalex_quote_status', 'accepted' );
							$order->save();

							update_post_meta( $request_data['wsxview'], '__conversation_status', 'resolved' );
							WC()->cart->empty_cart( true );
							$this->quote_accepted( $order_id );
						} else {
							wc_print_notice( __('Quote is already accepted/rejected.','wholesalex-pro'), 'error' );
							return;
						}
					}
				} else {
					wc_print_notice( __('You are not authorized to view this quote.','wholesalex-pro'), 'error' );
					return;
				}
				break;
			case 'reject_quote':
				if ( $this->verify_unique_quote_token( $request_data['quote_nonce'], $request_data['wsxview'], $order_id, $request_data['status'], $__user_id ) ) {
					$order  = wc_get_order( $order_id );
					$status = $order->get_status();
					if ( 'whx-quote-pending' === $status ) {
						$order->update_status( 'whx-quote-reject' );
						$order->save();
					} else {
						wc_print_notice( __('Quote is already accepted/rejected.','wholesalex-pro'), 'error' );
						return;
					}
				} else {
					wc_print_notice( __('You are not authorized to view this quote.','wholesalex-pro'), 'error' );
					return;
				}
				break;

			default:
				break;
		}
	}

	/**
	 * Create Unique Quote Token
	 *
	 * @param string $conversation_id Conversation Id.
	 * @param string $order_id Order ID.
	 * @param string $status Status.
	 * @param string $user_id User ID.
	 * @return string
	 */
	public function create_unique_quote_token( $conversation_id, $order_id, $status, $user_id ) {
		return wp_hash( $conversation_id . '|' . $order_id . '|' . $status . '|' . $user_id, 'wholesalex_quote_request' );
	}

	/**
	 * Create Unique Quote Token
	 *
	 * @param string $token Quote Nonce Token.
	 * @param string $conversation_id Conversation Id.
	 * @param string $order_id Order ID.
	 * @param string $status Status.
	 * @param string $user_id User ID.
	 * @return bool
	 */
	public function verify_unique_quote_token( $token, $conversation_id, $order_id, $status, $user_id ) {
		$expected = wp_hash( $conversation_id . '|' . $order_id . '|' . $status . '|' . $user_id, 'wholesalex_quote_request' );
		if ( hash_equals( $expected, $token ) ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Accept Quote Request and Add To Cart The Quote Items and Discounted Price.
	 *
	 * @param int|string $order_id Order ID.
	 * @return void
	 * @since 1.0.5
	 */
	public function quote_accepted( $order_id ) {
		// If not defined wc checkout, define it.
		if ( ! defined( 'WOOCOMMERCE_CHECKOUT' ) ) {
			define( 'WOOCOMMERCE_CHECKOUT', true );
		}
		// Clear Current Cart, if any.
		WC()->cart->empty_cart( true );
		WC()->cart->get_cart_from_session();
		WC()->session->set( 'order_awaiting_payment', $order_id );
		WC()->cart->set_session();

		$order = wc_get_order( $order_id );

		if ( ! $order ) {
			return;
		}

		// Order status should be make pending before add to cart.
		do_action( 'wholesalex_quote_before_pending', $order_id );
		$order->update_status( 'pending' );

		// Major Part: Set quote items to cart.
		foreach ( $order->get_items() as $item ) {
			$product_id   = (int) apply_filters( 'woocommerce_add_to_cart_product_id', $item->get_product_id() );
			$quantity     = $item->get_quantity();
			$variation_id = (int) $item->get_variation_id();
			$variations   = array();
			$product      = $item->get_product();
			if ( ! $product ) {
				continue;
			}

			// Prevent reordering variable products if no selected variation.
			if ( ! $variation_id && $product->is_type( 'variable' ) ) {
				continue;
			}

			// Prevent reordering items specifically out of stock.
			if ( ! $product->is_in_stock() ) {
				continue;
			}

			foreach ( $item->get_meta_data() as $meta ) {
				if ( taxonomy_is_product_attribute( $meta->key ) ) {
					$term                                    = get_term_by( 'slug', $meta->value, $meta->key );
					$variations[ 'attribute_' . $meta->key ] = $term ? $term->value : $meta->value;
				} elseif ( meta_is_product_attribute( $meta->key, $meta->value, $product_id ) ) {
					$variations[ 'attribute_' . $meta->key ] = $meta->value;
				}
			}

			$cart_item_data = apply_filters( 'woocommerce_order_again_cart_item_data', array(), $item, $order );
			$cart_item_data = apply_filters( 'wholesalex_quote_cart_item_data', $cart_item_data, $item, $order );

			$meta_data = $item->get_meta_data();

			if ( ! empty( $meta_data ) ) {

				foreach ( $meta_data as $meta ) {
					$cart_item_data['meta'][] = $meta->get_data();
				}
			}

			/**
			 * For Compatibility With WCPA (WooCommerce Custom Product Addon Plugin)
			 *
			 * @since
			 */
			if ( defined( 'WCPA_VERSION' ) ) {
				$wcpa_meta_data = wc_get_order_item_meta( $item->get_id(), '_WCPA_order_meta_data', true );
				if ( ! empty( $wcpa_meta_data ) ) {
					$cart_item_data['wcpa_data'] = $wcpa_meta_data;
				}
			}

			$cart_item_data = apply_filters( 'wholesalex_quote_cart_item_date_before_add_to_cart', $cart_item_data, $order );
			WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $variations, $cart_item_data );
		}

		$order->save();

		WC()->cart->calculate_totals();

		wp_safe_redirect( wc_get_checkout_url() );
		exit;
	}

	/**
	 * Set Cart Hash
	 *
	 * @param string $value string .
	 *
	 * @return string
	 * @since 1.0.5
	 */
	public function set_cart_hash( $value ) {

		$order_id = $this->get_current_order_id();
		$order    = wc_get_order( $order_id );

		if ( $order ) {
			$order_status = $order->get_meta( '__wholesalex_quote_status', true );

			if ( 'accepted' === $order_status ) {
				$hash = method_exists( WC()->cart, 'get_cart_hash' ) ? WC()->cart->get_cart_hash() : md5( wp_json_encode( wc_clean( WC()->cart->get_cart_for_session() ) ) . WC()->cart->total );
				$order->set_cart_hash( $hash );
				$order->save();
			}
		}

		return $value;
	}

	/**
	 * Get Current Order ID.
	 *
	 * @return int Order ID.
	 * @since 1.0.5
	 */
	public function get_current_order_id() {
		if ( is_null( WC()->session ) ) {
			return 0;
		}
		$order_id = absint( WC()->session->get( 'order_awaiting_payment' ) );

		return $order_id;
	}

	/**
	 * Get Order Content.
	 *
	 * @param string|int $order_id Order Id.
	 * @return array
	 */
	public function get_order_content( $order_id ) {
		$data  = array();
		$order = wc_get_order( $order_id );

		foreach ( $order->get_items() as $item ) {
			$quantity = $item->get_quantity();

			$product_id   = $item->get_product_id();
			$quantity     = $item->get_quantity();
			$variation_id = $item->get_variation_id();

			$product_title   = $item->get_name();
			$subtotal        = $item->get_subtotal();
			$total           = $item->get_total();
			$tax             = $item->get_subtotal_tax();
			$data['items'][] = array(
				'product_id'    => $product_id,
				'variation_id'  => $variation_id,
				'product_title' => $product_title,
				'quantity'      => $quantity,
				'subtotal'      => $subtotal,
				'total'         => $total,
				'tax'           => $tax,
			);
		}

		$total_discount         = $order->get_total_discount( false );
		$subtotal               = $order->get_subtotal();
		$total                  = $order->get_total();
		$data['total_discount'] = $total_discount;
		$data['subtotal']       = $subtotal;
		$data['total']          = $total;

		return $data;

	}

	/**
	 * Get Order Table for Quote
	 *
	 * @param array $order_data Order Data.
	 * @return void
	 * @since 1.0.5
	 */
	public static function get_order_table( $quote_data, $is_backend = false, $is_email = false ) {
		include __DIR__ . '/views/html-order-table.php';
	}

	/**
	 * Add Discounts on Cart
	 * If approved quote items and cart items matches, then quote discounts / quote price will be apply.
	 *
	 * @since 1.0.5
	 */
	public function add_discounts_on_cart() {
		if ( is_null( WC()->session ) ) {
			return;
		}
		$order_id = WC()->session->get( 'order_awaiting_payment' );

		if ( ! $order_id ) {
			return;
		}
		$cart_items = array();
		foreach ( WC()->cart->get_cart_contents() as $key => $item ) {
			$cart_items[ $item['product_id'] ][] = array(
				'variation_id' => $item['variation_id'],
				'quantity'     => $item['quantity'],
			);
		}

		$order_items = array();
		foreach ( wc_get_order( $order_id )->get_items() as $item ) {
			$order_items[ $item->get_product_id() ][] = array(
				'variation_id' => $item->get_variation_id(),
				'quantity'     => $item->get_quantity(),
			);
		}

		if ( $cart_items === $order_items ) {
			if ( $order_id ) {
				$order = wc_get_order( $order_id );

				$order_status = $order->get_meta( '__wholesalex_quote_status', true );
				if ( 'accepted' === $order_status ) {
					$conversation_id = get_post_meta( $order_id, '__wholesalex_quote_id', true );
					WC()->cart->add_fee( __('Quote Discount #','wholesalex-pro') . $conversation_id, -$order->get_total_discount( true ), true );
				}
			}
		}
	}

	/**
	 * Register Quote option metabox on edit order page.
	 *
	 * @return void
	 */
	public function quote_option_metabox_register() {
		$request = $_REQUEST; //phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$post    = isset( $request['post'] ) ? $request['post'] : ( isset( $request['post_ID'] ) ? $request['post_ID'] : 0 );
		$post_type = get_post_type($post);

		if ( ( $post && 'shop_order' === $post_type ) ) {
			$is_quote = get_post_meta( $post, '__wholesalex_quote_status', true );
			if('accepted' !== $is_quote) {
				$post     = get_post( $post );
				add_action( 'add_meta_boxes', array( $this, 'quote_option_meta_box' ) );
			}
		}

	}

	/**
	 * Add Quote option metabox on edit order page.
	 *
	 * @return void
	 */
	public function quote_option_meta_box() {
		add_meta_box( 'wholesalex_quote_option', 'Quote Options', array( $this, 'quote_option_metabox_content' ), 'shop_order', 'side', 'high' );
	}

	/**
	 * Quote option metabox content.
	 *
	 * @return void
	 */
	public function quote_option_metabox_content() {
		global $post;
		$expire_date         = get_post_meta( $post->ID, '__wholesalex_quote_expire_on', true );
		$reminder_email_date = get_post_meta( $post->ID, '__wholesalex_quote_reminder_on', true );
		?>
			<div class="wholesalex_quote_expire_date">
				<label for="wholesalex_quote_expire_date"><?php echo esc_html_e( 'This quote expire on:', 'wholesalex-pro' ); ?></label>
				<input type="date" name="wholesalex_quote_expire_date" id="wholesalex_quote_expire_date" value="<?php echo ( $expire_date ? esc_attr( $expire_date ) : '' ); //phpcs:ignore ?>"></input>
			</div>
			<div class="wholesalex_reminder_email_date">
				<label for="wholesalex_reminder_email_date"><?php echo esc_html_e( 'Send reminder email on:', 'wholesalex-pro' ); ?></label>
				<input type="date" name="wholesalex_reminder_email_date" id="wholesalex_reminder_email_date" value="<?php echo ( $reminder_email_date ? esc_attr($reminder_email_date) : '' ); //phpcs:ignore ?>"></input>
			</div>
		<?php
	}

	/**
	 * Save Quote Option Metabox Data on Order Page
	 *
	 * @param string|int $post_id Order ID.
	 * @param object     $post Order.
	 * @param bool       $is_update Is Update.
	 * @return void
	 */
	public function save_quote_option_metabox_data( $post_id, $post, $is_update ) {
		if ( ! $is_update ) {
			return;
		}

		$expire_date = isset( $_POST['wholesalex_quote_expire_date'] ) ? sanitize_text_field( $_POST['wholesalex_quote_expire_date'] ) : ''; //phpcs:ignore		
		$reminder_date = isset( $_POST['wholesalex_reminder_email_date'] ) ? sanitize_text_field( $_POST['wholesalex_reminder_email_date'] ) : ''; //phpcs:ignore	

		$order = wc_get_order( $post_id );

		if ( $expire_date ) {
			update_post_meta( $post_id, '__wholesalex_quote_expire_on', $expire_date );
			do_action( 'wholesalex_after_update_quote_expire_date', $post_id, $expire_date, $order->get_status() );
		}

		if ( $reminder_date ) {
			update_post_meta( $post_id, '__wholesalex_quote_reminder_on', $reminder_date );
			do_action( 'wholesalex_after_update_quote_reminder_date', $post_id, $reminder_date, $order->get_status() );
		}
	}

	/**
	 * Register Request a quote cron schedules.
	 *
	 * @return void
	 */
	public function register_request_a_quote_cron_jobs() {
		// Register a cron schedules to check quote request orders, are they expire or not.
		if ( ! wp_next_scheduled( 'wholesalex_raq_expire_date_check' ) ) {
			$gmt_offset = get_option( 'gmt_offset' );
			wp_schedule_event( strtotime( '00:00 tomorrow ' . ( $gmt_offset > 0 ? '+' : '-' ) . $gmt_offset . ' HOURS' ), 'daily', 'wholesalex_raq_expire_date_check' );
		}

		// Remove Procced to checkout button while add to cart replaced with add to quote.
		$__role_id      = wholesalex()->get_current_user_role();
		$__role_content = wholesalex()->get_roles( 'by_id', $__role_id );
		if ( isset( $__role_content['_raq_replace_add_to_cart_with_quote'] ) && 'yes' === $__role_content['_raq_replace_add_to_cart_with_quote'] ) {
			remove_all_actions( 'woocommerce_proceed_to_checkout' );
		}
	}

	/**
	 * Request a quote order expire date check.
	 * If expire, then change order status to expired.
	 *
	 * @return void
	 */
	public function raq_orders_expire_date_check() {
		$args = apply_filters(
			'wholesalex_raq_expire_date_check_args',
			array(
				'status'       => 'whx-quote-pending',
				'meta_key'     => '__wholesalex_quote_expire_on',
				'meta_value'   => '',
				'meta_compare' => '!=',
			)
		);

		$orders = wc_get_orders( $args );
		foreach ( $orders as $order ) {
			$expire_date        = get_post_meta( $order->get_id(), '__wholesalex_quote_expire_on', true );
			$reminder_mail_date = get_post_meta( $order->get_id(), '__wholesalex_quote_reminder_on', true );

			$conversation_id = get_post_meta( $order->get_id(), '__wholesalex_quote_id', true );
			$__user_email    = $this->get_customer_email( $order, $conversation_id );

			if ( $reminder_mail_date < $expire_date ) {
				$cur_date      = strtotime( date( 'Y/m/d' ) );
				$reminder_date = strtotime( $reminder_mail_date );
				if ( $cur_date === $reminder_date ) {
					do_action( 'wholesalex_quote_reminder_email', $__user_email, $order->get_id(), $conversation_id );
				}
			}
			$gmt_offset = get_option( 'gmt_offset' );
			// Site Time.
			// $expire_date = strtotime( $expire_date . ( $gmt_offset > 0 ? '-' : '+' ) . $gmt_offset . ' HOURS' );
			// GMT +0 Timestamp.
			$expire_date = strtotime( $expire_date );

			$expire_date += ( 24 * 60 * 60 ) - 1;

			$current_time = time();

			if ( $current_time > $expire_date ) {
				// Expired.
				do_action( 'wholesalex_raq_before_expire', $order );
				$order->update_status( 'whx-quote-expired' );
				$order->save();
				do_action( 'wholesalex_raq_after_expire', $__user_email, $order->get_id(), $conversation_id );
			}
		}
	}


	/**
	 * Add Quote Discount
	 *
	 * @return void
	 */
	public function add_quote_discount() {
		if ( is_null( WC()->session ) ) {
			return;
		}
		$order_id = WC()->session->get( 'order_awaiting_payment' );

		if ( ! $order_id ) {
			return;
		}
		$cart_items = array();
		foreach ( WC()->cart->get_cart_contents() as $key => $item ) {
			$cart_items[ $item['product_id'] ][] = array(
				'variation_id' => $item['variation_id'],
				'quantity'     => $item['quantity'],
			);
		}

		$order_items = array();
		foreach ( wc_get_order( $order_id )->get_items() as $item ) {

			$order_items[ $item->get_product_id() ][] = array(
				'variation_id' => $item->get_variation_id(),
				'quantity'     => $item->get_quantity(),
			);

		}

		if ( $cart_items === $order_items ) {
			if ( $order_id ) {
				$order = wc_get_order( $order_id );

				$order_status = $order->get_meta( '__wholesalex_quote_status', true );
				if ( 'accepted' === $order_status ) {
					$conversation_id = get_post_meta( $order_id, '__wholesalex_quote_id', true );
					WC()->cart->add_fee( __('Quote Discount #','wholesalex-pro') . $conversation_id, -$order->get_total_discount( true ) );
				}
			}
		}

	}

	/**
	 * Change Order Status after update expire date.
	 *
	 * @param string|int $order_id Order ID.
	 * @param string     $updated_date Updated Expire Date.
	 * @return void
	 */
	public function change_order_status_after_update_expire_date( $order_id, $updated_date ) {
		$order       = wc_get_order( $order_id );
		$expire_date = strtotime( $updated_date );

		$expire_date += ( 24 * 60 * 60 ) - 1;

		$current_time = time();

		$conversation_id = get_post_meta( $order->get_id(), '__wholesalex_quote_id', true );
		$__user_email    = $this->get_customer_email( $order, $conversation_id );
		if ( $current_time > $expire_date ) {
			// Expired.
			do_action( 'wholesalex_raq_before_expire', $order );
			$order->update_status( 'whx-quote-expired' );
			do_action( 'wholesalex_raq_after_expire', $__user_email, $order->get_id(), $conversation_id );
		} else {
			$order->update_status( 'whx-quote-pending' );
			do_action( 'wholesalex_raq_pending', $__user_email, $order->get_id(), $conversation_id );
		}
		$order->save();
	}

	/**
	 * Get Smart Tag Data
	 *
	 * @param string              $type Smart Tag Type.
	 * @param string|int          $conversation_id Conversation ID.
	 * @param string|int Order ID $order_id Order ID.
	 * @param array               $quote_data Quote Data.
	 * @return string
	 */
	public function get_smart_tag_data( $type, $conversation_id, $order_id, $quote_data ) {
		$data          = '';
		$customer_id   = get_post_field( 'post_author', $conversation_id, 'edit' );
		$customer_name = '';
		if ( $customer_id ) {
			$customer_name = get_post_field( 'display_name', $conversation_id, 'display' );
		} else {
			$customer_name = isset( $quote_data['name'] ) ? $quote_data['name'] : __( 'Guest', 'wholesalex-pro' );
		}
		$order_data = $this->get_order_content( $order_id );

		switch ( $type ) {
			case 'accept_button':
				$link = $this->get_quote_link( $conversation_id, $order_id, 'accept_quote', $customer_id );
				ob_start();
				?>
					<a href="<?php echo esc_url( $link ); ?>"><?php echo esc_html_e( 'Accept', 'wholesalex-pro' ); ?></a>
				<?php
				$data = ob_get_clean();
				break;
			case 'reject_button':
				$link = $this->get_quote_link( $conversation_id, $order_id, 'reject_quote', $customer_id );
				ob_start();
				?>
					<a href="<?php echo esc_url( $link ); ?>"><?php echo esc_html_e( 'Reject', 'wholesalex-pro' ); ?></a>
				<?php
				$data = ob_get_clean();
				break;
			case 'quote_table':
				$items = $order_data['items'];
				ob_start();
				self::get_order_table( $items, true, true );
				$data = ob_get_clean();
				break;
			case 'total_discount_amount':
				ob_start();
				echo wc_price( $order_data['total_discount'] ); //phpcs:ignore
				$data = ob_get_clean();
				break;
			case 'total_payable_amount':
				ob_start();
				echo wc_price( $order_data['total'] ); //phpcs:ignore
				$data = ob_get_clean();
				break;
			case 'expire_date':
				$data = get_post_meta( $order_id, '__wholesalex_quote_expire_on', true );
				$data = $data ? $data : '';
				break;
			case 'customer_name':
				$data = $customer_name;
				break;
			case 'quote_title':
				$data = get_the_title( $conversation_id );
				break;

			default:
				break;
		}
		return $data;
	}

	/**
	 * Send Quote Offer to Guest User Email
	 *
	 * @param array      $message Error/Success Message. Added on v1.0.6.
	 * @param string     $to Receiver Email Address.
	 * @param string|int $order_id Order ID.
	 * @param string|int $conversation_id Conversation Id.
	 * @param array      $quote_data Quote Data.
	 * @return void
	 * @since 1.0.5
	 * @since 1.0.6 Return Messages as array instead of json
	 */
	public function send_email_to_user( $message, $to, $order_id, $conversation_id, $quote_data ) {

		$customer_id = get_post_field( 'post_author', $conversation_id, 'edit' );
		$accept_url  = $this->get_quote_link( $conversation_id, $order_id, 'accept_quote', $customer_id );
		$reject_url  = $this->get_quote_link( $conversation_id, $order_id, 'reject_quote', $customer_id );

		$expire_date   = get_post_meta( $order_id, '__wholesalex_quote_expire_on', true );
		$customer_name = $this->get_smart_tag_data( 'customer_name', $conversation_id, $order_id, $quote_data );
		$quote_title   = $this->get_smart_tag_data( 'quote_title', $conversation_id, $order_id, $quote_data );

		$email  = $this->emails['WholesaleX_Quote_Make_Offer_Email'];
		$status = $email->trigger( $order_id, $customer_name, $to, $accept_url, $reject_url, $expire_date, $quote_title );

		$message = array();

		if ( $status ) {
			$__email_send_count = get_post_meta( $order_id, '__wholesalex_quote_email_sent_count', true );
			if ( $__email_send_count ) {
				$__email_send_count++;
				update_post_meta( $order_id, '__wholesalex_quote_email_sent_count', $__email_send_count );
			} else {
				update_post_meta( $order_id, '__wholesalex_quote_email_sent_count', 1 );
			}
			$message = array(
				'type'    => 'success',
				'message' => __( 'Email Sent Successfull.', 'wholesalex-pro' ),
			);
			return $message;
		} else {
			$message = array(
				'type'    => 'failed',
				'message' => __( 'Email Sent Failed!', 'wholesalex-pro' ),
			);
		}
		return $message;
	}

	/**
	 * Get Quote Link (Accept/Reject Quote)
	 *
	 * @param string|int $conversation_id Conversation ID.
	 * @param string|int $order_id Order ID.
	 * @param string     $type Link Type (accept_quote, reject_quote).
	 * @param string|int $customer_id Customer ID.
	 * @return string url.
	 */
	public function get_quote_link( $conversation_id, $order_id, $type, $customer_id ) {
		$url = add_query_arg(
			array(
				'wsxview'     => $conversation_id,
				'status'      => $type,
				'quote_nonce' => $this->create_unique_quote_token( $conversation_id, $order_id, $type, $customer_id ),
			),
			wc_get_account_endpoint_url( apply_filters( 'wholesalex_conversation_addon_my_account_end_point', __( 'wsx-conversation', 'wholesalex-pro' ) ) )
		);
		return $url;
	}

	/**
	 * Request a quote smart tags.
	 *
	 * @param array $tags Smart Tags.
	 * @return array
	 */
	public function request_a_quote_email_smart_tags( $tags, $template_name ) {
		if ( 'wholesalex_raq_make_offer' === $template_name || 'wholesalex_raq_expiring' === $template_name ) {
			$tags                          = array();
			$tags['date']                  = __( 'Show the current date inside the Email.', 'wholesalex-pro' );
			$tags['accept_button']         = __( 'Show the offer accepting button of the Quote Request.', 'wholesalex-pro' );
			$tags['reject_button']         = __( 'Show the offer rejecting button of the Quote Request.', 'wholesalex-pro' );
			$tags['expire_date']           = __( 'Show the expiry date of the Quote Request', 'wholesalex-pro' );
			$tags['customer_name']         = __( 'Show customer Name of Quote Request.', 'wholesalex-pro' );
			$tags['quote_table']           = __( 'Show the quote table of the Quote Request.', 'wholesalex-pro' );
			$tags['total_discount_amount'] = __( 'Show the total discount amount of the Quote Request.', 'wholesalex-pro' );
			$tags['total_payable_amount']  = __( 'Show the total payable amount of the Quote Request.', 'wholesalex-pro' );
			$tags['site_name']             = __( 'Show the Site Name.', 'wholesalex-pro' );
			$tags['admin_name']            = __( 'Show the Super Admin Name.', 'wholesalex-pro' );
			$tags['quote_title']           = __( 'Show the Quote Title.', 'wholesalex-pro' );
		}

		return $tags;
	}

	/**
	 * Rolewise Request A Quote Settings
	 *
	 * @param array $sections Role Settings.
	 * @return array
	 * @since 1.0.5
	 */
	public function raq_role_settings( $sections ) {
		$raq_settings = array(
			'_raq_replace_add_to_cart_with_quote' => array(
				'type'    => 'switch',
				'label'   => __( 'Replace Add To Cart with Add To Quote', 'wholesalex-pro' ),
				'desc'    => __( 'Replace Cart Button', 'wholesalex-pro' ),
				'help'    => '',
				'default' => 'no',
			),
		);

		$sections['_role']['attr']['settings_section']['attr'] = array_merge( $sections['_role']['attr']['settings_section']['attr'], $raq_settings );

		return $sections;
	}


	/**
	 * Enqueue Scripts
	 *
	 * @return void
	 */
	public function enqueue_scripts() {
		wp_enqueue_style( 'wholesalex-raq', WHOLESALEX_PRO_URL . 'addons/raq/assets/css/wholesalex_raq.css', array(), WHOLESALEX_PRO_VER );
	}

	/**
	 * Send Email After Expire Quote
	 *
	 * @param string     $to Customer/User Email.
	 * @param string|int $order_id Order ID.
	 * @param string|int $conversation_id Conversation ID.
	 * @return void
	 */
	public function send_expiring_email( $to, $order_id, $conversation_id ) {

		$quote_data = $this->get_quote_data( $conversation_id );

		$customer_id = get_post_field( 'post_author', $conversation_id, 'edit' );
		$accept_url  = $this->get_quote_link( $conversation_id, $order_id, 'accept_quote', $customer_id );
		$reject_url  = $this->get_quote_link( $conversation_id, $order_id, 'reject_quote', $customer_id );

		$expire_date   = get_post_meta( $order_id, '__wholesalex_quote_expire_on', true );
		$customer_name = $this->get_smart_tag_data( 'customer_name', $conversation_id, $order_id, $quote_data );
		$quote_title   = $this->get_smart_tag_data( 'quote_title', $conversation_id, $order_id, $quote_data );

		$email  = $this->emails['WholesaleX_Quote_Expiring_Offer_Email'];
		$status = $email->trigger( $order_id, $customer_name, $to, $accept_url, $reject_url, $expire_date, $quote_title );

		if ( $status ) {
			$__email_send_count = get_post_meta( $order_id, '__wholesalex_quote_email_sent_count', true );
			if ( $__email_send_count ) {
				$__email_send_count++;
				update_post_meta( $order_id, '__wholesalex_quote_email_sent_count', $__email_send_count );
			} else {
				update_post_meta( $order_id, '__wholesalex_quote_email_sent_count', 1 );
			}

			wp_send_json_success( array( 'message' => __( 'Email Sent Successfull.' ) ) );
		} else {
			wp_send_json_error( array( 'message' => __( 'Email Sent Failed!' ) ) );
		}

	}
	/**
	 * Send Email After Quote Request Received (Admin)
	 *
	 * @return void
	 */
	public function send_quote_request_received_email( $conversation_id, $display_name, $items ) {

		$email  = $this->emails['WholesaleX_Quote_Request_Recieved_Admin'];
		$status = $email->trigger( $conversation_id,$display_name,$items);

	}

	/**
	 * Get Quote Data
	 *
	 * @param string|int $conversation_id Conversation ID.
	 * @return array quote data | empty
	 */
	public function get_quote_data( $conversation_id = '' ) {
		if ( ! empty( $conversation_id ) ) {
			$quote_data = get_post_meta( $conversation_id, '__wholesalex_quote_data', true );
			$quote_data = json_decode( $quote_data, true );
			return $quote_data;
		} else {
			return array();
		}
	}

	/**
	 * Replace Add to cart text with Add to quote
	 *
	 * @param string $text Add to cart text.
	 * @return string modified text.
	 * @since 1.0.5
	 * @since 1.1.5 Is Product Object Check Added
	 */
	public function change_add_to_cart_text( $text ) {
		global $product;
		$__role_id      = wholesalex()->get_current_user_role();
		$__role_content = wholesalex()->get_roles( 'by_id', $__role_id );
		if ( is_object( $product ) && is_a( $product, 'WC_Product' ) && $product->is_purchasable() && isset( $__role_content['_raq_replace_add_to_cart_with_quote'] ) && 'yes' === $__role_content['_raq_replace_add_to_cart_with_quote'] ) {
			if ( $product->is_type( 'simple' ) ) {
				$text = apply_filters( 'wholesalex_raq_add_to_quote_text', __( 'Add to quote', 'wholesalex-pro' ) );
			}
		}

		return $text;
	}


	/**
	 * Replace Add to cart text with Add to quote
	 *
	 * @param string $text Add to cart text.
	 * @return string modified text.
	 * @since 1.0.5
	 */
	public function change_single_product_add_to_cart_text( $text ) {
		global $product;
		$__role_id      = wholesalex()->get_current_user_role();
		$__role_content = wholesalex()->get_roles( 'by_id', $__role_id );
		if ( $product->is_purchasable() && isset( $__role_content['_raq_replace_add_to_cart_with_quote'] ) && 'yes' === $__role_content['_raq_replace_add_to_cart_with_quote'] ) {
			$text = apply_filters( 'wholesalex_raq_add_to_quote_text', __( 'Add to quote', 'wholesalex-pro' ) );
		}

		return $text;
	}

	/**
	 * Request A Quote setting on Global Settings
	 *
	 * @param array $sections Settings Section.
	 * @return array
	 * @since 1.0.5
	 */
	public function raq_settings( $sections ) {
		$raq_settings                     = array(
			'_raq_clear_cart_after_request' => array(
				'type'    => 'switch',
				'label'   => __( 'Clear Cart After Request Custom Quote', 'wholesalex-pro' ),
				'desc'    => __( 'Clear Cart ', 'wholesalex-pro' ),
				'help'    => '',
				'default' => 'no',
			),
		);
		if( !isset($sections['conversation']['attr'])) {
			$sections['conversation']['attr'] = array();
		}
		$sections['conversation']['attr'] = array_merge( $sections['conversation']['attr'], $raq_settings );
		return $sections;
	}

	/**
	 * Change My Account Conversation Menu Title
	 *
	 * @param string $title Previous Title.
	 * @return string
	 * @since 1.0.5
	 */
	public function change_my_account_menu_title( $title ) {
		$title = __( 'Conversation & Quote', 'wholesalex-pro' );
		return $title;
	}


	/**
	 * Quote Submission Success Message
	 *
	 * @return void
	 * @since 1.0.6
	 */
	public function quote_submission_success_message() {
		if ( ! is_admin() && isset( $_GET['nonce'], $_GET['status'] ) && wp_verify_nonce( sanitize_key( wp_unslash( $_GET['nonce'] ) ), 'quote_request_success' ) ) {
			$status = sanitize_text_field( wp_unslash( $_GET['status'] ) );
			switch ( $status ) {
				case 'success':
					$user_data = get_userdata( get_current_user_id() );
					/* translators: User Display Name */
					wc_add_notice( sprintf( __( 'Hello %s, thank you for requesting a quote. Our team is working on getting you the best deal possible.', 'wholesalex-pro' ), $user_data->display_name ), 'success' );
					break;

				default:
					// code...
					break;
			}
		}
	}

	/**
	 * Add Request a Quote columns on Role Exporter Columns
	 *
	 * @param array $columns Role Exporter Columns
	 * @return void
	 */
	public function add_raq_fields_on_role_exporter( $columns ) {
		$columns['_raq_replace_add_to_cart_with_quote'] = __( 'Replace Add to Cart With Add to Quote', 'wholesalex-pro' );
		return $columns;
	}

	/**
	 * Get Role
	 *
	 * @param [type] $role
	 * @return void
	 */
	public function get_column_value__raq_replace_add_to_cart_with_quote( $value, $role ) {

		return isset( $role['_raq_replace_add_to_cart_with_quote'] ) ? $role['_raq_replace_add_to_cart_with_quote'] : '';
	}

	public function add_quote_table_before_message_content($post_id,$reply_val) {
		if(isset($reply_val['attrs']['is_requested_quote']) && $reply_val['attrs']['is_requested_quote']) {
			$quote_data = get_post_meta($post_id,'__wholesalex_quote_data',true);
			if($quote_data) {
				$quote_data = (array)json_decode($quote_data,true);
				echo '<span class="quote_request_items_heading">'. esc_html__('Requested Items','wholesalex-pro').'</span>';
				self::get_order_table((array) $quote_data['items'], is_admin()  );
			}
		}

		if(isset($reply_val['attrs']['is_offered_quote']) && $reply_val['attrs']['is_offered_quote']) {
			
			$quote_data = get_post_meta($post_id,'wholesalex_quote_offer_'.$reply_val['attrs']['id'],true);
			if($quote_data) {	
				$quote_data = (array)json_decode($quote_data,true);
				$accepted_url = get_post_meta( $post_id, '__quote_accept_url',true );
				$rejected_url = get_post_meta( $post_id, '__quote_reject_url',true );
				?>
				<div class="quote_request_heading"><span class="quote_request_items_heading"><?php echo esc_html__('Offered Items','wholesalex-pro'); ?></span> <span class="quote-buttons"><span id="quote_reject_button"><a href="<?php echo esc_url( $rejected_url); ?>"> <?php echo  esc_html__( 'Reject', 'wholesalex-pro' ) ?></a></span><span id="quote_accept_button"><a href="<?php echo esc_url($accepted_url); ?>"> <?php  echo esc_html__( 'Accept', 'wholesalex-pro' ) ?></a></span></span></span></div><br>
				<?php
				self::get_order_table((array) $quote_data['items'],true );
			}
		}		

		?>
		<?php
	}
}
