<?php
/**
 * Class wholesalex new user Registration admin notification file.
 *
 * @package WHOLESALEX
 * @since 1.2.3
 */

 namespace WHOLESALEX_PRO;

 use WHOLESALEX\WHOLESALEX_Email_Manager;

use WC_Email;
use WP_User;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}


/**
 * Customer New Account.
 *
 * An email sent to the wholesalex customer when they create an account.
 *
 * @extends     WC_Email
 */
class WholesaleX_Quote_Request_Recieved_Admin extends WC_Email {

    /**
     * User login name.
     *
     * @var string
     */
    public $user_login;

    public $display_name;

    /**
     * User email.
     *
     * @var string
     */
    public $user_email;

    /**
     * User password.
     *
     * @var string
     */
    public $user_pass;

    /**
     * User parent.
     *
     * @var string
     */
    public $parent_user_id;

    /**
     * Is the password generated?
     *
     * @var bool
     */
    public $password_generated;

    /**
     * Magic link to verify email.
     *
     * @var string
     */
    public $confirmation_url;


    public $accept_url;
    public $reject_url;
    public $expire_date;
    public $quote_title;

    /**
     * Constructor.
     */
    public function __construct() {
        $this->id             = 'wholesalex_raq_admin_new_quote_request';
        $this->title          = __( 'WholesaleX: Quote Request Received', 'wholesalex-pro' );
        $this->description    = __( 'WholesaleX: Quote Request Received emails are sent to the admin when store received any quote request from user.', 'wholesalex-pro' );
        $this->template_base  = WHOLESALEX_PRO_PATH.'addons/raq/emails/templates/';
        $this->template_html  = 'wholesalex-raq-received-admin.php';
        $this->placeholders = apply_filters('wholesalex_raq_admin_new_quote_request_smart_tags',array(
            	'{date}'                => gmdate( 'd-m-Y' ),
            	'{admin_name}'          => WHOLESALEX_Email_Manager::get_admin_display(),
            	'{site_name}'           => get_option( 'blogname' )
        ));
        $this->recipient     = $this->get_option( 'recipient', get_option( 'admin_email' ) );

        // Call parent constructor.
        parent::__construct();

        // add_action( 'wholesalex_registration_form_user_status_admin_approve_notification', array( $this, 'trigger'), 10, 3 );
    }

    /**
     * Get email subject.
     *
     * @since  1.2.3
     * @return string
     */
    public function get_default_subject() {
        return __('You have received a new Quote Request','wholesalex-pro');
    }

    /**
     * Get email heading.
     *
     * @since  1.2.3
     * @return string
     */
    public function get_default_heading() {
        return __('You have received a new Quote Request','wholesalex-pro');
    }

    /**
     * Trigger.
     *
     * @param int    $user_id User ID.
     * @param string $user_pass User password.
     * @param bool   $password_generated Whether the password was generated automatically or not.
     */
    public function trigger($conversation_id, $display_name, $items) {

        $this->setup_locale();

        $this->placeholders['{admin_name}'] = WHOLESALEX_Email_Manager::get_admin_display();             
        $this->placeholders['{date}'] = gmdate( 'd-m-Y' );             
        $this->placeholders['{site_name}'] = get_option( 'blogname' );         
        $this->display_name = $display_name;    


        $status = false;

        
        if ( $this->is_enabled() && $this->get_recipient() ) {
           $status = $this->send( $this->get_recipient(), $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments() );
        }

        $this->restore_locale();
        return $status;
    }

    /**
     * Get content html.
     *
     * @return string
     */
    public function get_content_html() {
        return wc_get_template_html(
            $this->template_html,
            array(
                'email_heading'      => $this->get_heading(),
                'additional_content' => $this->get_additional_content(),
                'blogname'           => $this->get_blogname(),
                'sent_to_admin'      => true,
                'plain_text'         => false,
                'email'              => $this,
                'display_name'       => $this->display_name
            ),
            $this->template_base, $this->template_base
        );
    }

    /**
     * Default content to show below main email content.
     *
     * @since 3.7.0
     * @return string
     */
    public function get_default_additional_content() {
        return __( '', 'wholesalex-pro' );
    }

    /**
 * Initialise Settings Form Fields - these are generic email options most will use.
 */
public function init_form_fields() {
    /* translators: %s: list of placeholders */
    $placeholder_text  = sprintf( __( 'Available placeholders: %s', 'wholesalex-pro' ), '<code>' . esc_html( implode( '</code>, <code>', array_keys( $this->placeholders ) ) ) . '</code>' );
    $this->form_fields = array(
        'enabled'            => array(
            'title'   => __( 'Enable/Disable', 'wholesalex-pro' ),
            'type'    => 'checkbox',
            'label'   => __( 'Enable this email notification', 'wholesalex-pro' ),
            'default' => 'yes',
        ),
        'subject'            => array(
            'title'       => __( 'Subject', 'wholesalex-pro' ),
            'type'        => 'text',
            'desc_tip'    => true,
            'description' => 'This control the email subject. To change default email subject, makes changes here.',
            'placeholder' => $this->get_default_subject(),
            'default'     => '',
        ),
        'heading'            => array(
            'title'       => __( 'Email heading', 'wholesalex-pro' ),
            'type'        => 'text',
            'desc_tip'    => true,
            'description' => 'This control the email heading. To change default email heading, makes changes here.',
            'placeholder' => $this->get_default_heading(),
            'default'     => '',
        ),
        'additional_content' => array(
            'title'       => __( 'Additional content', 'wholesalex-pro' ),
            'description' => __( 'Text to appear below the main email content.', 'wholesalex-pro' ) . ' ' . $placeholder_text,
            'css'         => 'width:400px; height: 75px;',
            'placeholder' => __( 'N/A', 'wholesalex-pro' ),
            'type'        => 'textarea',
            'default'     => $this->get_default_additional_content(),
            'desc_tip'    => true,
        ),
        'email_type'         => array(
            'title'       => __( 'Email type', 'wholesalex-pro' ),
            'type'        => 'select',
            'description' => __( 'Choose which format of email to send.', 'wholesalex-pro' ),
            'default'     => 'html',
            'class'       => 'email_type wc-enhanced-select',
            'options'     => array(
                'html'      => __( 'HTML', 'woocommerce' ),
                'multipart' => __( 'Multipart', 'woocommerce' )
            ),
            'desc_tip'    => true,
        ),
    );
}
}



return new WholesaleX_Quote_Request_Recieved_Admin();
