<?php

/**
 * Request A Quote Addon Init
 *
 * @package WHOLESALEX
 * @since 1.0.5
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

add_action( 'init', 'wholesalex_raq_init' );
add_filter( 'wholesalex_wsx_addon_raq_error', 'wholesalex_raq_activation_deps_check', 1, 2 );
add_action( 'wholesalex_wsx_addon_conversation_after_status_update', 'wholesalex_conversation_deactive_deps_check' );
add_filter( 'wholesalex_addon_conversation_title', 'wholesalex_change_conversation_title' );

/**
 * WholesaleX Request A Quote Init
 *
 * @return void
 */
function wholesalex_raq_init() {
	$is_conversation_enable = wholesalex()->get_setting( 'wsx_addon_conversation' );
	$is_raq_enable          = wholesalex()->get_setting( 'wsx_addon_raq' );	

	if ( 'yes' === $is_conversation_enable && 'yes' === $is_raq_enable ) {
		require_once WHOLESALEX_PRO_PATH . 'addons/raq/RequestQuote.php';
		new \WHOLESALEX_PRO\RequestQuote();		

		wholesalex_register_quote_order_status();
		add_filter( 'wholesalex_addon_conversation_types', 'wholesalex_add_quote_type_on_conversation' );
		/**
		 * Request A Quote Make Offer Email Status Default Enabled
		 *
		 * @since 1.0.6
		 */
		update_option( 'wholesalex_raq_make_offer_email_status', true );
		do_action('wholesalex_after_request_a_quote_addon_enabled');

	}
}
/**
 * Register quote order status.
 *
 * @return void
 * @since 1.0.5
 */
function wholesalex_register_quote_order_status() {
	register_post_status(
		'wc-whx-quote-pending',
		array(
			'label'                     => _x( 'Pending Quote Request', 'Order status', 'wholesalex-pro' ),
			'public'                    => false,
			'exclude_from_search'       => false,
			'show_in_admin_all_list'    => true,
			'show_in_admin_status_list' => true,
			/* translators: %s: number of orders */
			'label_count'               => _n_noop( 'Pending Quote Request <span class="count">(%s)</span>', 'Pending Quote Request <span class="count">(%s)</span>', 'wholesalex-pro' ),
		)
	);
	register_post_status(
		'wc-whx-quote-expired',
		array(
			'label'                     => _x( 'Expired Quote Request', 'Order status', 'wholesalex-pro' ),
			'public'                    => false,
			'exclude_from_search'       => false,
			'show_in_admin_all_list'    => true,
			'show_in_admin_status_list' => true,
			/* translators: %s: number of orders */
			'label_count'               => _n_noop( 'Expired Quote Request <span class="count">(%s)</span>', 'Expired Quote Request <span class="count">(%s)</span>', 'wholesalex-pro' ),
		)
	);
	register_post_status(
		'wc-whx-quote-reject',
		array(
			'label'                     => _x( 'Rejected Quote Request', 'Order status', 'wholesalex-pro' ),
			'public'                    => false,
			'exclude_from_search'       => false,
			'show_in_admin_all_list'    => true,
			'show_in_admin_status_list' => true,
			/* translators: %s: number of orders */
			'label_count'               => _n_noop( 'Rejected Quote Request <span class="count">(%s)</span>', 'Rejected Quote Request <span class="count">(%s)</span>', 'wholesalex-pro' ),
		)
	);
}

/**
 * Request A Quote Activation Dependency Check
 *
 * @param string $error Error Message.
 * @param string $status Addon Status.
 * @return string
 * @since 1.0.5
 */
function wholesalex_raq_activation_deps_check( $error, $status ) {
	if ( 'yes' === $status && function_exists( 'wholesalex' ) ) {
		if ( 'yes' !== wholesalex()->get_setting( 'wsx_addon_conversation' ) ) {
			$error = __( 'Request a Quote addon is dependent on the Conversation addon. So, you need to turn on the Conversation addon first.', 'wholesalex-pro' );
		}
	} else {
		$timestamp = wp_next_scheduled( 'wholesalex_raq_expire_date_check' );
		wp_unschedule_event( $timestamp, 'wholesalex_raq_expire_date_check' );
	}

	return $error;
}

/**
 * Conversation Deactive Dependency Check
 * While conversation addon deactive, also deactive request a quote addon.
 *
 * @param string $status Conversation Addon Status.
 * @return void
 */
function wholesalex_conversation_deactive_deps_check( $status ) {
	if ( 'yes' !== $status && function_exists( 'wholesalex' ) ) {
		wholesalex()->set_setting( 'wsx_addon_raq', '' );
	}
}

/**
 * Change Conversation Title If Request a quote is enabled.
 *
 * @param string $title Previous Title.
 * @return string updated title.
 */
function wholesalex_change_conversation_title( $title ) {

	$is_raq_enable = wholesalex()->get_setting( 'wsx_addon_raq' );
	if ( 'yes' === $is_raq_enable ) {
		$title = __( 'Conversation & Quote', 'wholesalex-pro' );
	}

	return $title;
}

/**
 * Add Quote Type On Conversation
 *
 * @param array $types Conversation Types.
 * @return array Updated Conversation Types.
 */
function wholesalex_add_quote_type_on_conversation( $types ) {
	$types['quote'] = wholesalex()->get_language_n_text( '_language_conversations_list_type_quote', __( 'Quote', 'wholesalex-pro' ) );
	return $types;
}
