<?php
/**
 * WholesaleX Subaccount Parent Approval Gateway
 *
 * @package WHOLESALEX
 * @since 1.0.9
 */

namespace WHOLESALEX_PRO;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
use WC_Payment_Gateway;
use Exception;
if ( class_exists( 'WC_Payment_Gateway' ) ) {
	/**
	 * WholesaleX Payment Gateway
	 */
	class ParentApprovalGateway extends WC_Payment_Gateway {
		/**
		 * Constructor
		 */
		public function __construct() {
			$this->initial_setup();

			// Load the settings.
			$this->init_form_fields();
			$this->init_settings();
			// Get settings.
			$this->title        = $this->get_option( 'title' );
			$this->description  = $this->get_option( 'description' );
			$this->instructions = $this->get_option( 'instructions' );

			$this->supports = array(
				'products',
				'refunds',
			);

			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		}

		/**
		 * WholesaleX Payment Gateway Initial Properties
		 *
		 * @since 1.0.9
		 */
		private function initial_setup() {
			$this->id                 = 'wholesalex_subaccount_order_approval';
			$this->method_title       = __( 'Subaccount Order Approval Required', 'wholesalex-pro' );
			$this->method_description = '';
			$this->has_fields         = false;
		}

		/**
		 * WholesaleX Payment Gateway Settings Form Fields Initialize
		 *
		 * @since 1.0.9
		 */
		public function init_form_fields() {
			$this->form_fields = array(
				'enabled'      => array(
					'title'       => __( 'Enable/Disable', 'wholesalex-pro' ),
					'label'       => __( 'Enable Subaccount Order Approval', 'wholesalex-pro' ),
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no',
				),
				'title'        => array(
					'title'       => __( 'Title', 'wholesalex-pro' ),
					'type'        => 'text',
					'description' => __( 'This controls the title which the user sees during checkout.', 'wholesalex-pro' ),
					'default'     => __( 'Notify Main Account for Approval', 'wholesalex-pro' ),
					'desc_tip'    => true,
				),
				'description'  => array(
					'title'       => __( 'Description', 'wholesalex-pro' ),
					'type'        => 'textarea',
					'description' => __( 'Payment method description that the customer will see on your checkout.', 'wholesalex-pro' ),
					'default'     => __( 'This order will be send to main account for approval', 'wholesalex-pro' ),
					'desc_tip'    => true,
				),
				'instructions' => array(
					'title'       => __( 'Instructions', 'wholesalex-pro' ),
					'type'        => 'textarea',
					'description' => __( 'Instructions that will be added to the thank you page.', 'wholesalex-pro' ),
					'default'     => __( 'This order will be send to main account for approval', 'wholesalex-pro' ),
					'desc_tip'    => true,
				),
			);
		}

		/**
		 * Process ParentApprovalGateway
		 *
		 * @param int $order_id Order Id.
		 * @return array process payment results.
		 * @since 1.0.9
		 */
		public function process_payment( $order_id ) {
			$order = wc_get_order( $order_id );

			$order->set_status( 'whx-approval-need', __( 'Order Send to Parent Account For Approval', 'wholesalex-pro' ) );
			$order->save();

			update_post_meta(
				$order_id,
				'__wholesalex_subaccount_order_staus',
				array(
					'status'      => 'pending',
					'user_id' => get_current_user_id(),
				)
			);

			// Reduce stock levels.
			wc_reduce_stock_levels( $order_id );

			// Remove cart.
			WC()->cart->empty_cart();

			return array(
				'result'   => 'success',
				'redirect' => $this->get_return_url( $order ),
			);
		}
	}
}
