<?php
/**
 * WholesaleX SubAccount
 *
 * @package WHOLESALEX_PRO
 * @since 1.0.9
 */

namespace WHOLESALEX_PRO;

use WC_Emails;
use WHOLESALEX\WHOLESALEX_Email_Manager;
use WP_User;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Sub Account Page Class
 *
 * @since 1.0.9
 */
class SubAccount {

	/**
	 * Current User ID
	 *
	 * @var string|int
	 */
	public $current_user_id = '';


	/**
	 * Parent ID
	 *
	 * @var string
	 */
	public $parent_id = '';
	/**
	 * Account Type
	 *
	 * @var string
	 */
	public $account_type = '';

	/**
	 * Account Permissions
	 *
	 * @var array
	 */
	public $permissions = array();

	public $emails = array();

	public $emails_class = '';



	/**
	 * Constructor
	 */
	public function __construct() {
		// Set Current User ID.
		$this->current_user_id = get_current_user_id();
		$this->parent_id       = $this->current_user_id;

		// Get Current User Account type.
		$account_type       = get_user_meta( $this->current_user_id, '__wholesalex_account_type', true );
		$this->account_type = $account_type;

		add_filter('wholesalex_addon_subaccount_my_account_menu_title', array($this,'subaccount_menu_title'));
		add_filter('wholesalex_addon_subaccount_my_account_menu_endpoint', array($this,'subaccount_menu_endpoint'));
		/**
		 * Set Subaccount Name on Conversation Reply
		 */
		add_filter( 'wholesalex_addon_conversation_reply_display_name', array( $this, 'conversation_reply_display_name' ), 10, 3 );

		/**
		 * Add a Placed By Column on Orders Page (My Account).
		 */
		add_filter( 'woocommerce_account_orders_columns', array( $this, 'placed_by_column' ) );

		/**
		 * Set Placed By Column Content.
		 */
		add_action( 'woocommerce_my_account_my_orders_column_order-placed-by', array( $this, 'placed_by_column_content' ) );

		/**
		 * Add Account Type Column on WholesaleX Users Page.
		 */
		add_filter( 'wholesalex_users_columns', array( $this, 'add_account_type_column' ) );

		/**
		 * Set Account Type Column Value.
		 */
		add_filter( 'wholesalex_users_column_value', array( $this, 'account_type_column_value' ), 10, 3 );

		// add_action( 'wholesalex_email_templates', array( $this, 'subaccounts_email_template' ), 5 );

		// add_action( 'wholesalex_subaccount_order', array( $this, 'send_subaccount_order_email_to_parent' ), 10, 3 );

		// add_filter( 'wholesalex_wallet_user_id', array( $this, 'set_current_user' ), 1 );

		add_filter( 'wholesalex_set_current_user', array( $this, 'set_current_user' ) );

		add_filter( 'wholesalex_wallet_order_payment_description', array( $this, 'wallet_order_payment_description' ), 10, 2 );

		add_filter( 'wholesalex_addon_wallet_topup_description', array( $this, 'wallet_topup_payment_description' ), 10, 2 );

		add_action( 'wholesalex_user_role_updated', array( $this, 'user_role_changed' ), 10, 3 );

		// Filter Subaccount user from users.
		add_filter( 'wholesalex_get_users_query', array( $this, 'filter_subaccount_holders' ) );

		// add_action( 'wholesalex_addon_subaccount_created', array( $this, 'subaccount_created' ), 10, 3 );

		add_filter( 'woocommerce_payment_gateways', array( $this, 'load_payment_gateways' ), 999999999 );

		add_filter( 'wc_order_statuses', array( $this, 'approval_need_order_status_add' ) );

		add_filter( 'woocommerce_available_payment_gateways', array( $this, 'control_payment_gateway' ) );

		add_filter( 'woocommerce_view_order', array( $this, 'order_approval_content_order_details_page' ) );

		// add_filter( 'wc_order_is_editable', array( $this, 'make_approval_required_order_editable' ), 10, 2 );

		// add_filter('woocommerce_order_is_pending_statuses', array($this, 'approval_need_status_as_pending'));
		add_filter( 'woocommerce_my_account_my_orders_actions', array( $this, 'my_account_order_actions' ), 10, 2 );
		add_filter( 'woocommerce_valid_order_statuses_for_payment', array( $this, 'approval_need_status_as_pending' ) );
		add_filter( 'woocommerce_valid_order_statuses_for_cancel', array( $this, 'approval_need_status_as_pending' ) );

		add_action( 'wp_ajax_order_approval_ajax_handler', array( $this, 'order_approval_ajax_handler' ) );

		add_action( 'woocommerce_order_status_changed', array( $this, 'order_status_changed_actions' ), 10, 4 );

		add_filter( 'wholesalex_role_fields', array( $this, 'subaccounts_field_on_role' ), 12 );
		add_filter( 'wholesalex_email_smart_tags', array( $this, 'email_smart_tags' ), 10, 2 );


		add_filter( 'woocommerce_email_classes', array($this, 'wholesalex_add_email_classes') );

		add_action('woocommerce_email',array($this,'handle_subaccount_email'));

		add_filter('woocommerce_email_actions', array($this,'wholesalex_add_email_actions'));



		if ( 'subaccount' === $account_type ) {
			$this->parent_id = get_user_meta( $this->current_user_id, '__wholesalex_parent_id', true );
			/**
			 * Set Parent User Wallet Id.
			 * If Current User is Subaccount, It inherit Parent Account Wallet Balance and transaction history.
			 */
			// add_filter( 'wholesalex_addon_wallet_user_id', array( $this, 'set_wallet_user_id' ) );

			// Get All Permissions of current user.
			$permissions       = get_user_meta( $this->current_user_id, '__wholesalex_permissions', true );
			$this->permissions = $permissions;

			// Place Order Permissions
			// If the user does not have place order permission, user can not able to place order as well as cart items.
			if ( ! in_array( 'place_order', $permissions ) ) { //phpcs:ignore

				remove_action( 'woocommerce_after_shop_loop_item', 'woocommerce_template_loop_add_to_cart', 10 );
				remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_add_to_cart', 30 );
				remove_action( 'woocommerce_simple_add_to_cart', 'woocommerce_simple_add_to_cart', 30 );
				remove_action( 'woocommerce_grouped_add_to_cart', 'woocommerce_grouped_add_to_cart', 30 );
				remove_action( 'woocommerce_variable_add_to_cart', 'woocommerce_variable_add_to_cart', 30 );
				remove_action( 'woocommerce_external_add_to_cart', 'woocommerce_external_add_to_cart', 30 );
				remove_action( 'woocommerce_single_variation', 'woocommerce_single_variation', 10 );
				remove_action( 'woocommerce_single_variation', 'woocommerce_single_variation_add_to_cart_button', 20 );

				add_filter( 'woocommerce_is_purchasable', '__return_false' );

				add_action( 'woocommerce_after_checkout_validation', array( $this, 'place_order_permission' ), 10, 2 );

			}

			/**
			 * If the subaccount has view all conversations permission, account holder will be able to view all subaccounts conversations including parent account converstions.
			 */
			if ( in_array( 'view_all_conversations', $permissions ) ) { //phpcs:ignore

				add_filter( 'wholesalex_addon_conversation_has_eligibility_to_view_conversation', array( $this, 'view_conversation_eligibility' ), 10, 2 );

				add_filter( 'wholesalex_addon_conversation_view_author_ids', array( $this, 'parent_n_subaccount_ids_for_conversation' ), 10, 2 );
			}

			/**
			 * If the subaccount has view all purchase lists permission, account holder will be able to view all subaccounts purchase lists including parent purchase lists.
			 */
			if ( in_array( 'view_all_purchase_lists', $permissions ) ) { //phpcs:ignore
				add_filter( 'wholesalex_addon_bulkorder_purchase_lists', array( $this, 'purchase_lists' ), 10, 2 );
			}

			/**
			 * If the subaccount has view all orders permission, account holder will be able to view all subaccount orders including parent orders.
			 */
			if ( in_array( 'view_all_orders', $permissions ) ) { //phpcs:ignore
				// add_filter( 'woocommerce_my_account_my_orders_columns', array( $this, 'placed_by_column' ) );
				add_filter( 'woocommerce_my_account_my_orders_query', array( $this, 'add_subaccount_orders_in_query' ) );
				add_action( 'woocommerce_after_order_details', array( $this, 'include_order_details_file' ) );
				add_filter( 'user_has_cap', array( $this, 'set_caps_to_view_orders' ), 10, 3 );

			}

			if(!in_array('wallet_topup_access',$permissions)) {
				add_filter( 'wholesalex_wallet_fields', array( $this, 'add_request_balance_field' ) );
				add_filter( 'woocommerce_is_purchasable', array( $this, 'make_topup_product_not_purchasable' ), 10, 2 );

			}

			if ( ! in_array( 'wallet_access', $permissions ) ) {
				// add_filter( 'wholesalex_wallet_fields', array( $this, 'remove_add_balance_fields' ) );
				add_filter( 'woocommerce_is_purchasable', array( $this, 'make_topup_product_not_purchasable' ), 10, 2 );
				add_filter( 'wholesalex_addon_wallet_balance_payment_status', '__return_false' );
				add_filter( 'woocommerce_account_menu_items', array( $this, 'remove_wallet_from_menu' ), 9999999 );
				add_filter( 'wholesalex_addon_wallet_endpoint_access', '__return_false' );
			}

			if ( in_array( 'order_approval_required', $permissions ) ) {
				add_filter( 'woocommerce_thankyou_order_received_text', array( $this, 'subaccount_order_manual_approval_order_received_text' ), 10, 2 );
			}

			// Add order note to subaccount order.
			add_action( 'woocommerce_thankyou', array( $this, 'add_order_note' ) );

			add_filter( 'wholesalex_dynamic_rule_user_id', array( $this, 'set_current_user' ) );

			return;
		}
		$this->subaccount_endpoint();

		add_action( 'woocommerce_account_'.$this->get_endpoint().'_endpoint', array( $this, 'subaccounts' ) );

		add_filter( 'woocommerce_get_query_vars', array( $this, 'add_subaccount_to_woocommerce_query_vars' ) );

		add_filter( 'woocommerce_account_menu_items', array( $this, 'subaccount_link_my_account' ) );

		add_action( 'rest_api_init', array( $this, 'subaccount_create_restapi_init' ) );

		add_filter( 'wholesalex_addon_conversation_has_eligibility_to_view_conversation', array( $this, 'view_conversation_eligibility' ), 10, 2 );

		add_filter( 'wholesalex_addon_conversation_view_author_ids', array( $this, 'parent_n_subaccount_ids_for_conversation' ), 10, 2 );

		add_filter( 'wholesalex_addon_bulkorder_purchase_lists', array( $this, 'purchase_lists' ), 10, 2 );

		add_filter( 'woocommerce_my_account_my_orders_query', array( $this, 'add_subaccount_orders_in_query' ) );

		add_filter( 'user_has_cap', array( $this, 'set_caps_to_view_orders' ), 10, 3 );

		add_action( 'woocommerce_after_order_details', array( $this, 'include_order_details_file' ) );

		add_filter( 'manage_edit-shop_order_columns', array( $this, 'add_placed_by_column_on_order_page' ) );

		add_action( 'manage_shop_order_posts_custom_column', array( $this, 'populate_data_on_placed_by_column' ), 10, 2 );

		add_filter( 'wholesalex_wallet_fields', array( $this, 'subaccount_wallet_fields' ) );

        add_filter( 'wholesalex_user_export_default_columns', array($this,  'account_type_column_on_exporter') );

        add_filter( 'wholesalex_user_export_column_data', array($this, 'populate_data_on_account_type_column_exporter') );

        add_action('wholesalex_import_userdata', array($this,'process_import'),10,3);

        add_filter( 'wholesalex_setting_fields', array($this,'subaccount_text_settings') );

		/**
		 * Set Parent Account Id as customer id, if the order is placed by subaccount
		 * So that parent account pay this order
		 * 
		 * @since 1.2.3
		 */
		add_filter('woocommerce_order_get_customer_id',array($this,'set_parent_id_as_subaccount_order_customer_id'));
		add_filter('wholesalex_role_exporter_default_columns', array($this,'add_subaccount_fields_on_role_exporter'));

		add_filter('wholesalex_role_export_role_column__subaccount_status', array($this,'get_column_value__subaccount_status'),10,2);
		add_filter('wholesalex_role_export_role_column__exclude_subaccounts', array($this,'get_column_value__exclude_subaccounts'),10,2);

		add_filter('wholesalex_csv_role_import_mapping_options', array($this,'set_import_column_value'));
		add_filter('wholesalex_csv_role_import_mapping_default_columns', array($this,'set_import_mapping_default_column'));

		add_filter('wholesalex_role_importer_parsed_data', array($this, 'role_importer_parse_subaccount_data'));

	}

	public function wholesalex_add_email_classes($email_classes) {
		$email_classes['WholesaleX_New_Subaccount_Create_Email'] = include WHOLESALEX_PRO_PATH .'/addons/subaccount/emails/class-wholesalex-new-subaccount-create.php';
		$email_classes['WholesaleX_Subaccount_Order_Placed_Email'] = include WHOLESALEX_PRO_PATH .'/addons/subaccount/emails/class-wholesalex-subaccount-order-placed.php';
		$email_classes['WholesaleX_Subaccount_Order_Approval_Required_Email'] = include WHOLESALEX_PRO_PATH .'/addons/subaccount/emails/class-wholesalex-subaccount-order-approval-require.php';
		$email_classes['WholesaleX_Subaccount_Order_Approved_Email'] = include WHOLESALEX_PRO_PATH .'/addons/subaccount/emails/class-wholesalex-subaccount-order-approved.php';
		$email_classes['WholesaleX_Subaccount_Order_Pending_Email'] = include WHOLESALEX_PRO_PATH .'/addons/subaccount/emails/class-wholesalex-subaccount-order-pending.php';
		$email_classes['WholesaleX_Subaccount_Order_Reject_Email'] = include WHOLESALEX_PRO_PATH .'/addons/subaccount/emails/class-wholesalex-subaccount-order-reject.php';

		return $email_classes;
	}

	public function wholesalex_add_email_actions($actions) {
		$actions[]='wholesalex_addon_subaccount_created';
		$actions[]='wholesalex_subaccount_order';
		$actions[]='wholesalex_addon_subaccount_parent_approve_order';
		$actions[]='wholesalex_addon_subaccount_parent_reject_order';
		$actions[]='wholesalex_addon_subaccount_new_order_for_approval';
		return $actions;
	}


	public function handle_subaccount_email($wc_emails){
		$this->emails = $wc_emails->get_emails();

		add_action( 'wholesalex_addon_subaccount_created_notification', array( $this, 'subaccount_created' ), 10, 3 );
		add_action( 'wholesalex_subaccount_order_notification', array( $this, 'send_subaccount_order_email_to_parent' ), 10, 3 );
		// add_action( 'woocommerce_order_status_pending_to_processing_notification', array( $this, 'send_subaccount_order_email_to_parent' ), 10, 3 );
		// add_action( 'woocommerce_order_status_pending_to_on-hold_notification', array( $this, 'send_subaccount_order_email_to_parent' ), 10, 3 );
		add_action( 'wholesalex_addon_subaccount_parent_approve_order_notification', array( $this, 'approve_order_send_email' ), 10, 2 );
		add_action( 'wholesalex_addon_subaccount_parent_reject_order_notification', array( $this, 'reject_order_send_email' ), 10, 2 );
		add_action( 'wholesalex_addon_subaccount_new_order_for_approval_notification', array( $this, 'new_order_for_admin_approval_email' ), 10, 2 );
		add_action( 'wholesalex_addon_subaccount_new_order_for_approval_notification', array( $this, 'new_order_pending_for_approval' ), 10, 2 );
	}

	// public function wholesalex_add_email_actions($actions) {
	// 	$actions[]='wholesalex_user_email_verified';
	// 	return $actions;
	// }

	/**
	 * Subaccount Rest API Init
	 *
	 * @return void
	 * @since 1.0.9
	 */
	public function subaccount_create_restapi_init() {
		register_rest_route(
			'wholesalex/v1',
			'/subaccount/',
			array(
				array(
					'methods'             => 'POST',
					'callback'            => array( $this, 'subaccount_restapi_callback' ),
					'permission_callback' => function () {
						$__user_id = get_current_user_id();
						$__account_status = get_user_meta( $__user_id, '__wholesalex_status', true );
						$__have_permission_to_create_subaccount = apply_filters( 'wholesalex_user_subaccount_create_permission', true, $__user_id );
						return ( ( 'active' === $__account_status && $__have_permission_to_create_subaccount ) || current_user_can( 'manage_options' ) );
					},
					'args'                => array(),
				),
			)
		);
	}

	/**
	 * Subaccount Rest API Callback
	 *
	 * @param object $server Server Object.
	 * @return void
	 * @since 1.0.9
	 */
	public function subaccount_restapi_callback( $server ) {
		$post = $server->get_params();
		if ( ! ( isset( $post['nonce'] ) && wp_verify_nonce( sanitize_key( $post['nonce'] ), 'wholesalex_subaccount' ) ) ) {
			return;
		}
		
		switch ( $post['type'] ) {
			case 'create_account':
				$form_data = json_decode( $post['data'], true );
				$result    = $this->create( $form_data );
				
				wp_send_json_success( $result );
				break;
			case 'get_accounts':
				wp_send_json_success( array( 'accounts' => $this->get_subaccounts( get_current_user_id() ) ) );
				break;
			case 'delete_account':
				$user_id = sanitize_text_field( $post['id'] );
				wp_send_json_success( $this->delete( $user_id ) );
				break;
			case 'update':
				$form_data = json_decode( $post['data'], true );

				$result = $this->update( $form_data['id'], $form_data );

				wp_send_json_success( $result );

				break;
		}
	}

	/**
	 * Update Subaccount.
	 *
	 * @param string|int $user_id User ID.
	 * @param object     $user_data User Data.
	 * @return array
	 * @since 1.0.9
	 */
	public function update( $user_id, $user_data ) {
		$user         = get_userdata( $user_id );
		$email        = isset( $user_data['email'] ) ? sanitize_email( $user_data['email'] ) : '';
		$display_name = isset( $user_data['full_name'] ) ? sanitize_text_field( $user_data['full_name'] ) : $user->display_name;
		$job_title    = isset( $user_data['job_title'] ) ? sanitize_text_field( $user_data['job_title'] ) : '';
		$phone        = isset( $user_data['phone'] ) ? sanitize_text_field( $user_data['phone'] ) : '';
		$password     = isset( $user_data['password'] ) ? sanitize_text_field( $user_data['password'] ) : '';
		$permissions  = isset( $user_data['permissions'] ) ? wholesalex()->sanitize( $user_data['permissions'] ) : array();
		$parent_user_id = get_user_meta($user_id,'__wholesalex_parent_id',true);
		$parent_data = get_userdata($parent_user_id);

		
		if( in_array('wallet_topup_access',$permissions) && !in_array('wallet_access',$permissions)) {
			// Check has wallet access or not.
			$result = array(
				'status'  => false,
				'message' => __( 'Invalid Permission! You have to give wallet access permission to topup wallet.', 'wholesalex-pro' ),
			);

			return $result;
		}

		if ( ! empty( $email ) && ! is_email( $email ) ) {
			$result = array(
				'status'  => false,
				'message' => __( 'Please enter a valid email address.', 'wholesalex-pro' ),
			);
			return $result;
		}

		if ( $user->user_email != $email && email_exists( $email ) ) {
			$result = array(
				'status'  => false,
				'message' => __( 'Email address already exist!', 'wholesalex-pro' ),
			);
			return $result;
		}

		if ( empty( $display_name ) ) {
			$result = array(
				'status'  => false,
				'message' => __( 'Please fill all required fields vvvvvvvvvv.', 'wholesalex-pro' ),
			);

			return $result;
		}

		$update_data = array(
			'ID'           => $user_id,
			'user_email'   => $email,
			'display_name' => $display_name,
		);

		if ( $password ) {
			$update_data['user_pass'] = $password;
		}

		if('yes' === wholesalex()->get_setting('wsx_addon_wallet')) {
			$balance_data = array();
			$procced_transaction = false;
			$wallet_balance = isset($user_data['wallet_balance'])? floatval($user_data['wallet_balance']): 0;
			$previous_balance = wholesalex_wallet()->get_wholesalex_balance($user_id,'edit');
			$balance_diff = $previous_balance - $wallet_balance;
			$parent_available_balance = wholesalex_wallet()->get_wholesalex_balance(get_current_user_id(),'edit');

			$balance_data['amount'] = abs($balance_diff);
			if($balance_diff>0) {
				$balance_data['type'] = 'debit';
			} else if($balance_diff<0) {
				$balance_data['type'] = 'credit';
			}

			if('credit' === $balance_data['type']) {
				if((floatval($parent_available_balance)-floatval($balance_data['amount']))>=0) {
					$procced_transaction = true;
				} else {
					$procced_transaction = false;
					$result = array(
						'status'  => false,
						'message' => __( 'Subaccount Update Failed: Credit Limit Exceeded!', 'wholesalex-pro' ),
					);
					return $result;
				}
			} else if('debit' === $balance_data['type']) {
				$procced_transaction = true;
			}
			
			if($procced_transaction && !in_array('wallet_access',$permissions)) {
				$result = array(
					'status'  => false,
					'message' => __( 'Without Wallet Access, You cannot assign credit balance!', 'wholesalex-pro' ),
				);
				return $result;
			}
		}
		$user_id = wp_update_user( $update_data );
		if ( ! is_wp_error( $user_id ) ) {
			update_user_meta( $user_id, '__job_title', $job_title );
			update_user_meta( $user_id, '__phone', $phone );
			update_user_meta( $user_id, '__wholesalex_permissions', $permissions );
			if(isset($procced_transaction) && $procced_transaction && $balance_data) {
				switch ($balance_data['type']) {
					case 'debit':
						$transation_id = wholesalex_wallet()->initiate_transaction($user_id,$balance_data['amount'],'debit', sprintf("Balance Debited by $parent_data->user_login") );
						if($transation_id) {
							$status = wholesalex_wallet()->initiate_transaction($parent_user_id,$balance_data['amount'],'credit', sprintf("Balance Credited From $user->user_login") );
							if(!$status) {
								$transation_id = wholesalex_wallet()->initiate_transaction($user_id,$balance_data['amount'],'creit', sprintf("Balance Reversal From $parent_data->user_login") );
							}
						}
						break;
					case 'credit':
						$transation_id = wholesalex_wallet()->initiate_transaction($user_id,$balance_data['amount'],'credit', sprintf("Balance Credited by $parent_data->user_login") );
						if($transation_id) {
							$status = wholesalex_wallet()->initiate_transaction($parent_user_id,$balance_data['amount'],'debit', sprintf("Balance Credited on $user->user_login") );
							if(!$status) {
								$transation_id = wholesalex_wallet()->initiate_transaction($user_id,$balance_data['amount'],'debit', sprintf("Balance Reversal By $parent_data->user_login") );
							}
						}
						break;
					
					default:
						
						break;
				}
			}
		} else {
			$result = array(
				'status'  => false,
				'message' => __( 'Subaccount Update Failed!', 'wholesalex-pro' ),
			);
			return $result;
		}

		$result = array(
			'status'  => true,
			'message' => __( 'Subaccount Successfully Updated.', 'wholesalex-pro' ),
			'id'      => $user_id,
		);
		return $result;
	}

	/**
	 * Get All Subaccounts For Given User Id.
	 *
	 * @param string $user_id User ID.
	 * @return array
	 * @since 1.0.9
	 */
	public function get_subaccounts( $user_id ) {
		$users = get_users(
			array(
				'meta_query' => array(
					array(
						'key'     => '__wholesalex_parent_id',
						'value'   => $user_id,
						'compare' => '===',
					),
					array(
						'key'     => '__wholesalex_account_type',
						'value'   => 'subaccount',
						'compare' => '===',
					),
				),
				'orderby' => 'registered',
				'order' => 'DESC'
			)
		);

		$subaccounts = array();

		foreach ( $users as $key => $user ) {
			$subaccount = array();

			$user_id = $user->ID;

			$subaccount['id']          = $user_id;
			$subaccount['username']    = $user->user_login;
			$subaccount['full_name']   = $user->display_name;
			$subaccount['email']       = $user->user_email;
			$subaccount['avatar_url']  = get_avatar_url( $user_id );
			$subaccount['job_title']   = get_user_meta( $user_id, '__job_title', true );
			$subaccount['phone']       = get_user_meta( $user_id, '__phone', true );
			$subaccount['permissions'] = get_user_meta( $user_id, '__wholesalex_permissions', true );
			if('yes' === wholesalex()->get_setting('wsx_addon_wallet')) {
				$subaccount['wallet_balance_html'] = wholesalex_wallet()->get_wholesalex_balance($user_id,'view');
				$subaccount['wallet_balance'] = wholesalex_wallet()->get_wholesalex_balance($user_id,'edit');
				if(empty($subaccount['wallet_balance'])) {
					$subaccount['wallet_balance_html'] = wc_price(0);
				}
			}
			$subaccounts[]             = $subaccount;
		}

		return $subaccounts;
	}

	/**
	 * Delete Subaccount
	 *
	 * @param string|int $user_id User ID.
	 * @return bool Delete Status
	 * @since 1.0.9
	 */
	public function delete( $user_id ) {
		// check current user has privilage to delete subaccount or not.
		$parent_id = get_user_meta( $user_id, '__wholesalex_parent_id', true );

		if ( get_current_user_id() == $parent_id ) {
			// User has right to delete this subaccount
			// delete subaccount and reassign to owner.
			require_once ABSPATH . 'wp-admin/includes/user.php';
			if('yes' === wholesalex()->get_setting('wsx_addon_wallet')) {
				// Delete Subaccount Credit Add to Master account.
				$is_deleted_subaccount_credit_add_to_master = apply_filters('wholesalex_wallet_is_deleted_subaccount_added_to_master',true);
				if($is_deleted_subaccount_credit_add_to_master) {
					$subaccount_balance = wholesalex_wallet()->get_wholesalex_balance($user_id,'edit');
					$credit_add_reason = sprintf("Deleted Subaccount #$user_id Balance");
					$transation_id = wholesalex_wallet()->initiate_transaction($parent_id,$subaccount_balance,'credit',$credit_add_reason);
				}
			}
			$status = wp_delete_user( $user_id, $parent_id );
			return $status;
		}
		return false;
	}

	/**
	 * Create Subaccount
	 *
	 * @param array $user_data User Data.
	 * @return array
	 * @since 1.0.9
	 */
	public function create( $user_data = array() ) {
		$email        = isset( $user_data['email'] ) ? sanitize_email( $user_data['email'] ) : '';
		$username     = isset( $user_data['username'] ) ? sanitize_user( $user_data['username'] ) : '';
		$display_name = isset( $user_data['full_name'] ) ? sanitize_text_field( $user_data['full_name'] ) : '';
		$job_title    = isset( $user_data['job_title'] ) ? sanitize_text_field( $user_data['job_title'] ) : '';
		$phone        = isset( $user_data['phone'] ) ? sanitize_text_field( $user_data['phone'] ) : '';
		$password     = isset( $user_data['password'] ) ? sanitize_text_field( $user_data['password'] ) : '';
		$permissions  = isset( $user_data['permissions'] ) ? wholesalex()->sanitize( $user_data['permissions'] ) : array();
		if('yes' === wholesalex()->get_setting('wsx_addon_wallet')) {
			$procced_transaction = false;
			$wallet_balance = isset($user_data['wallet_balance'])? floatval($user_data['wallet_balance']): 0;
			if($wallet_balance>0) {
				$parent_available_balance = wholesalex_wallet()->get_wholesalex_balance(get_current_user_id(),'edit');
				if((floatval($parent_available_balance)-floatval($wallet_balance))>=0) {
					$procced_transaction = true;
				} else {
					$result = array(
						'status'  => false,
						'message' => __( 'You dont have sufficient credit balance to allocate credit on subaccount.', 'wholesalex-pro' ),
					);
					return $result;
				}

			}
			
			if($procced_transaction && !in_array('wallet_access',$permissions)) {
				$result = array(
					'status'  => false,
					'message' => __( 'Without Wallet Access, You cannot assign credit balance!', 'wholesalex-pro' ),
				);
				return $result;
			}
		}
		if( in_array('wallet_topup_access',$permissions) && !in_array('wallet_access',$permissions)) {
			// Check has wallet access or not.
			$result = array(
				'status'  => false,
				'message' => __( 'Invalid Permission! You have to give wallet access permission to topup wallet.', 'wholesalex-pro' ),
			);

			return $result;
		}
		
		if ( empty( $email ) || ! is_email( $email ) ) {
			$result = array(
				'status'  => false,
				'message' => __( 'Please enter a valid email address.', 'wholesalex-pro' ),
			);
			return $result;
		}

		if ( email_exists( $email ) ) {
			$result = array(
				'status'  => false,
				'message' => __( 'Email address already exist!', 'wholesalex-pro' ),
			);
			return $result;
		}

		if ( empty( $username ) || ! validate_username( $username ) ) {
			$result = array(
				'status'  => false,
				'message' => __( 'Please enter a valid username', 'wholesalex-pro' ),
			);
			return $result;
		}

		if ( empty( $display_name ) || empty( $password ) || empty( $password ) ) {
			$result = array(
				'status'  => false,
				'message' => __( 'Please fill all required fields.', 'wholesalex-pro' ),
			);

			return $result;
		}


		$role = isset( $user_data['role'] ) ? $user_data['role'] : wholesalex()->get_current_user_role();

		if ( ! $role ) {
			$result = array(
				'status'  => false,
				'message' => __( 'Invalid Role! Please Contact With Site Administration.', 'wholesalex-pro' ),
			);
			return $result;
		}

		$subaccount_user_data = array(
			'user_login'   => $username,
			'user_pass'    => $password,
			'user_email'   => $email,
			'role'         => 'customer',
			'display_name' => $display_name,
		);

		$user_id = wp_insert_user( $subaccount_user_data );
		if ( ! is_wp_error( $user_id ) ) {
			update_user_meta( $user_id, '__wholesalex_role', $role );
			update_user_meta( $user_id, '__job_title', $job_title );
			update_user_meta( $user_id, '__wholesalex_status', 'active' );
			update_user_meta( $user_id, '__phone', $phone );
			update_user_meta( $user_id, '__wholesalex_permissions', $permissions );

			update_user_meta( $user_id, '__wholesalex_account_type', 'subaccount' );
			update_user_meta( $user_id, '__wholesalex_parent_id', get_current_user_id() );

			$parent_user_id = get_current_user_id();
			$subaccounts    = get_user_meta( $parent_user_id, '__wholesalex_subaccount_lists', true );
			if ( ! $subaccounts ) {
				$subaccounts = array();
			}
			$subaccounts[] = $user_id;
			update_user_meta( $parent_user_id, '__wholesalex_subaccount_lists', $subaccounts );

			if(isset($procced_transaction) && $procced_transaction && $wallet_balance>0) {
				
				$parent_user_data = get_userdata(get_current_user_id());
				
				$credit_add_reason_parent = apply_filters('wholesalex_subaccount_wallet_credit_add_reason_parent',sprintf(__('Balance Added to %s','wholesalex-pro'),$username));	
			
				$transation_id = wholesalex_wallet()->initiate_transaction(get_current_user_id(),$wallet_balance,'debit',$credit_add_reason_parent);

				if($transation_id) {
					$credit_add_reason_child = apply_filters('wholesalex_subaccount_wallet_credit_add_reason_subaccount',sprintf(__('Balance Added By %s','wholesalex-pro'),$parent_user_data->user_login));
					$status = wholesalex_wallet()->initiate_transaction($user_id,$wallet_balance,'credit',$credit_add_reason_child);
					if(!$status) {
						// Subaccount Transaction Failed!
						// Reverse to Parent Account
						$status = wholesalex_wallet()->initiate_transaction(get_current_user_id(),$wallet_balance,'credit','Balance Reversal For '.$transation_id);

						do_action('wholesalex_wallet_subaccount_credit_added',$transation_id,$user_id);
					}
				}
			}
			/**
			 * Assign Parent Account WholesaleX Role To Subaccount WordPress Role
			 *
			 * @since 1.1.2
			 */
			$subaccount_user = new WP_User( $user_id );
			$roles           = (array) $subaccount_user->roles;
			if ( ! in_array( $role, $roles, true ) ) {
				$subaccount_user->add_role( $role );
			}
			do_action( 'wholesalex_addon_subaccount_created', $user_id, $parent_user_id, $password );
		} else {
			$result = array(
				'status'  => false,
				'message' => __( 'Subaccount Create Failed!', 'wholesalex-pro' ),
			);
			return $result;
		}

		$result = array(
			'status'  => true,
			'message' => __( 'Subaccount Successfully Created.', 'wholesalex-pro' ),
			'id'      => $user_id,
		);

		return $result;

	}

	/**
	 * Rewrite subaccount Endpoint and Save subaccount Content.
	 *
	 * @return void
	 * @since 1.0.9
	 */
	public function subaccount_endpoint() {
		add_rewrite_endpoint( $this->get_endpoint(), EP_ROOT | EP_PAGES );
	}

	/**
	 * Add Custom Query Vars in WooCommerce
	 *
	 * @param array $query Query.
	 * @since 1.0.9
	 */
	public function add_subaccount_to_woocommerce_query_vars( $query ) {
		$query[$this->get_endpoint()] = $this->get_endpoint();
		return $query;
	}

	/**
	 * Add subaccount Link On My Account Page
	 *
	 * @param array $items array of all links.
	 * @return array updated links.
	 * @since 1.0.9
	 */
	public function subaccount_link_my_account( $items ) {
		$items      = array_slice( $items, 0, 5, true )
		+ array( $this->get_endpoint()=> $this->get_endpoint_title() )
		+ array_slice( $items, 5, null, true );
		return $items;
	}

	/**
	 * Render Subaccounts Page.
	 *
	 * @return void
	 * @since 1.0.9
	 */
	public function subaccounts() {
		$user_id      = get_current_user_id();
		$account_type = get_user_meta( $user_id, '__wholesalex_account_type', true );
		if ( 'subaccount' !== $account_type ) {
			wp_enqueue_script( 'wholesalex_subaccount' );
			wp_localize_script(
				'wholesalex_subaccount',
				'wholesalex_subaccount',
				array(
					'nonce'     => wp_create_nonce( 'wholesalex_subaccount' ),
					'end_point' => wc_get_account_endpoint_url( 'whx-bulkorder' ),
					'settings'  => wholesalex()->get_setting(),
					'accounts'	=> $this->get_subaccounts( get_current_user_id() ),
					'i18n' => array(
						'error_occured' => __('Error Occured!','wholesalex-pro'),
						'update_subaccount' => __('Update Subaccount','wholesalex-pro'),
						'create_subaccount' => __('Create Subaccount','wholesalex-pro'),
						'go_back' => __('Go Back','wholesalex-pro'),
						'sub_account_information' => __('Sub Account Information','wholesalex-pro'),
						'name' => __('Name','wholesalex-pro'),
						'enter_account_holder_name_here' => __('Enter Account holder name here','wholesalex-pro'),
						'username' => __('Username','wholesalex-pro'),
						'enter_the_subaccount_username' => __('Enter the subaccount username','wholesalex-pro'),
						'email' => __('Email','wholesalex-pro'),
						'enter_email' => __('Enter email','wholesalex-pro'),
						'password' => __('Password','wholesalex-pro'),
						'enter_password' => __('Enter password','wholesalex-pro'),
						'job_title' => __('Job Title','wholesalex-pro'),
						'enter_job_title' => __('Enter Job Title','wholesalex-pro'),
						'phone_number' => __('Phone Number','wholesalex-pro'),
						'enter_phone_number' => __('Enter Phone Number','wholesalex-pro'),
						'credit_balance' => __('Credit Balance','wholesalex-pro'),
						'be_careful_when_setting_the_credit_balance' => __('Be careful when setting the credit balance.','wholesalex-pro'),
						'back' => __('Back','wholesalex-pro'),
						'proceed' => __('Proceed','wholesalex-pro'),
						'sub_account_permission' => __('Sub Account Permissions','wholesalex-pro'),
						'place_an_order' => __('Place an Order','wholesalex-pro'),
						'view_all_account_orders' => __('View all account orders','wholesalex-pro'),
						'view_all_account_conversations' => __('View all account conversations','wholesalex-pro'),
						'view_all_account_purchase_lists' => __('View all account purchase lists','wholesalex-pro'),
						'order_approval_required' => __('Order Approval Required','wholesalex-pro'),
						'can_access_wallet' => __('Can Access Wallet','wholesalex-pro'),
						'can_topup_wallet' => __('Can TopUp Wallet','wholesalex-pro'),
						'update_subaccount' => __('Update Subaccount','wholesalex-pro'),
						'create_subaccount' => __('Create Sub Account','wholesalex-pro'),
						'confirm_delete' => __('Confirm Delete','wholesalex-pro'),
						'subaccount_delete_popup_warning_message' => sprintf(__('Do you want to delete %s? Be careful, this procedure is irreversible. Do you want to proceed?','wholesalex-pro'),'##'),
						'cancel' => __('Cancel','wholesalex-pro'),
						'delete' => __('Delete','wholesalex-pro'),
						'account_name' => __('Account Name','wholesalex-pro'),
						'job_title' => __('Job Title','wholesalex-pro'),
						'phone' => __('Phone','wholesalex-pro'),
						'permissions' => __('Permissions','wholesalex-pro'),
						'actions' => __('Actions','wholesalex-pro'),
						'wallet_balance' => __('Wallet Balance','wholesalex-pro'),
						'place_an_order' => __('Place an Order','wholesalex-pro'),
						'delete_account' => __('Delete Account','wholesalex-pro'),
						'previous' => __('Previous','wholesalex-pro'),
						'next' => __('Next','wholesalex-pro'),
						'delete_subaccount_message' => __('Delete account successfull','wholesalex-pro'),
						'subaccounts' => __('Subaccounts','wholesalex-pro'),
						'create_subaccounts' => __('Create Subaccounts','wholesalex-pro'),
						'no_subaccounts_message' =>__('No subaccounts were found. To create one,','wholesalex-pro'),
						'click_here' => __('Click Here','wholesalex-pro'),
						'edit' => __('Edit','wholesalex-pro'),
					)
				)
			);
			wp_enqueue_style( 'dashicons' );

			echo '<div id="wholesalex_subaccounts"></div>';
		}

	}


	/**
	 * Check If Current account type is subaccount then, Current user will display parent account balance and transactions.
	 *
	 * @param int|string $user_id User ID.
	 * @return int|string
	 */
	public function set_wallet_user_id( $user_id ) {
		$__account_type = get_user_meta( $user_id, '__wholesalex_account_type', true );
		if ( 'subaccount' === $__account_type ) {
			$user_id = get_user_meta( $user_id, '__wholesalex_parent_id', true );
		}
		return $user_id;
	}

	/**
	 * Add Account Type Column On WholesaleX Users Page.
	 *
	 * @param array $columns Columns.
	 * @return array
	 * @since 1.0.9
	 */
	public function add_account_type_column( $columns ) {
		$columns = wholesalex()->insert_into_array( $columns, array( 'account_type' => __( 'Account Type', 'wholesalex-pro' ) ), 3 );
		return $columns;
	}

	/**
	 * Set Account Type Column Value on WholesaleX Users Page.
	 * By Default All Account Type is Main Account, If Any Main Account Holder Create any Subaccount Then, Account Type will Be Sub Account.
	 *
	 * @param string $column_value Column Previous/Default Value.
	 * @param array  $item Column Value Array.
	 * @param string $column_name Column Name.
	 * @return string
	 * @since 1.0.9
	 */
	public function account_type_column_value( $column_value, $item, $column_name ) {		
		if ( 'account_type' === $column_value ) {
			$user_id      = $item['ID'];
			$account_type = get_user_meta( $user_id, '__wholesalex_account_type', true );
			if ( 'subaccount' === $account_type ) {
				$parent_account_id   = get_user_meta( $user_id, '__wholesalex_parent_id', true );
				$parent_account_data = get_userdata( $parent_account_id );
				$column_value        = sprintf(
					'<div class="wholesalex_placed_by_column">
					<div class="subaccount">Sub Account</div>
					<div class="parent"> Parent: <span class="name"> %s </span></div>
					</div>',
					$parent_account_data->display_name
				);
			} else {
				$column_value = __( 'Main Account', 'wholesalex-pro' );
			}
		}
		return $column_value;
	}

	/**
	 * Add Placed By Columns on Order Page.
	 *
	 * @param array $columns Columns.
	 * @return array
	 * @since 1.0.9
	 */
	public function placed_by_column( $columns ) {
		$columns = wholesalex()->insert_into_array( $columns, array( 'order-placed-by' => __( 'Placed By', 'wholesalex-pro' ) ), 3 );
		return $columns;
	}

	/**
	 * Placed By Column Content on Orders Page.
	 *
	 * @param object $order Order Object.
	 * @return void
	 * @since 1.0.9
	 */
	public function placed_by_column_content( $order ) {
		$customer_id  = $order->get_customer_id();
		$account_type = get_user_meta( $customer_id, '__wholesalex_account_type', true );

		$customer_data = get_userdata( $customer_id );
		$customer_name = $customer_data->user_login;
		if ( '' !== $customer_data->display_name ) {
			$customer_name = $customer_data->display_name;
		}
		if ( 'subaccount' === $account_type ) {
			wp_enqueue_style( 'dashicons' );
			$parent_id   = get_user_meta( $customer_id, '__wholesalex_parent_id', true );
			$parent_data = get_userdata( $parent_id );
			$parent_name = $parent_data->user_login;
			if ( '' !== $parent_data->display_name ) {
				$parent_name = $parent_data->display_name;
			}
			?>
				<div class="wholesalex_placed_by_column">
					<div class="parent"> Parent: <span class="name"><?php echo esc_html( $parent_name ); ?></span></div>
					<div class="subaccount"> <span class="dashicons dashicons-editor-break"></span> SubAcc: <span class="name"><?php echo esc_html( $customer_name ); ?></span>  </div>
				</div>
			<?php
		} else {
			echo esc_html( $customer_name );
		}
	}


	/**
	 * Add Subaccount Orders In Query.
	 * If account type is subaccount and account holder has permission to view all orders, set all subaccounts ids including parent id in query to get all orders.
	 * If account type is parent account, set all subaccounts ids including parent id in query to get all orders.
	 *
	 * @param array $query Orders Query.
	 * @return array
	 * @since 1.0.9
	 */
	public function add_subaccount_orders_in_query( $query ) {
		$account_type = get_user_meta( $this->current_user_id, '__wholesalex_account_type', true );
		if ( 'subaccount' === $account_type ) {
			$parent_id          = get_user_meta( $this->current_user_id, '__wholesalex_parent_id', true );
			$subaccount_lists   = get_user_meta( $parent_id, '__wholesalex_subaccount_lists', true );
			$subaccount_lists[] = $parent_id;
			$query['customer']  = $subaccount_lists;
		} else {
			$subaccount_lists = get_user_meta( $this->current_user_id, '__wholesalex_subaccount_lists', true );
			if ( is_array( $subaccount_lists ) && ! empty( $subaccount_lists ) ) {
				$subaccount_lists[] = $this->current_user_id;
				$query['customer']  = $subaccount_lists;
			}
		}

		return $query;
	}


	/**
	 * Set View Order and Order Again Capabilities
	 * If account type is subaccount and account holder has permission to view all orders, set view order and order again capabilites to user.
	 * If account type is parent account, set view order and order again capabilites to main account holder.
	 *
	 * @param array $caps All Capabilities.
	 * @param array $cap Capability.
	 * @param array $args Order Args.
	 * @return array Updated Capabilities.
	 * @since 1.0.9
	 */
	public function set_caps_to_view_orders( $caps, $cap, $args ) {
		if ( is_array( $cap ) && ( in_array( 'view_order', $cap ) || in_array( 'order_again', $cap ) || in_array( 'pay_for_order', $cap ) || in_array( 'cancel_order', $cap ) ) ) {
			$account_type     = get_user_meta( $this->current_user_id, '__wholesalex_account_type', true );
			$subaccount_lists = array();
			$parent_id        = '';
			if ( 'subaccount' === $account_type ) {
				$parent_id          = get_user_meta( $this->current_user_id, '__wholesalex_parent_id', true );
				$subaccount_lists   = get_user_meta( $parent_id, '__wholesalex_subaccount_lists', true );
				$subaccount_lists[] = $parent_id;
			} else {
				$subaccount_lists = get_user_meta( $this->current_user_id, '__wholesalex_subaccount_lists', true );
				if ( is_array( $subaccount_lists ) && ! empty( $subaccount_lists ) ) {
					$subaccount_lists[] = $this->current_user_id;
				}
			}

			if ( ! is_array( $subaccount_lists ) ) {
				$subaccount_lists = array();
			}

			$order_id = isset( $args[2] ) ? $args[2] : '';
			if ( $order_id ) {
				$order       = wc_get_order( $order_id );
				$customer_id = $order->get_customer_id();
				if ( in_array( $customer_id, $subaccount_lists ) ) {
					$caps['view_order']  = true;
					$caps['order_again'] = true;
				}

				$parent_id = get_user_meta( $customer_id, '__wholesalex_parent_id', true );

				if ( $parent_id == $this->current_user_id ) {
					$caps['pay_for_order'] = true;
					$caps['cancel_order']  = true;
				}
			}
		}

		return $caps;

	}


	/**
	 * Include Customer Order Details File On Orders Page.
	 * If account type is subaccount and account holder has permission to view all orders,
	 * include customer order details file for the orders that is not owned by account holder,
	 * but owned by any subaccount holder of same parent to view order shipping and billing address.
	 * If account type is parent account, include customer order details file for subaccounts order to view shipping and billing address of these orders.
	 *
	 * @param object $order Order Object.
	 * @return void
	 * @since 1.0.9
	 */
	public function include_order_details_file( $order ) {
		$account_type     = get_user_meta( $this->current_user_id, '__wholesalex_account_type', true );
		$subaccount_lists = array();
		if ( 'subaccount' === $account_type ) {
			$parent_id          = get_user_meta( $this->current_user_id, '__wholesalex_parent_id', true );
			$subaccount_lists   = get_user_meta( $parent_id, '__wholesalex_subaccount_lists', true );
			$subaccount_lists[] = $parent_id;
		} else {
			$subaccount_lists = get_user_meta( $this->current_user_id, '__wholesalex_subaccount_lists', true );
			if ( is_array( $subaccount_lists ) && ! empty( $subaccount_lists ) ) {
				$subaccount_lists[] = $this->current_user_id;
			}
		}

		if ( ! is_array( $subaccount_lists ) ) {
			$subaccount_lists = array();
		}

		$customer_id = $order->get_customer_id();
		if ( in_array( $customer_id, $subaccount_lists ) && $customer_id !== $this->current_user_id ) {
			wc_get_template( 'order/order-details-customer.php', array( 'order' => $order ) );
		}
	}

	/**
	 * Set Purchase List Which will be visible on Purchase Lists Page.
	 * If account type is subaccount and account holder has permission to view all purchase list,
	 * set purchase lists that is owned by account holder and all the subaccounts from the same parent including parent.
	 * If account type is parent account, set all purchase lists of all subaccounts including own.
	 *
	 * @param array $lists Purchase Lists owned by current user.
	 * @return array
	 * @since 1.0.9
	 */
	public function purchase_lists( $lists ) {
		$parent_id        = get_user_meta( $this->current_user_id, '__wholesalex_parent_id', true );
		$subaccount_lists = get_user_meta( $parent_id ? $parent_id : $this->current_user_id, '__wholesalex_subaccount_lists', true );
		if ( is_array( $subaccount_lists ) && ! empty( $subaccount_lists ) ) {
			$lists = array();
			$ids   = $subaccount_lists;
            if(!is_array($ids)) {
                $ids = array();
            }
			$ids[] = $parent_id ? $parent_id : $this->current_user_id;
			foreach ( $ids as $id ) {
				$__lists = get_user_meta( $id, '__bulkorder_purchase_lists', true );
				if ( is_array( $__lists ) && ! empty( $subaccount_lists ) ) {
					$lists += $__lists;
				}
			}
		}
		return $lists;
	}

	/**
	 * Set Subaccount User Display Name On Conversation Reply.
	 *
	 * @param string     $display_name Conversation Reply Display Name.
	 * @param string|int $user_id User ID.
	 * @param string|int $author_id Conversation Author ID.
	 * @return string
	 * @since 1.0.9
	 */
	public function conversation_reply_display_name( $display_name, $user_id, $author_id ) {
		$account_type = get_user_meta( $user_id, '__wholesalex_account_type', true );
		if ( 'subaccount' === $account_type ) {
			$parent_id   = get_user_meta( $user_id, '__wholesalex_parent_id', true );
			$parent_data = get_userdata( $parent_id );
			$parent_name = $parent_data->display_name;

			$display_name = sprintf( 'Parent: <span class="reply-name"> %s </span>  %s Sub Acc: <span class="reply-name">%s</span>', $parent_name, '<span class="dashicons dashicons-arrow-right-alt"></span>', $display_name );
		}

		return $display_name;

	}

	/**
	 * Set Parent and Subaccount Ids of Current Account for Conversations.
	 *
	 * @param array      $ids Ids of user, whose conversation will be visible on myaccount conversation tab.
	 * @param string|int $author_id Conversation Author ID.
	 * @return array
	 * @since 1.0.9
	 */
	public function parent_n_subaccount_ids_for_conversation( $ids, $author_id ) {
		$parent_id        = get_user_meta( $author_id, '__wholesalex_parent_id', true );
		$subaccount_lists = get_user_meta( $parent_id ? $parent_id : $this->current_user_id, '__wholesalex_subaccount_lists', true );
		if ( is_array( $subaccount_lists ) && ! empty( $subaccount_lists ) ) {
			$ids   = $subaccount_lists;
			$ids[] = $parent_id ? $parent_id : $this->current_user_id;
		}
		return $ids;
	}

	/**
	 * Set Eligibility to View Conversations on Sub account holders and Parent Account.
	 * If account type is sub account, and has permission to view all conversation, then set account holder as eligible view conversations of subaccount of same parent including parent.
	 * If account type is parent account,then set account holder as eligible view conversations of all subaccounts.
	 *
	 * @param bool       $status Conversation View Eligibility Status.
	 * @param string|int $author_id Conversation Author Id.
	 * @return bool
	 * @since 1.0.9
	 */
	public function view_conversation_eligibility( $status, $author_id ) {
		$parent_id          = get_user_meta( $author_id, '__wholesalex_parent_id', true );
		$cur_user_parent_id = get_user_meta( $this->current_user_id, '__wholesalex_parent_id', true );
		$status             = ( $parent_id == $cur_user_parent_id ) || ( $author_id == $cur_user_parent_id ) || ( $this->current_user_id == $parent_id );
		return $status;
	}


	/**
	 * Set Error Message for the users that does not have place order permission.
	 *
	 * @param object $data Order Data.
	 * @param object $errors Error Messages.
	 * @return void
	 * @since 1.0.9
	 */
	public function place_order_permission( $data, $errors ) {
		$errors->add( 'validation', esc_html__( 'Your account does have permission to place order/checkout', 'wholesalex-pro' ) );
	}


	/**
	 * Add Account Type Column On Orders Page. (Admin Interface).
	 *
	 * @param array $columns Columns.
	 * @return array
	 * @since 1.0.9
	 */
	public function add_placed_by_column_on_order_page( $columns ) {
		$columns = wholesalex()->insert_into_array( $columns, array( 'wholesalex_placed_by' => __( 'Placed By' ) ), 4 );
		return $columns;
	}

	/**
	 * Populate Data on Account Type Column On Orders Page. (Admin Interface).
	 *
	 * @param string     $column Column Name.
	 * @param int|string $order_id Order ID.
	 * @return void
	 */
	public function populate_data_on_placed_by_column( $column, $order_id ) {
		if ( 'wholesalex_placed_by' === $column ) {
			$order        = wc_get_order( $order_id );
			$customer_id  = $order->get_customer_id();
			$account_type = get_user_meta( $customer_id, '__wholesalex_account_type', true );

			$customer_data = get_userdata( $customer_id );
			if ( ! is_object( $customer_data ) ) {
				echo 'Guest User';
				return;
			}
			$customer_name = $customer_data->user_login;
			if ( '' !== $customer_data->display_name ) {
				$customer_name = $customer_data->display_name;
			}
			if ( 'subaccount' === $account_type ) {
				wp_enqueue_style( 'dashicons' );
				$parent_id   = get_user_meta( $customer_id, '__wholesalex_parent_id', true );
				$parent_data = get_userdata( $parent_id );
				$parent_name = $parent_data->user_login;
				if ( '' !== $parent_data->display_name ) {
					$parent_name = $parent_data->display_name;
				}
				?>
					<div class="wholesalex_placed_by_column">
						<div class="parent"> Parent: <span class="name"><?php echo esc_html( $parent_name ); ?></span></div>
						<div class="subaccount"> <span class="dashicons dashicons-editor-break"></span> SubAcc: <span class="name"><?php echo esc_html( $customer_name ); ?></span>  </div>
					</div>
				<?php
			} else {
				echo esc_html( $customer_name );
			}
		}
	}


	/**
	 * Add Order Note on Subaccounts Order
	 *
	 * @param int|string $order_id Order ID.
	 * @return void
	 * @since 1.0.9
	 */
	public function add_order_note( $order_id ) {
		$order        = wc_get_order( $order_id );
		$customer_id  = $order->get_customer_id();
		$account_type = get_user_meta( $customer_id, '__wholesalex_account_type', true );
		if ( 'subaccount' === $account_type ) {
			$parent_account_id = get_user_meta( $customer_id, '__wholesalex_parent_id', true );
			$parent_data       = get_userdata( $parent_account_id );
			$order_note        = sprintf( 'This order placed by a subaccount of %s', $parent_data->display_name );
			$order->add_order_note( $order_note );

			// Does Notify Parent Account for subaccounts order?
			do_action( 'wholesalex_subaccount_order', $order_id, $customer_id, $parent_account_id );

			// $permissions = get_user_meta( $this->current_user_id, '__wholesalex_permissions', true );
			// if ( in_array( 'order_approval_required', $permissions ) ) {
			// $order->update_status( 'whx-approval-need', __( 'Send to Main Account For Approval', 'wholesalex-pro' ) );
			// }
		}
	}


	/**
	 * Send Subaccount Order Details to Parent By Email
	 *
	 * @param int|string $order_id Order Id.
	 * @param int|string $customer_id Customer ID.
	 * @param int|string $parent_id Parent ID.
	 * @return void
	 * @since 1.0.9
	 */
	public function send_subaccount_order_email_to_parent( $order_id, $customer_id, $parent_account_id) {
		$order = wc_get_order($order_id);
		$order_status = $order->get_status();
		
		if ( 'whx-approval-need' === $order_status ) {
			return;
		}

		$email = $this->emails['WholesaleX_Subaccount_Order_Placed_Email'];
		$email->trigger( $order_id,$customer_id,$parent_account_id );
	}

	/**
	 * Send An Email to subaccount user.
	 *
	 * @param string|int $subaccount_user_id Subaccount User ID.
	 * @param string|int $parent_user_id Parent User ID.
	 * @return void
	 * @since 1.0.9
	 */
	public function subaccount_created( $subaccount_user_id, $parent_user_id, $subaccount_password ) {
		$subaccount_data = get_userdata( $subaccount_user_id );
		$parent_data     = get_userdata( $parent_user_id );


		$email = $this->emails['WholesaleX_New_Subaccount_Create_Email'];
		$email->trigger( $subaccount_user_id, $parent_user_id,$subaccount_password );
	}
	/**
	 * Remove Add Balance Fields From My Account Wallet Page For Subaccounts those does not have wallet topup permission.
	 *
	 * @param array $wallet_fields Wallet Fields.
	 * @return array
	 * @since 1.0.9
	 */
	public function remove_add_balance_fields( $wallet_fields ) {
		unset( $wallet_fields['_add_balance_section'] );
		unset( $wallet_fields['_button_section']['attr']['_add_balance'] );
		return $wallet_fields;
	}
	/**
	 * Remove Add Balance Fields From My Account Wallet Page For Subaccounts those does not have wallet topup permission.
	 *
	 * @param array $wallet_fields Wallet Fields.
	 * @return array
	 * @since 1.0.9
	 */
	public function add_request_balance_field( $wallet_fields ) {
		// $wallet_fields['_add_balance_section']
		// unset( $wallet_fields['_button_section']['attr']['_add_balance'] );
		$wallet_fields['_add_balance_section'] = array(
			'attr' => array(
				'_request_amount' => array(
					'type'        => 'number',
					/* translators: %s: Currency Symbol */
					'label'       => wholesalex()->get_language_n_text( '_language_wallet_request_amount_text' , sprintf( __( 'Enter Amount (%s)', 'wholesalex-pro' ), get_woocommerce_currency_symbol() ) ),
					'default'     => '',
					'placeholder' => '',
				),
				'_request_btn'   => array(
					'type'        => 'button',
					'label'       => wholesalex()->get_language_n_text( '_language_wallet_request_submit_text' , __( 'Request', 'wholesalex-pro' ) ),
					'redirect_to' => wc_get_checkout_url(),
				),
			),
		) ;
		$wallet_fields['_button_section']['attr']['_add_balance']= array(
			'type'       => 'button',
			'icon_class' => 'dashicons dashicons-money-alt',
			'label'      => wholesalex()->get_language_n_text( '_language_wallet_request_balance_text', __( 'Request Balance', 'wholesalex-pro' ) ),
		);
		return $wallet_fields;
	}

	/**
	 * Make Wallet Topup Not Purchaseable For Subaccounts those doesnot have wallet topup permissions.
	 *
	 * @param bool   $is_purchasable Is Purchaseable Status.
	 * @param object $product Product Object.
	 * @return bool
	 * @since 1.0.9
	 */
	public function make_topup_product_not_purchasable( $is_purchasable, $product ) {
		$_topup_product_id = get_option( '__wholesalex_wallet_topup_product' );
		if ( $_topup_product_id ) {
			if ( intval( $_topup_product_id ) === intval( $product->get_id() ) ) {
				$is_purchasable = false;
			}
		}
		return $is_purchasable;
	}


	/**
	 * Set Parent Account ID As User Instead of Subaccount.
	 *
	 * @param string|int $user_id Subaccount User ID.
	 * @return string|int
	 * @since 1.0.9
	 */
	public function set_current_user( $user_id ) {
		$account_type = get_user_meta( $user_id, '__wholesalex_account_type', true );
		if ( 'subaccount' === $account_type ) {
			$user_id = get_user_meta( $user_id, '__wholesalex_parent_id', true );
		}
		return $user_id;
	}

	/**
	 * Set Wallet Payment Desciption Message For Subaccount Wallet Payment.
	 *
	 * @param string $description Wallet Payment Description.
	 * @param object $order Order Object.
	 * @return string
	 * @since 1.0.9
	 */
	public function wallet_order_payment_description( $description, $order ) {
		$account_type = get_user_meta( $order->get_customer_id(), '__wholesalex_account_type', true );
		if ( 'subaccount' === $account_type ) {
			$customer_data = get_userdata( $order->get_customer_id() );
			/* translators: %1s: Order ID %2s: Customer Name */
			$description = sprintf( __( 'For order payment #%1$s, Paid By %2$s', 'wholesalex-pro' ), $order->get_order_number(), $customer_data->display_name );
		}
		return $description;
	}

	/**
	 * Set Wallet Topup Desciption Message For Subaccount.
	 *
	 * @param string $description Wallet Payment Description.
	 * @param object $order Order Object.
	 * @return string
	 * @since 1.0.9
	 */
	public function wallet_topup_payment_description( $description, $order ) {
		$account_type = get_user_meta( $order->get_customer_id(), '__wholesalex_account_type', true );
		if ( 'subaccount' === $account_type ) {
			$customer_data = get_userdata( $order->get_customer_id() );
			/* translators: %1s: Order ID %2s: Customer Name */
			$description = sprintf( __( 'Topup Through #%1$s, Paid By %2$s', 'wholesalex-pro' ), $order->get_order_number(), $customer_data->display_name );
		}
		return $description;
	}


	/**
	 * Subaccount Role Changed.
	 * If parent account role changed, then all subaccounts of this parent role also changed.
	 *
	 * @param int    $user_id User ID.
	 * @param string $previous_role Previous Role ID.
	 * @param string $updated_role Updated Role Role ID.
	 * @return void
	 */
	public function user_role_changed( $user_id, $previous_role, $updated_role ) {
		$account_type = get_user_meta( $user_id, '__wholesalex_account_type', true );
		if ( 'subaccount' != $account_type ) {
			$subaccount_lists = get_user_meta( $user_id, '__wholesalex_subaccount_lists', true );
			if ( is_array( $subaccount_lists ) && ! empty( $subaccount_lists ) ) {
				foreach ( $subaccount_lists as $sid ) {
					wholesalex()->change_role( $sid, $updated_role, $previous_role );
				}
			}
		}
	}


	/**
	 * Filter Subaccount Holder From Dynamic Rule Users List.
	 * Subaccount Inherit Dynamic Rules From Their Parent Account.
	 *
	 * @param array $query WP Query.
	 * @return array
	 * @since 1.0.9
	 */
	public function filter_subaccount_holders( $query ) {
		$query['meta_query'] = array(
			'relation' => 'OR',
			array(
				'key'     => '__wholesalex_account_type',
				'compare' => 'NOT EXISTS',
			),
			array(
				'key'     => '__wholesalex_account_type',
				'value'   => 'subaccount',
				'compare' => '!=',
			),
		);
		return $query;
	}

	/**
	 * Remove Wallet From Menu Item, if account holder does not have wallet access permission
	 *
	 * @param array $menu_items Menu Items.
	 * @return array
	 */
	public function remove_wallet_from_menu( $menu_items ) {
		if ( isset( $menu_items['wholesalex-wallet'] ) ) {
			unset( $menu_items['wholesalex-wallet'] );
		}
		return $menu_items;
	}


	/**
	 * Load Wholesalex Main Account Approval Required Payment Gateways
	 *
	 * @param array $gateways Payment Gateways.
	 * @return array
	 * @since 1.0.9
	 */
	public function load_payment_gateways( $gateways ) {
		$gateways[] = 'WHOLESALEX_PRO\ParentApprovalGateway';
		return $gateways;
	}


	/**
	 * Main Account Approval Required Order Status Added.
	 *
	 * @param array $status Order Statues.
	 * @return array
	 * @since 1.0.9
	 * @todo Content Deps
	 */
	public function approval_need_order_status_add( $status ) {
		$status['wc-whx-approval-need'] = apply_filters('wholesalex_subaccount_addon_main_account_approval_needed_status_label', _x( 'Main Account Approval Needed', 'Order status', 'woocommerce' ));
		return $status;
	}

	/**
	 * Get User Account Type
	 *
	 * @param int|string $user_id User ID.
	 * @return bool
	 * @since 1.0.9
	 */
	public function get_account_type( $user_id ) {
		$account_type = get_user_meta( $user_id, '__wholesalex_account_type', true );
		return $account_type;
	}

	/**
	 * Check Given User ID is a subaccount or not.
	 *
	 * @param int|string $user_id User ID.
	 * @since 1.0.9
	 */
	public function is_subaccount( $user_id ) {
		$account_type = $this->get_account_type( $user_id );
		return 'subaccount' === $account_type;
	}

	/**
	 * Get Parent ID of a subaccount
	 *
	 * @param int|string $user_id User ID.
	 * @return int|string|false
	 * @since 1.0.9
	 */
	public function get_parent_id( $user_id ) {
		$parent_id = false;
		if ( $this->is_subaccount( $user_id ) ) {
			$parent_id = get_user_meta( $user_id, '__wholesalex_parent_id', true );
		}
		return $parent_id;
	}

	/**
	 * Check Current Subaccount Has A Specific Permission or not
	 *
	 * @param array $permission Permission.
	 * @return boolean
	 * @since 1.0.9
	 */
	public function has_permission( $permission = '' ) {
		return in_array( $permission, $this->permissions );
	}



	/**
	 * Restrict Approval Needed Payment Gateways for users.
	 *
	 * @param array $available_gateways Available Payment Gateways.
	 * @return array
	 * @since 1.0.9
	 */
	public function control_payment_gateway( $available_gateways ) {
		if ( $this->is_subaccount( $this->current_user_id ) && $this->has_permission( 'order_approval_required' ) ) {
			$order_id        = '';
			$approval_status = false;
			if ( is_wc_endpoint_url( 'order-pay' ) ) {
				$order_id = get_query_var( 'order-pay' );
				$order    = wc_get_order( get_query_var( 'order-pay' ) );
				if ( $order ) {
					$order_status = get_post_meta( $order_id, '__wholesalex_subaccount_order_staus', true );
					if ( isset( $order_status['status'] ) && ( 'approved' === $order_status['status'] ) ) {
						$approval_status = true;
					}
				}
			}
			if ( ! $approval_status ) {
				foreach ( $available_gateways as $key => $value ) {
					if ( 'wholesalex_subaccount_order_approval' !== $key ) {
						unset( $available_gateways[ $key ] );
					}
				}
				return $available_gateways;
			}
		}

		if ( isset( $available_gateways['wholesalex_subaccount_order_approval'] ) ) {
			unset( $available_gateways['wholesalex_subaccount_order_approval'] );
		}
		return $available_gateways;
	}

	/**
	 * Subaccount Order Manual Approval Order Received Text
	 *
	 * @param string $message Order Received Message.
	 * @return string
	 * @since 1.0.9
	 */
	public function subaccount_order_manual_approval_order_received_text( $message, $order ) {
		$order_id                = $order->get_id();
		$subaccount_order_status = get_post_meta( $order_id, '__wholesalex_subaccount_order_staus', true );

		if ( isset( $subaccount_order_status['status'] ) && 'pending' === $subaccount_order_status['status'] ) {
			$message = __( 'Thank you for your order. Once it gets approved, you can make the payment to complete the order.', 'wholesalex-pro' );
		}
		return $message;
	}

	/**
	 * Order Approval Content on Order Details Page. (Frontend)
	 * This content will be visible by parent account if any of his subaccount needs order approval.
	 *
	 * @param int|string $order_id Order ID.
	 * @todo Content Deps
	 * @since 1.0.9
	 */
	public function order_approval_content_order_details_page( $order_id ) {
		$order = wc_get_order( $order_id );

		$order_status = $order->get_status();

        $approval_notice_text = wholesalex()->get_setting('_language_subaccount_review_the_order_text',__('Please review the order and accept or reject it using the following buttons.','wholesalex-pro'));
        $approve_order_text = wholesalex()->get_setting('_language_subaccount_approve_order_text',__('Approve','wholesalex-pro'));
        $reject_order_text = wholesalex()->get_setting('_language_subaccount_reject_order_text',__('Reject','wholesalex-pro'));

		if ( 'whx-approval-need' === $order_status && 'subaccount' != $this->account_type ) {
			?>
                <div class="wholesalex_circular_loading__wrapper" id="wholesalex_subaccount_orders__loading">
                <div class="wholesalex_loading_spinner">
                    <svg viewBox="25 25 50 50" class="move_circular">
                        <circle
                            cx="50"
                            cy="50"
                            r="20"
                            fill="none"
                            class="wholesalex_circular_loading_icon"
                        ></circle>
                    </svg>
                </div>
            </div>
				<div class="wholesalex_subaccount_order_approval">
					<div class="order_approval_notice"><?php echo esc_html($approval_notice_text); ?> </div>
					<button class="button approve" id="approve_order" onclick="approveOrder()"><?php echo esc_html($approve_order_text); ?></button>
					<button class="button reject" id="reject_order"><a href="<?php echo esc_url_raw( $order->get_cancel_order_url( wc_get_endpoint_url( 'orders' ) ) ); ?>"><?php echo esc_html($reject_order_text); ?></a></button>
				</div>

				<style>
					.wholesalex_subaccount_order_approval{
						margin-top: 2.5rem;
					}
					#approve_order {
                        font-size: 14px;
                        line-height: 22px;
                        color: #fff;
                        font-weight: 500;
                        background-color: #25a85d;
                        padding: 7px 12px;
                        border-radius: 4px;
                        /* font-weight: bold; */
                    }
					#reject_order {
                        font-size: 14px;
                        line-height: 22px;
                        color: #fff;
                        font-weight: 500;
                        background-color: #ff3131;
                        padding: 7px 18px;
                        border-radius: 4px;
                    }

                    button#approve_order:hover {
                        background-color: #1b7c44;
                    }

                    button#reject_order:hover {
                        background-color: #ae2222;
                    }
					#reject_order a {
						color: white;
					}

                    .wholesalex_subaccount_order_approval {
                        background-color: #fafaff;
                        border: 1px solid var(--wholesalex-border-color);
                        padding: 20px;
                    }
                    .order_approval_notice {
                        margin-bottom: 10px;
                    }
                    .wholesalex_circular_loading__wrapper {
                        background-color: hsla(0, 0%, 100%, 0.5);
                        bottom: 0;
                        left: 0;
                        margin: 0;
                        position: absolute;
                        right: 0;
                        top: 0;
                        transition: opacity 0.3s;
                        z-index: 9999;
                        display: none;
                    }
                    .wholesalex_loading_spinner {
                        margin-top: -21px;
                        position: absolute;
                        text-align: center;
                        top: 50%;
                        width: 100%;
                    }
                    .wholesalex_circular_loading_icon {
                        stroke-dasharray: 90, 150;
                        stroke-dashoffset: 0;
                        stroke-width: 2;
                        stroke: var(--wholesalex-primary-color);
                        stroke-linecap: round;
                        animation: wholesalex_circular_loading 1.5s ease-in-out infinite;
                    }
                    @keyframes wholesalex_circular_loading {
                        0% {
                            stroke-dasharray: 1, 140;
                            stroke-dashoffset: 0;
                        }
                    }
                    .wholesalex_loading_spinner .move_circular {
                        animation: circular_rotate 2s linear infinite;
                        height: 42px;
                        width: 42px;
                    }
                    @keyframes circular_rotate {
                        100% {
                            transform: rotate(1turn);
                        }
                    }
				</style>

				<script>
					const sendRequest = (type='approve') => {
						// const nonce = event.target.getAttribute('data-security');
                        document.getElementById('wholesalex_subaccount_orders__loading').style.display = 'block';

						const formData = new FormData();
						formData.append('action', 'order_approval_ajax_handler');
						formData.append('type',type);
						formData.append('id', <?php echo esc_attr($order_id); ?>);
						formData.append('nonce', "<?php echo esc_attr(wp_create_nonce( 'wholesalex_subaccount_order_approval' )); ?>");
						// formData.append('nonce', nonce);


						fetch(
							wholesalex.ajax, {
							method: 'POST',
							body: formData,
						})
						.then(res => res.json())
						.then(res => {
                            document.getElementById('wholesalex_subaccount_orders__loading').style.display = 'none';

							if(res.success) {
								window.alert(res.data.message);
								location.replace(res.data.redirect);
							}
						})
					}
					const approveOrder = ()=> {
						sendRequest('approve');
					}
				</script>
			<?php
		}

	}


	/**
	 * Make approval need status order editable.
	 *
	 * @param bool   $status Previous Order Status.
	 * @param object $order Order Object.
	 * @return bool
	 * @since 1.0.9
	 */
	public function make_approval_required_order_editable( $status, $order ) {
		$__order_status = $order->get_status();
		return ( 'whx-approval-need' == $__order_status );
	}

	/**
	 * Set Approval Needs Order Status equivalen to Pending order status.
	 *
	 * @param array $status Status.
	 * @return array
	 * @since 1.0.9
	 */
	public function approval_need_status_as_pending( $status ) {
		$status[] = 'whx-approval-need';
		return $status;
	}


	/**
	 * My Account Order Actions
	 *
	 * @param array  $actions Actions.
	 * @param object $order Order.
	 * @return array
	 * @since 1.0.9
	 */
	public function my_account_order_actions( $actions, $order ) {
		$status = $order->get_status();
		if ( 'whx-approval-need' === $status ) {
			unset( $actions['pay'] );
			if ( $this->is_subaccount( $order->get_customer_id() ) && $this->get_parent_id( $order->get_customer_id() ) == $this->current_user_id ) {
				unset( $actions['cancel'] );
			}
		}
		return $actions;
	}


	/**
	 * Order Approval Ajax Handler
	 *
	 * @return void
	 * @todo Content Deps
	 * @since 1.0.9
	 */
	public function order_approval_ajax_handler() {
		if ( ! ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'wholesalex_subaccount_order_approval' ) ) ) {
			wp_send_json_error( 'Nonce Verification Failed!' );
			die();
		}
		$type     = isset( $_POST['type'] ) ? sanitize_text_field( wp_unslash( $_POST['type'] ) ) : '';
		$order_id = isset( $_POST['id'] ) ? sanitize_text_field( wp_unslash( $_POST['id'] ) ) : '';

		$order = wc_get_order( $order_id );
		if ( ! $order ) {
			wp_send_json_error( 'Invalid Order!' );
			die();
		}

		switch ( $type ) {
			case 'approve':
				$order->update_status( 'pending', __( 'Order Approved', 'wholesalex-pro' ) );
				update_post_meta(
					$order_id,
					'__wholesalex_subaccount_order_staus',
					array(
						'status'  => 'approved',
						'user_id' => $this->current_user_id,
					)
				);
				wp_send_json_success(
					array(
						'redirect' => wc_get_endpoint_url( 'orders', '', get_permalink( get_option( 'woocommerce_myaccount_page_id' ) ) ),
						'message'  => __(
							'Order Approved.',
							'wholesalex-pro'
						),
					)
				);
				break;
		}
		die();

	}


	/**
	 * Order Status Changed Actions
	 *
	 * @param int|string $order_id Order ID.
	 * @param string     $old_status Old Status.
	 * @param string     $new_status New Status.
	 * @param obejct     $order Order.
	 * @return void
	 * @since 1.0.9
	 */
	public function order_status_changed_actions( $order_id, $old_status, $new_status, $order ) {
		if ( 'whx-approval-need' === $new_status ) {
			do_action( 'wholesalex_addon_subaccount_new_order_for_approval', $order_id, $order );
		}
		if ( 'whx-approval-need' === $old_status && 'pending' === $new_status ) {
			do_action( 'wholesalex_addon_subaccount_parent_approve_order', $order_id, $order );
		}
		if ( 'whx-approval-need' === $old_status && 'cancelled' === $new_status ) {
			do_action( 'wholesalex_addon_subaccount_parent_reject_order', $order_id, $order );
		}
	}

	/**
	 * Admin Approve Order, Send Email to Subaccount
	 *
	 * @param string|int $order_id Order ID.
	 * @param object     $order Order Object.
	 * @return void
	 * @since 1.0.9
	 */
	public function approve_order_send_email( $order_id, $order ) {
		// $email_templates = get_option( '__wholesalex_email_templates', array() );
		// $email_template  = $email_templates['wholesalex_subaccount_approve_order'];

		$customer_id   = $order->get_customer_id();
		$customer_data = get_userdata( $customer_id );

		if ( ! $this->is_subaccount( $customer_id ) ) {
			return;
		}
		$parent_account_id   = $this->get_parent_id( $customer_id );
		$parent_account_data = get_userdata( $parent_account_id );


		$email = $this->emails['WholesaleX_Subaccount_Order_Approved_Email'];
		$email->trigger( $order_id,$customer_id,$parent_account_id );
		
	}
	/**
	 * Admin Rejected Order, Email Send to customer.
	 *
	 * @param int|string $order_id Order ID.
	 * @param object     $order Order Object.
	 * @return void
	 * @since 1.0.9
	 */
	public function reject_order_send_email( $order_id, $order ) {
		// $email_templates = get_option( '__wholesalex_email_templates', array() );
		// $email_template  = $email_templates['wholesalex_subaccount_reject_order'];

		$customer_id   = $order->get_customer_id();
		$customer_data = get_userdata( $customer_id );

		if ( ! $this->is_subaccount( $customer_id ) ) {
			return;
		}
		$parent_account_id   = $this->get_parent_id( $customer_id );
		$parent_account_data = get_userdata( $parent_account_id );

		$email = $this->emails['WholesaleX_Subaccount_Order_Reject_Email'];
		$email->trigger( $order_id,$customer_id,$parent_account_id );
	}


	/**
	 * New Order For Approval.
	 *
	 * @param int|string $order_id Order ID.
	 * @param object     $order Order Object.
	 * @return void
	 * @since 1.0.9
	 */
	public function new_order_for_admin_approval_email( $order_id, $order ) {
		// $email_templates = get_option( '__wholesalex_email_templates', array() );
		// $email_template  = $email_templates['wholesalex_subaccount_new_order_approval'];

		$customer_id   = $order->get_customer_id();
		$customer_data = get_userdata( $customer_id );

		if ( ! $this->is_subaccount( $customer_id ) ) {
			return;
		}
		$parent_account_id   = $this->get_parent_id( $customer_id );
		$parent_account_data = get_userdata( $parent_account_id );

		$email = $this->emails['WholesaleX_Subaccount_Order_Approval_Required_Email'];
		$email->trigger( $order_id,$customer_id,$parent_account_id );

	}
	/**
	 * New Order For Approval.
	 *
	 * @param int|string $order_id Order ID.
	 * @param object     $order Order Object.
	 * @return void
	 * @since 1.0.9
	 */
	public function new_order_pending_for_approval( $order_id, $order ) {
		// $email_templates = get_option( '__wholesalex_email_templates', array() );
		// $email_template  = $email_templates['wholesalex_subaccount_order_pending_for_approval'];

		$customer_id   = $order->get_customer_id();
		$customer_data = get_userdata( $customer_id );

		if ( ! $this->is_subaccount( $customer_id ) ) {
			return;
		}
		$parent_account_id   = $this->get_parent_id( $customer_id );
		$parent_account_data = get_userdata( $parent_account_id );

		$email = $this->emails['WholesaleX_Subaccount_Order_Pending_Email'];
		$email->trigger( $order_id,$customer_id,$parent_account_id );

	}
	/**
	 * Subaccount Related Fields on Roles
	 *
	 * @param array $fields Fields.
	 * @since 1.0.9
	 * @todo Content Deps
	 */
	public function subaccounts_field_on_role( $fields ) {
		$subaccount_fields = array(
			'_subaccount_status'   => array(
				'type'     => 'switch',
				'label'    => __( 'Enable Subaccounts Creation Option for This Role', 'wholesalex-pro' ),
				'desc'     => __( 'Enable', 'wholesalex-pro' ),
				'help'     => '',
				'default'  => '',
				'excludes' => array( 'wholesalex_guest', 'wholesalex_b2c_users' ),
			),
			'_exclude_subaccounts' => array(
				'type'       => 'multiselect',
				'label'      => __( 'Exclude Users ', 'wholesalex-pro' ),
				'desc'       => __( 'Enable', 'wholesalex-pro' ),
				'help'       => '',
				'default'    => '',
				'options'    => array(),
				'depends_on' => array(
					array(
						'key'   => '_subaccount_status',
						'value' => 'yes',
					),
				),
				'excludes'   => array( 'wholesalex_guest', 'wholesalex_b2c_users' ),
			),
		);

		$fields['_role']['attr']['settings_section']['attr'] = wholesalex()->insert_into_array( $fields['_role']['attr']['settings_section']['attr'], $subaccount_fields, 4 );
		return $fields;

	}


	/**
	 * Subaccount Email Templates Smart Tags
	 *
	 * @param array  $tags Tags.
	 * @param string $template_name Template Name.
	 * @return array
	 * @since 1.0.9
	 */
	public function email_smart_tags( $prev_tags, $template_name ) {
		$tags = array();

		if ( 'wholesalex_subaccount_order' === $template_name || 'wholesalex_subaccount_create' === $template_name || 'wholesalex_subaccount_approve_order' === $template_name || 'wholesalex_subaccount_reject_order' === $template_name || 'wholesalex_subaccount_new_order_approval' === $template_name || 'wholesalex_subaccount_order_pending_for_approval' === $template_name ) {
			$tags['date']                = __( 'Show the current date inside the Email.', 'wholesalex-pro' );
			$tags['subaccount_name']     = __( 'Show the subaccount name', 'wholesalex-pro' );
			$tags['main_account_name']   = __( 'Show the main account name', 'wholesalex-pro' );
			$tags['subaccount_username'] = __( 'Show the subaccount username.', 'wholesalex-pro' );
			$tags['site_name']           = __( 'Show the Site Name.', 'wholesalex-pro' );
			$tags['admin_name']          = __( 'Show the Super Admin Name.', 'wholesalex-pro' );
		}
		if ( 'wholesalex_subaccount_create' === $template_name ) {
			$tags['subaccount_password'] = __( 'Show the subaccount password', 'wholesalex-pro' );

		}

		if ( 'wholesalex_subaccount_order' === $template_name || 'wholesalex_subaccount_approve_order' === $template_name || 'wholesalex_subaccount_reject_order' === $template_name || 'wholesalex_subaccount_new_order_approval' === $template_name || 'wholesalex_subaccount_order_pending_for_approval' === $template_name ) {
			$tags['order_id'] = __( 'Show the order id.', 'wholesalex-pro' );
			$tags['view_order'] = __( 'Show the view order url.', 'wholesalex-pro' );
			
		}

		if('wholesalex_subaccount_approve_order' ===  $template_name) {
			$tags['order_pay_url']          = __( 'Get The Order Pay URL', 'wholesalex-pro' );
		}

		if(!empty($tags)) {
			$prev_tags = $tags;
		}
		return $prev_tags;
	}

	public function subaccount_wallet_fields( $fields ) {

		$wallet_balance_request = get_user_meta(get_current_user_id(),'__wholesalex_wallet_balance_requests',true);
		if(!is_array($wallet_balance_request)) {
			$wallet_balance_request = array();
		}
		foreach ($wallet_balance_request as $key => $request) {
			$user_data = get_userdata($request['from']);
			$wallet_balance_request[$key]['subaccount_name'] =  isset($user_data->display_name)?$user_data->display_name:'';
			$wallet_balance_request[$key]['amount_html'] =  wc_price($request['amount']);
		}
		$pending_request_button = array(
			'type'       => 'button',
			'icon_class' => 'dashicons dashicons-image-rotate',
			'label'      => wholesalex()->get_language_n_text( '_language_wallet_pending_request_from_subaccount', __( 'Pending Request From Subaccount', 'wholesalex-pro' ) ),
		);

		if ( isset( $fields['_button_section']['attr'] ) ) {
			$fields['_button_section']['attr']['_subaccount_pending_request'] = $pending_request_button;
		}

		if ( is_array( $fields ) ) {
			$fields['_subaccount_pending_request_section'] = array(
				'attr' => array(
					'_pending_requested_table' => array(
						'type'             => 'table',
						'item_per_page'    => apply_filters( 'wholesalex_wallet_pending_request_table_item_per_page', 50 ),
						'columns'          => array(
							'_requested_subaccount_name' => wholesalex()->get_language_n_text( '_language_wallet_balance_request_table_subaccount_name_text', __( 'Subaccount Name', 'wholesalex-pro' ) ),
							'_requested_wallet_credit_text'  => wholesalex()->get_language_n_text( '_language_wallet_balance_request_table_requested_credit_text', __( 'Requested Balance', 'wholesalex-pro' ) ),
							// '_request_date'              => wholesalex()->get_language_n_text( '_language_wallet_balance_request_table_requested_date_text', __( 'Date', 'wholesalex-pro' ) ),
							'_request_actions_text'           => wholesalex()->get_language_n_text( '_language_wallet_balance_request_table_requested_action_text', __( 'Actions', 'wholesalex-pro' ) ),
						),
						'data'             => $wallet_balance_request,
						'pagination_label' => array(
							'prev' => wholesalex()->get_language_n_text( '_language_pagination_label_prev', __( 'Previous', 'wholesalex-pro' ) ),
							'next' => wholesalex()->get_language_n_text( '_language_pagination_label_next', __( 'Next', 'wholesalex-pro' ) ),
						),
						'price_formatter'  => wc_price( 0 ),
					),
				),
			);
		}

		return $fields;
	}

    public function convert_subaccount_to_parent($user_id) {
        update_user_meta( $user_id, '__wholesalex_account_type', 'parent' );
        $prev_parent_id = get_user_meta($user_id,'__wholesalex_parent_id',true);
        update_user_meta( $user_id,'__wholesalex_parent_id', '' );
        if(''!==$prev_parent_id) {
            update_user_meta( $user_id,'__wholesalex_prev_parent_id',$prev_parent_id );
        }
        $subaccounts    = get_user_meta( $prev_parent_id, '__wholesalex_subaccount_lists', true );
        if (($key = array_search($user_id, $subaccounts)) !== false) {
            unset($subaccounts[$key]);
        }
        update_user_meta( $prev_parent_id, '__wholesalex_subaccount_lists', $subaccounts );
    }
    public function convert_parent_account_to_subaccount($user_id, $parent_id, $password='', $permissions = '', $phone='', $job_title='') {
		update_user_meta( $user_id, '__job_title', $job_title );
        update_user_meta( $user_id, '__wholesalex_status', 'active' );
        update_user_meta( $user_id, '__phone', $phone );

		$permissions = explode(';',$permissions);
		
        update_user_meta( $user_id, '__wholesalex_permissions', $permissions);
        update_user_meta( $user_id, '__wholesalex_account_type', 'subaccount' );
        update_user_meta( $user_id, '__wholesalex_parent_id', $parent_id );

        $subaccounts    = get_user_meta( $parent_id, '__wholesalex_subaccount_lists', true );
        if ( ! $subaccounts ) {
            $subaccounts = array();
        }
        $subaccounts[] = $user_id;
        update_user_meta( $parent_id, '__wholesalex_subaccount_lists', $subaccounts );
        $parent_user_role = get_user_meta($parent_id, '__wholesalex_role', true);
        wholesalex()->change_role($user_id,$parent_user_role);

		do_action( 'wholesalex_addon_subaccount_created', $user_id, $parent_id, $password );

    }


    public function account_type_column_on_exporter($columns) {
        $columns['wholesalex_account_type'] = __('WholesaleX Account Type','wholesalex-pro');
        $columns['wholesalex_parent_account'] = __('WholesaleX Parent Account','wholesalex-pro');
        $columns['wholesalex_subaccount_phone'] = __('WholesaleX Subaccount: Phone','wholesalex-pro');
        $columns['wholesalex_subaccount_job_title'] = __('WholesaleX Subaccount: Job Title','wholesalex-pro');
        $columns['wholesalex_subaccount_permissions'] = __('WholesaleX Subaccount: Permissions','wholesalex-pro');
        return $columns;
    }

    public function populate_data_on_account_type_column_exporter($data) {
        $user_id = $data['id'];

        $account_type = get_user_meta( $user_id, '__wholesalex_account_type', true );
        if(!$account_type) {
            $account_type = 'parent';
        }

		$permissions = get_user_meta( $user_id, '__wholesalex_permissions', true );
		$data['wholesalex_subaccount_permissions'] = '';
		if($permissions) {
			$data['wholesalex_subaccount_permissions'] = implode(";",$permissions);
		}		
		$data['wholesalex_account_type'] = $account_type;
		$data['wholesalex_parent_account'] = get_user_meta( $user_id, '__wholesalex_parent_id', true );
		
		$phone = get_user_meta($user_id,'__phone',true);
		$data['wholesalex_subaccount_phone']  = '';
		
		if($phone) {
			$data['wholesalex_subaccount_phone'] = $phone;
		}

		$job_title = get_user_meta($user_id,'__job_title', true);
		$data['wholesalex_subaccount_job_title'] = '';
		if($job_title) {
			$data['wholesalex_subaccount_job_title']= $job_title;
		}

        return $data;
    }

    public function process_import($user_id, $data, $mapped_column) {
		if(!isset($data[$mapped_column['wholesalex_account_type']])) {
			return;
		}
        $account_type = $data[$mapped_column['wholesalex_account_type']];
        switch ($account_type) {
            case 'subaccount':
                $prev_account_type = get_user_meta($user_id,'__wholesalex_account_type',true);
                if('subaccount' !== $prev_account_type) {
                    $parent_id = $data[$mapped_column['wholesalex_parent_account']];
					$password = $data[$mapped_column['password']];
					$permissions = $data[$mapped_column['wholesalex_subaccount_permissions']];
					$job_title = $data[$mapped_column['wholesalex_subaccount_job_title']];
					$phone = $data[$mapped_column['wholesalex_subaccount_phone']];
                    if($parent_id) {
                        $this->convert_parent_account_to_subaccount($user_id,$parent_id,$password,$permissions,$phone,$job_title);
                    }
                } else {
					//update subaccount
					$parent_id = $data[$mapped_column['wholesalex_parent_account']];
					$password = $data[$mapped_column['password']];
					$permissions = $data[$mapped_column['wholesalex_subaccount_permissions']];
					$job_title = $data[$mapped_column['wholesalex_subaccount_job_title']];
					$phone = $data[$mapped_column['wholesalex_subaccount_phone']];
					$parent_account_type = get_user_meta($parent_id,'__wholesalex_account_type',true);
                    if($parent_id && 'subaccount'!=$parent_account_type) {
                        $this->update_subaccount_info($user_id,$parent_id,$password,$permissions,$phone,$job_title);
                    }
				}
                break;
            case 'parent':
            case '':
                $prev_account_type = get_user_meta($user_id,'__wholesalex_account_type',true);
                if('subaccount' === $prev_account_type) {
                    $this->convert_subaccount_to_parent($user_id);
                }


                break;
            
            default:
                # code...
                break;
        }
    }

	public function update_subaccount_info($user_id, $parent_id, $password='', $permissions = '', $phone='', $job_title='') {
		$previous_parent_id = get_user_meta($user_id,'__wholesalex_parent_id',true);		
		$subaccount_updated_data = array();
		if ( $job_title != get_user_meta($user_id,'__job_title',true)) {
			update_user_meta( $user_id, '__job_title', $job_title );
			$subaccount_updated_data[] = '__job_title';
		}
		if ( $phone != get_user_meta($user_id,'__phone',true)) {
			update_user_meta( $user_id, '__phone', $phone );
			$subaccount_updated_data[] = '__phone';
		}
		if ( $parent_id != get_user_meta($user_id,'__wholesalex_parent_id',true)) {
			update_user_meta( $user_id, '__wholesalex_parent_id', $parent_id );
			$subaccount_updated_data[] = '__wholesalex_parent_id';
		}
        update_user_meta( $user_id, '__wholesalex_status', 'active' );
		update_user_meta( $user_id, '__wholesalex_account_type', 'subaccount' );

		if($permissions) {
			$permissions = explode(';',$permissions);
			if ( $permissions != get_user_meta($user_id,'__wholesalex_permissions',true)) {
				update_user_meta( $user_id, '__wholesalex_permissions', $permissions );
				$subaccount_updated_data[] = '__wholesalex_permissions';
			}
		}

		//Filter Hook For Getting Updated User Data To Sent Tha Email
		add_filter('wholesalex_profile_updated_data', function( $sub_account_data, $us_id ) use( $subaccount_updated_data, $user_id ) {
			if( $us_id == $user_id ) {
				return array_merge( $sub_account_data , $subaccount_updated_data );
			}
			return $sub_account_data;
		}, 10, 2);
		
        $subaccounts    = get_user_meta( $parent_id, '__wholesalex_subaccount_lists', true );
        if ( ! $subaccounts ) {
            $subaccounts = array();
        }
        $subaccounts[] = $user_id;
        update_user_meta( $parent_id, '__wholesalex_subaccount_lists', $subaccounts );
        $parent_user_role = get_user_meta($parent_id, '__wholesalex_role', true);
        wholesalex()->change_role($user_id,$parent_user_role);

		if(''!==$previous_parent_id) {
            update_user_meta( $user_id,'__wholesalex_prev_parent_id',$previous_parent_id );
        }
        $subaccounts    = get_user_meta( $previous_parent_id, '__wholesalex_subaccount_lists', true );
        if (($key = array_search($user_id, $subaccounts)) !== false) {
            unset($subaccounts[$key]);
        }
        update_user_meta( $previous_parent_id, '__wholesalex_subaccount_lists', $subaccounts );

		$permissions = get_user_meta( $user_id, '__wholesalex_permissions', true );
		if($permissions) {
			$data['wholesalex_subaccount_permissions'] = implode(";",$permissions);
		}

	}

    public function subaccount_text_settings($sections) {
        if ( isset( $sections['language_n_text']['attr'] ) && is_array( $sections['language_n_text']['attr'] ) ) {
			$__fields = array(
				'_language_subaccount_review_the_order_text'     => array(
					'type'        => 'text',
					'label'       => __( 'WholesaleX Subaccount Review Order Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __('Please review the order and accept or reject it using the following buttons.','wholesalex-pro'),
				),
				'_language_subaccount_approve_order_text' => array(
					'type'        => 'text',
					'label'       => __( 'WholesaleX Subaccount Approve Order Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Approve', 'wholesalex-pro' ),
				),
				'_language_subaccount_reject_order_text'              => array(
					'type'        => 'text',
					'label'       => __( 'WholesaleX Subaccount Reject Order Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __('Reject','wholesalex-pro'),
				)
			);

			$sections['language_n_text']['attr'] = array_merge( $sections['language_n_text']['attr'], $__fields );
		}

		$subaccount_sections= array(
            'subaccounts' => array(
                'label' => __( 'Subaccounts', 'wholesalex-pro' ),
                'attr'  => array(
                    '_subaccount_my_account_endpoint' => array(
						'type'        => 'text',
						'label'       => __( 'Subaccount My Account Page Endpoint', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => 'whx-subaccount' ,
					),
					'_subaccount_my_account_endpoint_title'     => array(
						'type'        => 'text',
						'label'       => __( 'Subaccount My Account Endpoint Title', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Subaccounts', 'wholesalex-pro' ),
					),
					'subaccount_order_approval_need_status_label'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Subaccount Order Approval Need Status Label', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => _x( 'Main Account Approval Needed', 'Order status', 'woocommerce' ),
                    ),
                ),
            )
           
        );

        $sections = wholesalex()->insert_into_array( $sections, $subaccount_sections);

        return $sections;
    }


	/**
	 * Set Parent ID as customer id if the order is placed by subaccount
	 * If the order is placed by a subaccount, and current user is parent account of the subaccount
	 * then we set parent account id as customer id for this order, so that parent account can pay this order
	 *
	 * @return string customer id.
	 * @since 1.2.3
	 */
	public function set_parent_id_as_subaccount_order_customer_id($customer_id) {

		if ( is_checkout() &&  is_wc_endpoint_url('order-received') ) {
			
			$__account_type = get_user_meta( $customer_id, '__wholesalex_account_type', true );
			
			if('subaccount' === $__account_type) {
				$parent_id = get_user_meta( $customer_id, '__wholesalex_parent_id', true );
				if($parent_id) {
					$customer_id = $parent_id;
				}
			}
		}
				
		return absint( $customer_id);
	}

	/**
	 * Add Subaccounts columns on Role Exporter Columns
	 *
	 * @param array $columns Role Exporter Columns
	 * @return void
	 */
	public function add_subaccount_fields_on_role_exporter($columns) {
		$columns['_subaccount_status'] = __('Allow Subaccount Creation For this role','wholesalex-pro');
		$columns['_exclude_subaccounts'] = __('Exclude Users to Create Subaccount','wholesalex-pro');
		return $columns;
	}

	/**
	 * Get Role
	 *
	 * @param [type] $role
	 * @return void
	 */
	public function get_column_value__subaccount_status($value,$role) {
		
		return isset($role['_subaccount_status'])?$role['_subaccount_status']:'';
	}
	/**
	 * Get Role
	 *
	 * @param [type] $role
	 * @return void
	 */
	public function get_column_value__exclude_subaccounts($value,$role) {
		
		if(isset($role['_exclude_subaccounts']) && is_array($role['_exclude_subaccounts'])) {
			$data = [];
			foreach ($role['_exclude_subaccounts'] as $user) {
				$data[]=$user['value'];
			}
			return implode(';',$data);
		}
		return $value;
	}


	public function set_import_column_value($options) {
		if(!isset($options['_subaccount_status'])) {
			$options['_subaccount_status'] =  __('Allow Subaccount Creation For this role','wholesalex-pro');
		}
		if(!isset($options['_exclude_subaccounts'])) {
			$options['_exclude_subaccounts'] =   __('Exclude Users to Create Subaccount','wholesalex-pro');
		}

		return $options;

	}


	public function set_import_mapping_default_column($columns) {
		if(!isset($columns[ __('Allow Subaccount Creation For this role','wholesalex-pro')])) {
			$columns[ __('Allow Subaccount Creation For this role','wholesalex-pro')] = '_subaccount_status';
		}
		if(!isset($columns[  __('Exclude Users to Create Subaccount','wholesalex-pro')])) {
			$columns[  __('Exclude Users to Create Subaccount','wholesalex-pro')] = '_exclude_subaccounts';
		}

		return $columns;
	}


	public function role_importer_parse_subaccount_data($data) {
		if(isset($data['_exclude_subaccounts'])) {
			$data['_exclude_subaccounts'] = $this->parse_exclude_subaccounts_field($data['_exclude_subaccounts']);
		}
		
		return $data;
	}

	public function parse_exclude_subaccounts_field($value) {
		$data = array();
		if(!empty($value) && $value) {
			$value  = str_replace( '\\,', '::separator::', $value );
			$values = explode( ';', $value );
			$values = array_map( array( $this, 'explode_values_formatter' ), $values );

			foreach ($values as $user) {
				if(!is_numeric($user)) {
					$id = explode('_',$user);
					if(isset($id[1])) {
						$id = $id[1];
					}
				}
				$user_obj = get_user_by('id', $id);
				if($user_obj) {
					$data[]=array('name'=>$user_obj->user_login, 'value'=>'user_'.$id);
				}
 
			}
		}

		return $data;
	}


	/**
	 * Remove formatting and trim each value.
	 *
	 * @since  1.2.5
	 * @param  string $value Value to format.
	 * @return string
	 */
	public function explode_values_formatter( $value ) {
		return trim( str_replace( '::separator::', ';', $value ) );
	}
	
	public function get_endpoint_title() {
		$menu_title = apply_filters( 'wholesalex_addon_subaccount_my_account_menu_title', __( 'Subaccounts', 'wholesalex-pro' ) );
		return $menu_title;
	}

	public function get_endpoint() {
		$menu_endpoint = apply_filters('wholesalex_addon_subaccount_my_account_menu_endpoint','wsx-subaccount');
		return $menu_endpoint;
	}


	public function subaccount_menu_title($title) {
		if(''!=wholesalex()->get_setting('_subaccount_my_account_endpoint_title','Subaccounts')) {
			$title = wholesalex()->get_setting('_subaccount_my_account_endpoint_title','Subaccounts');
		}
		return $title;
	}

	public function subaccount_menu_endpoint($endpoint) {
		if(''!=wholesalex()->get_setting('_subaccount_my_account_endpoint','wsx-subaccount')) {
			$endpoint = wholesalex()->get_setting('_subaccount_my_account_endpoint','wsx-subaccount');
		}
		return $endpoint;
	}


	public function subaccount_approval_need_order_status($label) {

        if(''!=wholesalex()->get_setting('subaccount_order_approval_need_status_label',_x( 'Main Account Approval Needed', 'Order status', 'woocommerce' ))) {
            $label = wholesalex()->get_setting('subaccount_order_approval_need_status_label',_x( 'Main Account Approval Needed', 'Order status', 'woocommerce' ) );
        }
        return $label;
    }
}
