<?php
/**
 * Class wholesalex new user Registration admin notification file.
 *
 * @package WHOLESALEX
 * @since 1.2.3
 */

namespace WHOLESALEX_PRO;

use WHOLESALEX\WHOLESALEX_Email_Manager;

use WC_Email;
use WP_User;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}


/**
 * Customer New Account.
 *
 * An email sent to the wholesalex customer when they create an account.
 *
 * @extends     WC_Email
 */
class WholesaleX_Subaccount_Order_Approved_Email extends WC_Email {

    /**
     * User login name.
     *
     * @var string
     */
    public $user_login;

    /**
     * User email.
     *
     * @var string
     */
    public $user_email;

    /**
     * User password.
     *
     * @var string
     */
    public $user_pass;

    /**
     * User parent.
     *
     * @var string
     */
    public $parent_user_id;

    /**
     * Is the password generated?
     *
     * @var bool
     */
    public $password_generated;

    /**
     * Magic link to verify email.
     *
     * @var string
     */
    public $confirmation_url;

    public $parent_email;

    /**
     * Constructor.
     */
    public function __construct() {
        $this->customer_email = true;
        $this->id             = 'wholesalex_subaccount_order_approved';
        $this->title          = __( 'WholesaleX: Subaccount Order Approved', 'wholesalex-pro' );
        $this->description    = __( 'WholesaleX: Subaccount Order Approved emails are sent to the subaccount holder who create this order', 'wholesalex-pro' );
        $this->template_base  = WHOLESALEX_PRO_PATH.'addons/subaccount/emails/templates/';
        $this->template_html  = 'wholesalex-subaccount-order-approved.php';
        $this->placeholders = apply_filters('wholesalex_subaccount_order_approved_smart_tags',array(
            '{subaccount_name}'=>$this->user_login,
            '{order_date}'=>'',
            '{order_number}'=>'',
            '{order_billing_full_name}'=>'',
            '{view_order}'=>'',
            '{admin_name}'=>'',
            '{date}'=>'',
            '{site_name}'=>'',
            '{subaccount_email}'=>'',
    ));

        // Call parent constructor.
        parent::__construct();


        // add_action( 'wholesalex_subaccount_order_notification', array( $this, 'trigger'), 10, 3 );
    }

    /**
     * Get email subject.
     *
     * @since  1.2.3
     * @return string
     */
    public function get_default_subject() {
        return __( 'Your Order #{order_number} Has Been Approved', 'wholesalex-pro' );
    }

    /**
     * Get email heading.
     *
     * @since  1.2.3
     * @return string
     */
    public function get_default_heading() {
        return __( 'Your Order #{order_number} Has Been Approved', 'wholesalex-pro' );
    }

    /**
     * Trigger the sending of this email.
     *
     * @param int            $order_id The order ID.
     * @param WC_Order|false $order Order object.
     */
    public function trigger( $order_id, $customer_id='',$parent_id='' ) {
        $this->setup_locale();

        if ( $order_id  ) {
            $order = wc_get_order( $order_id );
        }

        if ( is_a( $order, 'WC_Order' ) ) {
            $this->object                                    = $order;
            $this->placeholders['{order_date}']              = wc_format_datetime( $this->object->get_date_created() );
            $this->placeholders['{order_number}']            = $this->object->get_order_number();
            $this->placeholders['{order_billing_full_name}'] = $this->object->get_formatted_billing_full_name();
            $this->placeholders['{view_order}'] = $this->object->get_view_order_url();             
            if ( $customer_id ) {
    
                $user = new WP_User( $customer_id );
                
                $this->user_login         = stripslashes( $user->user_login );
                $this->user_email         = stripslashes( $user->user_email );
                $this->parent_user_id=$parent_id;
                $parent = get_user_by('ID',$this->parent_user_id);
                $this->recipient          = $this->user_email;
                $this->placeholders['{subaccount_name}'] = $this->user_login;
                $this->placeholders['{admin_name}'] = WHOLESALEX_Email_Manager::get_admin_display();             
                $this->placeholders['{date}'] = gmdate( 'd-m-Y' );             
                $this->placeholders['{site_name}'] = get_option( 'blogname' );             
                $this->placeholders['{subaccount_email}'] = $this->user_email;   
            }
    
            
    
            if ( $this->is_enabled() && $this->get_recipient() ) {
                $this->send( $this->get_recipient(), $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments() );
            }
        }


        $this->restore_locale();
    }

    /**
     * Get content html.
     *
     * @return string
     */
    public function get_content_html() {
        return wc_get_template_html(
            $this->template_html,
            array(
                'email_heading'      => $this->get_heading(),
                'additional_content' => $this->get_additional_content(),
                'user_login'         => $this->user_login,
                'user_pass'          => $this->user_pass,
                'parent_id'          => $this->parent_user_id,
                'blogname'           => $this->get_blogname(),
                'sent_to_admin'      => false,
                'plain_text'         => false,
                'email'              => $this,
                'order'              => $this->object,
            ),
            $this->template_base, $this->template_base
        );
    }

    /**
     * Default content to show below main email content.
     *
     * @since 3.7.0
     * @return string
     */
    public function get_default_additional_content() {
        return __( '', 'wholesalex-pro' );
    }

    /**
 * Initialise Settings Form Fields - these are generic email options most will use.
 */
public function init_form_fields() {
    /* translators: %s: list of placeholders */
    $placeholder_text  = sprintf( __( 'Available placeholders: %s', 'wholesalex-pro' ), '<code>' . esc_html( implode( '</code>, <code>', array_keys( $this->placeholders ) ) ) . '</code>' );
    $this->form_fields = array(
        'enabled'            => array(
            'title'   => __( 'Enable/Disable', 'wholesalex-pro' ),
            'type'    => 'checkbox',
            'label'   => __( 'Enable this email notification', 'wholesalex-pro' ),
            'default' => 'yes',
        ),
        'subject'            => array(
            'title'       => __( 'Subject', 'wholesalex-pro' ),
            'type'        => 'text',
            'desc_tip'    => true,
            'description' => 'This control the email subject. To change default email subject, makes changes here.',
            'placeholder' => $this->get_default_subject(),
            'default'     => '',
        ),
        'heading'            => array(
            'title'       => __( 'Email heading', 'wholesalex-pro' ),
            'type'        => 'text',
            'desc_tip'    => true,
            'description' => 'This control the email heading. To change default email heading, makes changes here.',
            'placeholder' => $this->get_default_heading(),
            'default'     => '',
        ),
        'additional_content' => array(
            'title'       => __( 'Additional content', 'wholesalex-pro' ),
            'description' => __( 'Text to appear below the main email content.', 'wholesalex-pro' ) . ' ' . $placeholder_text,
            'css'         => 'width:400px; height: 75px;',
            'placeholder' => __( 'N/A', 'wholesalex-pro' ),
            'type'        => 'textarea',
            'default'     => $this->get_default_additional_content(),
            'desc_tip'    => true,
        ),
        'email_type'         => array(
            'title'       => __( 'Email type', 'wholesalex-pro' ),
            'type'        => 'select',
            'description' => __( 'Choose which format of email to send.', 'wholesalex-pro' ),
            'default'     => 'html',
            'class'       => 'email_type wc-enhanced-select',
            'options'     => array(
                'html'      => __( 'HTML', 'woocommerce' ),
                'multipart' => __( 'Multipart', 'woocommerce' )
            ),
            'desc_tip'    => true,
        ),
    );
}
}



return new WholesaleX_Subaccount_Order_Approved_Email();
