<?php

/**
 * Subaccount Addon Init
 *
 * @package WHOLESALEX
 * @since 1.1.0
 */

use WHOLESALEX_PRO\WHOLESALEX_PRO_Initialization;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

add_action( 'init', 'wholesalex_subaccount_init' );

/**
 * WholesaleX Subaccount Init
 *
 * @return void
 */
function wholesalex_subaccount_init() {
	$is_enable = wholesalex()->get_setting( 'wsx_addon_subaccount' );

	if ( 'yes' === $is_enable ) {
		register_approval_needed_status();
		if ( ! wp_script_is( 'wholesalex_subaccount', 'registered' ) ) {
			WHOLESALEX_PRO_Initialization::register_scripts();
		}
		require_once WHOLESALEX_PRO_PATH . 'addons/subaccount/SubAccount.php';
		require_once WHOLESALEX_PRO_PATH . 'addons/subaccount/ParentApprovalGateway.php';
		if ( is_eligible_for_subaccount() || is_admin() || current_user_can( 'manage_options' ) ) {
			new \WHOLESALEX_PRO\SubAccount();
		}
	}
}


/**
 * Register quote order status.
 *
 * @return void
 * @since 1.0.5
 */
function register_approval_needed_status() {
	$approval_needed_post_status_label = apply_filters( 'wholesalex_subaccount_addon_main_account_approval_needed_status_label', _x( 'Main Account Approval Needed', 'Order status', 'wholesalex-pro' ) );
	register_post_status(
		'wc-whx-approval-need',
		array(
			'label'                     => $approval_needed_post_status_label,
			'public'                    => false,
			'exclude_from_search'       => false,
			'show_in_admin_all_list'    => true,
			'show_in_admin_status_list' => true,
			/* translators: %s: number of orders */
			'label_count'               => _n_noop( $approval_needed_post_status_label . ' <span class="count">(%s)</span>', $approval_needed_post_status_label . ' <span class="count">(%s)</span>', 'wholesalex-pro' ), //phpcs:ignore
		)
	);
}

/**
 * Check current user is eligible for subaccount features or not.
 *
 * @return boolean
 * @since 1.0.9
 */
function is_eligible_for_subaccount() {
	$status        = false;
	$__user_id     = get_current_user_id();
	$_account_type = get_user_meta( $__user_id, '__wholesalex_account_type', true );
	if ( 'subaccount' === $_account_type ) {
		$__user_id = get_user_meta( $__user_id, '__wholesalex_parent_id', true );
	}

	$__roles = array_values( wholesalex()->get_roles() );
	if ( empty( $__roles ) ) {
		$__roles = array(
			array(
				'id'    => 1,
				'label' => 'New Role',
			),
		);
	}
	$__user_role = get_user_meta( $__user_id, '__wholesalex_role', true );

	foreach ( $__roles as $role ) {
		if ( ! isset( $role['_subaccount_status'] ) ) {
			continue;
		}
		if ( $__user_role === $role['id'] ) {
			if ( 'yes' === $role['_subaccount_status'] ) {
				// check is exclude.
				$status = true;

				if ( is_array( $role['_exclude_subaccounts'] ) && ! empty( $role['_exclude_subaccounts'] ) ) {
					foreach ( $role['_exclude_subaccounts'] as $user ) {
						if ( 'user_' . $__user_id === $user['value'] ) {
							$status = false;
						}
					}
				}
			}
		}
	}

	return $status;
}

add_action( 'woocommerce_blocks_loaded', function(){
    $is_enable = wholesalex_pro()->get_setting( 'wsx_addon_subaccount' );
        if (  'yes' === $is_enable && class_exists( 'Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType' ) ) {
            require_once WHOLESALEX_PRO_PATH . 'addons/subaccount/ParentApprovalGatewayBlock.php';
            add_action(
                'woocommerce_blocks_payment_method_type_registration',
                function( Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry ) {
                    $payment_method_registry->register( new \WHOLESALEX_PRO\WholesaleX_Parent_Approval_Gateway() );
                }
            );
        }
} );
