<?php
/**
 * WholesaleX Wallet Manage
 *
 * @package WHOLESALEX
 * @since 1.0.0
 */

namespace WHOLESALEX_PRO;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}
/**
 * WholesaleX Wallet Mange
 */
class WHOLESALEX_Wallet_Manage {

	/**
	 * TopUP Ammount
	 *
	 * @var integer
	 */
	public $topup_amount = 0;

	/**
	 * Construtor
	 */
	public function __construct() {
		add_action( 'wp_loaded', array( $this, 'plugin_loaded_topup_product_add' ) );

		add_filter( 'wholesalex_addon_wallet_menu_item_text', array( $this, 'wallet_menu_item_text_change' ) );

		add_filter( 'wholesalex_addon_wallet_my_account_menu_endpoint', array( $this, 'wallet_menu_item_endpoint_change' ) );

		if ( $this->is_eligible_for_wallet() ) {
			add_filter( 'woocommerce_account_menu_items', array( $this, 'wholesalex_wallet_menu_item' ) );
			add_action( 'init', array( $this, 'wholesalex_wallet_menu_item_add_end_point' ) );
			add_filter( 'woocommerce_get_query_vars', array( $this, 'wholesalex_add_woocommerce_query_vars' ) );
			add_action( 'woocommerce_account_' . $this->get_endpoint() . '_endpoint', array( $this, 'wholesalex_wallet_page' ) );
			add_action( 'woocommerce_cart_loaded_from_session', array( $this, 'woocommerce_cart_loaded_from_session' ) );
			add_filter( 'woocommerce_is_purchasable', array( $this, 'make_topup_product_purchasable' ), 10, 2 );
			add_action( 'woocommerce_before_calculate_totals', array( $this, 'set_topup_amount_as_product_price' ) );
			add_filter( 'woocommerce_add_to_cart_validation', array( $this, 'restrict_other_items_from_add_to_cart' ) );
			add_filter( 'woocommerce_available_payment_gateways', array( $this, 'topup_available_payment_gateways' ) );
			add_action( 'woocommerce_order_status_completed', array( $this, 'process_topup' ), 999999999 );
			add_action( 'woocommerce_order_status_refunded', array( $this, 'process_topup_refund' ), 999999999 );
			add_filter( 'wholesalex_profile_setting_data', array( $this, 'process_transaction_profile_page' ), 10, 2 );
			add_filter( 'wholesalex_get_user_settings_data', array( $this, 'set_credit_balance_profile_page' ), 10, 2 );
			add_action( 'wholesalex_user_role_updated', array( $this, 'process_transaction_after_role_change' ), 10, 3 );
			add_filter( 'woocommerce_payment_gateways', array( $this, 'load_payment_gateway' ) );
			add_action( 'rest_api_init', array( $this, 'wallet_restapi_callback' ) );
			add_action( 'woocommerce_review_order_after_order_total', array( $this, 'add_partial_payment_checkbox' ) );

			add_filter( 'woocommerce_checkout_order_processed', array( $this, 'process_partial_payment' ) );

			add_filter( 'woocommerce_order_status_cancelled', array( $this, 'process_partial_payment_cancelled_order' ) );
			add_filter( 'woocommerce_after_order_fee_item_name', array( $this, 'add_wallet_partial_payment_refund_button' ), 10, 2 );

			add_filter( 'woocommerce_available_payment_gateways', array( $this, 'remove_wallet_balance_gateway' ), 999999999 );
			add_action( 'woocommerce_cart_calculate_fees', array( $this, 'wholesalex_wallet_add_partial_payment_fee' ) );

			add_filter( 'woocommerce_cart_totals_get_fees_from_cart_taxes', array( $this, 'remove_partial_payment_tax' ), 10, 2 );


		}

		add_filter( 'wholesalex_role_fields', array( $this, 'add_credit_field_in_role' ), 11 );
		add_filter( 'wholesalex_profile_fields', array( $this, 'add_credit_field_in_profile' ), 11 );
		add_filter( 'wholesalex_role_table_columns', array( $this, 'add_credit_column' ), 10 );

		add_filter( 'wholesalex_setting_fields', array( $this, 'wallet_settings_field' ) );

		add_filter( 'wholesalex_users_columns', array( $this, 'wallet_balance_column_on_wholesalex_users' ) );

		add_filter( 'wholesalex_users_column_value', array( $this, 'wallet_balance_field_value_on_wholesalex_users' ), 10, 2 );

		add_filter( 'wholesalex_user_export_default_columns', array( $this, 'wallet_balance_column_on_exporter' ) );

		add_filter( 'wholesalex_user_export_column_data', array( $this, 'populate_data_on_balance_column_exporter' ) );

		add_action( 'wholesalex_import_userdata', array( $this, 'process_import' ), 10, 3 );

		add_filter( 'wholesalex_wallet_partial_payment_status', array( $this, 'wallet_partial_payment_status' ) );

		add_action( 'wp_ajax_wholesalex_partial_payment_update_session', array( $this, 'update_wallet_partial_pay_in_session' ) );

		add_filter( 'wholesalex_wallet_payment_gateway_available', array( $this, 'make_unavailable_wallet_payment_gateway' ) );

		// add_filter('woocommerce_update_cart_action_cart_updated', array($this, 'remove_partial_payment_after_cart'));

		add_action( 'wp_ajax_wholesalex_wallet_partial_payment_refund', array( $this, 'wholesalex_wallet_partial_payment_refund' ) );

		add_filter( 'wholesalex_role_exporter_default_columns', array( $this, 'add_wallet_fields_on_role_exporter' ) );

		add_filter( 'wholesalex_role_export_role_column_credit_limit', array( $this, 'get_column_value_credit_limit' ), 10, 2 );
		add_filter( 'wholesalex_role_export_role_column_disable_wallet', array( $this, 'get_column_value_disable_wallet' ), 10, 2 );

		add_filter( 'wholesalex_csv_role_import_mapping_options', array( $this, 'set_import_column_value' ) );
		add_filter( 'wholesalex_csv_role_import_mapping_default_columns', array( $this, 'set_import_mapping_default_column' ) );

		add_filter( 'wholesalex_role_importer_parsed_data', array( $this, 'role_importer_parse_wallet_data' ) );

        /**
         * Register a shortcode for show wallet available balance
         * @since 1.2.9
         */
        add_shortcode( 'wsx_wallet_balance', array($this,'wallet_available_balance_shortcode') );

        add_action( 'admin_enqueue_scripts',array($this,'enqueue_wallet_admin_scripts'));

        add_action( 'wp_ajax_wholesalex_wallet_process_refund', array($this,'wholesalex_wallet_process_refund'));

        add_filter( 'rest_authentication_errors', array($this,'make_restapi_whitelist'), 999999999999 );


	}

    /**
     * Check current user is eligible for wallet features or not.
     *
     * @return boolean
     * @since 1.1.10
     */
    public function is_eligible_for_wallet() {
        $status        = true;
        $__user_id     = get_current_user_id();
        
        $__roles = array_values( wholesalex()->get_roles() );
        if ( empty( $__roles ) ) {
            $__roles = array(
                array(
                    'id'    => 1,
                    'label' => 'New Role',
                ),
            );
        }
        $__user_role = get_user_meta( $__user_id, '__wholesalex_role', true );

        foreach ( $__roles as $role ) {
            if ( $__user_role === $role['id'] && isset( $role['disable_wallet'] ) && 'yes' === $role['disable_wallet'] ) {
                $status=false;
            }
            
        }

        return $status;
    }


	/**
	 * Wallet Rest API Callback
	 *
	 * @since 1.0.0
	 * @since 1.0.9 wholesalex_addon_wallet_endpoint_access filter added.
	 */
	public function wallet_restapi_callback() {
		register_rest_route(
			'wholesalex/v1',
			'/wallet_action/',
			array(
				array(
					'methods'             => 'POST',
					'callback'            => array( $this, 'wallet_restapi_action' ),
					'permission_callback' => function () {
						$__user_id = get_current_user_id();
						$__account_status = get_user_meta( $__user_id, '__wholesalex_status', true );
						$has_access = apply_filters( 'wholesalex_addon_wallet_endpoint_access', true, $__user_id );
						return ( ( 'active' === $__account_status && $has_access ) || current_user_can( 'manage_options' ) );
					},
					'args'                => array(),
				),
			)
		);
	}

	/**
	 * Get Wallet actions
	 *
	 * @param object $server Server.
	 * @return void
	 * @since 1.0.0
	 */
	public function wallet_restapi_action( $server ) {

		$post = $server->get_params();
		if ( ! ( isset( $post['nonce'] ) && wp_verify_nonce( $post['nonce'], 'wholesalex-registration' ) ) ) {
			return;
		}

		$type = isset( $post['type'] ) ? sanitize_text_field( $post['type'] ) : '';

		if ( 'get' == $type ) {
			wp_send_json_success();
		} elseif ( 'post' == $type ) {
			if ( isset( $post['amount'] ) && ! empty( $post['amount'] ) ) {
				$this->topup_amount = floatval( sanitize_text_field( $post['amount'] ) );
			}
			if ( isset( $post['is_request'] ) && $post['is_request'] ) {
				if ( 'yes' === wholesalex()->get_setting( 'wsx_addon_wallet' ) ) {
					$user_id        = get_current_user_id();
					$parent_user_id = get_user_meta( $user_id, '__wholesalex_parent_id', true );

					$status = wholesalex_wallet()->send_balance_request( get_current_user_id(), $parent_user_id, $this->topup_amount, __('Request From Subaccount to Master Account','wholesalex-pro') );
					if ( $status ) {
						do_action( 'wholesalex_wallet_subaccount_balance_request_send', $user_id, $parent_user_id, $this->topup_amount );
					}
					wp_send_json_success( $status );
				}
			} else {
				$_topup_product_id = get_option( '__wholesalex_wallet_topup_product' );

				// First Load WC Cart.
				wc_load_cart();

				// add to cart top up product.
				wc()->cart->empty_cart();
				wc()->cart->add_to_cart( $_topup_product_id, 1, 0, array(), array( 'topup_amount' => $this->topup_amount ) );
				wp_send_json_success();
			}
		} elseif ( 'wallet_request' === $type ) {
			$request_type = isset( $post['requestType'] ) ? sanitize_text_field( $post['requestType'] ) : '';
			$request_id   = isset( $post['requestId'] ) ? sanitize_text_field( $post['requestId'] ) : '';
			$status       = array();
			switch ( $request_type ) {
				case 'approve':
					$status = wholesalex_wallet()->approve_balance_request( $request_id );
					if ( is_array( $status ) ) {
						$status['fields'] = $this->get_wallet_page_fields();
					}
					break;
				case 'reject':
					$status = wholesalex_wallet()->reject_balance_request( $request_id );
					if ( is_array( $status ) ) {
						$status['fields'] = $this->get_wallet_page_fields();
					}
					break;

				default:
					break;
			}

			wp_send_json_success( $status );
		}
	}

	/**
	 * Add Credit Field In Role Page
	 *
	 * @param array $fields Role Fields.
	 * @return array $fields.
	 * @since 1.0.0
	 * @since 1.0.4 Excludes option and wholesalex_credit_limit_fields_exclude_from Hook added.
	 */
	public function add_credit_field_in_role( $fields ) {
		$credit_field            = array(
			'credit_limit_section' => array(
				'label' => '',
				'type'  => 'credit_limit',
				'_id'   => 1,
				'attr'  => array(
					'credit_limit' => array(
						'type'        => 'number',
						'label'       => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_credit_limit' : wholesalex()->get_setting( '_language_credit_limit' ), __( 'Credit Limit', 'wholesalex-pro' ) ),
						'placeholder' => __( '100', 'wholesalex-pro' ),
						'default'     => '',
						'help'        => '',
						'excludes'    => apply_filters( 'wholesalex_credit_limit_fields_exclude_from', array( 'wholesalex_guest' ) ),
					),
				),
			),
		);
		$fields['_role']['attr'] = wholesalex()->insert_into_array( $fields['_role']['attr'], $credit_field, 2 );

		$wallet_status_fields = array(
			'disable_wallet' => array(
				'type'    => 'switch',
				'label'   => __( 'Disable Wallet Addon for This Role', 'wholesalex-pro' ),
				'desc'    => __( 'Disable', 'wholesalex-pro' ),
				'help'    => '',
				'default' => 'no',
			),
		);

		$fields['_role']['attr']['settings_section']['attr'] = wholesalex()->insert_into_array( $fields['_role']['attr']['settings_section']['attr'], $wallet_status_fields, 5 );
		return $fields;
	}
	/**
	 * Add Credit Fields in Profile
	 *
	 * @param array $fields Profile Fields.
	 * @return array $fields.
	 * @since 1.0.0
	 */
	public function add_credit_field_in_profile( $fields ) {
		$credit_field                        = array(
			'_credit_section' => array(
				'label' => '',
				'attr'  => array(
					'_wholesalex_add_credit'     => array(
						'type'        => 'number',
						'label'       => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_wholesalex_add_credit' : wholesalex()->get_setting( '_language_wholesalex_add_credit' ), __( 'Credit/Debit', 'wholesalex-pro' ) ),
						'default'     => '',
						'placeholder' => '',
						'help'        => 'For Credit Use Positive Value and For Debit Use Negative Value',
					),
					'_wholesalex_credit_balance' => array(
						'type'        => 'number',
						'label'       => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_wholesalex_credit_balance' : wholesalex()->get_setting( '_language_wholesalex_credit_balance' ), __( 'Credit Balance', 'wholesalex-pro' ) ),
						'default'     => '',
						'placeholder' => '',
						'help'        => '',
						'is_disable'  => true,
					),
				),
			),
		);
		$fields['_profile_settings']['attr'] = wholesalex()->insert_into_array( $fields['_profile_settings']['attr'], $credit_field, 2 );
		return $fields;
	}

	/**
	 * Add Credit Column In Role Table
	 *
	 * @param array $columns Role Table Columns.
	 * @return array $columns.
	 * @since 1.0.0
	 */
	public function add_credit_column( $columns ) {
		$columns['credit_limit'] = __( 'Credit Limit', 'wholesalex-pro' );
		return $columns;
	}

	/**
	 * WooCommerce wallet payment gateway loader
	 *
	 * @param array $load_gateways All WooCommerce Payment Gateways.
	 * @return array $load_gateways Updated WooCommerce Payment Gateways.
	 */
	public function load_payment_gateway( $load_gateways ) {
		$load_gateways[] = 'WHOLESALEX_PRO\WHOLESALEX_Payment_Gateway';
		return $load_gateways;
	}

	/**
	 * Create topup product
	 *
	 * @return void
	 */
	public function plugin_loaded_topup_product_add() {
		if ( ! wp_doing_ajax() ) {
			$this->set_wallet_partial_payment_in_session( false );
		}

		$topup_product = wc_get_product( get_option( '__wholesalex_wallet_topup_product' ) );

		if ( ! get_option( '__wholesalex_wallet_topup_product' ) || ! $topup_product ) {
			$this->create_topup_product();
		}
	}

	/**
	 * Process TopUP
	 *
	 * @param int $order_id Topup Ammount Order ID.
	 * @since 1.0.0
	 * @since 1.0.9 add wholesalex_wallet_user_id filter.
	 */
	public function process_topup( $order_id ) {
		$topup_product_id = get_option( '__wholesalex_wallet_topup_product' );
		if ( ! $topup_product_id ) {
			return;
		}
		$order          = wc_get_order( $order_id );
		$items          = $order->get_items();
		$is_topup_order = false;
		foreach ( $items as $item ) {
			$product_id = $item->get_product_id();
			if ( intval( $product_id ) !== intval( $topup_product_id ) ) {
				return;
			}
			if ( intval( $product_id ) === intval( $topup_product_id ) ) {
				$is_topup_order = true;
			}
		}
		if ( ! $is_topup_order ) {
			return;
		}

		$topup_amount = $order->get_subtotal( 'edit' );
		$user_id      = apply_filters( 'wholesalex_wallet_user_id', $order->get_customer_id() );

		$transaction_id = wholesalex_wallet()->initiate_transaction( $user_id, $topup_amount, 'credit', apply_filters( 'wholesalex_addon_wallet_topup_description', __( 'TopUP Through #', 'wholesalex-pro' ) . $order->get_order_number(), $order ) );
		update_post_meta(
			$order_id,
			'_wholesalex_topup_success',
			array(
				'status'         => 'success',
				'transaction_id' => $transaction_id,
			)
		);
	}

	/**
	 * Process TopUP Refund
	 *
	 * @param int $order_id Order_ID.
	 * @return void
	 * @since 1.0.0
	 * @since 1.0.9 add wholesalex_wallet_user_id filter.
	 */
	public function process_topup_refund( $order_id ) {
		$topup_product_id = get_option( '__wholesalex_wallet_topup_product' );
		if ( ! $topup_product_id ) {
			return;
		}
		$_topup_success = get_post_meta( $order_id, '_wholesalex_topup_success', true );
		$_topup_refund  = get_post_meta( $order_id, '_wholesalex_topup_refund', true );

		if ( ! $_topup_success || $_topup_refund ) {
			return;
		}
		if ( 'success' !== $_topup_success['status'] ) {
			return;
		}
		$order = wc_get_order( $order_id );

		$refund_amount  = $order->get_subtotal( 'edit' );
		$user_id        = apply_filters( 'wholesalex_wallet_user_id', $order->get_customer_id() );
		$transaction_id = wholesalex_wallet()->initiate_transaction( $user_id, $refund_amount, 'debit', __( 'Refund For #', 'wholesalex-pro' ) . $order->get_order_number() );
		update_post_meta(
			$order_id,
			'_wholesalex_topup_refund',
			array(
				'status'         => 'success',
				'transaction_id' => $transaction_id,
			)
		);
	}

	/**
	 * Create a rechargeable product for topup wholesalex balance
	 */
	private function create_topup_product() {
		$_is_exist = get_option( '__wholesalex_wallet_topup_product' );

		if ( $_is_exist ) {
			$product = wc_get_product( $_is_exist );
			if ( ! $product ) {
				$_is_exist = 0;
			}
		}
		if ( ! $_is_exist ) {
			$product_attr = array(
				'post_title'   => wc_clean( 'WholesaleX Wallet TopUP' ),
				'post_status'  => 'private',
				'post_type'    => 'product',
				'post_excerpt' => '',
				'post_content' => 'WholesaleX Auto Generate Product For TopUP WholesaleX Balance',
				'post_author'  => get_current_user_id(),
			);
			$product_id   = wp_insert_post( $product_attr );
			if ( ! is_wp_error( $product_id ) ) {
				$product = wc_get_product( $product_id );
				wp_set_object_terms( $product_id, 'simple', 'product_type' );
				update_post_meta( $product_id, '_stock_status', 'instock' );
				update_post_meta( $product_id, 'total_sales', '0' );
				update_post_meta( $product_id, '_downloadable', 'no' );
				update_post_meta( $product_id, '_virtual', 'yes' );
				update_post_meta( $product_id, '_regular_price', '' );
				update_post_meta( $product_id, '_sale_price', '' );
				update_post_meta( $product_id, '_purchase_note', '' );
				update_post_meta( $product_id, '_featured', 'no' );
				update_post_meta( $product_id, '_weight', '' );
				update_post_meta( $product_id, '_length', '' );
				update_post_meta( $product_id, '_width', '' );
				update_post_meta( $product_id, '_height', '' );
				update_post_meta( $product_id, '_sku', 'wholesalex_wallet_topup' );
				update_post_meta( $product_id, '_product_attributes', array() );
				update_post_meta( $product_id, '_sale_price_dates_from', '' );
				update_post_meta( $product_id, '_sale_price_dates_to', '' );
				update_post_meta( $product_id, '_price', '' );
				update_post_meta( $product_id, '_sold_individually', 'yes' );
				update_post_meta( $product_id, '_manage_stock', 'no' );
				update_post_meta( $product_id, '_backorders', 'no' );
				update_post_meta( $product_id, '_stock', '' );
				$product->set_reviews_allowed( false );
				$product->set_catalog_visibility( 'hidden' );
				$product->save();

				update_option( '__wholesalex_wallet_topup_product', $product_id );
			}
		}
	}

	/**
	 * Check if cart contains WholesaleX TopUP Product
	 *
	 * @return boolean
	 * @since 1.0.0
	 */
	public function is_cart_contain_wholesalex_topup() {
		$status = false;
		if ( ! is_null( wc()->cart ) && count( wc()->cart->get_cart() ) > 0 ) {
			$_topup_product_id = get_option( '__wholesalex_wallet_topup_product' );
			if ( ! $_topup_product_id ) {
				return false;
			}
			foreach ( wc()->cart->get_cart() as $key => $cart_item ) {
				if ( intval( $_topup_product_id ) === intval( $cart_item['product_id'] ) ) {
					$status = true;
					break;
				}
			}
		}
		return $status;
	}

	/**
	 * Remove WholesaleX TopUp Product If cart contain other products
	 *
	 * @param WC_Cart $cart Card Object.
	 */
	public function woocommerce_cart_loaded_from_session( $cart ) {
		if ( count( $cart->get_cart() ) > 1 ) {
			$_topup_product_id = get_option( '__wholesalex_wallet_topup_product' );
			foreach ( $cart->get_cart() as $cart_item_key => $cart_item ) {
				$product_id = $cart_item['variation_id'] ? $cart_item['variation_id'] : $cart_item['product_id'];
				if ( intval( $product_id ) === intval( $_topup_product_id ) ) {
					WC()->cart->remove_cart_item( $cart_item_key );
				}
			}
		}
	}

	/**
	 * Add WholesaleX Wallet in Account Menu Items
	 *
	 * @since 1.0.0
	 * @since 1.0.10 wholesalex_addon_wallet_menu_item_text Filter Added.
	 */
	public function wholesalex_wallet_menu_item( $menu_links ) {
		$menu_links = array_slice( $menu_links, 0, 5, true )
		+ array( $this->get_endpoint() => $this->get_endpoint_title() )
		+ array_slice( $menu_links, 5, null, true );
		return $menu_links;
	}

	/**
	 * WholesaleX Wallet Add End Point
	 */
	public function wholesalex_wallet_menu_item_add_end_point() {
		add_rewrite_endpoint( $this->get_endpoint(), EP_PAGES );
	}
	/**
	 * WholesaleX Wallet Page
	 */
	public function wholesalex_wallet_page() {
		if ( ! wp_script_is( 'wholesalex_wallet', 'registered' ) ) {
			WHOLESALEX_PRO_Initialization::register_scripts();
		}
		wp_enqueue_script( 'wholesalex_wallet' );

		wp_localize_script(
			'wholesalex_wallet',
			'wholesalex_wallet',
			array(
				'settings' => wholesalex()->get_setting(),
				'fields'   => $this->get_wallet_page_fields(),
				'i18n' => array(
					'approve'=> __('Approve','wholesalex-pro'),
					'decline'=> __('Decline','wholesalex-pro'),
					'rejected'=> __('Rejected','wholesalex-pro'),
					'approved'=> __('Approved','wholesalex-pro'),
					'no_requests_found'=> __('No Requests Found!','wholesalex-pro'),
					'do_you_really_want_to_decline_this_request'=> __('Do you really want to decline this request?','wholesalex-pro'),
					'back'=> __('Back','wholesalex-pro'),
					'request_send_successful'=> __('Request Send Successful.','wholesalex-pro'),
				)
			)
		);

		wp_enqueue_style( 'dashicons' );
		echo '<div id="_wholesalex_wallet_page"></div>';
	}


	/**
	 * Set TopUP ammount as Product Price
	 *
	 * @param WC_Cart $cart WooCommerce Cart Object.
	 * @return void
	 */
	public function set_topup_amount_as_product_price( $cart ) {
		if ( empty( $cart->cart_contents ) ) {
			return;
		}
		$_topup_product_id = get_option( '__wholesalex_wallet_topup_product' );

		foreach ( $cart->cart_contents as $cart_item ) {
			if ( isset( $cart_item['topup_amount'] ) && $_topup_product_id == $cart_item['product_id'] ) {
				$cart_item['data']->set_price( $cart_item['topup_amount'] );
			}
		}
	}

	/**
	 * Restrict customer to order other product along with wallet topup
	 *
	 * @param boolean $status Restriction Status.
	 * @return boolean $status Status of Restriction
	 */
	public function restrict_other_items_from_add_to_cart( $status ) {
		if ( $this->is_cart_contain_wholesalex_topup() ) {
			wc_add_notice( __( 'You cannot purchase other product while topup item in cart', 'wholesalex-pro' ), 'error' );
			$status = false;
		}
		return $status;
	}

	/**
	 * Make TopUp product purchasable
	 *
	 * @param boolean           $is_purchasable Purchasable Status.
	 * @param WC_Product object $product WooCommerce Product Object.
	 * @return boolean $is_purchaseable Purchaseable Status.
	 */
	public function make_topup_product_purchasable( $is_purchasable, $product ) {
		$_topup_product_id = get_option( '__wholesalex_wallet_topup_product' );
		if ( $_topup_product_id ) {
			if ( intval( $_topup_product_id ) === intval( $product->get_id() ) ) {
				$is_purchasable = true;
			}
		}
		return $is_purchasable;
	}

	/**
	 * Filter WooCommerce available payment gateway while topup
	 *
	 * @param array $_available_gateways Avaiable gateways.
	 * @return array $_available_gateways Updated Gateway List.
	 * @since 1.0.0
	 * @since 1.0.9 wholesalex_addon_wallet_balance_payment_status filter added.
	 */
	public function topup_available_payment_gateways( $_available_gateways ) {
		$can_use_wallet_balance = apply_filters( 'wholesalex_addon_wallet_balance_payment_status', true );
		if ( $this->is_cart_contain_wholesalex_topup() || ! $can_use_wallet_balance ) {
			foreach ( $_available_gateways as $gateway_id => $gateway ) {
					unset( $_available_gateways['wholesalex'] );
			}
		}
		return $_available_gateways;
	}

	/**
	 * Add Custom Query Vars in WooCommerce
	 *
	 * @param array $query Query.
	 */
	public function wholesalex_add_woocommerce_query_vars( $query ) {
		$query[ $this->get_endpoint() ] = $this->get_endpoint();
		return $query;
	}

	/**
	 * WholesaleX Process Wallet In Profile Page
	 *
	 * @param array $profile_settings Profile Settings.
	 * @param int   $user_id User ID.
	 * @return array Profile Settings Data.
	 */
	public function process_transaction_profile_page( $profile_settings, $user_id ) {
		if ( isset( $profile_settings['_wholesalex_add_credit'] ) && ! empty( $profile_settings['_wholesalex_add_credit'] ) ) {
			$amount = floatval( $profile_settings['_wholesalex_add_credit'] );

			$available_balance = wholesalex_wallet()->get_wholesalex_balance( $user_id, 'edit' );
			$transaction_type  = $amount > 0 ? 'credit' : 'debit';
			switch ( $transaction_type ) {
				case 'credit':
					$transaction_id = wholesalex_wallet()->initiate_transaction( $user_id, $amount, 'credit', apply_filters( 'wholesalex_wallet_admin_credited_message', __( 'Credited By Admin', 'wholesalex-pro' ) ) );
					break;
				case 'debit':
					$transaction_id = wholesalex_wallet()->initiate_transaction( $user_id, min( abs( $amount ), $available_balance ), 'debit', apply_filters( 'wholesalex_wallet_admin_debited_message', __( 'Debited By Admin', 'wholesalex-pro' ) ) );
					break;

				default:
					// code...
					break;
			}

			unset( $profile_settings['_wholesalex_add_credit'] );
		}
		return $profile_settings;
	}

	/**
	 * WholesaleX Set Credit Balance On Profile Page
	 *
	 * @param array $profile_settings Profile Settings.
	 * @param int   $user_id User ID.
	 * @return array Profile Settings Data.
	 */
	public function set_credit_balance_profile_page( $profile_settings, $user_id ) {
		$profile_settings['_wholesalex_credit_balance'] = wholesalex_wallet()->get_wholesalex_balance( $user_id, 'edit' );
		return $profile_settings;
	}

	/**
	 * WholesaleX Process Transaction after role change
	 *
	 * @param int    $user_id User ID.
	 * @param string $previous_role Previous Role.
	 * @param string $updated_role Updated Role.
	 * @return void
	 */
	public function process_transaction_after_role_change( $user_id, $previous_role, $updated_role ) {
		$previous_role  = $previous_role;
		$__role_content = wholesalex()->get_roles( 'by_id', $updated_role );
		if ( ! empty( $__role_content ) ) {
			$__already_credit_from_role = get_user_meta( $user_id, '__wholesalex_credit_from_role', true );
			if ( ! $__already_credit_from_role && isset( $__role_content['credit_limit'] ) && isset( $__role_content['_role_title'] ) ) {
				$amount         = $__role_content['credit_limit'];
				$name           = $__role_content['_role_title'];
				$transaction_id = wholesalex_wallet()->initiate_transaction( $user_id, $amount, 'credit', __( 'Bonus For Migrate Role  #', 'wholesalex-pro' ) . $name );
				if ( $transaction_id ) {
					update_user_meta(
						$user_id,
						'__wholesalex_credit_from_role',
						array(
							'status'         => true,
							'transaction_id' => $transaction_id,
						)
					);
					do_action( 'wholesalex_role_initial_balance_added', $user_id, $name, $amount, $transaction_id );
				}
			}
		}
	}

	/**
	 * Get Wallet Page Fields
	 *
	 * @since 1.0.9 Add Filter wholesalex_wallet_user_id
	 */
	public function get_wallet_page_fields() {

		$__user_id      = apply_filters( 'wholesalex_wallet_user_id', get_current_user_id() );
		$__transactions = get_user_meta( $__user_id, '__wholesalex_transactions_details', true );

		if ( $__transactions && ! empty( $__transactions ) ) {
			foreach ( $__transactions as $key => $value ) {
				$__transactions[ $key ]['formated_amount'] = wc_price( $__transactions[ $key ]['amount'] );
			}
		} else {
			$__transactions = array();
		}

		return apply_filters(
			'wholesalex_wallet_fields',
			array(
				'_balance_section'             => array(
					'attr' => array(
						'_balance' => array(
							'type'             => 'balance',
							'label'            => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_balance' : wholesalex()->get_setting( '_language_balance' ), __( 'Wallet Balance', 'wholesalex-pro' ) ),
							'value'            => wholesalex_wallet()->get_wholesalex_balance( $__user_id, 'edit' ),
							'currency_symbol'  => get_woocommerce_currency_symbol(),
							'formated_balance' => wholesalex_wallet()->get_wholesalex_balance( $__user_id, 'view' ),
						),
					),
				),
				'_button_section'              => array(
					'attr' => array(
						'_add_balance'         => array(
							'type'       => 'button',
							'icon_class' => 'dashicons dashicons-money-alt',
							'label'      => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_add_balance' : wholesalex()->get_setting( '_language_add_balance' ), __( 'Add Balance', 'wholesalex-pro' ) ),
						),
						'_transaction_history' => array(
							'type'       => 'button',
							'icon_class' => 'dashicons dashicons-chart-bar',
							'label'      => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_transaction_history' : wholesalex()->get_setting( '_language_transaction_history' ), __( 'Transactions', 'wholesalex-pro' ) ),
						),
					),
				),
				'_add_balance_section'         => array(
					'attr' => array(
						'_topup_amount' => array(
							'type'        => 'number',
							/* translators: %s: Currency Symbol */
							'label'       => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_topup_amount' : wholesalex()->get_setting( '_language_topup_amount' ), sprintf( __( 'Enter Amount (%s)', 'wholesalex-pro' ), get_woocommerce_currency_symbol() ) ),
							'default'     => '',
							'placeholder' => '',
						),
						'_submit_btn'   => array(
							'type'        => 'button',
							'label'       => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_add_balance_submit_btn' : wholesalex()->get_setting( '_language_add_balance_submit_btn' ), __( 'Proceed', 'wholesalex-pro' ) ),
							'redirect_to' => wc_get_checkout_url(),
						),
					),
				),
				'_transaction_history_section' => array(
					'attr' => array(
						'_transaction_table' => array(
							'type'             => 'table',
							'item_per_page'    => apply_filters( 'wholesalex_wallet_transaction_table_item_per_page', 50 ),
							'columns'          => array(
								'transaction_id'  => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_transaction_table_transaction_id' : wholesalex()->get_setting( '_language_transaction_table_transaction_id' ), __( 'Transactions ID', 'wholesalex-pro' ) ),
								'transction_type' => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_transaction_table_transction_type' : wholesalex()->get_setting( '_language_transaction_table_transction_type' ), __( 'Type', 'wholesalex-pro' ) ),
								'amount'          => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_transaction_table_amount' : wholesalex()->get_setting( '_language_transaction_table_amount' ), __( 'Amount', 'wholesalex-pro' ) ),
								'date'            => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_transaction_table_date' : wholesalex()->get_setting( '_language_transaction_table_date' ), __( 'Date', 'wholesalex-pro' ) ),
								'details'         => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_transaction_table_details' : wholesalex()->get_setting( '_language_transaction_table_details' ), __( 'Details', 'wholesalex-pro' ) ),
							),
							'data'             => $__transactions,
							'pagination_label' => array(
								'prev' => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_pagination_label_prev' : wholesalex()->get_setting( '_language_pagination_label_prev' ), __( 'Previous', 'wholesalex-pro' ) ),
								'next' => wholesalex()->get_language_n_text( version_compare( WHOLESALEX_VER, '1.0.2', '>=' ) ? '_language_pagination_label_next' : wholesalex()->get_setting( '_language_pagination_label_next' ), __( 'Next', 'wholesalex-pro' ) ),
							),
							'price_formatter'  => wc_price( 0 ),
						),
					),
				),
			),
		);
	}

	/**
	 * Wallet Settings Fields
	 *
	 * @param array $sections Setting Sections.
	 */
	public function wallet_settings_field( $sections ) {
		if ( isset( $sections['language_n_text']['attr'] ) && is_array( $sections['language_n_text']['attr'] ) ) {
			$__fields = array(
				'_language_credit_limit'              => array(
					'type'        => 'text',
					'label'       => __( 'Credit Limit Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Credit Limit', 'wholesalex-pro' ),
				),
				'_language_wholesalex_add_credit'     => array(
					'type'        => 'text',
					'label'       => __( 'Credit Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Credit', 'wholesalex-pro' ),
				),
				'_language_wholesalex_credit_balance' => array(
					'type'        => 'text',
					'label'       => __( 'Credit Balance Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Credit Balance', 'wholesalex-pro' ),
				),
				'_language_balance'                   => array(
					'type'        => 'text',
					'label'       => __( 'Wallet Balance Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Wallet Balance', 'wholesalex-pro' ),
				),
				'_language_add_balance'               => array(
					'type'        => 'text',
					'label'       => __( 'Add Balance Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Add Balance', 'wholesalex-pro' ),
				),
				'_language_transaction_history'       => array(
					'type'        => 'text',
					'label'       => __( 'Transactions Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Transactions', 'wholesalex-pro' ),
				),
				'_language_topup_amount'              => array(
					'type'        => 'text',
					'label'       => __( 'Enter Amount Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Enter Amount', 'wholesalex-pro' ),
				),
				'_language_add_balance_submit_btn'    => array(
					'type'        => 'text',
					'label'       => __( 'Proceed Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Proceed', 'wholesalex-pro' ),
				),
				'_language_transaction_table_transaction_id' => array(
					'type'        => 'text',
					'label'       => __( 'Transactions ID Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Transactions ID', 'wholesalex-pro' ),
				),
				'_language_transaction_table_transction_type' => array(
					'type'        => 'text',
					'label'       => __( 'Transactions Type Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Type', 'wholesalex-pro' ),
				),
				'_language_transaction_table_amount'  => array(
					'type'        => 'text',
					'label'       => __( 'Transactions Amount Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Amount', 'wholesalex-pro' ),
				),
				'_language_transaction_table_date'    => array(
					'type'        => 'text',
					'label'       => __( 'Transactions Date Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Date', 'wholesalex-pro' ),
				),
				'_language_transaction_table_details' => array(
					'type'        => 'text',
					'label'       => __( 'Transactions Details Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Details', 'wholesalex-pro' ),
				),
				'_language_pagination_label_prev'     => array(
					'type'        => 'text',
					'label'       => __( 'Transactions Pagination Previous Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Previous', 'wholesalex-pro' ),
				),
				'_language_pagination_label_next'     => array(
					'type'        => 'text',
					'label'       => __( 'Transactions Pagination Next Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Next', 'wholesalex-pro' ),
				),
				'_language_wallet_pending_request_from_subaccount' => array(
					'type'        => 'text',
					'label'       => __( 'Pending Request From Subaccount Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Pending Request From Subaccount', 'wholesalex-pro' ),
				),
				'_language_wallet_balance_request_table_subaccount_name_text' => array(
					'type'        => 'text',
					'label'       => __( 'Wallet Pending Request Subaccount Column Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Subaccount Name', 'wholesalex-pro' ),
				),
				'_language_wallet_balance_request_table_requested_credit_text' => array(
					'type'        => 'text',
					'label'       => __( 'Wallet Pending Request Requested Balance Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Requested Balance', 'wholesalex-pro' ),
				),
				'_language_wallet_balance_request_table_requested_action_text' => array(
					'type'        => 'text',
					'label'       => __( 'Wallet Pending Request Actions Text', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Actions', 'wholesalex-pro' ),
				),
				'_language_hidden_price_request_quote_btn_text' => array(
					'type'        => 'text',
					'label'       => __( 'Request Quote', 'wholesalex-pro' ),
					'placeholder' => '',
					'help'        => '',
					'default'     => __( 'Request Quote', 'wholesalex-pro' ),
				),
			);

			$sections['language_n_text']['attr'] = array_merge( $sections['language_n_text']['attr'], $__fields );
		}

		/**
		 * Wallet Partial Payment Settings Added
		 *
		 * @since 1.1.9
		 */
		$settings                    = array(
			'wholesalex_wallet_refund_for_all_gateway'    => array(
				'type'    => 'switch',
				'label'   => __( 'Enable Refund to Wallet Balance ', 'wholesalex-pro' ),
				'desc'    => __( 'Enable', 'wholesalex-pro' ),
				'help'    => 'If enable, then admin can refund paid orders as wallet balance',
				'default' => 'no',
			),
			'wholesalex_wallet_partial_payment_status'    => array(
				'type'    => 'switch',
				'label'   => __( 'Enable Parital Payment Through Wallet Balance ', 'wholesalex-pro' ),
				'desc'    => __( 'Enable', 'wholesalex-pro' ),
				'help'    => 'If enable, then user can pay partially through the wallet balance and pay rest of the amount using different payment gateway.',
				'default' => 'no',
			),
			'wallet_balance_shortcode'                => array(
				'type'    => 'shortcode',
				'label'   => __( 'Wallet Balance Shortcode', 'wholesalex-pro' ),
				'desc'    => __( '[wsx_wallet_balance] ', 'wholesalex-pro' ),
				'help'    => '',
				'default' => '[wsx_wallet_balance]',
				'tooltip' => __( 'Click on the button to directly copy the shortcode.', 'wholesalex-pro' ),
			),
			'_language_wallet_menu_item_text'             => array(
				'type'        => 'text',
				'label'       => __( 'My Account Menu Item Text', 'wholesalex-pro' ),
				'placeholder' => '',
				'help'        => '',
				'default'     => __( 'WholesaleX Wallet', 'wholesalex-pro' ),
			),
			'_language_wallet_menu_item_endpoint_text'    => array(
				'type'        => 'text',
				'label'       => __( 'My Account Menu Item Endpoint', 'wholesalex-pro' ),
				'placeholder' => '',
				'help'        => '',
				'default'     => __( 'wholesalex-wallet', 'wholesalex-pro' ),
			),
			'_language_wholesalex_payment_gateway_icon_text' => array(
				'type'        => 'text',
				'label'       => __( 'Payment Gateway Icon Text', 'wholesalex-pro' ),
				'placeholder' => '',
				'help'        => 'Smart Tags : {remaining_balance} => Remaining Wallet Balance',
				'default'     => __( 'Remaining WholesaleX Balance: {remaining_balance}', 'wholesalex-pro' ),
			),
			'_language_wallet_partial_pay_text'           => array(
				'type'        => 'text',
				'label'       => __( 'Partially Pay Text', 'wholesalex-pro' ),
				'placeholder' => '',
				'help'        => '',
				'default'     => __( 'Partially Pay with WholesaleX Wallet', 'wholesalex-pro' ),
			),
			'_language_wallet_partial_paid_checkout_text' => array(
				'type'        => 'text',
				'label'       => __( 'Wallet Pay Text', 'wholesalex-pro' ),
				'placeholder' => '',
				'help'        => '',
				'default'     => __( 'WholesaleX Wallet Pay', 'wholesalex-pro' ),
			),
		);
		$sections['wallet']['label'] = __( 'Wallet', 'wholesalex-pro' );
		$sections['wallet']['attr']  = $settings;
		return $sections;
	}


	/**
	 * Change Wallet Menu Item Text
	 *
	 * @param string $menu_item_text Menu Item Text.
	 * @return string
	 * @since 1.0.10
	 */
	public function wallet_menu_item_text_change( $menu_item_text ) {
		if ( wholesalex()->get_setting( '_language_wallet_menu_item_text' ) ) {
			$menu_item_text = wholesalex()->get_setting( '_language_wallet_menu_item_text' );
		}

		return $menu_item_text;
	}

	/**
	 * Change Wallet Menu Item Endpoint
	 *
	 * @param string $menu_item_text Menu Item endpoint.
	 * @return string
	 * @since 1.2.4
	 */
	public function wallet_menu_item_endpoint_change( $menu_item_text ) {
		if ( wholesalex()->get_setting( '_language_wallet_menu_item_endpoint_text', 'wholesalex-wallet' ) ) {
			$menu_item_text = wholesalex()->get_setting( '_language_wallet_menu_item_endpoint_text', 'wholesalex-wallet' );
		}

		return $menu_item_text;
	}


	/**
	 * Wallet Balance Column on WholesaleX Users Page
	 *
	 * @since 1.1.5
	 */
	public function wallet_balance_column_on_wholesalex_users( $columns ) {
		if ( is_array( $columns ) ) {
			$columns = wholesalex()->insert_into_array( $columns, array( 'wallet_balance' => __( 'Wallet Balance', 'wholesalex-pro' ) ) );
		}
		return $columns;
	}


	public function wallet_balance_field_value_on_wholesalex_users( $value, $user ) {
		if ( 'wallet_balance' === $value ) {
			$user_id = $user['ID'];
			$value   = wholesalex_wallet()->get_wholesalex_balance( $user_id );

		}
		return $value;
	}



	public function wallet_balance_column_on_exporter( $columns ) {
		$columns['wholesalex_wallet_balance'] = __( 'Wallet Balance', 'wholesalex-pro' );
		return $columns;
	}

	public function populate_data_on_balance_column_exporter( $data ) {
		$user_id                           = $data['id'];
		$data['wholesalex_wallet_balance'] = wholesalex_wallet()->get_wholesalex_balance( $user_id, 'edit' );
		return $data;
	}

	public function process_import( $user_id, $row_data, $mapped_column ) {
		$wallet_balance = floatval( wholesalex_wallet()->get_wholesalex_balance( $user_id, 'edit' ) );
		$new_balance    = floatval( $row_data[ $mapped_column['wholesalex_wallet_balance'] ] );
		if ( $wallet_balance != $new_balance ) {
			$transaction_type = $new_balance > $wallet_balance ? 'credit' : 'debit';
			$amount           = abs( $wallet_balance - $new_balance );
			switch ( $transaction_type ) {
				case 'credit':
					$transaction_id = wholesalex_wallet()->initiate_transaction( $user_id, $amount, 'credit', apply_filters( 'wholesalex_wallet_admin_credited_message', __( 'Credited By Admin', 'wholesalex-pro' ) ) );
					break;
				case 'debit':
					$transaction_id = wholesalex_wallet()->initiate_transaction( $user_id, min( abs( $amount ), $wallet_balance ), 'debit', apply_filters( 'wholesalex_wallet_admin_debited_message', __( 'Debited By Admin', 'wholesalex-pro' ) ) );
					break;

				default:
					// code...
					break;
			}
			//Filter Hook For Getting Updated Wholesalex Wallet Balance to Sent The Email
			add_filter('wholesalex_profile_updated_data', function( $sub_account_data, $us_id ) use( $user_id) {
				if ( $user_id == $us_id) {
					return array_merge( $sub_account_data, ['__wholesalex_balance'] );
				}
				return $sub_account_data;
			}, 10, 2);
			
		}
	}

	/**
	 * Add Parital Payment Checkbox on Checkout Page
	 */

	public function add_partial_payment_checkbox() {
		if ( 'no' == wholesalex()->get_setting( 'wholesalex_wallet_partial_payment_status','no' ) || ! is_user_logged_in() ) {
			return;
		}

		if ( $this->is_cart_contain_wholesalex_topup() ) {
			$this->set_wallet_partial_payment_in_session( false );
			return;
		}

		if ( ! floatval( wholesalex_wallet()->get_wholesalex_balance( get_current_user_id(), 'edit' ) ) > 0 ) {
			return;
		}

		// If current cart total is greater than, available wallet balance
		// Hide Wallet Balance Payment Gateway
		// add checkbox to get the value

		$is_eligible_for_partial_pay = ( WC()->cart->get_total( 'edit' ) > wholesalex_wallet()->get_wholesalex_balance( get_current_user_id(), 'edit' ) );
		$partial_pay_text            = apply_filters( 'wholesalex_wallet_partial_pay_checkout_text', wholesalex()->get_language_n_text( '_language_wallet_partial_pay_text', 'Partially Pay with WholesaleX Wallet' ) );

		if ( $is_eligible_for_partial_pay ) {
			?>
			<tr class="wholesalex-wallet-partial-pay">
				<th> <?php echo sprintf( '%s ( %s )', esc_html( $partial_pay_text ), wholesalex_wallet()->get_wholesalex_balance( get_current_user_id() ) ); //phpcs:ignore ?> </th>
				<td data-title="<?php echo esc_html( $partial_pay_text ); ?>"><input type="checkbox"  style="vertical-align: middle;" id="wholesalex_wallet_partial_pay" <?php checked( $this->get_wallet_partial_payment_status_from_session(), true, true ); ?> name="wholesalex_wallet_partial_pay" class="wholesalex_wallet_partial_pay" /></td>
			</tr>
			<script>
				jQuery(function ($) {
					$(document).on('change', '#wholesalex_wallet_partial_pay', function (e) {
					e.stopImmediatePropagation();
					let attr = {
						action: 'wholesalex_partial_payment_update_session',
						partial_pay: $(this).is(':checked')?'yes':'no',
						nonce: "<?php echo esc_attr( wp_create_nonce( 'wholesalex_wallet_partial_pay' ) ); ?>"
					};
					$.post('<?php echo esc_url( admin_url( 'admin-ajax.php' ) ); ?>', attr, function () {
						$(document.body).trigger('update_checkout');
					});
				});
			});
			</script>
			<?php
		}

	}


	/**
	 * Remove Wallet Balance Gateway For If Partial Payment Found
	 *
	 * @param object $gateways WooCommerce Payment Gateways
	 * @since 1.1.8
	 */

	public function remove_wallet_balance_gateway( $gateways ) {

		if ( ! is_admin() && ( is_checkout() || is_cart() ) ) {
			$enable_partial_payment      = wholesalex()->get_setting( 'wholesalex_wallet_partial_payment_status' );
			$is_eligible_for_partial_pay = ( WC()->cart->get_total( 'edit' ) > wholesalex_wallet()->get_wholesalex_balance( get_current_user_id(), 'edit' ) ) && apply_filters( 'wholesalex_wallet_partial_payment_status', false );

			if ( 'yes' === $enable_partial_payment && ( $is_eligible_for_partial_pay || $this->get_wallet_partial_payment_status_from_session() ) ) {
				if ( isset( $gateways['wholesalex'] ) ) {
					unset( $gateways['wholesalex'] );
				}
			} else {
				$this->set_wallet_partial_payment_in_session( false );
			}
		}

		return $gateways;
	}

	/**
	 * Check is partial payment enable on not
	 *
	 * @param boolean $status Status
	 * @return void
	 * @since 1.1.8
	 */
	public function wallet_partial_payment_status( $status ) {
		if ( 'yes' === wholesalex()->get_setting( 'wholesalex_wallet_partial_payment_status' ) ) {
			$status = true;
		}
		return $status;
	}

	/**
	 * Update Wallet Partial Pay in WC Session
	 *
	 * @return void
	 */
	public function update_wallet_partial_pay_in_session() {
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'wholesalex_wallet_partial_pay' ) ) {
			if ( isset( $_POST['partial_pay'] ) && 'yes' === sanitize_text_field( wp_unslash( $_POST['partial_pay'] ) ) ) {
				$this->set_wallet_partial_payment_in_session( true );
			} else {
				$this->set_wallet_partial_payment_in_session( false );
			}
		}
		wp_send_json_success( 'Wallet Balance Partially Pay Updated.' );
		wp_die();
	}

	/**
	 * Set Wallet Partial Pay in session
	 *
	 * @param boolean $status Status.
	 * @return void
	 */
	public function set_wallet_partial_payment_in_session( $status ) {
		$status = apply_filters( 'wholesalex_wallet_set_partial_payment_status_in_session', $status );
		if ( defined( 'WC_VERSION' ) && ! is_null( WC()->session ) ) {
			WC()->session->set( 'wholesalex_wallet_partial_payment', $status );
		}
	}



	/**
	 * Get Wallet Partial Pay in session
	 *
	 * @return void
	 */
	public function get_wallet_partial_payment_status_from_session() {
		$status = ( ! is_null( wc()->session ) && wc()->session->get( 'wholesalex_wallet_partial_payment', false ) );
		return apply_filters( 'wholesalex_wallet_get_partial_payment_status_from_session', $status );
	}


	/**
	 * Add Partial Payment as fee
	 *
	 * @param object $cart Cart object
	 * @return void
	 */
	public function wholesalex_wallet_add_partial_payment_fee( $cart ) {

		if ( 'yes' === wholesalex()->get_setting( 'wholesalex_wallet_partial_payment_status' ) && $this->get_wallet_partial_payment_status_from_session() ) {

			// apply partial payment
			$wallet_balance = floatval( wholesalex_wallet()->get_wholesalex_balance( get_current_user_id(), 'edit' ) );

			$partial_payment_amount = apply_filters( 'wholesalex_wallet_partial_payment_amount', $wallet_balance );

			$fee_name = apply_filters( 'wholesalex_wallet_partial_pay_fee_name', wholesalex()->get_language_n_text( '_language_wallet_partial_paid_checkout_text', 'WholesaleX Wallet Pay' ) );

			$fee = array(
				'id'        => '__wholesalex_wallet_partial_payment',
				'name'      => $fee_name,
				'amount'    => -1.00 * floatval( $partial_payment_amount ),
				'taxable'   => false,
				'tax_class' => '',
			);

			// WC()->cart->add_fee($fee_name,-1*$partial_payment_amount,false,'');
			// Fees api use because we need to add an fees id to detect this fees in future/for other process.
			if ( $partial_payment_amount > 0 ) {
				$cart->fees_api()->add_fee( $fee );
			}
		} else {
			// clear the fee
			$fees = $cart->fees_api()->get_fees();
			if ( isset( $fees['__wholesalex_wallet_partial_payment'] ) ) {
				unset( $fees['__wholesalex_wallet_partial_payment'] );
				$cart->fees_api()->set_fees( $fees );
			}
		}
	}


	public function remove_partial_payment_tax( $taxes, $fee ) {
		if ( '__wholesalex_wallet_partial_payment' === $fee->object->id ) {
			$taxes = array();
		}
		return $taxes;
	}

	public function make_unavailable_wallet_payment_gateway( $status ) {
		if ( 'yes' === wholesalex()->get_setting( 'wholesalex_wallet_partial_payment_status' ) && $this->get_wallet_partial_payment_status_from_session() ) {
			$status = false;
		}
		return $status;
	}


	/**
	 * Process Partial Payment
	 *
	 * @param int|string $order_id Order ID.
	 * @return void
	 */
	public function process_partial_payment( $order_id ) {
		wholesalex_wallet()->process_partial_payment( $order_id );
	}

	/**
	 * Process Cancelled Partial Payment
	 *
	 * @param int|string $order_id Order ID.
	 * @return void
	 */
	public function process_partial_payment_cancelled_order( $order_id ) {
		wholesalex_wallet()->process_partial_payment_cancel_order( $order_id );
	}

	/**
	 * Load Partial Refund Script
	 *
	 * @return void
	 */
	public function load_partial_refund_script() {
		?>
		<script> 
			jQuery(function ($) {
				$(document).on('click', '.wholesalex_partial_pay_refund_btn', function (e) {
					e.preventDefault();
					if (window.confirm("<?php echo esc_html__( 'Do You realy want to refund?', 'wholesalex-pro' ); ?>")) {
						let attr = {
							action: 'wholesalex_wallet_partial_payment_refund',
							order_id: woocommerce_admin_meta_boxes.post_id,
							nonce: "<?php echo esc_attr( wp_create_nonce( 'wholesalex_wallet_refund' ) ); ?>"
						};
						$.post('<?php echo esc_url( admin_url( 'admin-ajax.php' ) ); ?>', attr, function (res) {
							if ('success' === res.data.status) {
								window.location.reload();
							}
						});
				}
				});
			});
		</script>
		<?php
	}

	/**
	 * Partial Payment Refund Button
	 *
	 * @param int    $item_id Item Id
	 * @param object $item Item
	 * @return void
	 */
	public function add_wallet_partial_payment_refund_button( $item_id, $item ) {
		global $post;
		if ( ! wholesalex_wallet()->is_partial_payment_order( $item_id, $item ) && ! get_post_meta( $post->ID, '__wholesalex_wallet_partial_payment_refund_transaction_id', true ) ) {
			return;
		}

		// is already refunded.
		$refund_tranx_id = get_post_meta( $post->ID, '__wholesalex_wallet_partial_payment_refund_transaction_id', true );
		if ( ! $refund_tranx_id ) {
			?>
				<button type="button" class="button wholesalex_partial_pay_refund_btn"><?php echo esc_html__( 'Refund', 'wholesalex-pro' ); ?></button>
			<?php
			$this->load_partial_refund_script();
		} else {
			?>
				<span class="wholesalex_partial_pay_refunded"><?php echo esc_html__( 'Refunded', 'wholesalex-pro' ); ?></button>
			<?php
		}

	}

    /**
	 * Handle a refund via the edit order screen.
	 *
	 * @throws Exception To return errors.
	 */
	public function wholesalex_wallet_process_refund() {
		ob_start();

		check_ajax_referer( 'order-item', 'security' );

		if ( ! current_user_can( 'edit_shop_orders' ) ) {
			wp_die( -1 );
		}

		$order_id               = isset( $_POST['order_id'] ) ? absint( $_POST['order_id'] ) : 0;
		$refund_amount          = isset( $_POST['refund_amount'] ) ? wc_format_decimal( sanitize_text_field( wp_unslash( $_POST['refund_amount'] ) ), wc_get_price_decimals() ) : 0;
		$refunded_amount        = isset( $_POST['refunded_amount'] ) ? wc_format_decimal( sanitize_text_field( wp_unslash( $_POST['refunded_amount'] ) ), wc_get_price_decimals() ) : 0;
		$refund_reason          = isset( $_POST['refund_reason'] ) ? sanitize_text_field( wp_unslash( $_POST['refund_reason'] ) ) : __('Refund to Wallet #'.$order_id,'wholesalex-pro');
		$line_item_qtys         = isset( $_POST['line_item_qtys'] ) ? json_decode( sanitize_text_field( wp_unslash( $_POST['line_item_qtys'] ) ), true ) : array();
		$line_item_totals       = isset( $_POST['line_item_totals'] ) ? json_decode( sanitize_text_field( wp_unslash( $_POST['line_item_totals'] ) ), true ) : array();
		$line_item_tax_totals   = isset( $_POST['line_item_tax_totals'] ) ? json_decode( sanitize_text_field( wp_unslash( $_POST['line_item_tax_totals'] ) ), true ) : array();
		$api_refund             = isset( $_POST['api_refund'] ) && 'true' === $_POST['api_refund'];
		$restock_refunded_items = isset( $_POST['restock_refunded_items'] ) && 'true' === $_POST['restock_refunded_items'];
		$refund                 = false;
		$response               = array();

		try {
			$order      = wc_get_order( $order_id );
			$max_refund = wc_format_decimal( $order->get_total() - $order->get_total_refunded(), wc_get_price_decimals() );

			if ( ( ! $refund_amount && ( wc_format_decimal( 0, wc_get_price_decimals() ) !== $refund_amount ) ) || $max_refund < $refund_amount || 0 > $refund_amount ) {
				throw new Exception( __( 'Invalid refund amount', 'woocommerce' ) );
			}

			if ( wc_format_decimal( $order->get_total_refunded(), wc_get_price_decimals() ) !== $refunded_amount ) {
				throw new Exception( __( 'Error processing refund. Please try again.', 'woocommerce' ) );
			}

			// Prepare line items which we are refunding.
			$line_items = array();
			$item_ids   = array_unique( array_merge( array_keys( $line_item_qtys ), array_keys( $line_item_totals ) ) );

			foreach ( $item_ids as $item_id ) {
				$line_items[ $item_id ] = array(
					'qty'          => 0,
					'refund_total' => 0,
					'refund_tax'   => array(),
				);
			}
			foreach ( $line_item_qtys as $item_id => $qty ) {
				$line_items[ $item_id ]['qty'] = max( $qty, 0 );
			}
			foreach ( $line_item_totals as $item_id => $total ) {
				$line_items[ $item_id ]['refund_total'] = wc_format_decimal( $total );
			}
			foreach ( $line_item_tax_totals as $item_id => $tax_totals ) {
				$line_items[ $item_id ]['refund_tax'] = array_filter( array_map( 'wc_format_decimal', $tax_totals ) );
			}



			// Create the refund object.
			$refund = wc_create_refund(
				array(
					'amount'         => $refund_amount,
					'reason'         => $refund_reason,
					'order_id'       => $order_id,
					'line_items'     => $line_items,
					'refund_payment' => $api_refund,
					'restock_items'  => $restock_refunded_items,
				)
			);

            if(!is_wp_error( $refund )) {
                $transaction_id = wholesalex_wallet()->initiate_transaction( $order->get_customer_id(), $refund_amount, 'credit', $refund_reason );
                if(!$transaction_id) {
                    throw new Exception(__('Refund not credited to customer wallet','wholesalex-pro'));
                }
                do_action( 'wholesalex_refunded_to_wallet', $order, $transaction_id, $refund );
            }

			if ( is_wp_error( $refund ) ) {
				throw new Exception( $refund->get_error_message() );
			}

			if ( did_action( 'woocommerce_order_fully_refunded' ) ) {
				$response['status'] = 'fully_refunded';
			}
		} catch ( Exception $e ) {
			wp_send_json_error( array( 'error' => $e->getMessage() ) );
		}

		// wp_send_json_success must be outside the try block not to break phpunit tests.
		wp_send_json_success( $response );
	}

	/**
	 * Process Partial Payment Refund
	 *
	 * @return void
	 */
	public function wholesalex_wallet_partial_payment_refund() {

		$res = array(
			'status'         => false,
			'transaction_id' => '',
		);

		if ( isset( $_POST['nonce'], $_POST['order_id'] ) && wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'wholesalex_wallet_refund' ) && ! empty( $_POST['order_id'] ) ) {

			$order_id = sanitize_text_field( wp_unslash( $_POST['order_id'] ) );
			$order    = wc_get_order( $order_id );
			if ( $order ) {
				$refund_amount  = wholesalex_wallet()->get_partial_payment_amount( $order_id );
				$reason         = apply_filters( 'wholesalex_wallet_partial_payment_refund_reason', __( 'Refund Wallet Balance , #', 'wholesalex-pro' ) . $order->get_order_number(), $order );
				$transaction_id = wholesalex_wallet()->initiate_transaction( $order->get_customer_id(), $refund_amount, 'credit', $reason );
				if ( $transaction_id ) {
					$order_note = apply_filters( 'wholesalex_wallet_partial_payment_order_note', sprintf( 'Wallet Balance %s refunded to the customer', wc_price( $refund_amount ) ), $refund_amount, $order );
					$order->add_order_note( $order_note );
					delete_post_meta( $order_id, '__wholesalex_wallet_partial_payment_transaction_id' );
					update_post_meta( $order_id, '__wholesalex_wallet_partial_payment_refund_transaction_id', $transaction_id );
					$res = array(
						'status'         => 'success',
						'transaction_id' => $transaction_id,
					);
					do_action( 'wholesalex_wallet_partial_payment_refunded', $order, $transaction_id );
				}
			}
		}

		wp_send_json_success( $res );
	}

	/**
	 * Add Request a Quote columns on Role Exporter Columns
	 *
	 * @param array $columns Role Exporter Columns
	 * @return void
	 */
	public function add_wallet_fields_on_role_exporter( $columns ) {
		$columns['credit_limit']   = __( 'Credit', 'wholesalex-pro' );
		$columns['disable_wallet'] = __( 'Disable Wallet Addon For this Role', 'wholesalex-pro' );
		return $columns;
	}

	/**
	 * Get Role
	 *
	 * @param [type] $role
	 * @return void
	 */
	public function get_column_value_credit_limit( $value, $role ) {

		return isset( $role['credit_limit'] ) ? $role['credit_limit'] : '';
	}
	/**
	 * Get Role
	 *
	 * @param [type] $role
	 * @return void
	 */
	public function get_column_value_disable_wallet( $value, $role ) {

		return isset( $role['disable_wallet'] ) ? $role['disable_wallet'] : '';
	}

	public function set_import_column_value( $options ) {
		if ( ! isset( $options['credit_limit'] ) ) {
			$options['credit_limit'] = __( 'Credit', 'wholesalex-pro' );
		}
		if ( ! isset( $options['disable_wallet'] ) ) {
			$options['disable_wallet'] = __( 'Disable Wallet Addon For this Role', 'wholesalex-pro' );
		}

		return $options;

	}


	public function set_import_mapping_default_column( $columns ) {
		if ( ! isset( $columns[ __( 'Credit', 'wholesalex-pro' ) ] ) ) {
			$columns[ __( 'Credit', 'wholesalex-pro' ) ] = 'credit_limit';
		}
		if ( ! isset( $columns[ __( 'Disable Wallet Addon For this Role', 'wholesalex-pro' ) ] ) ) {
			$columns[ __( 'Disable Wallet Addon For this Role', 'wholesalex-pro' ) ] = 'disable_wallet';
		}

		return $columns;
	}


	public function role_importer_parse_wallet_data( $data ) {
		if ( isset( $data['credit_limit'] ) ) {
			$data['credit_limit'] = floatval( $data['credit_limit'] );
		}

		return $data;
	}


	/**
	 * Get Wallet Endpoint Title
	 *
	 * @return string
	 * @since 1.2.4
	 */
	public function get_endpoint_title() {
		$title = apply_filters( 'wholesalex_addon_wallet_menu_item_text', __( 'WholesaleX Wallet', 'wholesalex-pro' ) );
		return $title;
	}


	/**
	 * Get Wallet Endpoint
	 *
	 * @return string
	 * @since 1.2.4
	 */
	public function get_endpoint() {
		$endpoint = apply_filters( 'wholesalex_addon_wallet_my_account_menu_endpoint', 'wholesalex-wallet' );
		return $endpoint;
	}


    /**
     * Register a shortcode for show wallet available balance
     * @since 1.2.9
     */
    public function wallet_available_balance_shortcode($atts) {
        $user_id = isset($atts['user_id'])?$atts['user_id']:'';
        $class = isset($atts['class'])?$atts['class']:'';
        return "<span class='wsx_wallet_balance ".$class." '>".wholesalex_wallet()->get_wholesalex_balance($user_id)."</span>";
    }

    public function enqueue_wallet_admin_scripts() {
		if('yes'===wholesalex_pro()->get_setting('wholesalex_wallet_refund_for_all_gateway','no')) {
			$screen    = get_current_screen();
			$screen_id = $screen? $screen->id: '';
			if('shop_order' === $screen_id) {
				// Load Order Page Scripts
				global $post;
				$order_id = $post->ID;
				$order = wc_get_order($order_id);
				$eligible_for_wallet_refund = apply_filters( 'wholesalex_wallet_eligible_for_wallet_refund',true, $order );
				wp_enqueue_script( 'whx_wallet_admin_order_script');
				$data = array(
					'order_id' => $order_id,
					'eligible_for_wallet_refund' => $eligible_for_wallet_refund,
					'amount_placeholder' => wc_price( 0 ),
					'i18n' => array(
						'refund' => __('Refund','wholesalex-pro'),
						'to_wholesalex_wallet' => __('to wallet','wholesalex-pro'),
					)
				);
				wp_add_inline_script('whx_wallet_admin_order_script', 'const wholesalex_wallet_admin_order_data='.wp_json_encode($data),'before' );
			}
		}

    }

    /**
     * Make Wallet Route Whitelist
     *
     * @param WP_Error $error Error.
     * @return true|WP_Error
     * @since 1.0.3
     */
    public function make_restapi_whitelist( $error ) {
        if ( is_wp_error( $error ) ) {
            $rest_route = $GLOBALS['wp']->query_vars['rest_route'];
            if ( '/wholesalex/v1/wallet_action' === $rest_route ) {
                return true;
            }
            return $error;
        }
        return $error;
    }

}
