<?php
/**
 * WholesaleX White Label
 *
 * @package WHOLESALEX_PRO
 * @since 1.2.5
 */

namespace WHOLESALEX_PRO;
use WHOLESALEX\Scripts;


if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * WhiteLabel Class
 *
 * @since 1.2.5
 */
class WhiteLabel {


	/**
	 * Constructor
	 */
	public function __construct() {
        add_filter( 'wholesalex_setting_fields', array($this,'whitelable_settings') );

        add_filter('all_plugins', array($this, 'change_plugin_meta_data'));
        add_filter('wholesalex_addons_config', array($this, 'all_addons'));

        add_filter('plugin_row_meta', array($this, 'change_plugin_row_meta'),10,2);

        add_filter('wholesalex_menu_icon', array($this, 'change_plugin_menu_icon'));

        add_filter('wholesalex_logo_url', array($this, 'change_logo_url'));
        
        add_filter('wholesalex_plugin_name', array($this,'plugin_name'));

        add_filter('wholesalex_plugin_menu_name', array($this,'plugin_menu_name'));

        add_filter('wholesalex_plugin_menu_slug', array($this, 'plugin_menu_slug'));

        add_filter('wholesalex_dynamic_rules_submenu_slug', array($this, 'dynamic_rule_submenu_slug'));

        add_filter('wholesalex_emails_submenu_slug', array($this, 'emails_submenu_slug'));

        add_filter('wholesalex_registration_form_builder_submenu_slug', array($this, 'registration_form_buidler_submenu_slug'));

        add_filter('wholesalex_role_submenu_slug', array($this, 'role_submenu_slug'));

        add_filter('wholesalex_settings_submenu_slug', array($this, 'settings_submenu_slug'));

        add_filter('wholesalex_users_submenu_slug', array($this, 'users_submenu_slug'));

        add_filter('wholesalex_addons_submenu_slug', array($this, 'addons_submenu_slug'));

        add_filter('wholesalex_wallet_addon_payment_gateway_title',array($this,'set_wallet_payment_gateway_title'));

        add_filter( 'wholesalex_wallet_addon_payment_gateway_description', array($this, 'set_wallet_payment_gateway_desc'));

        add_filter('wholesalex_help_center_menu_page_enabled','__return_false');

        add_filter('wholesalex_features_menu_page_enabled','__return_false');

        add_filter('wholesalex_setup_wizard_menu_page_enabled','__return_false');

        if(version_compare(WHOLESALEX_VER,'1.3.0','<')) {
            $registration_page_slug = wholesalex()->get_setting('registration_form_buidler_submenu_slug');
            if(''!=$registration_page_slug) {
                add_shortcode($registration_page_slug.'_login_registration', array($this,'login_registration_shortcode'));
                add_shortcode($registration_page_slug.'_registration', array($this,'registration_shortcode'));
            }
        }

        add_filter('wholesalex_addons_config', array($this, 'remove_whitelabel_addon'));

		add_filter('wholesalex_addon_conversation_endpoint', array($this,'conversation_menu_endpoint'));

        add_filter('wholesalex_role_fields',array($this,'remove_role_id_section'));

        add_filter('wholesalex_bulkorder_addon_shortcode', array($this,'bulkorder_form_shortcode'));

        add_filter('woocommerce_email_title', array($this,'change_email_title'));
        
        add_filter('woocommerce_email_description', array($this,'change_email_title'));
        
        if ( defined('WHOLESALEX_BASE') ) {
            add_filter( 'plugin_action_links_' . WHOLESALEX_BASE, array( $this, 'plugin_action_links_callback' ) );
        }
       
	}

    /**
     * Settings WholesaleX Update Link
     *
     * @since v.1.4.3
     * @return array
     */
    public function plugin_action_links_callback( $links ) {
        $links['wholesalex_settings'] = '<a href="' . esc_url( admin_url( 'admin.php?page='. wholesalex()->get_setting('settings_submenu_slug', 'wholesalex-settings') ) ) . '">' . esc_html__( 'Settings', 'wholesalex-pro' ) . '</a>';
        return $links;
    }

    /**
	 * Get All Addons
	 *
	 * @return array
	 */
	public function all_addons( $addons ) {
        if ( ''!=wholesalex()->get_setting('free_plugin_name') ) {
            if ( isset( $addons['wsx_addon_wallet']['name'] ) ){
                $addons['wsx_addon_wallet']['name'] = str_ireplace(["wholesalex", "Wholesale"], wholesalex()->get_setting('free_plugin_name') , $addons['wsx_addon_wallet']['name'] );
            }
            if ( isset( $addons['wsx_addon_wcfm_integration']['name'] ) ){
                $addons['wsx_addon_wcfm_integration']['name'] = str_ireplace(["wholesalex", "Wholesale"], wholesalex()->get_setting('free_plugin_name'), $addons['wsx_addon_wcfm_integration']['name'] );
            }
            if ( isset( $addons['wsx_addon_dokan_integration']['name'] ) ){
                $addons['wsx_addon_dokan_integration']['name'] = str_ireplace(["wholesalex", "Wholesale"], wholesalex()->get_setting('free_plugin_name'), $addons['wsx_addon_dokan_integration']['name'] );
            }
        }
        foreach ($addons as &$addon) {
            if ( isset($addon['docs']) ) {
                $addon['docs'] = '';
            }
            if ( isset($addon['video']) ) {
                $addon['video'] = '';
            }
        }
        unset($addon);
        
		return $addons;
	}


    /**
     * Change Plugin Meta Data on Plugins Page
     *
     * @param array $plugins Plugins
     * @return array
     */
    public function change_plugin_meta_data( $plugins ) {
        $free_data = $plugins['wholesalex/wholesalex.php'];
        $pro_data = $plugins['wholesalex-pro/wholesalex-pro.php'];

        //Change Free Plugin Name
        if(''!=wholesalex()->get_setting('free_plugin_name')) {
            $free_data['Name'] = wholesalex()->get_setting('free_plugin_name');
            $wholesalex_plugin_name = wholesalex()->get_setting('free_plugin_name');
            $plugins_to_check = [
                'wholesalex-wcfm-b2b-multivendor-marketplace/wholesalex-wcfm-b2b-multivendor-marketplace.php',
                'multi-vendor-marketplace-b2b-for-wholesalex-dokan/multi-vendor-marketplace-b2b-for-wholesalex-dokan.php'
            ];
            foreach ( $plugins_to_check as $plugin_slug ) {
                if ( isset( $plugins[$plugin_slug] ) ) {
                    $integration_data = $plugins[$plugin_slug];
                    $integration_data['Name'] = str_ireplace(["wholesalex", "Wholesale"], $wholesalex_plugin_name, $integration_data['Name']);
                    $integration_data['Title'] = str_ireplace(["wholesalex", "Wholesale"], $wholesalex_plugin_name, $integration_data['Title']);
                    $integration_data['Author'] = str_ireplace(["wholesalex", "Wholesale"], $wholesalex_plugin_name, $integration_data['Author']);
                    $integration_data['AuthorURI'] = $free_data['AuthorURI'];
                    $integration_data['Description'] = str_ireplace(["wholesalex", "Wholesale"], $wholesalex_plugin_name, $integration_data['Description']);
                    $plugins[$plugin_slug] = $integration_data;
                }
            }
        }
        //Change Free Plugin Title
        if(''!=wholesalex()->get_setting('free_plugin_name')) {
            $free_data['Title'] = wholesalex()->get_setting('free_plugin_name');
        }
        //Change Free Plugin Author Name
        if(''!=wholesalex()->get_setting('free_plugin_author_name')) {
            $free_data['Author'] = wholesalex()->get_setting('free_plugin_author_name');
        }
        //Change Free Plugin Author URL
        if(''!=wholesalex()->get_setting('free_plugin_author_url')) {
            $free_data['AuthorURI'] = wholesalex()->get_setting('free_plugin_author_url');
        }
        //Change Free Plugin Name
        if(''!=wholesalex()->get_setting('free_plugin_description')) {
            $free_data['Description'] = wholesalex()->get_setting('free_plugin_description');
        }

        //Change Pro Plugin Name
        if(''!=wholesalex()->get_setting('pro_plugin_name')) {
            $pro_data['Name'] = wholesalex()->get_setting('pro_plugin_name');
        }
        //Change Pro Plugin Title
        if(''!=wholesalex()->get_setting('pro_plugin_name')) {
            $pro_data['Title'] = wholesalex()->get_setting('pro_plugin_name');
        }
        //Change Pro Plugin Author Name
        if(''!=wholesalex()->get_setting('pro_plugin_author_name')) {
            $pro_data['Author'] = wholesalex()->get_setting('pro_plugin_author_name');
        }
        //Change Pro Plugin Author URL
        if(''!=wholesalex()->get_setting('pro_plugin_author_url')) {
            $pro_data['AuthorURI'] = wholesalex()->get_setting('pro_plugin_author_url');
        }
        //Change Pro Plugin Name
        if(''!=wholesalex()->get_setting('pro_plugin_description')) {
            $pro_data['Description'] = wholesalex()->get_setting('pro_plugin_description');
        }

        $plugins['wholesalex/wholesalex.php'] = $free_data;
        $plugins['wholesalex-pro/wholesalex-pro.php'] = $pro_data;

        return $plugins;
    }


    /**
     * Change Plugin Row Meta
     *
     * @param array $links Plugin Row Meta Links.
     * @param string $file plugin file name.
     * @return array
     */
    public function change_plugin_row_meta($links, $file) {
        if ( strpos( $file, 'wholesalex.php' ) !== false ) {
            if(isset($links['wholesalex_docs'])) {
                unset($links['wholesalex_docs']);
            }
            if(isset($links['wholesalex_support'])) {
                unset($links['wholesalex_support']);
            }

            if(isset($links[2])) {
                unset($links[2]);
            }
		}

        if ( strpos( $file, 'wholesalex-pro.php' ) !== false ) {
            if(isset($links[2])) {
                unset($links[2]);
            }
		}
		return $links;
        
    }


    /**
     * WholesaleX Menu Icon
     *
     * @param string|url $icon Icon URL/ Dashicon Class Name
     * @return string|url
     */
    public function change_plugin_menu_icon($icon) {
        //Change Plugin Icon
        if(''!=wholesalex()->get_setting('plugin_icon')) {
            $icon = wholesalex()->get_setting('plugin_icon');
        }
        return $icon;
    }

    /**
     * Change Plugin Logo
     *
     * @param string $url Logo URL
     * @return string
     */
    public function change_logo_url($url) {
        //Change Plugin Logo
        if(''!=wholesalex()->get_setting('plugin_logo')) {
            $url = wholesalex()->get_setting('plugin_logo');
        }
        return $url;
    }


    /**
     * Whitelabel Settings Section
     *
     * @param array $sections WholesaleX Settings Sections
     * @return array
     * @since 1.2.5
     */
    public function whitelable_settings($sections) {
        $white_label_sections= array(
            'whitelabel' => array(
                'label' => __( 'White Label', 'wholesalex-pro' ),
                'attr'  => array(
                    'plugin_menu_name'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Plugin Menu Name', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'plugin_menu_slug'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Plugin Menu Slug', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'free_plugin_name'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Free Plugin Name', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'free_plugin_author_name'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Free Plugin Developer Name', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => __('WPXPO','wholesalex'),
                    ),
                    'free_plugin_author_url'     => array(
                        'type'        => 'url',
                        'label'       => __( 'Free Plugin Developer URL', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'free_plugin_description'     => array(
                        'type'        => 'textarea',
                        'label'       => __( 'Free Plugin Description', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'pro_plugin_name'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Pro Plugin Name', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'pro_plugin_author_name'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Pro Plugin Developer Name', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => __('WPXPO','wholesalex'),
                    ),
                    'pro_plugin_author_url'     => array(
                        'type'        => 'url',
                        'label'       => __( 'Pro Plugin Developer URL', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'pro_plugin_description'     => array(
                        'type'        => 'textarea',
                        'label'       => __( 'Pro Plugin Description', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'plugin_logo'     => array(
                        'type'        => 'url',
                        'label'       => __( 'Plugin Logo URL', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'plugin_icon'     => array(
                        'type'        => 'url',
                        'label'       => __( 'Plugin Icon URL', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'dynamic_rule_submenu_slug'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Dynamic Rule Submenu Slug', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'emails_submenu_slug'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Emails Submenu Slug', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'registration_form_buidler_submenu_slug'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Registration Form Builder Slug', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'role_submenu_slug'     => array(
                        'type'        => 'text',
                        'label'       => __( 'WholesaleX Roles Slug', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'settings_submenu_slug'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Settings Submenu Slug', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'users_submenu_slug'     => array(
                        'type'        => 'text',
                        'label'       => __( 'WholesaleX Users Slug', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'addons_submenu_slug'     => array(
                        'type'        => 'text',
                        'label'       => __( 'WholesaleX Addons Slug', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => '',
                    ),
                    'bulkorder_form_shortcode'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Bulkorder Form Shortcode', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => 'wholesalex_bulkorder_form',
                    ),
                    'wallet_payment_gateway_title'     => array(
                        'type'        => 'text',
                        'label'       => __( 'Wallet Payment Gateway Title', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     => __('WholesaleX Wallet','wholesalex'),
                    ),
                    'wallet_payment_gateway_description'     => array(
                        'type'        => 'textarea',
                        'label'       => __( 'Wallet Payment Description', 'wholesalex-pro' ),
                        'placeholder' => '',
                        'help'        => '',
                        'default'     =>  __( 'Enable the WholesaleX Wallet to let the customers add funds via the wallet and use it for future payments.', 'wholesalex-pro' ),
                    ),
                ),
            )
           
        );

        $is_whitelabel_addon_remove = apply_filters('wholesalex_whitelabel_addon_remove',false);
        if(!$is_whitelabel_addon_remove) {
            $sections = wholesalex()->insert_into_array( $sections, $white_label_sections);
        }
        return $sections;

    }




    public function set_wallet_payment_gateway_title($title) {
        if(''!=wholesalex()->get_setting('wallet_payment_gateway_title',__( 'WholesaleX Wallet', 'wholesalex-pro' ))) {
            $title = wholesalex()->get_setting('wallet_payment_gateway_title',__( 'WholesaleX Wallet', 'wholesalex-pro' ) );
        }
        return $title;
    }

    public function set_wallet_payment_gateway_desc($desc) {
        $default = __( 'Enable the WholesaleX Wallet to let the customers add funds via the wallet and use it for future payments.', 'wholesalex-pro' );
        if(''!=wholesalex()->get_setting('wallet_payment_gateway_description',$default)) {
            $desc = wholesalex()->get_setting('wallet_payment_gateway_description',$default );
        }
        return $desc;
    }


    public function plugin_name($name) {
        if(''!=wholesalex()->get_setting('free_plugin_name')) {
            $name = wholesalex()->get_setting('free_plugin_name');
        }
        return $name;
    }
    public function plugin_menu_name($name) {
        if(''!=wholesalex()->get_setting('plugin_menu_name')) {
            $name = wholesalex()->get_setting('plugin_menu_name');
        }
        return $name;
    }

    public function plugin_menu_slug($slug) {
        if(''!=wholesalex()->get_setting('plugin_menu_slug')) {
            $slug = wholesalex()->get_setting('plugin_menu_slug');
        }
        return $slug;
    }
    public function dynamic_rule_submenu_slug($slug) {
        if(''!=wholesalex()->get_setting('dynamic_rule_submenu_slug')) {
            $slug = wholesalex()->get_setting('dynamic_rule_submenu_slug');
        }
        return $slug;
    }
    public function emails_submenu_slug($slug) {
        if(''!=wholesalex()->get_setting('emails_submenu_slug')) {
            $slug = wholesalex()->get_setting('emails_submenu_slug');
        }
        return $slug;
    }
    public function registration_form_buidler_submenu_slug($slug) {
        if(''!=wholesalex()->get_setting('registration_form_buidler_submenu_slug')) {
            $slug = wholesalex()->get_setting('registration_form_buidler_submenu_slug');
        }
        return $slug;
    }
    public function role_submenu_slug($slug) {
        if(''!=wholesalex()->get_setting('role_submenu_slug')) {
            $slug = wholesalex()->get_setting('role_submenu_slug');
        }
        return $slug;
    }
    public function settings_submenu_slug($slug) {
        if(''!=wholesalex()->get_setting('settings_submenu_slug')) {
            $slug = wholesalex()->get_setting('settings_submenu_slug');
        }
        return $slug;
    }
    public function users_submenu_slug($slug) {
        if(''!=wholesalex()->get_setting('users_submenu_slug')) {
            $slug = wholesalex()->get_setting('users_submenu_slug');
        }
        return $slug;
    }
    public function addons_submenu_slug($slug) {
        if(''!=wholesalex()->get_setting('addons_submenu_slug')) {
            $slug = wholesalex()->get_setting('addons_submenu_slug');
        }
        return $slug;
    }

    /**
	 * Login And Registration Form
	 *
	 * @param  array $atts Shortcode Attributes. Default Empty.
	 * @return string Shortcode output.
	 * @since 1.0.1
	 * @since 1.2.4 _settings_redirect_url_login Field Deafult Settings Param Added
	 */
	public function login_registration_shortcode( $atts = array() ) {
		if ( is_admin() ) {
			return;
		}

		if ( ! wp_script_is( 'wholesalex_form', 'registered' ) ) {
			Scripts::register_frontend_scripts();
		}

		/**
		 * Enqueue Script
		 *
		 * @since 1.1.0 Enqueue Script (Reconfigure Build File)
		 */
		wp_enqueue_script( 'wholesalex_form' );

		if ( is_user_logged_in() && is_singular() ) {
			$__form_view_for_logged_in_user = wholesalex()->get_setting( '_settings_show_form_for_logged_in' );
			$__message_for_logged_in_user   = wholesalex()->get_setting( '_settings_message_for_logged_in_user' );
			if ( 'yes' !== $__form_view_for_logged_in_user ) {
				if ( is_admin() || ! function_exists( 'wc_add_notice' ) || ! function_exists( 'wc_print_notices' ) ) {
					return;
				}
				?> <div> 
				<?php
				wc_add_notice( $__message_for_logged_in_user, 'error' );
				wc_print_notices();
				?>
				<a href="<?php echo esc_url_raw( wp_logout_url( get_permalink() ) ); ?>"><?php echo esc_html( wholesalex()->get_language_n_text( '_language_logout_to_see_this_form', __( 'Logout to See this form', 'wholesalex-pro' ) ) ); ?></a>
				</div>
				<?php
				return;
			}
		}
		
		if ( isset( $atts['registration_role'] ) ) {
			
			if('all_b2b'===$atts['registration_role']) {
				wp_localize_script(
					'wholesalex_form',
					'builder',
					array(
						'form_data'         => wholesalex()->get_form_data('','registration',true),
						'registration_role' => '',
					)
				);
			} else {
				wp_localize_script(
					'wholesalex_form',
					'builder',
					array(
						'form_data'         => wholesalex()->get_form_data( $atts['registration_role'] ),
						'registration_role' => $atts['registration_role'],
					)
				);
			}

		} 
		else {
			wp_localize_script(
				'wholesalex_form',
				'builder',
				array(
					'form_data'         => wholesalex()->get_form_data(),
					'registration_role' => '',
				)
			);
		}
		ob_start();
		?>
		<div id="_wholesalex_form">
			<div class="wholesalex_notice_wrapper">
				<div class="wholesalex_notices">
				<?php
				if ( function_exists( 'wc_print_notices' ) && ! is_admin() ) {
					wc_print_notices();
				}
				?>
				</div>
			</div>
			<div  class="wholesalex_login_registration">
				<div id="wholesalex_login">
					<h2><?php echo esc_html( wholesalex()->get_language_n_text( '_language_registraion_from_combine_login_text', __( 'Login', 'wholesalex-pro' ) ) ); ?></h2>
					<?php woocommerce_login_form( array( 'redirect' => wholesalex()->get_setting( '_settings_redirect_url_login', get_permalink( get_option( 'woocommerce_shop_page_id' ) ) ) ) ); ?>
				</div>
				<div id="wholesalex_registration">
					<h2><?php echo esc_html( wholesalex()->get_language_n_text( '_language_registraion_from_combine_registration_text', __( 'Registration', 'wholesalex-pro' ) ) ); ?></h2>
						<div id="_wholesalex_registration_form" class="">
						Loading..
						</div>
				</div>
			</div>
		</div>
		<?php
		return ob_get_clean();

	}

	/**
	 * Registration Form
	 *
	 * @param array $atts    Shortcode attributes. Default empty.
	 * @return string Shortcode output.
	 * @since 1.0.0
	 */
	public function registration_shortcode( $atts = array() ) {
		do_action( 'wholesalex_before_registration_form_render' );

		/**
		 * Enqueue Script
		 *
		 * @since 1.1.0 Enqueue Script (Reconfigure Build File)
		 */
		if ( ! wp_script_is( 'wholesalex_form', 'registered' ) ) {
			Scripts::register_frontend_scripts();
		}
		wp_enqueue_script( 'wholesalex_form' );

		$atts = array_change_key_case( (array) $atts, CASE_LOWER );

		if ( is_user_logged_in() && is_singular() ) {
			$__form_view_for_logged_in_user = wholesalex()->get_setting( '_settings_show_form_for_logged_in' );
			$__message_for_logged_in_user   = wholesalex()->get_setting( '_settings_message_for_logged_in_user' );
			if ( 'yes' !== $__form_view_for_logged_in_user ) {
				if ( is_admin() || ! function_exists( 'wc_add_notice' ) || ! function_exists( 'wc_print_notices' ) ) {
					return;
				}
				?>
				<div>
				<?php
				wc_add_notice( $__message_for_logged_in_user, 'error' );
				wc_print_notices();
				?>
					<a href="<?php echo esc_url_raw( wp_logout_url( get_permalink() ) ); ?>"> <?php echo esc_html( wholesalex()->get_language_n_text( '_language_logout_to_see_this_form', __( 'Logout to See this form', 'wholesalex-pro' ) ) ); ?></a>
					</div>
				<?php
				return;
			}
		}
		if ( isset( $atts['registration_role'] ) ) {
			if('all_b2b'===$atts['registration_role']) {
				wp_localize_script(
					'wholesalex_form',
					'builder',
					array(
						'form_data'         => wholesalex()->get_form_data('','registration',true),
						'registration_role' => '',
					)
				);
			} else {
				wp_localize_script(
					'wholesalex_form',
					'builder',
					array(
						'form_data'         => wholesalex()->get_form_data( $atts['registration_role'] ),
						'registration_role' => $atts['registration_role'],
					)
				);
			}

		} else {
			wp_localize_script(
				'wholesalex_form',
				'builder',
				array(
					'form_data'         => wholesalex()->get_form_data(),
					'registration_role' => '',
				)
			);
		}

		return '<div id="_wholesalex_form" class="woocommerce"><div class="wholesalex_notice_wrapper">
		<div class="wholesalex_notices"></div>
	</div></div><div id="_wholesalex_registration_form">Loading...</div>';
	}


    public function remove_whitelabel_addon($addons) {

        $is_whitelabel_addon_remove = apply_filters('wholesalex_whitelabel_addon_remove',false);
        if($is_whitelabel_addon_remove) {
            if(isset($addons['wsx_addon_whitelabel'])) {
                unset($addons['wsx_addon_whitelabel']);
            }
        }

        return $addons;
    }


    public function conversation_menu_title($title) {
		if(''!=wholesalex()->get_setting('_conversation_my_account_endpoint_title','Conversation')) {
			$title = wholesalex()->get_setting('_conversation_my_account_endpoint_title','Conversation');
		}
		return $title;
	}

	public function conversation_menu_endpoint($endpoint) {
		if(''!=wholesalex()->get_setting('_conversation_my_account_endpoint','whx-conversation')) {
			$endpoint = wholesalex()->get_setting('_conversation_my_account_endpoint','whx-conversation');
		}
		return $endpoint;
	}

    public function remove_role_id_section($sections) {
        if(isset($sections['_role']['attr']['role_id_section'])) {
            unset($sections['_role']['attr']['role_id_section']);
        }
        return $sections;
    }

    public function bulkorder_form_shortcode() {
        return wholesalex()->get_setting('bulkorder_form_shortcode','wholesalex_bulkorder_form');
    }

    public function change_email_title($title) {

        if(wholesalex_pro()->starts_with($title,__('WholesaleX:','wholesalex'))) {
            $plugin_name = apply_filters( 'wholesalex_plugin_name', __( 'WholesaleX', 'wholesalex-pro' ) );
            $title = str_replace( __('WholesaleX:','wholesalex-pro'), $plugin_name.':', $title );
        }

        return $title;
    }
}
