(function ($) {
    'use strict';
    var wholesalex_wallet_admin_order = {
        init: function () {
            if(wholesalex_wallet_admin_order_data?.eligible_for_wallet_refund) {
                const element = `<button type="button" class="wholesalex-primary-btn button-primary do-wholesalex-wallet-refund tips" data-tip="Refund to the wholesalex wallet."> ${wholesalex_wallet_admin_order_data?.i18n?.refund} <span class="wc-order-refund-amount">${wholesalex_wallet_admin_order_data?.amount_placeholder}</span> ${wholesalex_wallet_admin_order_data?.i18n?.to_wholesalex_wallet}</button>`;
                $('.refund-actions .do-manual-refund').before(element);
                $('#woocommerce-order-items').on('click', '.refund-actions .do-wholesalex-wallet-refund', wholesalex_wallet_admin_order.do_refund);

            }
        },
        do_refund: function() {
            wholesalex_wallet_admin_order.block();

            if ( window.confirm( woocommerce_admin_meta_boxes.i18n_do_refund ) ) {
                var refund_amount   = $( 'input#refund_amount' ).val();
                var refund_reason   = $( 'input#refund_reason' ).val();
                var refunded_amount = $( 'input#refunded_amount' ).val();

                // Get line item refunds
                var line_item_qtys       = {};
                var line_item_totals     = {};
                var line_item_tax_totals = {};

                $( '.refund input.refund_order_item_qty' ).each(function( index, item ) {
                    if ( $( item ).closest( 'tr' ).data( 'order_item_id' ) ) {
                        if ( item.value ) {
                            line_item_qtys[ $( item ).closest( 'tr' ).data( 'order_item_id' ) ] = item.value;
                        }
                    }
                });

                $( '.refund input.refund_line_total' ).each(function( index, item ) {
                    if ( $( item ).closest( 'tr' ).data( 'order_item_id' ) ) {
                        line_item_totals[ $( item ).closest( 'tr' ).data( 'order_item_id' ) ] = accounting.unformat(
                            item.value,
                            woocommerce_admin.mon_decimal_point
                        );
                    }
                });

                $( '.refund input.refund_line_tax' ).each(function( index, item ) {
                    if ( $( item ).closest( 'tr' ).data( 'order_item_id' ) ) {
                        var tax_id = $( item ).data( 'tax_id' );

                        if ( ! line_item_tax_totals[ $( item ).closest( 'tr' ).data( 'order_item_id' ) ] ) {
                            line_item_tax_totals[ $( item ).closest( 'tr' ).data( 'order_item_id' ) ] = {};
                        }

                        line_item_tax_totals[ $( item ).closest( 'tr' ).data( 'order_item_id' ) ][ tax_id ] = accounting.unformat(
                            item.value,
                            woocommerce_admin.mon_decimal_point
                        );
                    }
                });

                var data = {
                    action                : 'wholesalex_wallet_process_refund',
                    order_id              : woocommerce_admin_meta_boxes.post_id,
                    refund_amount         : refund_amount,
                    refunded_amount       : refunded_amount,
                    refund_reason         : refund_reason,
                    line_item_qtys        : JSON.stringify( line_item_qtys, null, '' ),
                    line_item_totals      : JSON.stringify( line_item_totals, null, '' ),
                    line_item_tax_totals  : JSON.stringify( line_item_tax_totals, null, '' ),
                    api_refund            : $( this ).is( '.do-api-refund' ),
                    restock_refunded_items: $( '#restock_refunded_items:checked' ).length ? 'true': 'false',
                    security              : woocommerce_admin_meta_boxes.order_item_nonce
                };

                $.ajax( {
                    url:     woocommerce_admin_meta_boxes.ajax_url,
                    data:    data,
                    type:    'POST',
                    success: function( response ) {
                        if ( true === response.success ) {
                            if('fully_refunded'==response.status) {
                                // Redirect to same page for show the refunded status
                                window.location.reload();
                            } else  {
                                wholesalex_wallet_admin_order.reload_items();
                            }
                        } else {
                            window.alert( response.data.error );
                            wholesalex_wallet_admin_order.reload_items();
                            wholesalex_wallet_admin_order.unblock();
                        }
                    },
                    complete: function() {
                        window.wcTracks.recordEvent( 'order_edit_refunded', {
                            order_id: data.order_id,
                            status: $( '#order_status' ).val(),
                            api_refund: data.api_refund,
                            has_reason: Boolean( data.refund_reason.length ),
                            restock: 'true' === data.restock_refunded_items
                        } );
                    }
                } );
            } else {
                wholesalex_wallet_admin_order.unblock();
            }
        },
        block: function() {
			$( '#woocommerce-order-items' ).block({
				message: null,
				overlayCSS: {
					background: '#fff',
					opacity: 0.6
				}
			});
		},

		unblock: function() {
			$( '#woocommerce-order-items' ).unblock();
		},
        reload_items: function() {
			var data = {
				order_id: woocommerce_admin_meta_boxes.post_id,
				action:   'woocommerce_load_order_items',
				security: woocommerce_admin_meta_boxes.order_item_nonce
			};

			wholesalex_wallet_admin_order.block();

			$.ajax({
				url:  woocommerce_admin_meta_boxes.ajax_url,
				data: data,
				type: 'POST',
				success: function( response ) {
					$( '#woocommerce-order-items' ).find( '.inside' ).empty();
					$( '#woocommerce-order-items' ).find( '.inside' ).append( response );
					wholesalex_wallet_admin_order.reloaded_items();
					wholesalex_wallet_admin_order.unblock();
				}
			});
		},

		reloaded_items: function() {
			wholesalex_wallet_admin_order.init_tiptip();
			wholesalex_wallet_admin_order.stupidtable.init();
		},
        init_tiptip: function () {
            $('#tiptip_holder').removeAttr('style');
            $('#tiptip_arrow').removeAttr('style');
            $('.tips').tipTip({
                'attribute': 'data-tip',
                'fadeIn': 50,
                'fadeOut': 50,
                'delay': 200
            });
        },
        stupidtable: {
			init: function() {
				$( '.woocommerce_order_items' ).stupidtable();
				$( '.woocommerce_order_items' ).on( 'aftertablesort', this.add_arrows );
			},

			add_arrows: function( event, data ) {
				var th    = $( this ).find( 'th' );
				var arrow = data.direction === 'asc' ? '&uarr;' : '&darr;';
				var index = data.column;
				th.find( '.wc-arrow' ).remove();
				th.eq( index ).append( '<span class="wc-arrow">' + arrow + '</span>' );
			}
		}
    };
    wholesalex_wallet_admin_order.init();
})(jQuery);