<?php //phpcs:ignore
namespace WHOLESALEX_PRO;


defined( 'ABSPATH' ) || exit;
/**
 * Category Class
 */
class Category {

	/**
	 * Constructor
	 */
	public function __construct() {
		if ( ! wholesalex_pro()->is_active() ) {
			return;
		}

		add_filter( 'wholesalex_category_b2b_roles_tier_fields', array( $this, 'category_role_based_discounts_fields' ), 1 );

		add_filter( 'wholesalex_category_discount_action', array( $this, 'category_discounts' ) );
	}


	/**
	 * Category Role Based Discount Fields
	 *
	 * @param array $field Fields.
	 * @return array
	 */
	public function category_role_based_discounts_fields( $field ) {
		if ( ! ( wholesalex_pro()->is_free_ready() && version_compare( WHOLESALEX_VER, '1.0.10', '<' ) ) ) {
			return $field;
		}
		$__roles_options = wholesalex()->get_roles( 'b2b_roles_option' );
		$__b2b_roles     = array();
		foreach ( $__roles_options as $role ) {
			if ( ! ( isset( $role['value'] ) && isset( $role['name'] ) ) ) {
				continue;
			}
			$__b2b_roles[ $role['value'] ] = array(
				'label'  => $role['name'],
				'type'   => 'tiers',
				'is_pro' => true,
				'attr'   =>
				array(
					$role['value'] . 'tier' => array(
						'type'   => 'tier',
						'_tiers' => array(
							'columns' => array(
								__( 'Discount Type', 'wholesalex-pro' ),
								/* translators: %s: WholesaleX Role Name */
								sprintf( __( ' %s Price', 'wholesalex-pro' ), $role['name'] ),
								__( 'Min Quantity', 'wholesalex-pro' ),
							),
							'data'    => array(
								'_discount_type'   => array(
									'type'    => 'select',
									'options' => array(
										''            => __( 'Choose Discount Type...', 'wholesalex-pro' ),
										'amount'      => __( 'Discount Amount', 'wholesalex-pro' ),
										'percentage'  => __( 'Discount Percentage', 'wholesalex-pro' ),
										'fixed_price' => __( 'Fixed Price', 'wholesalex-pro' ),
									),
									'default' => '',
								),
								'_discount_amount' => array(
									'type'        => 'number',
									'placeholder' => '',
									'default'     => '',
								),
								'_min_quantity'    => array(
									'type'        => 'number',
									'placeholder' => '',
									'default'     => '',
								),
							),
							'add'     => array(
								'type'  => 'button',
								'label' => __( 'Add Price Tier', 'wholesalex-pro' ),
							),
						),
					),
				),
			);

		}
		return $__b2b_roles;
	}


	/**
	 * Category Discounts
	 *
	 * @param array $data Data.
	 * @return array
	 * @since 1.0.0
	 * @since 1.0.3 ProductX Currency Switcher Compatibility Fixed
	 */
	public function category_discounts( $data ) {
		if ( ! ( wholesalex_pro()->is_free_ready() && version_compare( WHOLESALEX_VER, '1.0.10', '<' ) ) ) {
			return $data;
		}
		
		$sale_price = $data['sale_price'];
		$product    = $data['product'];

		$__override_sale_price = false;
		$__parent_id           = $product->get_parent_id();
		$__product_id          = $product->get_id();
		$__user_id             = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );

		// delete_transient( 'wholesalex_pricing_tiers_category_' . $__user_id );

		$__discounts = wholesalex()->get_discounts( 'category', 0 !== $__parent_id ? $__parent_id : $__product_id, );
		if ( empty( $__discounts ) ) {
			return $data;
		}
		$__tiers  = $__discounts['tiers'];
		$__cat_id = $__discounts['cat_id'];

		/**
		 * ProductX Currency Switcher Compatibility Fixed
		 *
		 * @since 1.0.3
		 */
		$__regular_price = wholesalex_pro()->price_after_currency_changed( $product->get_regular_price() );

		// If product category has no discounts, then return as usual sale price.
		if ( empty( $__tiers ) ) {
			return $data;
		}

		/**
		 * Hook Added For Support Bulk Order Addon (wholesalex_category_discounts_cart_count)
		 *
		 * @since 1.0.4
		 */
		$__quantity = apply_filters( 'wholesalex_category_discounts_cart_count', 0, $__cat_id, ( 0 !== $__parent_id ) ? $__parent_id : $__product_id );
		$__quantity = $__quantity ? $__quantity : wholesalex()->category_cart_count( $__cat_id );

		foreach ( $__tiers as $tier ) {
			$__min_quantity = isset( $tier['_min_quantity'] ) ? $tier['_min_quantity'] : 999999999;
			if ( $__quantity >= $__min_quantity ) {
				$sale_price             = wholesalex()->calculate_sale_price( $tier, $__regular_price );
				$__override_sale_price  = true;
				$data['active_tier_id'] = isset( $tier['_id'] ) ? $tier['_id'] : '';
			}
		}

		$data['sale_price'] = $sale_price;

		set_transient( 'wholesalex_pricing_tiers_category_' . $__user_id, $__tiers );

		if ( $__override_sale_price ) {
			remove_all_filters( 'woocommerce_sale_flash' );
			$data['discount_src'] = 'category';
		}

		return $data;

	}
}
