<?php //phpcs:ignore
namespace WHOLESALEX_PRO;


defined( 'ABSPATH' ) || exit;

use WHOLESALEX\WHOLESALEX_Dynamic_Rules;

/**
 * Functions Class
 */
class DynamicRules {

	/**
	 * Dynamic Rules Min Order Quantity Session Name
	 *
	 * @var string
	 * @since 1.0.6
	 */
	public $min_order_qty_session_name = '';
	/**
	 * Dynamic Rules Max Order Quantity Session Name
	 *
	 * @var string
	 * @since 1.0.6
	 */
	public $max_order_qty_session_name = '';

	/**
	 * Restrict Product Visibility Include Products
	 *
	 * @var array
	 * @since 1.0.8
	 */
	public $rpv_include_products = array();
	/**
	 * Restrict Product Visibility Include Categories
	 *
	 * @var array
	 * @since 1.0.8
	 */
	public $rpv_include_cats = array();
	/**
	 * Restrict Product Visibility Include Product Variation
	 *
	 * @var array
	 * @since 1.0.8
	 */
	public $rpv_include_variations = array();
	/**
	 * Restrict Product Visibility Exclude Products
	 *
	 * @var array
	 * @since 1.0.8
	 */
	public $rpv_exclude_products = array();
	/**
	 * Restrict Product Visibility Exclude Categories
	 *
	 * @var array
	 * @since 1.0.8
	 */
	public $rpv_exclude_cats = array();
	/**
	 * Restrict Product Visibility Exclude Product Variations
	 *
	 * @var array
	 * @since 1.0.8
	 */
	public $rpv_exclude_variations = array();
	/**
	 * Is all product hidden
	 *
	 * @var array
	 * @since 1.0.8
	 */
	public $rpv_is_all_products = false;

	/**
	 * Rule Wise Products
	 *
	 * @var array
	 * @since 1.0.9
	 */
	public $rules_products = array();

	/**
	 * Contain BXGY Rules
	 *
	 * @var array
	 */
	public $bxgy_rules = array();



	/**
	 * Current User ID.
	 *
	 * @var string
	 * @since 1.0.9
	 */
	public $current_user_id = '';

	private $valid_rules = array();

	/**
	 * Constructor
	 */
	public function __construct() {
		if ( ! wholesalex_pro()->is_active() ) {
			return;
		}

		add_filter( 'wholesalex_dynamic_rules_field', array( $this, 'dynamic_rule_fields' ), 1 );

		/**
		 * Dynamic Rules Condtions : User Order Count
		 *
		 * @since 1.0.4
		 */
		add_filter( 'wholesalex_dynamic_rules_condition_options', array( $this, 'options_pro_remove' ), 20 );

		add_filter( 'wholesalex_dynamic_rules_rule_type_options', array( $this, 'options_pro_remove' ), 20 );

		add_filter( 'wholesalex_dynamic_rules_rule_for_options', array( $this, 'options_pro_remove' ), 20 );

		add_filter( 'wholesalex_dynamic_rules_product_filter_options', array( $this, 'options_pro_remove' ), 20 );

		add_filter( 'wholesalex_dynamic_rule_quantity_based_action', array( $this, 'quantity_based_rules' ), 20 );

		add_action( 'wholesalex_dynamic_rules_extra_charge_action', array( $this, 'extra_charge_rules' ), 1, 3 );

		/**
		 * Perform Actions After Updating Dynamic Rules.
		 *
		 * @since 1.0.6
		 */
		add_action( 'wholesalex_dynamic_rules_updated', array( $this, 'dynamic_rules_updated' ) );

		/**
		 * Set Restrict Product Visibility Pricing Plan
		 *
		 * @since 1.0.8
		 */

		/**
		 * Dynamic Rules: Minimum Order Quantity Rule Added
		 *                Maximum Order Quantity Rule Added
		 *
		 * @since 1.0.6
		 */
		/**
		 * Handle Dynamic Rules: Restrict Product Visibility
		 *
		 * @since 1.0.8
		 */

		// add_action( 'wp_loaded', array( $this, 'handle_dynamic_rules' ) );

		$this->current_user_id = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );

		add_filter( 'woocommerce_product_subcategories_args', array( $this, 'exclude_category_from_shop' ) );

		add_action('woocommerce_blocks_enqueue_cart_block_scripts_after',function(){
			wp_enqueue_script( 'wholesalex_cart_discounts', WHOLESALEX_PRO_URL . 'assets/js/whx_cart_discounts.js', array('jquery'), WHOLESALEX_PRO_VER ,true );
		});

		add_action('wholesalex_valid_dynamic_rules',array($this,'handle_pro_dynamic_rules'));

	}

	/**
	 * Exclude Hidden Category From Catalog Mode Category Lists
	 *
	 * @param array $args Args
	 * @return array
	 */
	public function exclude_category_from_shop( $args ) {
		if ( ! empty( $this->rpv_include_cats ) ) {
			$args['exclude'] = $this->rpv_include_cats;
		}
		if ( ! empty( $this->rpv_exclude_cats ) ) {
			$args['include'] = $this->rpv_exclude_cats;
		}
		return $args;
	}


	/**
	 * Profile Discounts Fields
	 *
	 * @param array $sections Dynamic Rule Sections.
	 * @since 1.0.0
	 * @since 1.0.4 Buy X Get Y Added
	 */
	public function dynamic_rule_fields( $sections ) {
		$__sections = array(
			'quantity_based' => array(
				'label'      => __( 'Quantity Based Discount', 'wholesalex-pro' ),
				'type'       => 'tiers',
				'depends_on' => array(
					array(
						'key'   => '_rule_type',
						'value' => 'quantity_based',
					),
				),
				'attr'       =>
				array(
					'_quantity_based_tier' => array(
						'type'   => 'tier',
						'_tiers' => array(
							'columns' => array(
								__( 'Discount Type', 'wholesalex-pro' ),
								__( 'Min Quantity', 'wholesalex-pro' ),
								__( 'Amount', 'wholesalex-pro' ),
								__( 'Disc. name(optional)', 'wholesalex-pro' ),
							),
							'data'    => array(
								'_discount_type'   => array(
									'type'    => 'select',
									'options' => array(
										''           => __( 'Choose Discount Type...', 'wholesalex-pro' ),
										'amount'     => __( 'Discount Amount', 'wholesalex-pro' ),
										'percentage' => __( 'Discount Percentage', 'wholesalex-pro' ),
										'fixed'      => __( 'Fixed Price', 'wholesalex-pro' ),
									),
									'default' => '',
									'label'   => __( 'Discount Type', 'wholesalex-pro' ),
								),
								'_min_quantity'    => array(
									'type'        => 'number',
									'placeholder' => '',
									'default'     => '',
									'label'       => __( 'Min Quantity', 'wholesalex-pro' ),
								),
								'_discount_amount' => array(
									'type'        => 'number',
									'placeholder' => '',
									'default'     => '',
									'label'       => __( 'Amount', 'wholesalex-pro' ),
								),
								'_discount_name'   => array(
									'type'        => 'text',
									'placeholder' => '',
									'default'     => '',
									'label'       => __( 'Disc. name(optional)', 'wholesalex-pro' ),
								),
							),
						),
					),
				),
			),
			'extra_charge'   => array(
				'label'      => __( 'Extra Charge', 'wholesalex-pro' ),
				'type'       => 'payment_extra_charge',
				'depends_on' => array(
					array(
						'key'   => '_rule_type',
						'value' => 'extra_charge',
					),
				),
				'attr'       => array(
					'_payment_gateways' => array(
						'type'        => 'multiselect',
						'label'       => __( 'Payment Gateways', 'wholesalex-pro' ),
						// 'options'     => $payment_gateways,
						'options'     => array(),
						'default'     => '',
						'placeholder' => '',
						'help'        => '',
						'is_ajax'     => true,
						'ajax_action' => 'get_payment_gateways',
						'ajax_search' => false,
					),
					'_charge_type'      => array(
						'type'    => 'select',
						'label'   => __( 'Charge Type', 'wholesalex-pro' ),
						'options' => array(
							'percentage' => __( 'Percentage', 'wholesalex-pro' ),
							'amount'     => __( 'Amount', 'wholesalex-pro' ),
						),
						'default' => 'percentage',
					),
					'_charge_amount'    => array(
						'type'        => 'number',
						'label'       => __( 'Amount', 'wholesalex-pro' ),
						'default'     => '',
						'placeholder' => '',
						'help'        => '',
					),
					'_charge_name'      => array(
						'type'        => 'text',
						'label'       => __( 'Extra Charge name(optional)', 'wholesalex-pro' ),
						'default'     => '',
						'placeholder' => __( 'Add Extra Charge Name here', 'wholesalex-pro' ),
						'help'        => '',
					),
				),
			),
			'buy_x_get_y'    => array(
				'label'      => __( 'Buy X Get Y', 'wholesalex-pro' ),
				'type'       => 'buy_x_get_y',
				'depends_on' => array(
					array(
						'key'   => '_rule_type',
						'value' => 'buy_x_get_y',
					),
				),
				'attr'       => array(
					'_minimum_purchase_count' => array(
						'type'        => 'number',
						'label'       => __( 'Minimum Product Quantity (X)', 'wholesalex-pro' ),
						'default'     => '',
						'placeholder' => '',
						'help'        => '',
					),
					'_free_item'              => array(
						'type'        => 'multiselect',
						'label'       => __( 'Free Products', 'wholesalex-pro' ),
						// 'options'     => $products,
						'options'     => array(),
						'default'     => '',
						'placeholder' => '',
						'help'        => '',
						'is_ajax'     => true,
						'ajax_action' => 'get_products_with_variations',
						'ajax_search' => true,
					),
					'_free_item_count'        => array(
						'type'        => 'number',
						'label'       => __( 'Free Product Quantity (Y)', 'wholesalex-pro' ),
						'default'     => '',
						'placeholder' => '',
						'help'        => '',
					),
					'_per_cart_once' => array(
						'type'        => 'switch',
						'label'		  => __('Restrict Free Quantity','wholesalex'),
						'desc'       => __( 'Restrict to one offer per order', 'wholesalex-pro' ),
						'default'     => 'no',
						'placeholder' => '',
						'descTooltip'	  => __('Enabling this option will restrict the shoppers from availing multiple free offers from adding the required number of products more than one time to the cart.','wholesalex'),
					),
				),
			),
			'max_order_qty'  => array(
				'label'      => __( 'Maximum Order Quantity', 'wholesalex-pro' ),
				'type'       => 'max_order_qty',
				'depends_on' => array(
					array(
						'key'   => '_rule_type',
						'value' => 'max_order_qty',
					),
				),
				'attr'       => array(
					'_max_order_qty' => array(
						'type'        => 'number',
						'label'       => '',
						'default'     => '',
						'placeholder' => '',
						'help'        => '',
					),
				),
			),
		);
		if ( isset( $sections['_new_rule']['attr'] ) && is_array( $sections['_new_rule']['attr'] ) ) {
			$sections['_new_rule']['attr'] = wholesalex()->insert_into_array( $sections['_new_rule']['attr'], $__sections, 3 );
		}
		return $sections;
	}

	/**
	 * Dynamic Rule Type Pro Remove
	 *
	 * @param array $options Options.
	 * @return array $options.
	 * @since 1.0.0
	 * @since 1.0.4 Buy X Get Y Added
	 * @since 1.0.6 Existing Code deleted and rewrite for new prcing plan.
	 */
	public function options_pro_remove( $options ) {
		return wholesalex_pro()->unlock_options( $options );
	}

	/**
	 * Quantity Based Rules
	 *
	 * @param array $data Data.
	 * @return $data
	 */
	public function quantity_based_rules( $data ) {
		$rule_id         = $data['id'];
		$__tiers         = $data['tiers'];
		$__product_id    = $data['product_id'];
		$__cat_ids       = $data['cat_ids'];
		$__regular_price = $data['regular_price'];
		$__for           = $data['for'];
		$__src_id        = $data['src_id'];
		$__user_id       = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );

		// Remove Empty Tiers.
		$__tiers = wholesalex()->filter_empty_tier( $__tiers );

		if ( isset( $__tiers['_min_quantity'] ) ) {
			// Sort Tiers by min quantity.
			$__sort_colum = array_column( $__tiers, '_min_quantity' );
			array_multisort( $__sort_colum, SORT_ASC, $__tiers );
		}

		delete_transient( 'wholesalex_pricing_tiers_dynamic_rule_' . $__user_id );
		$__tiers_transient = get_transient( 'wholesalex_pricing_tiers_dynamic_rule_' . $__user_id );
		if ( $__tiers_transient && ! empty( $__tiers_transient ) ) {
			$__tiers_transient = array_merge( $__tiers_transient, $__tiers );
			set_transient( 'wholesalex_pricing_tiers_dynamic_rule_' . $__user_id, $__tiers_transient );

		} else {
			$__tiers_transient = $__tiers;
			set_transient( 'wholesalex_pricing_tiers_dynamic_rule_' . $__user_id, $__tiers_transient );
		}

		foreach ( $__tiers as $tier ) {
			if ( isset( $__for ) && isset( $__src_id ) ) {
				switch ( $__for ) {
					case 'cat':
						if ( in_array( $__src_id, $__cat_ids ) ) {
							/**
							 * Hook Added For Support Bulk Order Addon (wholesalex_dynamic_rules_category_cart_count)
							 *
							 * @since 1.0.4
							 */
							$__quantity     = apply_filters( 'wholesalex_dynamic_rules_category_cart_count', 0, $__src_id, $__product_id );
							$__quantity     = $__quantity ? $__quantity : wholesalex()->category_cart_count( $__src_id );
							$__min_quantity = isset( $tier['_min_quantity'] ) ? $tier['_min_quantity'] : 999999999;
							if ( $__quantity >= $__min_quantity ) {
								$data['sale_price'] = wholesalex()->calculate_sale_price( $tier, $__regular_price );
								wholesalex()->set_usages_dynamic_rule_id( $rule_id );
								$data['override_sale_price'] = true;
								$data['active_tier_id']      = isset( $tier['_id'] ) ? $tier['_id'] : '';
								$__discount_name             = isset( $tier['_discount_name'] ) ? $tier['_discount_name'] : '';
								// If discount has any name then discount name will replace the default woocommerce sale flash text.
								if ( ! empty( $__discount_name ) ) {
									add_filter(
										'woocommerce_sale_flash',
										function () use ( $__discount_name ) {
											return '<span class="onsale">' . $__discount_name . '</span>';
										}
									);
								}
							}
						}

						break;
					case 'product':
						/**
						 * @since 1.1.7
						 */
						/**
						 * Hook Added For Support Bulk Order Addon (wholesalex_dynamic_rules_cart_product_count)
						 *
						 * @since 1.0.4
						 */
						$__quantity = apply_filters( 'wholesalex_dynamic_rules_cart_product_count', 0, $__product_id );
						$__quantity = $__quantity ? $__quantity : wholesalex_pro()->product_cart_count( $__product_id );
						if ( $__quantity >= $tier['_min_quantity'] ) {
							$data['sale_price'] = wholesalex()->calculate_sale_price( $tier, $__regular_price );
							wholesalex()->set_usages_dynamic_rule_id( $rule_id );
							$data['override_sale_price'] = true;

							$data['active_tier_id'] = isset( $tier['_id'] ) ? $tier['_id'] : '';

							$__discount_name = isset( $tier['_discount_name'] ) ? $tier['_discount_name'] : '';

							// If discount has any name then discount name will replace the default woocommerce sale flash text.
							if ( ! empty( $__discount_name ) ) {
								add_filter(
									'woocommerce_sale_flash',
									function () use ( $__discount_name ) {

										return '<span class="onsale">' . $__discount_name . '</span>';
									}
								);
							}
						}
						break;
					case 'variation':
					case 'all_products':
						/**
						 * Hook Added For Support Bulk Order Addon (wholesalex_dynamic_rules_cart_product_count)
						 *
						 * @since 1.0.4
						 */
						$__quantity = apply_filters( 'wholesalex_dynamic_rules_cart_product_count', 0, $__product_id );
						$__quantity = $__quantity ? $__quantity : wholesalex()->cart_count( $__product_id );
						if ( $__quantity >= $tier['_min_quantity'] ) {
							$data['sale_price'] = wholesalex()->calculate_sale_price( $tier, $__regular_price );
							wholesalex()->set_usages_dynamic_rule_id( $rule_id );
							$data['override_sale_price'] = true;

							$data['active_tier_id'] = isset( $tier['_id'] ) ? $tier['_id'] : '';

							$__discount_name = isset( $tier['_discount_name'] ) ? $tier['_discount_name'] : '';

							// If discount has any name then discount name will replace the default woocommerce sale flash text.
							if ( ! empty( $__discount_name ) ) {
								add_filter(
									'woocommerce_sale_flash',
									function () use ( $__discount_name ) {

										return '<span class="onsale">' . $__discount_name . '</span>';
									}
								);
							}
						}
						break;
				}
			}
		}

		return $data;

	}

	/**
	 * Payment Discount Rules
	 *
	 * @param array  $__payment_discount Payment Discounts.
	 * @param string $__rule_id Dynamic Rule ID.
	 * @param mixed  $__product_id Prouduct ID.
	 * @return void
	 */
	public function payment_discount_rules( $__payment_discount, $__rule_id, $__product_id ) {
		add_action(
			'woocommerce_cart_calculate_fees',
			function( $cart_object ) use ( $__payment_discount, $__product_id, $__rule_id ) {
				$__gateways              = isset( $__payment_discount['_payment_gateways'] ) ? $__payment_discount['_payment_gateways'] : array();
				$__chosen_payment_method = WC()->session->get( 'chosen_payment_method' );
				foreach ( $__gateways as $gateway ) {
					if ( isset( $gateway['value'] ) && $__chosen_payment_method == $gateway['value'] && isset( $__payment_discount['_discount_amount'] ) && isset( $__payment_discount['_discount_type'] ) ) {

						wholesalex()->set_usages_dynamic_rule_id( $__rule_id );

						// $__cart_total     = (float) $cart_object->cart_contents_total;
						$__total_amount   = wholesalex()->get_cart_total( $__product_id );
						$__discount_value = (float) $__payment_discount['_discount_amount'];
						$__amount         = 'percentage' === $__payment_discount['_discount_type'] ? ( $__total_amount * $__discount_value ) / 100 : $__discount_value;
						$__amount         = max( 0, min( $__total_amount, $__amount ) );
						$__discount_name  = isset( $__payment_discount['_discount_name'] ) ? $__payment_discount['_discount_name'] : apply_filters( 'wholesalex_payment_gateway_default_discount_name', __( 'Payment Discount!', 'wholesalex-pro' ) );
						$__is_taxable     = apply_filters( 'wholesalex_payment_gateway_discount_is_taxable', false );

						$cart_object->add_fee( $__discount_name, -$__amount, $__is_taxable );
					}
				}
			}
		);

		add_action(
			'woocommerce_review_order_before_payment',
			function () {
				// Only on checkout page.
				if ( is_checkout() && ! is_wc_endpoint_url( 'order-received' ) ) :
					?>
						<script type="text/javascript">
							jQuery(function($) {
								// On payment method change.
								$('form.woocommerce-checkout').on('change', 'input[name="payment_method"]', function() {
									// Refresh checkout.
									$('body').trigger('update_checkout');
								});
							})
						</script>
					<?php
			endif;
			}
		);
	}


	/**
	 * Payment Order Quantity Discount Rules.
	 *
	 * @param array $__payment_gateways Payment Gateways.
	 * @param mixed $__quantity Quantity.
	 * @param mixed $__product_id Product ID.
	 * @return void
	 */
	public function payment_order_qty_discount_rules( $__payment_gateways, $__quantity, $__product_id ) {
		$__user_id                 = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
		$__payment_order_discounts = get_transient( 'wholesalex_payment_order_quantity_discounts_' . $__user_id );
		if ( $__payment_order_discounts && ! empty( $__payment_order_discounts ) ) {
			array_push(
				$__payment_order_discounts,
				array(
					'gateways'     => $__payment_gateways,
					'product_id'   => $__product_id,
					'min_quantity' => isset( $__quantity ) ? $__quantity : 99999999,
				)
			);
			set_transient( 'wholesalex_payment_order_quantity_discounts_' . $__user_id, $__payment_order_discounts );
		} else {
			$__payment_order_discounts = array(
				'gateways'     => $__payment_gateways,
				'product_id'   => $__product_id,
				'min_quantity' => isset( $__quantity ) ? $__quantity : 99999999,
			);
			set_transient( 'wholesalex_payment_order_quantity_discounts_' . $__user_id, array( $__payment_order_discounts ) );
		}
	}

	/**
	 * Tax Rules
	 *
	 * @param array  $__rules Tax Rules.
	 * @param string $__rule_id Rule ID.
	 * @param mixed  $__product_id Product ID.
	 * @return void
	 */
	public function tax_rules( $__rules, $__rule_id, $__product_id ) {
		$__user_id = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
		if ( isset( $__rules['_tax_exempted'] ) && 'yes' === $__rules['_tax_exempted'] ) {
			wholesalex()->set_usages_dynamic_rule_id( $__rule_id );

			add_filter(
				'woocommerce_product_get_tax_status',
				function ( $tax_status, $product ) use ( $__rules, $__product_id, $__rule_id ) {
					if ( $product->get_id() === $__product_id && isset( $__rules['_tax_exempted'] ) && 'yes' === $__rules['_tax_exempted'] ) {
						wholesalex()->set_usages_dynamic_rule_id( $__rule_id );
						return 'none';
					} else {
						return $tax_status;
					}
				},
				10,
				2
			);

			add_filter(
				'woocommerce_product_variation_get_tax_status',
				function ( $tax_status, $product ) use ( $__rules, $__product_id, $__rule_id ) {
					if ( $product->get_id() === $__product_id && isset( $__rules['_tax_exempted'] ) && 'yes' === $__rules['_tax_exempted'] ) {
						wholesalex()->set_usages_dynamic_rule_id( $__rule_id );
						return 'none';
					} else {
						return $tax_status;
					}
				},
				10,
				2
			);

			// set_transient( 'wholesalex_tax_exemption_' . $__user_id, true );
		} else {
			delete_transient( 'wholesalex_tax_exemption_' . $__user_id, false );
			add_filter(
				'woocommerce_product_get_tax_class',
				function ( $tax_class, $product ) use ( $__rules, $__product_id, $__rule_id ) {
					if ( $product->get_id() === $__product_id && isset( $__rules['_tax_class'] ) ) {
						wholesalex()->set_usages_dynamic_rule_id( $__rule_id );
						return $__rules['_tax_class'];
					} else {
						return $tax_class;
					}
				},
				10,
				2
			);

			add_filter(
				'woocommerce_product_variation_get_tax_class',
				function ( $tax_class, $product ) use ( $__rules, $__product_id, $__rule_id ) {
					if ( $product->get_id() === $__product_id && $__rules['_tax_class'] ) {
						wholesalex()->set_usages_dynamic_rule_id( $__rule_id );
						return $__rules['_tax_class'];
					} else {
						return $tax_class;
					}
				},
				10,
				2
			);

		}
	}


	/**
	 * Extra Charge Rules
	 *
	 * @param array  $__rules Tax Rules.
	 * @param string $__rule_id Rule ID.
	 * @param mixed  $__product_id Product ID.
	 * @return void
	 */
	public function extra_charge_rules( $__rules, $__rule_id, $__product_id ) {
		if ( ! empty( $__rules ) ) {
			add_action(
				'woocommerce_cart_calculate_fees',
				function( $cart_object ) use ( $__rules, $__product_id, $__rule_id ) {
					$__gateways              = isset( $__rules['_payment_gateways'] ) ? $__rules['_payment_gateways'] : array();
					$__chosen_payment_method = WC()->session->get( 'chosen_payment_method' );
					foreach ( $__gateways as $gateway ) {
						if ( isset( $gateway['value'] ) && $__chosen_payment_method == $gateway['value'] && isset( $__rules['_charge_amount'] ) && $__rules['_charge_type'] ) {
							wholesalex()->set_usages_dynamic_rule_id( $__rule_id );

							$__total_amount   = wholesalex()->get_cart_total( $__product_id );
							$__discount_value = (float) $__rules['_charge_amount'];
							$__amount         = 'percentage' === $__rules['_charge_type'] ? ( $__total_amount * $__discount_value ) / 100 : $__discount_value;
							$__amount         = max( 0, min( $__total_amount, $__amount ) );
							$__discount_name  = isset( $__rules['_charge_name'] ) ? $__rules['_charge_name'] : apply_filters( 'wholesalex_payment_gateway_default_charge_name', __( 'Extra Charge', 'wholesalex-pro' ) );
							$__is_taxable     = apply_filters( 'wholesalex_payment_gateway_charge_is_taxable', false );

							$cart_object->add_fee( $__discount_name, $__amount, $__is_taxable );
						}
					}
				}
			);

			add_action(
				'woocommerce_review_order_before_payment',
				function () {
					// Only on checkout page.
					if ( is_checkout() && ! is_wc_endpoint_url( 'order-received' ) ) :
						?>
							<script type="text/javascript">
								jQuery(function($) {
									// On payment method change.
									$('form.woocommerce-checkout').on('change', 'input[name="payment_method"]', function() {
										// Refresh checkout.
										$('body').trigger('update_checkout');
									});
								})
							</script>
						<?php
				endif;
				}
			);

		}
	}

	// /**
	// * Cart Discount Rules
	// *
	// * @param array  $__rules Tax Rules.
	// * @param string $__rule_id Rule ID.
	// * @param mixed  $__product_id Product ID.
	// * @return void
	// */
	// public function cart_discount_rules( $__rules, $__rule_id, $__product_id ) {
	// $__user_id             = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
	// $__cart_discounts      = get_transient( 'wholesalex_cart_total_discounts_' . $__user_id );
	// $__cart_discount_title = apply_filters( 'wholesalex_cart_discount_title', isset( $__rules['_discount_name'] ) ? $__rules['_discount_name'] : __( 'Cart Discounts' ) );
	// $__discount_value      = 'percentage' === $__rules['_discount_type'] ? ( wholesalex()->get_cart_total( $__product_id ) * $__rules['_discount_amount'] ) / 100.00 : $__rules['_discount_amount'];
	// if ( $__cart_discounts && ! empty( $__cart_discounts ) ) {
	// if ( $__rule_id === $__cart_discounts['id'] ) {
	// $__cart_discounts['data'][ $__product_id ] = $__discount_value;
	// set_transient( 'wholesalex_cart_total_discounts_' . $__user_id, $__cart_discounts );
	// }
	// } else {
	// $__cart_discounts = array(
	// 'id'    => $__rule_id,
	// 'title' => $__cart_discount_title,
	// 'data'  => array(
	// $__product_id => $__discount_value,
	// ),
	// );
	// set_transient( 'wholesalex_cart_total_discounts_' . $__user_id, $__cart_discounts );
	// }

	// wholesalex()->set_usages_dynamic_rule_id( $__rule_id );

	// }


	/**
	 * Shipping Rules
	 *
	 * @param array  $__rules Tax Rules.
	 * @param string $__rule_id Rule ID.
	 * @param mixed  $__product_id Product ID.
	 * @return void
	 */
	public function shipping_rules( $__rules, $__rule_id, $__product_id ) {

		$__user_id = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );

		$__zone_id               = $__rules['__shipping_zone'];
		$__shipping_zone_methods = $__rules['_shipping_zone_methods'];

		$__available_methods = array();

		foreach ( $__shipping_zone_methods as $method ) {
			$__available_methods[ $method['value'] ] = true;
		}

		$__shipping_method_transient = get_transient( 'wholesalex_shipping_methods_' . $__user_id );

		if ( $__shipping_method_transient && ! empty( $__shipping_method_transient ) ) {
			if ( isset( $__shipping_method_transient[ $__zone_id ] ) ) {

				if ( isset( $__shipping_method_transient[ $__zone_id ][ $__product_id ] ) ) {
					$__existing_shipping_methods = $__shipping_method_transient[ $__zone_id ][ $__product_id ];

					foreach ( $__existing_shipping_methods as $key => $value ) {
						$__shipping_method_transient[ $__zone_id ][ $__product_id ][ $key ] = $value;
					}
				} else {
					foreach ( $__available_methods as $key => $value ) {
						$__shipping_method_transient[ $__zone_id ][ $__product_id ][ $key ] = $value;
					}
				}
			} else {
				foreach ( $__available_methods as $key => $value ) {
					$__shipping_method_transient[ $__zone_id ][ $__product_id ][ $key ] = $value;
				}
			}

			set_transient( 'wholesalex_shipping_methods_' . $__user_id, $__shipping_method_transient );
		} else {
			$__temp_shipping_data                                = array();
			$__temp_shipping_data[ $__zone_id ][ $__product_id ] = $__available_methods;
			set_transient( 'wholesalex_shipping_methods_' . $__user_id, $__temp_shipping_data );
		}

		wholesalex()->set_usages_dynamic_rule_id( $__rule_id );

	}


	/**
	 * Min order add to cart validation
	 *
	 * @param bool       $status Add to cart Status.
	 * @param int|string $product_id Product ID.
	 * @param int        $quantity Cart Quantity.
	 * @return bool
	 * @since 1.0.6
	 */
	public function min_order_add_to_cart_validation( $status, $product_id, $quantity ) {
		$current_user_id                  = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
		$this->min_order_qty_session_name = 'wholesalex_dynamic_rules_min_order_qty_' . $current_user_id;

		$variation_id = 0;
		$product      = wc_get_product( $product_id );
		$variation    = false;
		if ( ! $product->is_type( 'simple' ) ) {
			$args         = func_get_args();
			$variation_id = isset( $args[3] ) ? $args[3] : 0;
			if ( $variation_id ) {
				$variation = wc_get_product( $variation_id );
			}
		}

		$session = get_transient( $this->min_order_qty_session_name );

		$has_session        = isset( $session[ $variation_id ] ) ? $session[ $variation_id ] : array();
		$has_parent_session = isset( $session[ $product_id ] ) ? $session[ $product_id ] : array();

		if ( ! empty( $has_session ) && $variation_id ) {
			$min = isset( $has_session['value'] ) ? $has_session['value'] : '';

			if ( ! empty( $min ) ) {
				$existing_qty = wholesalex()->cart_count( $variation_id );
				$new_qty      = $existing_qty + $quantity;
				if ( $new_qty < $min ) {
					$status = false;
					/* translators: 1: minimum quantity, 2: product name */
					wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_min_quantity_error_message', sprintf( __( 'You have to add minimun %1$d quantity of %2$s.', 'wholesalex-pro' ), $min, $variation->get_name() ) ), 'error' );
				}
			}
		}

		if ( ! empty( $has_parent_session ) & $status ) {
			$min = isset( $has_parent_session['value'] ) ? $has_parent_session['value'] : '';

			if ( ! empty( $min ) ) {
				$existing_qty = wholesalex()->cart_count( $product_id );
				$new_qty      = $existing_qty + $quantity;
				if ( $new_qty < $min ) {
					$status = false;
					wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_min_quantity_error_message', sprintf( __( 'You have to add minimun %1$d quantity of %2$s.', 'wholesalex-pro' ), $min, $variation_id ? $variation->get_name() : $product->get_name() ) ), 'error' );
				}
			}
		}

		return $status;

	}


	/**
	 * Minimum Order Quantity Update Cart Validation.
	 *
	 * @param bool       $status Update Cart Status.
	 * @param string     $cart_item_key Cart Items.
	 * @param array      $values Cart Item Data.
	 * @param int|string $quantity Update Quantity.
	 * @return bool
	 * @since 1.0.6
	 */
	public function min_order_update_cart_validation( $status, $cart_item_key, $values, $quantity ) {
		$current_user_id                  = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
		$this->min_order_qty_session_name = 'wholesalex_dynamic_rules_min_order_qty_' . $current_user_id;

		$product_id = $values['variation_id'] ? $values['variation_id'] : $values['product_id'];
		$product    = wc_get_product( $product_id );
		$session    = get_transient( $this->min_order_qty_session_name );

		$has_session        = isset( $session[ $values['variation_id'] ] ) ? $session[ $values['variation_id'] ] : array();
		$has_parent_session = isset( $session[ $values['product_id'] ] ) ? $session[ $values['product_id'] ] : array();

		if ( ! empty( $has_session ) ) {
			$min = isset( $has_session['value'] ) ? $has_session['value'] : '';

			$product_title = $product->get_name();
			if ( ! empty( $min ) ) {
				if ( $quantity < $min ) {
					$status = false;
					/* translators: 1: minimum quantity, 2: product name */
					wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_min_quantity_error_message', sprintf( __( 'You have to add minimum %1$d quantity of %2$s.', 'wholesalex-pro' ), $min, $product_title ) ), 'error' );
				}
			}
		}

		if ( ! empty( $has_parent_session ) & $status ) {
			$min = isset( $has_parent_session['value'] ) ? $has_parent_session['value'] : '';

			$product_title = $product->get_name();
			if ( ! empty( $min ) ) {
				if ( $quantity < $min ) {
					$status = false;
					/* translators: 1: minimum quantity, 2: product name */
					wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_min_quantity_error_message', sprintf( __( 'You have to add minimum %1$d quantity of %2$s.', 'wholesalex-pro' ), $min, $product_title ) ), 'error' );
				}
			}
		}

		return $status;
	}

	/**
	 * Min Order Product Variation Availability Set
	 *
	 * @param array  $variation_data Variation Data.
	 * @param object $product Product Object.
	 * @param object $variation Variation Product Object.
	 * @return array
	 * @since 1.0.6
	 */
	public function min_order_variation_available( $variation_data, $product, $variation ) {
		$current_user_id                  = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
		$this->min_order_qty_session_name = 'wholesalex_dynamic_rules_min_order_qty_' . $current_user_id;

		$parent_id    = $variation->get_parent_id();
		$variation_id = $variation->get_id();

		$session = get_transient( $this->min_order_qty_session_name );

		$has_session        = isset( $session[ $variation_id ] ) ? $session[ $variation_id ] : array();
		$has_parent_session = isset( $session[ $parent_id ] ) ? $session[ $parent_id ] : array();

		if ( ! empty( $has_parent_session ) ) {
			$min = isset( $has_parent_session['value'] ) ? $has_parent_session['value'] : '';
			if ( ! empty( $min ) ) {
				$variation_data['min_qty'] = $min;
			}
		}
		if ( ! empty( $has_session ) ) {
			$min = isset( $has_session['value'] ) ? $has_session['value'] : '';
			if ( ! empty( $min ) ) {
				$variation_data['min_qty'] = $min;
			}
		}

		return $variation_data;

	}

	/**
	 * Min order add to cart validation
	 *
	 * @param bool       $status Add to cart Status.
	 * @param int|string $product_id Product ID.
	 * @param int        $quantity Cart Quantity.
	 * @return bool
	 * @since 1.0.6
	 */
	public function max_order_add_to_cart_validation( $status, $product_id, $quantity ) {
		$current_user_id                  = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
		$this->max_order_qty_session_name = 'wholesalex_dynamic_rules_max_order_qty_' . $current_user_id;

		$variation_id = 0;
		$product      = wc_get_product( $product_id );
		$variation    = false;
		if ( ! $product->is_type( 'simple' ) ) {
			$args         = func_get_args();
			$variation_id = isset( $args[3] ) ? $args[3] : 0;
			if ( $variation_id ) {
				$variation = wc_get_product( $variation_id );
			}
		}
		$session = get_transient( $this->max_order_qty_session_name );

		$has_session        = isset( $session[ $variation_id ] ) ? $session[ $variation_id ] : array();
		$has_parent_session = isset( $session[ $product_id ] ) ? $session[ $product_id ] : array();

		if ( ! empty( $has_session ) && $variation_id ) {
			$max = isset( $has_session['value'] ) ? $has_session['value'] : '';

			if ( ! empty( $max ) ) {
				$existing_qty = wholesalex()->cart_count( $variation_id );
				$new_qty      = $existing_qty + $quantity;
				if ( $new_qty > $max ) {
					$status = false;
					/* translators: 1: maximum quantity, 2: product name */
					wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_max_quantity_error_message', sprintf( __( 'You can add maximun %1$d quantity of %2$s.', 'wholesalex-pro' ), $max, $variation->get_name() ) ), 'error' );
				}
			}
		}

		if ( ! empty( $has_parent_session ) & $status ) {
			$max = isset( $has_parent_session['value'] ) ? $has_parent_session['value'] : '';

			if ( ! empty( $max ) ) {
				$existing_qty = wholesalex()->cart_count( $product_id );
				$new_qty      = $existing_qty + $quantity;
				if ( $new_qty > $max ) {
					$status = false;
					/* translators: 1: maximum quantity, 2: product name */
					wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_max_quantity_error_message', sprintf( __( 'You can add maximun %1$d quantity of %2$s.', 'wholesalex-pro' ), $max, $variation_id ? $variation->get_name() : $product->get_name() ) ), 'error' );
				}
			}
		}

		return $status;

	}

	/**
	 * Maximum Order Quantity Update Cart Validation.
	 *
	 * @param bool       $status Update Cart Status.
	 * @param string     $cart_item_key Cart Items.
	 * @param array      $values Cart Item Data.
	 * @param int|string $quantity Update Quantity.
	 * @return bool
	 * @since 1.0.6
	 */
	public function max_order_update_cart_validation( $status, $cart_item_key, $values, $quantity ) {
		$current_user_id                  = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
		$this->max_order_qty_session_name = 'wholesalex_dynamic_rules_max_order_qty_' . $current_user_id;

		$product_id = $values['variation_id'] ? $values['variation_id'] : $values['product_id'];
		$product    = wc_get_product( $product_id );
		$session    = get_transient( $this->max_order_qty_session_name );

		$has_session        = isset( $session[ $values['variation_id'] ] ) ? $session[ $values['variation_id'] ] : array();
		$has_parent_session = isset( $session[ $values['product_id'] ] ) ? $session[ $values['product_id'] ] : array();
		if ( ! empty( $has_session ) ) {
			$max = isset( $has_session['value'] ) ? $has_session['value'] : '';

			$product_title = $product->get_name();
			if ( ! empty( $max ) ) {
				if ( $quantity > $max ) {
					$status = false;
					/* translators: 1: maximum quantity, 2: product name */
					wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_max_quantity_error_message', sprintf( __( 'You can add maximum %1$d quantity of %2$s.', 'wholesalex-pro' ), $max, $product_title ) ), 'error' );
				}
			}
		}

		if ( ! empty( $has_parent_session ) & $status ) {
			$max = isset( $has_parent_session['value'] ) ? $has_parent_session['value'] : '';

			$product_title = $product->get_name();
			if ( ! empty( $max ) ) {
				if ( $quantity > $max ) {
					$status = false;
					/* translators: 1: maximum quantity, 2: product name */
					wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_max_quantity_error_message', sprintf( __( 'You can add maximum %1$d quantity of %2$s.', 'wholesalex-pro' ), $max, $product_title ) ), 'error' );
				}
			}
		}

		return $status;
	}

	/**
	 * Max Order Product Variation Availability Set
	 *
	 * @param array  $variation_data Variation Data.
	 * @param object $product Product Object.
	 * @param object $variation Variation Product Object.
	 * @return array
	 * @since 1.0.6
	 */
	public function max_order_variation_available( $variation_data, $product, $variation ) {
		$current_user_id                  = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
		$this->max_order_qty_session_name = 'wholesalex_dynamic_rules_max_order_qty_' . $current_user_id;

		$parent_id    = $variation->get_parent_id();
		$variation_id = $variation->get_id();

		$session = get_transient( $this->max_order_qty_session_name );

		$has_session        = isset( $session[ $variation_id ] ) ? $session[ $variation_id ] : array();
		$has_parent_session = isset( $session[ $parent_id ] ) ? $session[ $parent_id ] : array();

		if ( ! empty( $has_parent_session ) ) {
			$max = isset( $has_parent_session['value'] ) ? $has_parent_session['value'] : '';
			if ( ! empty( $max ) ) {
				$variation_data['max_qty'] = $max;
			}
		}
		if ( ! empty( $has_session ) ) {
			$max = isset( $has_session['value'] ) ? $has_session['value'] : '';
			if ( ! empty( $max ) ) {
				$variation_data['max_qty'] = $max;
			}
		}
		return $variation_data;

	}


	/**
	 * Perform Actions After Dynamic Rules Updated.
	 *
	 * @return void
	 * @since 1.0.6
	 */
	public function dynamic_rules_updated() {
		delete_transient( $this->min_order_qty_session_name );
		delete_transient( $this->max_order_qty_session_name );
	}

	public function make_not_purchasable() {
		add_filter('woocommerce_is_purchasable',array($this,'modify_woocommerce_is_purchasable'),10,2);
		add_filter('woocommerce_variation_is_purchasable',array($this,'modify_woocommerce_is_purchasable'),10,2);
	}

	public function modify_woocommerce_is_purchasable($is_purchasable,$product){
		$hidden_prices = $non_purchasable = array(
			'include_product' => array(),
			'exclude_product' => array(),
			'include_cat'	  => array(),
			'exclude_cat'	  => array(),
			'include_variations' => array(),
			'exclude_variations' => array(),
			'is_all_products' => array()
		);
		if(isset($this->valid_rules['hidden_price'])) {
			$hidden_prices = $this->valid_rules['hidden_price'];

		}
		
		if(isset($this->valid_rules['non_purchasable'])) {

			$non_purchasable =$this->valid_rules['non_purchasable'];
		}

		$filter = array(
			'include_product' => array_merge($hidden_prices['include_product'],$non_purchasable['include_product']),
			'exclude_product' => array_merge($hidden_prices['exclude_product'],$non_purchasable['exclude_product']),
			'include_cat' => array_merge($hidden_prices['include_cat'],$non_purchasable['include_cat']),
			'exclude_cat' => array_merge($hidden_prices['exclude_cat'],$non_purchasable['exclude_cat']),
			'include_variations' => array_merge($hidden_prices['include_variations'],$non_purchasable['include_variations']),
			'exclude_variations' => array_merge($hidden_prices['exclude_variations'],$non_purchasable['exclude_variations']),
			'is_all_products' => $hidden_prices['is_all_products'] || $non_purchasable['is_all_products'],
		);


		$product_id = $product->get_id();

		$cats       = wc_get_product_term_ids( $product_id, 'product_cat' );

		if($filter['is_all_products'] ) {
			return false;
		}

		if(!empty($filter['include_product']) && in_array($product_id,$filter['include_product'])) {
			return false;
		} else if(!empty($filter['exclude_product']) && !in_array($product_id,$filter['exclude_product'])) {
			return false;
		} else if(!empty($filter['include_cat']) && !empty(array_intersect($cats,$filter['include_cat']))) {
			return false;
		} else if(!empty($filter['exclude_cat']) && empty(array_intersect($cats,$filter['exclude_cat']))) {
			return false;
		} else if(!empty($filter['include_variations']) && in_array($product_id,$filter['include_variations'])) {
			return false;
		} else if(!empty($filter['exclude_variations']) && !in_array($product_id,$filter['exclude_variations'])) {
			return false;
		}

		return $is_purchasable;
	}
	public function hidden_price_handler() {
		add_filter('wholesalex_get_price_html',array($this,'get_price_html_for_hidden_price'),10,2);
		add_filter('woocommerce_variation_price_html',array($this,'get_price_html_for_hidden_price'),10,2);
		do_action( 'wholesalex_after_hidden_price_quote',  (isset($this->valid_rules['hidden_price'])) ? $this->valid_rules['hidden_price'] : array() );
		$this->make_not_purchasable();
	}

	public function get_price_html_for_hidden_price($price_html, $product) {
		$hidden_price_text = apply_filters('wholesalex_hidden_price_text',wholesalex()->get_setting('_language_price_is_hidden',__('Price is hidden!','wholesalex-pro')));
		$hidden_prices = $this->valid_rules['hidden_price'];

		$product_id = $product->get_id();
		$cats       = wc_get_product_term_ids( $product_id, 'product_cat' );

		if($hidden_prices['is_all_products']  ) {
			return $hidden_price_text;
		}

		if(!empty($hidden_prices['include_product']) && in_array($product_id,$hidden_prices['include_product'])) {
			return $hidden_price_text;
		} else if(!empty($hidden_prices['exclude_product']) && !in_array($product_id,$hidden_prices['exclude_product'])) {
			return $hidden_price_text;
		} else if(!empty($hidden_prices['include_cat']) && !empty(array_intersect($cats,$hidden_prices['include_cat']))) {
			return $hidden_price_text;
		} else if(!empty($hidden_prices['exclude_cat']) && empty(array_intersect($cats,$hidden_prices['exclude_cat']))) {
			return $hidden_price_text;
		} else if(!empty($hidden_prices['include_variations']) && in_array($product_id,$hidden_prices['include_variations'])) {
			return $hidden_price_text;
		} else if(!empty($hidden_prices['exclude_variations']) && !in_array($product_id,$hidden_prices['exclude_variations'])) {
			return $hidden_price_text;
		}

		return $price_html;
	}


	/**
	 * Max Order Quantity
	 *
	 * @param  $max_order_qty
	 * @return void
	 */
	public function max_order_quantity($max_order_qty) {
		extract($max_order_qty);
		add_filter(
			'woocommerce_quantity_input_max',
			function( $quantity, $product ) use ( $max_quantity, $rule_id, $product_filter ) {
				if ( $this->is_rule_applicable( $product->get_ID(), $product_filter, $this->rules_products[ $rule_id ] ) ) {
					$this->max_order_qty_session_name = 'wholesalex_dynamic_rules_max_order_qty_' . $this->current_user_id;

					$session = get_transient( $this->max_order_qty_session_name );

					if ( ! is_array( $session ) ) {
						$session = array();
					}
					$session[ $product->get_ID() ] = array(
						'rule_id' => $rule_id,
						'value'   => $max_quantity,
					);

					set_transient( $this->max_order_qty_session_name, $session );
					return $max_quantity;
				} else {
					return $quantity;
				}
			},
			10,
			2
		);
		add_filter( 'woocommerce_add_to_cart_validation', array( $this, 'max_order_add_to_cart_validation' ), 10, 5 );

		add_filter( 'woocommerce_update_cart_validation', array( $this, 'max_order_update_cart_validation' ), 10, 4 );

		add_filter( 'woocommerce_available_variation', array( $this, 'max_order_variation_available' ), 10, 3 );
		

		add_filter('woocommerce_store_api_product_quantity_maximum',function ($value,$product,$cart_item) use($max_quantity,$product_filter,$rule_id){
			if ( $this->is_rule_applicable( $product->get_ID(), $product_filter, $this->rules_products[ $rule_id ] ) ) {
				return $max_quantity;
			}
			return $value;
		},10,3);
	}

	/**
	 * Restrict Product Visibility.
	 *
	 * @return void
	 * @since 1.0.8
	 */
	public function restrict_visibility() {
		add_filter( 'pre_get_posts', array( $this, 'filter_hidden_products' ), 999999 );
		add_filter( 'woocommerce_product_query', array( $this, 'filter_hidden_products' ), 999999 );
		add_action( 'template_redirect', array( $this, 'redirect_from_hidden_products' ) );
		add_action( 'woocommerce_check_cart_items', array( $this, 'prevent_checkout_hidden_products' ) );
		add_filter( 'woocommerce_get_children', array( $this, 'filter_variations' ), 10 );
		add_filter( 'woocommerce_product_related_posts_query',array($this,'modify_woocommerce_product_related_posts_query'));
		add_filter( 'rest_post_query',array($this,'modify_rest_post_query'));
	}

	public function modify_rest_post_query($args) {

		$data = $this->valid_rules['restrict_product_visibility'];

		if( $data['is_all_products'] ) {
			if(!isset($args['post__in'])) {
				$args['post__in']= array();
			}
			$args['post__in'] = array(999999999999);
		} else {
			if ( ! empty( $data['include_product'] ) ) {
				if(!isset($args['post__not_in'])) {
					$args['post__not_in']= array();
				}
				$args['post__not_in'] = array_merge( $args['post__not_in'], $data['include_product'] );
			}
			if ( ! empty(  $data['exclude_product'] ) ) {
				if(!isset($args['post__in'])) {
					$args['post__in']= array();
				}
				$args['post__in'] = array_merge( $args['post__in'], $data['exclude_product'] );
			}
			if ( ! empty( $data['include_cat'] ) ) {
				if(!isset($args['tax_query'])) {
					$args['tax_query']= array();
				}
				$args['tax_query'][] = array(
					'taxonomy'         => 'product_cat',
					'field'            => 'term_id',
					'terms'            => $data['include_cat'],
					'include_children' => false,
					'operator'         => 'NOT IN',
				);
			}
			if ( ! empty(  $data['exclude_cat'] ) ) {
				if(!isset($args['tax_query'])) {
					$args['tax_query']= array();
				}
				$args['tax_query'][] = array(
					'taxonomy'         => 'product_cat',
					'field'            => 'term_id',
					'terms'            => $data['exclude_cat'],
					'include_children' => false,
					'operator'         => 'IN',
				);
			}
		}
		
		return $args;
	}

	public function modify_woocommerce_product_related_posts_query($query) {
		global $wpdb;
		$data = $this->valid_rules['restrict_product_visibility'];

		if ( count( $data['include_cat']) ) {
			$query['join']  .= " LEFT JOIN ( SELECT object_id FROM {$wpdb->term_relationships} WHERE term_taxonomy_id IN ( " . implode( ',', array_map( 'absint', $data['include_cat'] ) ) . ' ) ) AS exclude_join ON exclude_join.object_id = p.ID';
			$query['where'] .= ' AND exclude_join.object_id IS NULL';
		}

		if ( count( $data['exclude_cat'] ) ) {
			$query['join'] .= " INNER JOIN ( SELECT object_id FROM {$wpdb->term_relationships} INNER JOIN {$wpdb->term_taxonomy} using( term_taxonomy_id ) WHERE term_id IN ( " . implode( ',', array_map( 'absint', $data['exclude_cat'] ) ) . ' ) ) AS include_join ON include_join.object_id = p.ID';
		}

		if ( count( $data['include_product'] ) ) {
			$query['where'] .= ' AND p.ID NOT IN ( ' . implode( ',', array_map( 'absint', $data['include_product'] ) ) . ' )';
		}

		return $query;
	}

	/**
	 * Filter Hidden Products
	 *
	 * @param object $q Query Object.
	 * @return object
	 * @since 1.0.8
	 */
	public function filter_hidden_products( $q ) {
		$post_type   = $q->get( 'post_type' );
		$tax_query   = $q->get( 'tax_query' );
		$product_cat = $q->get( 'product_cat' );

		$data = $this->valid_rules['restrict_product_visibility'];

		
		if ( ( 'product' === $post_type || '' != $product_cat ) && ! is_admin() ) {
			// If Restrict Product Visibiliy Enabled on All Product, We dont need to proceed further
			if ( $data['is_all_products'] ) {
				$q->set( 'post__in', array( '999999999999' ) );
			} else {
				if ( ! empty( $data['include_product'] ) ) {
					$q->set( 'post__not_in', $data['include_product'] );
				}
				if ( ! empty(  $data['exclude_product'] ) ) {
					$q->set( 'post__in', $data['exclude_product'] );
				}
			}
		}

		/**
		 * Post Type Check Added
		 *
		 * @since 1.1.3
		 */
		if ( is_array( $tax_query ) && ( 'product' === $post_type || '' != $product_cat ) ) {
			if ( ! empty( $data['include_cat']) ) {
				$tax_query[] = array(
					'taxonomy'         => 'product_cat',
					'field'            => 'term_id',
					'terms'            => $data['include_cat'],
					'include_children' => false,
					'operator'         => 'NOT IN',
				);
			}
			if ( ! empty( $data['exclude_cat']) ) {
				$tax_query[] = array(
					'taxonomy'         => 'product_cat',
					'field'            => 'term_id',
					'terms'            => $data['exclude_cat'],
					'include_children' => false,
					'operator'         => 'IN',
				);
			}
			$q->set( 'tax_query', $tax_query );
		}
		if ( ! empty( $data['include_cat']) ) {
			$q->set( '__wholesalex_exclude_cat', $data['include_cat']);
		}
		if ( ! empty( $data['exclude_cat'] ) ) {
			$q->set( '__wholesalex_include_cat', $data['exclude_cat'] );
		}
		return $q;
	}

	/**
	 * Redirect From Hidden Product
	 *
	 * @return void
	 * @since 1.0.8
	 */
	public function redirect_from_hidden_products() {
		if ( is_product() ) {
			$data = $this->valid_rules['restrict_product_visibility'];

			$__id = get_the_ID();
			if ( $data['is_all_products'] ) {
				wc_add_notice( sprintf( __( 'Sorry, you are not allowed to see %s product.', 'wholesalex-pro' ), get_the_title( get_the_ID() ) ), 'notice' );
				$redirect_url = get_permalink( get_option( 'woocommerce_shop_page_id' ) );
				wp_safe_redirect( $redirect_url );
				exit();
			}
			if ( ! empty( $data['exclude_product'] ) && ! in_array( $__id, $data['exclude_product'] ) ) {
				/* translators: %s: Product Name */
				wc_add_notice( sprintf( __( 'Sorry, you are not allowed to see %s product.', 'wholesalex-pro' ), get_the_title( get_the_ID() ) ), 'notice' );
				$redirect_url = get_permalink( get_option( 'woocommerce_shop_page_id' ) );
				wp_safe_redirect( $redirect_url );
				exit();
			}
			if ( ! empty( $data['include_product'] ) && in_array( $__id, $data['include_product'] ) ) {
				/* translators: %s: Product Name */
				wc_add_notice( sprintf( __( 'Sorry, you are not allowed to see %s product.', 'wholesalex-pro' ), get_the_title( get_the_ID() ) ), 'notice' );
				$redirect_url = get_permalink( get_option( 'woocommerce_shop_page_id' ) );
				wp_safe_redirect( $redirect_url );
				exit();
			}
			if (function_exists( 'wc_get_product_term_ids' ) ) {
				$cats = wc_get_product_term_ids( get_the_ID(), 'product_cat' );
				if ( ! empty( $data['exclude_cat'] ) && ! array_intersect( $cats, $data['exclude_cat'] ) ) {
					/* translators: %s: Product Name */
					wc_add_notice( sprintf( __( 'Sorry, you are not allowed to see %s product.', 'wholesalex-pro' ), get_the_title( get_the_ID() ) ), 'notice' );
					$redirect_url = get_permalink( get_option( 'woocommerce_shop_page_id' ) );
					wp_safe_redirect( $redirect_url );
					exit();
				}
				if ( ! empty( $data['include_cat'] ) && array_intersect( $cats, $data['include_cat'] ) ) {
					/* translators: %s: Product Name */
					wc_add_notice( sprintf( __( 'Sorry, you are not allowed to see %s product.', 'wholesalex-pro' ), get_the_title( get_the_ID() ) ), 'notice' );
					$redirect_url = get_permalink( get_option( 'woocommerce_shop_page_id' ) );
					wp_safe_redirect( $redirect_url );
					exit();
				}
			}
		}

	}

	/**
	 * Prevent Checkout If any cart has any wholesalex hidden product
	 *
	 * @since 1.0.8
	 * @since 1.1.0 Allow Hidden to checkout hook added
	 */
	public function prevent_checkout_hidden_products() {
		if ( ! ( isset( WC()->cart ) && ! empty( WC()->cart ) ) ) {
			return;
		}
		$allow_hidden_product_to_checkout = apply_filters( 'wholesalex_allow_hidden_product_to_checkout', false );
		if ( $allow_hidden_product_to_checkout ) {
			return;
		}
		$data = $this->valid_rules['restrict_product_visibility'];

		if ( $data['is_all_products'] ) {
			WC()->cart->empty_cart();
			wc_add_notice( sprintf( __( 'Sorry, you are not allowed to checkout products.', 'wholesalex-pro' ) ), 'error' );
		}
		foreach ( WC()->cart->get_cart() as $key => $cart_item ) {
			$cats = wc_get_product_term_ids( $cart_item['product_id'], 'product_cat' );
			if ( ! empty( $data['include_cat']  ) && array_intersect( $cats, $data['include_cat']  ) ) {
				// Remove Hidden Product From Cart.
				WC()->cart->remove_cart_item( $key );
				/* translators: %s: Product Name */
				wc_add_notice( sprintf( __( 'Sorry, you are not allowed to checkout %s product.', 'wholesalex-pro' ), get_the_title( $cart_item['product_id'] ) ), 'error' );
			}
			if ( ! empty( $data['exclude_cat'] ) && ! array_intersect( $cats, $data['exclude_cat'] ) ) {
				// Remove Hidden Product From Cart.
				WC()->cart->remove_cart_item( $key );
				/* translators: %s: Product Name */
				wc_add_notice( sprintf( __( 'Sorry, you are not allowed to checkout %s product.', 'wholesalex-pro' ), get_the_title( $cart_item['product_id'] ) ), 'error' );
			}
			if ( $cart_item['variation_id'] ) {
				if ( ! empty( $data['include_variations'] ) && in_array( $cart_item['variation_id'], $data['include_variations'] ) ) {
					// Remove Hidden Product From Cart.
					WC()->cart->remove_cart_item( $key );
					/* translators: %s: Product Name */
					wc_add_notice( sprintf( __( 'Sorry, you are not allowed to checkout %s product.', 'wholesalex-pro' ), get_the_title( $cart_item['variation_id'] ) ), 'error' );
				}
				if ( ! empty( $data['exclude_variations'] ) && ! in_array( $cart_item['variation_id'], $data['exclude_variations'] ) ) {
					// Remove Hidden Product From Cart.
					WC()->cart->remove_cart_item( $key );
					/* translators: %s: Product Name */
					wc_add_notice( sprintf( __( 'Sorry, you are not allowed to checkout %s product.', 'wholesalex-pro' ), get_the_title( $cart_item['variation_id'] ) ), 'error' );
				}
			} elseif ( $cart_item['product_id'] ) {
				if ( ! empty( $data['include_product'] ) && in_array( $cart_item['product_id'], $data['include_product'] ) ) {
					// Remove Hidden Product From Cart.
					WC()->cart->remove_cart_item( $key );
					/* translators: %s: Product Name */
					wc_add_notice( sprintf( __( 'Sorry, you are not allowed to checkout %s product.', 'wholesalex-pro' ), get_the_title( $cart_item['product_id'] ) ), 'error' );
				}
				if ( ! empty( $data['exclude_product'] ) && ! in_array( $cart_item['product_id'], $data['exclude_product'] ) ) {
					// Remove Hidden Product From Cart.
					WC()->cart->remove_cart_item( $key );
					/* translators: %s: Product Name */
					wc_add_notice( sprintf( __( 'Sorry, you are not allowed to checkout %s product.', 'wholesalex-pro' ), get_the_title( $cart_item['product_id'] ) ), 'error' );
				}
			}
		}
	}


	/**
	 * Filter Product Variations.
	 *
	 * @param array $children Children IDs.
	 * @return array
	 * @since 1.0.8
	 */
	public function filter_variations( $children ) {
		/**
		 * Doing AJAX Check Added
		 *
		 * @since 1.1.4
		 */
		if ( is_admin() && ! ( defined( 'DOING_AJAX' ) && DOING_AJAX ) ) {
			return $children;
		}

		$data = $this->valid_rules['restrict_product_visibility'];


		if ( ! empty( $data['include_variations'] ) ) {
			$children = array_diff( $children, $data['include_variations'] );
		}

		if ( ! empty( $data['exclude_variations'] ) ) {
			$children = array_intersect( $children, $data['exclude_variations'] );
		}

		return $children;

	}


	/**
	 * Add to cart free items.
	 *
	 * @param array      $main_product_ids Main Product ids.
	 * @param int|string $main_product_qty Main Product cart quantity.
	 * @param int|string $min_quantity Main Product Minimum Quantity to get free items.
	 * @param array      $free_items Free Items.
	 * @param int|string $free_items_quantity Free items quantity.
	 * @return void
	 * @since 1.0.9
	 */
	public function add_to_cart_free_items( $main_product_ids, $main_product_qty, $min_quantity, $free_items, $free_items_quantity,$apply_once=false ) {
		if ( $main_product_qty >= $min_quantity ) {
			// Need to add free item in cart.
			foreach ( $free_items as $item ) {
				$free_item_id = $item['value'];

				$cart_item_data = array(
					'unique_key'   => wp_unique_id( 'wsx_free' ),
					'free_product' => true,
					'free_for'     => $main_product_ids,
				);

				$free_qty = (int) ((int)$main_product_qty/(int)$min_quantity) * $free_items_quantity;
				if($apply_once) {
					// Apply Free Item per cart once
					$free_qty = $free_items_quantity;
				}
				WC()->cart->add_to_cart( $free_item_id, $free_qty, 0, array(), $cart_item_data );
			}
		}
	}


	/**
	 * Buy X Get Y Free Items Templates for Single Product Page.
	 *
	 * @param string     $min_purchase_text Text that visible before free items.
	 * @param array      $free_items Free Items.
	 * @param int|string $free_item_quantity Free Items quantity.
	 * @return void
	 * @since 1.0.9
	 */
	public function bxgy_free_items_template( $min_purchase_text, $free_items, $free_item_quantity ) {
		?>
		<?php echo esc_html( $min_purchase_text ); ?>
		<div class="wholesalex_free_items">
			<?php
			foreach ( $free_items as $item ) {
				$free_item_id = $item['value'];
				$product      = wc_get_product( $free_item_id );
				$image        = wp_get_attachment_image_src( get_post_thumbnail_id( $free_item_id ), 'thumbnail' );
				?>
			<div class="wholesalex_free_item">
				<img width="80px" src="<?php echo esc_url( $image[0] ); ?>">
				<div class="free_item_meta">
					<div class="item_title">
						<a href="<?php echo esc_url( $product->get_permalink() ); ?>"> <?php echo esc_html( $product->get_name() ); ?> </a>
					</div>
					<div class="free_quantity">
						Free Quantity: <?php echo esc_html( $free_item_quantity ); ?>
					</div>
					<div class="price">
						<?php echo '<del>' . wc_price( $product->get_price( 'edit' ) * esc_html( $free_item_quantity ) ) . '</del>'; //phpcs:ignore ?> Free!
					</div>
				</div>
			</div>
				<?php
			}
			?>
		</div>
			<?php

	}

	/**
	 * Buy X Get Y Products
	 *
	 * @param int|sting  $min_purchase_count Minimum Purchase Count.
	 * @param array      $free_items Free Items.
	 * @param int|string $free_item_quantity Free Items Quantity.
	 * @return void
	 * @since 1.0.9
	 */
	public function buy_x_get_y(  ) {

		$show_free_items_single_product_page = wholesalex()->get_setting( '_settings_show_bxgy_free_products_on_single_product_page' );

		if ( 'yes' === $show_free_items_single_product_page ) {	
			add_action('wholesalex_before_add_to_cart_form',function($product){
				$product_id = $product->get_ID();
				if ( $product->get_type() == 'simple' ) {
					$rules = $this->valid_rules['buy_x_get_y'];
					foreach ($rules as $rule_id => $rule) {
						if(!WHOLESALEX_Dynamic_Rules::check_rule_conditions($rule['conditions'])) {
							continue;
						}
						$id = $rule['id'];

						$min_purchase_count = isset( $rule['rule']['_minimum_purchase_count'] ) ? $rule['rule']['_minimum_purchase_count'] : PHP_INT_MAX - 10;
						$free_items         = isset( $rule['rule']['_free_item'] ) ? $rule['rule']['_free_item'] : false;
						$free_item_quantity = isset( $rule['rule']['_free_item_count'] ) ? $rule['rule']['_free_item_count'] : false;


						if ( ! $free_items || ! $free_item_quantity ) {
							continue;
						}
						

						if(method_exists(wholesalex(),'set_rule_data') && empty(wholesalex()->get_rule_data($product->get_id(), 'buy_x_get_y', $rule['id']))) {
							wholesalex()->set_rule_data(
								$id,
								$product->get_id(),
								'buy_x_get_y',
								array(
									'conditions' => $rule['conditions'],
									'who_priority' => $rule['who_priority'],
									'applied_on_priority' => $rule['applied_on_priority'],
									'end_date' => $rule['end_date'],
									'filter' => $rule['filter'],
									'min_purchase_count' => $min_purchase_count,
									'free_items'=>$free_items,
									'free_item_quantity' => $free_item_quantity
								)
							);
						}
					}

				}
			});	
			
			
			add_filter(
				'woocommerce_available_variation',
				function ( $data, $product, $variation ) {
					$variation_id = $variation->get_id();
					$product_id   = $product->get_ID();
					$rules = $this->valid_rules['buy_x_get_y'];
					foreach ($rules as $rule_id => $rule) {
						if(!WHOLESALEX_Dynamic_Rules::check_rule_conditions($rule['conditions'])) {
							continue;
						}
						$id = $rule['id'];

						$min_purchase_count = isset( $rule['rule']['_minimum_purchase_count'] ) ? $rule['rule']['_minimum_purchase_count'] : PHP_INT_MAX - 10;
						$free_items         = isset( $rule['rule']['_free_item'] ) ? $rule['rule']['_free_item'] : false;
						$free_item_quantity = isset( $rule['rule']['_free_item_count'] ) ? $rule['rule']['_free_item_count'] : false;


						if ( ! $free_items || ! $free_item_quantity ) {
							continue;
						}

						if(method_exists(wholesalex(),'set_rule_data') && empty(wholesalex()->get_rule_data($variation_id, 'buy_x_get_y', $rule['id']))) {
							wholesalex()->set_rule_data(
								$id,
								$variation_id,
								'buy_x_get_y',
								array(
									'conditions' => $rule['conditions'],
									'who_priority' => $rule['who_priority'],
									'applied_on_priority' => $rule['applied_on_priority'],
									'end_date' => $rule['end_date'],
									'filter' => $rule['filter'],
									'min_purchase_count' => $min_purchase_count,
									'free_items'=>$free_items,
									'free_item_quantity' => $free_item_quantity
								)
							);
						}
						
					}

					return $data;
				},
				10,
				3
			);
		}

		add_action('woocommerce_get_item_data',function($item_data, $cart_item) {
			if(isset($cart_item['free_product']) && $cart_item['free_product']) {
				$item_data[] = array('wsx_is_free_item'=>true,'name'=> 'Free Items','value'=>'This items is free!','hidden'=>true);
			}
			return $item_data;
		},10,2);

		add_action('woocommerce_check_cart_items',array($this,'check_cart_items'));

		add_action(
			'woocommerce_before_calculate_totals',
			function ( $cart ) {
				foreach ( $cart->cart_contents as $cart_item ) {										
					if ( isset( $cart_item['free_product'] ) && $cart_item['free_product'] == true ) {
						$cart_item['data']->set_price( 0 );
						$cart_item['data']->set_regular_price( 0 );
						$cart_item['data']->set_sale_price( 0 );
					}
				}
			},
			102
		);

		add_filter(
			'woocommerce_cart_item_name',
			function ( $cart_item_name, $cart_item ) {
				if ( isset( $cart_item['free_product'] ) && $cart_item['free_product'] == true ) {
					$product         = $cart_item['data'];
					$free_item_title = $product->get_name() . __( '(Free)', 'wholesalex-pro' );
					return $free_item_title;
				}

				return $cart_item_name;
			},
			10,
			2
		);

		add_filter(
			'woocommerce_cart_item_quantity',
			function ( $quantity_html, $item_key, $cart_item ) {
				if ( isset( $cart_item['free_product'] ) && $cart_item['free_product'] == true ) {
					return $cart_item['quantity'];
				} else {
					return $quantity_html;
				}
			},
			10,
			3
		);
		add_filter(
			'woocommerce_cart_item_price',
			function ( $price, $cart_item ) {
				if ( isset( $cart_item['free_product'] ) && $cart_item['free_product'] == true ) {
					return '';
				} else {
					return $price;
				}
			},
			10,
			2
		);
		add_filter(
			'woocommerce_cart_item_subtotal',
			function ( $price, $cart_item ) {
				if ( isset( $cart_item['free_product'] ) && $cart_item['free_product'] == true ) {
					return __( 'Free', 'wholesalex-pro' );
				} else {
					return $price;
				}
			},
			10,
			2
		);
		add_filter(
			'woocommerce_cart_item_remove_link',
			function ( $remove_html, $cart_item_key ) {
				$cart_item = WC()->cart->get_cart_item( $cart_item_key );
				if ( isset( $cart_item['free_product'] ) && $cart_item['free_product'] == true ) {
					return '';
				} else {
					return $remove_html;
				}
			},
			10,
			2
		);

		add_filter('woocommerce_store_api_product_quantity_minimum',function ($value,$product,$cart_item){
			if(isset($cart_item['free_product']) && $cart_item['free_product']) {
				return $cart_item['quantity'];
			}
			return $value;
		},10,3);
		
		add_filter('woocommerce_store_api_product_quantity_maximum',function ($value,$product,$cart_item){
			if(isset($cart_item['free_product']) && $cart_item['free_product']) {
				return $cart_item['quantity'];
			}
			return $value;
		},10,3);
	}



	/**
	 * Check Cart Items
	 * Add Free Items
	 *
	 * @return void
	 */
	public function check_cart_items() {
		$cart = WC()->cart;
		$rules = $this->valid_rules['buy_x_get_y'];
		$cart_items        = $cart->get_cart();


		foreach ( $cart_items as $key => $cart_item ) {
			if(isset($cart_item['free_product']) && $cart_item['free_product']) {
				WC()->cart->remove_cart_item( $key );
			}
		}
		foreach ($rules as $rule_id => $rule) {
			if(!WHOLESALEX_Dynamic_Rules::check_rule_conditions($rule['conditions'])) {
				continue;
			}
			$cart_items        = $cart->get_cart();
			$paid_products_qty = 0;
			$free_for          = array();

			$id = $rule['id'];

			$min_purchase_count = isset( $rule['rule']['_minimum_purchase_count'] ) ? $rule['rule']['_minimum_purchase_count'] : PHP_INT_MAX - 10;
			$free_items         = isset( $rule['rule']['_free_item'] ) ? $rule['rule']['_free_item'] : false;
			$free_item_quantity = isset( $rule['rule']['_free_item_count'] ) ? $rule['rule']['_free_item_count'] : false;
			$apply_once = isset( $rule['rule']['_per_cart_once'] ) && 'yes'==$rule['rule']['_per_cart_once'] ? true: false;


			if ( ! $free_items || ! $free_item_quantity ) {
				continue;
			}

			extract($rule['filter']);
			
			foreach ( $cart_items as $key => $cart_item ) {
				$product_id = $cart_item['product_id'];
					
				$cats       = wc_get_product_term_ids( $product_id, 'product_cat' );

				if(isset($cart_item['free_product']) && $cart_item['free_product']) {
					continue;
				}

				if ( ! empty( $include_products ) && in_array( $product_id, $include_products ) ) {
					
					$paid_products_qty += intval( $cart_item['quantity'] );
					$free_for[]         = $product_id;
				} elseif ( ! empty( $exclude_products ) && ! in_array( $product_id, $exclude_products ) ) {

					$paid_products_qty += intval( $cart_item['quantity'] );
					$free_for[]         = $product_id;
				} elseif ( ! empty( $include_cats ) && array_intersect( $cats, $include_cats ) ) {

					$paid_products_qty += intval( $cart_item['quantity'] );
					$free_for[]         = $product_id;
				} elseif ( ! empty( $exclude_cats ) && ! array_intersect( $cats, $exclude_cats ) ) {

					$paid_products_qty += intval( $cart_item['quantity'] );
					$free_for[]         = $product_id;
				} elseif ( ! empty( $include_variations ) && in_array( $cart_item['variation_id'], $include_variations ) ) {

					$paid_products_qty += intval( $cart_item['quantity'] );
					$free_for[]         = $product_id;
				} elseif ( ! empty( $exclude_variations ) && ! in_array( $cart_item['variation_id'], $exclude_variations ) ) {

					$paid_products_qty += intval( $cart_item['quantity'] );
					$free_for[]         = $product_id;
				} elseif ( $is_all_products ) {
					$paid_products_qty += intval( $cart_item['quantity'] );
					$free_for[]         = $product_id;
				} 
				
			}	
							
			$this->add_to_cart_free_items( $free_for, $paid_products_qty, $min_purchase_count, $free_items, $free_item_quantity, $apply_once );
		}
		
	}


	/**
	 * Check Specific Product Is Applicable Current Rule
	 *
	 * @param string|int $product_id Product ID.
	 * @param string     $filter_type Filter Type.
	 *
	 * @return boolean
	 * @since 1.0.9
	 */
	public function is_rule_applicable( $product_id, $filter_type, $products ) {
		$status = false;
		$cats   = wc_get_product_term_ids( $product_id, 'product_cat' );
		switch ( $filter_type ) {
			case 'all_products':
				$status = true;
				break;
			case 'products_in_list':
				if ( in_array( $product_id, $products['include_products'] ) ) {
					$status = true;
				}
				break;
			case 'products_not_in_list':
				if ( ! in_array( $product_id, $products['exclude_products'] ) ) {
					$status = true;
				}
				break;
			case 'cat_in_list':
				$cats = wc_get_product_term_ids( $product_id, 'product_cat' );
				if ( ! empty( $products['include_cats'] ) && array_intersect( $cats, $products['include_cats'] ) ) {
					$status = true;
				}

				break;

			case 'cat_not_in_list':
				$cats = wc_get_product_term_ids( $product_id, 'product_cat' );
				if ( ! ( ! empty( $products['exclude_cats'] ) && array_intersect( $cats, $products['exclude_cats'] ) ) ) {
					$status = true;
				}
				break;
			case 'attribute_in_list':
				if ( in_array( $product_id, $products['include_variations'] ) ) {
					$status = true;
				}
				break;
			case 'attribute_not_in_list':
				if ( ! in_array( $product_id, $products['exclude_variations'] ) ) {
					$status = true;
				}
				break;
		}
		return $status;
	}

	public function process_max_order_qty(){
		add_filter('wholesalex_dr_min_max_rules', function($data, $valid_rules){
			// Pro
			if(isset($valid_rules['max_order_qty']) && !empty($valid_rules['max_order_qty'])) {
				$data['max_order_qty'] = $valid_rules['max_order_qty'];
				$this->valid_rules['max_order_qty'] = $data['max_order_qty'];
			}
			return $data;
		},10,2);

		add_filter('wholesalex_dr_min_max_qty_input_args',function($args,$data,$product){
			if(!empty($data['max_order_qty'])) {
				foreach ($data['max_order_qty'] as $rule) {
					if( isset($rule['conditions']) && !WHOLESALEX_Dynamic_Rules::check_rule_conditions($rule['conditions'])) {
						continue;
					}
					if(WHOLESALEX_Dynamic_Rules::is_eligible_for_rule($product->get_parent_id()?$product->get_parent_id():$product->get_id(),$product->get_id(),$rule['filter'])) {
						$args['max_value'] = $rule['rule']['_max_order_qty'];
					}	

				}
			}
			return $args;
		},10,3);

		add_filter('wholesalex_dr_min_max_add_to_cart_validation',function($status,$data,$product_id,$variation_id,$quantity,$product,$variation){
			foreach ($data['max_order_qty'] as $rule) {
				if( isset($rule['conditions']) && !WHOLESALEX_Dynamic_Rules::check_rule_conditions($rule['conditions'])) {
					continue;
				}
				if(WHOLESALEX_Dynamic_Rules::is_eligible_for_rule($product_id,$variation_id,$rule['filter'])) {
					if($variation_id) {
						$existing_qty = wholesalex()->cart_count( $variation_id );
						$new_qty      = $existing_qty + $quantity;
						if ( $new_qty > $rule['rule']['_max_order_qty'] ) {
							$status = false;
							/* translators: 1: minimum quantity, 2: product name */
							wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_max_quantity_error_message', sprintf( __( 'You can add maximum %1$d quantity of %2$s. You already added maximum amount.', 'wholesalex-pro' ), $rule['rule']['_max_order_qty'], $variation->get_name() ) ), 'error' );
						}
					}

					if($status) {
						$existing_qty = wholesalex()->cart_count( $product_id );
						$new_qty      = $existing_qty + $quantity;
						if ( $new_qty > $rule['rule']['_max_order_qty'] ) {
							$status = false;
							/* translators: 1: minimum quantity, 2: product name */
							wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_max_quantity_error_message', sprintf( __( 'You can add maximum %1$d quantity of %2$s. You already added maximum amount.', 'wholesalex-pro' ), $rule['rule']['_max_order_qty'], $variation_id ? $variation->get_name() : $product->get_name() ) ), 'error' );
						}
					}

					
				}
			}
			return $status;
		},10,7);

		add_filter('wholesalex_dr_min_max_update_cart_validation',function($status,$data,$product_id,$variation_id,$quantity,$product){
			foreach ($data['max_order_qty'] as $rule) {
				if( isset($rule['conditions']) && !WHOLESALEX_Dynamic_Rules::check_rule_conditions($rule['conditions'])) {
					continue;
				}
				if(WHOLESALEX_Dynamic_Rules::is_eligible_for_rule($product_id,$variation_id,$rule['filter'])) {
					if(isset($values['variation_id'] ) && $values['variation_id'] ) {
						if ( $quantity > $rule['rule']['_max_order_qty'] ) {
							$status = false;
							/* translators: 1: minimum quantity, 2: product name */
							wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_max_quantity_error_message', sprintf( __( 'You can add maximum %1$d quantity of %2$s.', 'wholesalex-pro' ), $rule['rule']['_max_order_qty'], $product->get_name() ) ), 'error' );
						}
					}

					if($status) {
						if ( $quantity > $rule['rule']['_max_order_qty'] ) {
							$status = false;
							/* translators: 1: minimum quantity, 2: product name */
							wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_max_quantity_error_message', sprintf( __( 'You can add maximum %1$d quantity of %2$s.', 'wholesalex-pro' ), $rule['rule']['_max_order_qty'], $product->get_name() ) ), 'error' );
						}
					}

					
				}
			}
			return $status;
		},10,6);

		add_action('wholesalex_dr_min_max_check_cart_items',function($status,$data,$cart_item,$product_title){
			foreach ($data['max_order_qty'] as $rule) {
				if( isset($rule['conditions']) && !WHOLESALEX_Dynamic_Rules::check_rule_conditions($rule['conditions'])) {
					continue;
				}
				$max = $rule['rule']['_max_order_qty'];
				if(WHOLESALEX_Dynamic_Rules::is_eligible_for_rule($cart_item['variation_id']?0:$cart_item['product_id'], $cart_item['variation_id'],$rule['filter'])) {
					if ( $cart_item['quantity'] > $max ) {
						$status = false;
						/* translators: 1: minimum quantity, 2: product name */
						wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_max_quantity_error_message', sprintf( __( 'You can add maximun %1$d quantity of %2$s.', 'wholesalex-pro' ), $max, $product_title ) ), 'error' );
					}
				}
				if(WHOLESALEX_Dynamic_Rules::is_eligible_for_rule($cart_item['product_id'],0,$rule['filter']) && $status) {
					if ( $cart_item['quantity'] > $max ) {
						$status = false;
						/* translators: 1: minimum quantity, 2: product name */
						wc_add_notice( apply_filters( 'wholesalex_dynamic_rules_max_quantity_error_message', sprintf( __( 'You can add maximun %1$d quantity of %2$s.', 'wholesalex-pro' ), $max, $product_title ) ), 'error' );
					}
				}
			}
		},10,4);

		add_action('woocommerce_store_api_product_quantity_maximum',function($value,$product){
			if(!empty($this->valid_rules['max_order_qty'])) {
				foreach ($this->valid_rules['max_order_qty'] as $rule) {
					if( isset($rule['conditions']) && !WHOLESALEX_Dynamic_Rules::check_rule_conditions($rule['conditions'])) {
						continue;
					}
					if(WHOLESALEX_Dynamic_Rules::is_eligible_for_rule($product->get_parent_id()?$product->get_parent_id():$product->get_id(),$product->get_id(),$rule['filter'])) {
						$value= $rule['rule']['_max_order_qty'];
					}	

				}
			}
			return $value;
		},10,2);
	}

	public function handle_pro_dynamic_rules($valid_dynamic_rules) {

		// Quantity Based
		add_filter('wholesalex_dr_discounts',function($discounts_releated_data) use($valid_dynamic_rules){
			if(isset($valid_dynamic_rules['quantity_based']) && !empty($valid_dynamic_rules['quantity_based']) ) {
				$discounts_releated_data['quantity_based'] = $valid_dynamic_rules['quantity_based'];
			}
			return $discounts_releated_data;
		});
		
		// Extra Charge
		add_filter('wholesalex_dr_cart_related_data',function($cart_related_data) use($valid_dynamic_rules){
			if(isset($valid_dynamic_rules['extra_charge']) && !empty($valid_dynamic_rules['extra_charge']) ) {
				$cart_related_data['extra_charge'] = $valid_dynamic_rules['extra_charge'];
			}
			return $cart_related_data;
		});

		add_filter('wholesalex_dr_cart_fees',function($cart_fees,$rules,$selected_gateway){
			if(isset($rules['extra_charge']) && !empty($rules['extra_charge'])) {
				$hash = array();
				foreach ($rules['extra_charge'] as $rule) {

					if( isset($rule['conditions']) && !WHOLESALEX_Dynamic_Rules::check_rule_conditions($rule['conditions'])) {
						continue;
					}
	
					$allowed_gateways = WHOLESALEX_Dynamic_Rules::get_multiselect_values($rule['rule']['_payment_gateways']);
					
					if(in_array($selected_gateway,$allowed_gateways)) {
						foreach ($allowed_gateways as $key => $gateway) {
							$discount_amount = 0;
							$discount_name = '';
							$is_all_products = $rule['filter']['is_all_products'];
							$discount_type = $rule['rule']['_charge_type'];
							$hash_key = md5(serialize($rule['filter']));
							if($is_all_products) {
								$total_value = wholesalex()->get_cart_total();
								$discount_amount = ('percentage' == $discount_type) ? ( $total_value * floatval($rule['rule']['_charge_amount']) )/100 : floatval($rule['rule']['_charge_amount']);
								$discount_name =  apply_filters( 'wholesalex_payment_gateway_default_charge_name', isset( $rule['rule']['_charge_name'] ) ? $rule['rule']['_charge_name'] : __( 'Extra Charge','wholesalex' ) );
								if(isset($hash[$hash_key]) && is_array($hash[$hash_key])) {
									if($discount_amount>=$hash[$hash_key]['charge']) {
										$hash[md5(serialize($rule['filter']))] = array('charge'=> $discount_amount,'name'=> $discount_name);
										wholesalex()->set_usages_dynamic_rule_id( $rule['id'] );
									}
								} else {
									$hash[md5(serialize($rule['filter']))] = array('charge'=> $discount_amount,'name'=> $discount_name);
									wholesalex()->set_usages_dynamic_rule_id( $rule['id'] );
								}				
							} else {
								if ( ! isset( WC()->cart ) || null === WC()->cart->get_cart() ) {
									return;
								}
								$rule_total_amount = 0;
								foreach ( WC()->cart->get_cart() as $cart_item ) {
									if(WHOLESALEX_Dynamic_Rules::is_eligible_for_rule($cart_item['product_id'],$cart_item['variation_id'],$rule['filter'])) {						
										$rule_total_amount += $cart_item['line_total'];
										if(apply_filters('wholesalex_dr_extra_charge_on_tax',true)) {
											$rule_total_amount+=$cart_item['line_tax'];
										}
										
									}
								}				
								$discount_amount = ('percentage' == $discount_type) ? ( $rule_total_amount * floatval($rule['rule']['_charge_amount']) )/100 : floatval($rule['rule']['_charge_amount']);
								
								$discount_name =  apply_filters( 'wholesalex_payment_gateway_default_charge_name', isset( $rule['rule']['_charge_name'] ) ? $rule['rule']['_charge_name'] : __( 'Extra Charge','wholesalex' ) );
								if(isset($hash[$hash_key]) && is_array($hash[$hash_key])) {
									if($discount_amount>=$hash[$hash_key]['charge']) {
										$hash[md5(serialize($rule['filter']))] = array('charge'=> $discount_amount,'name'=> $discount_name);
										wholesalex()->set_usages_dynamic_rule_id( $rule['id'] );
									}
								} else {
									$hash[md5(serialize($rule['filter']))] = array('charge'=> $discount_amount,'name'=> $discount_name);
									wholesalex()->set_usages_dynamic_rule_id( $rule['id'] );
								}
								
							}
						}
					}
				}
				$cart_fees['extra_charge']=$hash;
			}
			return $cart_fees;
		},10,3);

		// Buy X Get Y
		if(isset($valid_dynamic_rules['buy_x_get_y']) && !empty($valid_dynamic_rules['buy_x_get_y'])) {
			$this->valid_rules['buy_x_get_y'] = $valid_dynamic_rules['buy_x_get_y'];
			$this->buy_x_get_y();
		}
		
		// Max Order Quantity
		$this->process_max_order_qty();

		// Restrict Product Visibility
		if(isset($valid_dynamic_rules['restrict_product_visibility']) && !empty($valid_dynamic_rules['restrict_product_visibility'])) {
			$this->valid_rules['restrict_product_visibility'] = $valid_dynamic_rules['restrict_product_visibility'];
			$data = array(
				'include_product' => array(),
				'exclude_product' => array(),
				'include_cat'	  => array(),
				'exclude_cat'	  => array(),
				'include_variations' => array(),
				'exclude_variations' => array(),
				'is_all_products' => array()
			);
	
			foreach ($valid_dynamic_rules['restrict_product_visibility'] as $rpv) {
				if(!WHOLESALEX_Dynamic_Rules::check_rule_conditions($rpv['conditions'])) {
					continue;
				}
	
				if(!empty($rpv['filter']['is_all_products'])) {
					$data['is_all_products'] = true;
					break;
				}
				if(!empty($rpv['filter']['include_products'])) {
					$data['include_product'] = array_merge($data['include_product'],$rpv['filter']['include_products']);
				}
				if(!empty($rpv['filter']['exclude_products'])) {
					$data['exclude_product'] = array_merge($data['exclude_product'],$rpv['filter']['exclude_products']);
				}
				if(!empty($rpv['filter']['include_cats'])) {
					$data['include_cat'] = array_merge($data['include_cat'],$rpv['filter']['include_cats']);
				}
				if(!empty($rpv['filter']['exclude_cats'])) {
					$data['exclude_cat'] = array_merge($data['exclude_cat'],$rpv['filter']['exclude_cats']);
				}
				if(!empty($rpv['filter']['include_variations'])) {
					$data['include_variations'] = array_merge($data['include_variations'],$rpv['filter']['include_variations']);
				}
				if(!empty($rpv['filter']['exclude_variations'])) {
					$data['exclude_variations'] = array_merge($data['exclude_variations'],$rpv['filter']['exclude_variations']);
				}			
			}
			$this->valid_rules['restrict_product_visibility'] = $data;

			$this->restrict_visibility();
		}

		// Hidden Price
		if(isset($valid_dynamic_rules['hidden_price']) && !empty($valid_dynamic_rules['hidden_price'])) {
			$this->valid_rules['hidden_price'] = array();
			$data = array(
				'include_product' => array(),
				'exclude_product' => array(),
				'include_cat'	  => array(),
				'exclude_cat'	  => array(),
				'include_variations' => array(),
				'exclude_variations' => array(),
				'is_all_products' => array(),
				'hidden_is_all_products' => array(),
				'hidden_request_product_list' => array(),
				'hidden_exclude_products' => array(),
				'hidden_include_cats' => array(),
				'hidden_exclude_cats' => array(),
				'hidden_include_variations' => array(),
				'hidden_exclude_variations' => array(),
				'request_quote_enable' => false
			);
	
			foreach ($valid_dynamic_rules['hidden_price'] as $rpv) {
				if(!WHOLESALEX_Dynamic_Rules::check_rule_conditions($rpv['conditions'])) {
					continue;
				}
	
				if(!empty($rpv['filter']['is_all_products']) ) {
					$data['is_all_products'] = true;
					if ( isset( $rpv['rule'] ) && isset( $rpv['rule']['_hidden_price_checkbox'] ) && $rpv['rule']['_hidden_price_checkbox'] == 'yes' ) {
						$data['hidden_is_all_products'] = true;
						$data['request_quote_enable'] = true;
					}
					break;
				}
				if(!empty($rpv['filter']['include_products'])) {
					$data['include_product'] = array_merge($data['include_product'],$rpv['filter']['include_products']);
				}
				if(!empty($rpv['filter']['exclude_products'])) {
					$data['exclude_product'] = array_merge($data['exclude_product'],$rpv['filter']['exclude_products']);
				}
				if(!empty($rpv['filter']['include_cats'])) {
					$data['include_cat'] = array_merge($data['include_cat'],$rpv['filter']['include_cats']);
				}
				if(!empty($rpv['filter']['exclude_cats'])) {
					$data['exclude_cat'] = array_merge($data['exclude_cat'],$rpv['filter']['exclude_cats']);
				}
				if(!empty($rpv['filter']['include_variations'])) {
					$data['include_variations'] = array_merge($data['include_variations'],$rpv['filter']['include_variations']);
				}
				if(!empty($rpv['filter']['exclude_variations'])) {
					$data['exclude_variations'] = array_merge($data['exclude_variations'],$rpv['filter']['exclude_variations']);
				}
				
				if ( isset( $rpv['rule'] ) && isset( $rpv['rule']['_hidden_price_checkbox'] ) && $rpv['rule']['_hidden_price_checkbox'] == 'yes' ) {
					$data['hidden_request_product_list'] = array_merge( $data['hidden_request_product_list'] , $rpv['filter']['include_products'] );
					$data['hidden_exclude_products'] = array_merge( $data['hidden_exclude_products'] , $rpv['filter']['exclude_products'] );
					$data['hidden_include_cats'] = array_merge( $data['hidden_include_cats'] , $rpv['filter']['include_cats'] );
					$data['hidden_exclude_cats'] = array_merge( $data['hidden_exclude_cats'] , $rpv['filter']['exclude_cats'] );
					$data['hidden_include_variations'] = array_merge( $data['hidden_include_variations'] , $rpv['filter']['include_variations'] );
					$data['hidden_exclude_variations'] = array_merge( $data['hidden_exclude_variations'] , $rpv['filter']['exclude_variations'] );
					$data['request_quote_enable'] = true;
				}
				
			}
			$this->valid_rules['hidden_price'] = $data;

			$this->hidden_price_handler();
		}

		// Non Purchasable
		if(isset($valid_dynamic_rules['non_purchasable']) && !empty($valid_dynamic_rules['non_purchasable'])) {
			$this->valid_rules['non_purchasable'] = $valid_dynamic_rules['non_purchasable'];
			$data = array(
				'include_product' => array(),
				'exclude_product' => array(),
				'include_cat'	  => array(),
				'exclude_cat'	  => array(),
				'include_variations' => array(),
				'exclude_variations' => array(),
				'is_all_products' => array()
			);
	
			foreach ($valid_dynamic_rules['non_purchasable'] as $rpv) {
				if(!WHOLESALEX_Dynamic_Rules::check_rule_conditions($rpv['conditions'])) {
					continue;
				}
	
				if(!empty($rpv['filter']['is_all_products'])) {
					$data['is_all_products'] = true;
					break;
				}
				if(!empty($rpv['filter']['include_products'])) {
					$data['include_product'] = array_merge($data['include_product'],$rpv['filter']['include_products']);
				}
				if(!empty($rpv['filter']['exclude_products'])) {
					$data['exclude_product'] = array_merge($data['exclude_product'],$rpv['filter']['exclude_products']);
				}
				if(!empty($rpv['filter']['include_cats'])) {
					$data['include_cat'] = array_merge($data['include_cat'],$rpv['filter']['include_cats']);
				}
				if(!empty($rpv['filter']['exclude_cats'])) {
					$data['exclude_cat'] = array_merge($data['exclude_cat'],$rpv['filter']['exclude_cats']);
				}
				if(!empty($rpv['filter']['include_variations'])) {
					$data['include_variations'] = array_merge($data['include_variations'],$rpv['filter']['include_variations']);
				}
				if(!empty($rpv['filter']['exclude_variations'])) {
					$data['exclude_variations'] = array_merge($data['exclude_variations'],$rpv['filter']['exclude_variations']);
				}
				
			}
			$this->valid_rules['non_purchasable'] = $data;
			$this->make_not_purchasable();
		}

		/**
		 * Restrict Checkout Link Disable Pro Feature
		 *
		 * @return void
		 */
		add_action(
			'woocommerce_check_cart_items',
			function ( ) use( $valid_dynamic_rules ) {
				//Check if apply to products
				if ( isset( $valid_dynamic_rules['restrict_checkout'] ) ) {
					$restrict_checkout_condition = $valid_dynamic_rules['restrict_checkout'];
					
					$cart_contents = WC()->cart->get_cart();
					$product_ids = array();
					$category_ids = array();
					$parent_ids = array();

					foreach ( $cart_contents as $cart_item_key => $cart_item ) {
						$product_id = $cart_item['product_id'];
						$variation_id = $cart_item['variation_id'];
						if ( $variation_id ) {
							$product_ids[] = $variation_id;
							$product_ids[] = $product_id;
							$parent_ids[]  = $product_id;
						}else {
							$product_ids[] = $product_id;
						}
						$product_categories = wp_get_post_terms( $product_id, 'product_cat' );
						foreach ( $product_categories as $category ) {
							$category_ids[] = $category->term_id;
						}
					}
					$category_ids 		= array_unique($category_ids);
					$product_ids 		= array_unique($product_ids);
					$is_apply_product 	= false;
					foreach ( $restrict_checkout_condition as $restrict_condition ) {
						$restrict_checkout_condition = $restrict_condition['filter'];
						if($restrict_checkout_condition['is_all_products']  ) {
							$is_apply_product = true;
						}
						if(!empty($restrict_checkout_condition['include_products']) && array_intersect($product_ids, $restrict_checkout_condition['include_products'])) {
							$is_apply_product = true;
						} 
						else if(!empty($restrict_checkout_condition['exclude_products']) ) {
							$exclude_products_diff_a 	= array_diff($product_ids, $restrict_checkout_condition['exclude_products']);
							$exclude_products_diff_b 	= array_diff($restrict_checkout_condition['exclude_products'], $product_ids);
							$uncommon_exclude_products 	= array_merge($exclude_products_diff_a, $exclude_products_diff_b);
							
							if ( array_intersect( $product_ids, $uncommon_exclude_products ) ) {
								$is_apply_product = true;
							}
						} 
						else if(!empty($restrict_checkout_condition['include_cats']) && !empty(array_intersect($category_ids, $restrict_checkout_condition['include_cats']))) {
							$is_apply_product = true;
						} 
						else if( !empty($restrict_checkout_condition['exclude_cats']) ) {
							$exclude_cats_diff_a 	= array_diff( $category_ids, $restrict_checkout_condition['exclude_cats'] );
							$exclude_cats_diff_b 	= array_diff( $restrict_checkout_condition['exclude_cats'], $category_ids );
							$uncommon_exclude_cats 	= array_merge( $exclude_cats_diff_a, $exclude_cats_diff_b );
							if ( array_intersect( $category_ids, $uncommon_exclude_cats ) ) {
								$is_apply_product = true;
							}
						} 
						else if(!empty($restrict_checkout_condition['include_variations']) && array_intersect($product_ids, $restrict_checkout_condition['include_variations'])) {
							$is_apply_product = true;
						} 
						else if( !empty($restrict_checkout_condition['exclude_variations']) ) {
							$exclude_variations_diff_a   = array_diff( $product_ids, $restrict_checkout_condition['exclude_variations'] );
							$exclude_variations_diff_b   = array_diff( $restrict_checkout_condition['exclude_variations'], $product_ids );
							$uncommon_exclude_variations = array_merge( $exclude_variations_diff_a, $exclude_variations_diff_b );
							if ( ! empty( $parent_ids ) ) {
								$uncommon_exclude_variations = array_diff($uncommon_exclude_variations, $parent_ids);
							}
							if ( array_intersect( $product_ids, $uncommon_exclude_variations ) ) {
								$is_apply_product = true;
							}
						}

					}


					$__discounts = $valid_dynamic_rules['restrict_checkout'];
					foreach ( $__discounts as $discount ) {
						$_conditions = $discount['conditions'];
						$is_checkout = WHOLESALEX_Dynamic_Rules::is_conditions_fullfiled( $_conditions['tiers'] );
					}
					$checkout_restrict_message = 'Please fulfill the cart limitation to checkout.';
					$check_messages = $__discounts[0]['conditions']['tiers'];
					foreach ( $check_messages as $check_message ) {
						$restrictionType = '';
						$settingKey = '';
						$placeholderKey = '';
						switch ( $check_message['_conditions_for'] ) {
							case 'cart_total_value':
								$restrictionType = 'amount';
								$settingKey 	 = 'only_total_cart_value_promo_text';
								$placeholderKey  = '{cart_value}';
								break;
							case 'cart_total_qty':
								$restrictionType = 'number of';
								$settingKey 	 = 'only_total_cart_quantity_promo_text';
								$placeholderKey  = '{cart_quantity}';
								break;
							case 'cart_total_weight':
								$restrictionType = 'cart weight';
								$settingKey 	 = 'only_total_cart_weight_promo_text';
								$placeholderKey  = '{cart_weight}';
								break;
						}
						if ( empty( $settingKey ) ){
							return;
						}
						$checkout_restrict_message 	= "Specify the minimum $restrictionType for checkout";
						$checkout_setting_message 	= wholesalex()->get_setting( $settingKey );
						$checkout_setting_message 	= str_replace( $placeholderKey, $check_message['_conditions_value'], $checkout_setting_message );
						$checkout_restrict_message 	= ( $checkout_setting_message ) ? $checkout_setting_message : $checkout_restrict_message;
					}
					if ( $is_checkout && $is_apply_product ) {
						remove_action( 'woocommerce_proceed_to_checkout', 'woocommerce_button_proceed_to_checkout', 20 );
						// echo '<a href="#" class="checkout-button button alt wc-forward">';
						// esc_html_e( 'Proceed to checkout', 'wholesalex-pro' );
						// echo '</a>';
						echo wc_add_notice( __( $checkout_restrict_message , 'wholesalex-pro' ), 'error' );
					}
				}
			},
			1
		);

	}


}
