<?php //phpcs:ignore
namespace WHOLESALEX_PRO;

use WHOLESALEX\WHOLESALEX_Email_Manager;

defined( 'ABSPATH' ) || exit;
/**
 * Email Class
 */
class Email {

	/**
	 * Contain Email Templates.
	 *
	 * @var array
	 */
	public $email_templates = '';

	/**
	 * Constructor
	 */
	public function __construct() {
		if ( ! wholesalex_pro()->is_active() ) {
			return;
		}

		$this->email_templates = get_option( '__wholesalex_email_templates', array() );

		// add_action( 'wholesalex_set_status_pending', array( $this, 'user_status_pending_email_trigger' ) );

		// add_action( 'wholesalex_set_status_reject', array( $this, 'user_status_reject_email_trigger' ) );
	}

	/**
	 * User Status : Pending -> Trigger Email
	 *
	 * @param int $user_id User ID.
	 */
	public function user_status_pending_email_trigger( $user_id ) {
		$email_template = $this->email_templates['new_wholesalex_user'];
		if ( $user_id ) {
			$user_data = get_userdata( $user_id );
			$to        = $user_data->user_email;
			if ( ! isset( $to ) || empty( $to ) ) {
				return;
			}
			$smart_tags = array(
				'{date}'         => gmdate( 'd-m-Y' ),
				'{username}'     => $user_data->user_login,
				'{email}'        => $user_data->user_email,
				'{display_name}' => $user_data->display_name,
				'{first_name}'   => $user_data->user_firstname,
				'{last_name}'    => $user_data->user_lastname,
				'{full_name}'    => $user_data->user_firstname . ' ' . $user_data->user_lastname,
				'{site_name}'    => get_option( 'blogname' ),
				'{admin_name}'   => WHOLESALEX_Email_Manager::get_admin_display()
			);
			if ( get_option( 'new_wholesalex_user_email_status' ) && ! empty( $to ) ) {
				$subject = isset( $email_template['subject'] ) ? $email_template['subject'] : '';
				$content = isset( $email_template['content'] ) ? $email_template['content'] : '';
				foreach ( $smart_tags as $key => $value ) {
					$subject = str_replace( $key, $value, $subject );
					$content = str_replace( $key, $value, $content );
				}
				$headers = array( 'Content-Type: text/html; charset=UTF-8' );

				if(wholesalex_pro()->is_free_ready() && version_compare(WHOLESALEX_VER,'1.2.0','>=')>0){
					$status  = wholesalex()->send_email('new_wholesalex_user',$to,$smart_tags);
				} else {
					$status  = wp_mail( $to, $subject, $content, $headers );
				}
				if ( isset( $email_template['admin_notification'] ) && $email_template['admin_notification'] ) {
					if(wholesalex_pro()->is_free_ready() && version_compare(WHOLESALEX_VER,'1.2.0','>=')>0){
						$status  = wholesalex()->send_email('new_wholesalex_user',$to,$smart_tags);
					} else {
						$status  = wp_mail( get_option( 'admin_email' ), $subject, $content, $headers );
					}
				}
			}
		}

	}
	/**
	 * User Status : Pending -> Trigger Email
	 *
	 * @param int $user_id User ID.
	 */
	public function user_status_reject_email_trigger( $user_id ) {
		$email_template = $this->email_templates['wholesalex_registration_decline'];
		if ( $user_id ) {
			$user_data = get_userdata( $user_id );
			$to        = $user_data->user_email;
			if ( ! isset( $to ) || empty( $to ) ) {
				return;
			}
			$smart_tags = array(
				'{date}'         => gmdate( 'd-m-Y' ),
				'{username}'     => $user_data->user_login,
				'{email}'        => $user_data->user_email,
				'{display_name}' => $user_data->display_name,
				'{first_name}'   => $user_data->user_firstname,
				'{last_name}'    => $user_data->user_lastname,
				'{full_name}'    => $user_data->user_firstname . ' ' . $user_data->user_lastname,
				'{site_name}'    => get_option( 'blogname' ),
				'{admin_name}'   => WHOLESALEX_Email_Manager::get_admin_display()
			);
			if ( get_option( 'wholesalex_registration_decline_email_status' ) && ! empty( $to ) ) {
				$subject = isset( $email_template['subject'] ) ? $email_template['subject'] : '';
				$content = isset( $email_template['content'] ) ? $email_template['content'] : '';
				foreach ( $smart_tags as $key => $value ) {
					$subject = str_replace( $key, $value, $subject );
					$content = str_replace( $key, $value, $content );
				}
				$headers = array( 'Content-Type: text/html; charset=UTF-8' );

				if(wholesalex_pro()->is_free_ready() && version_compare(WHOLESALEX_VER,'1.2.0','>=')>0){
					$status  = wholesalex()->send_email('wholesalex_registration_decline',$to,$smart_tags);
				} else {
					$status  = wp_mail( $to, $subject, $content, $headers );
				}
				if ( isset( $email_template['admin_notification'] ) && $email_template['admin_notification'] ) {
					if(wholesalex_pro()->is_free_ready() && version_compare(WHOLESALEX_VER,'1.2.0','>=')>0){
						$status  = wholesalex()->send_email('wholesalex_registration_decline',$to,$smart_tags);
					} else {
						$status  = wp_mail( get_option( 'admin_email' ), $subject, $content, $headers );
					}
				}
			}
		}

	}

}
