<?php //phpcs:ignore
namespace WHOLESALEX_PRO;

defined( 'ABSPATH' ) || exit;
use WOPB_PRO\Currency_Switcher_Action;

/**
 * Functions Class
 */
class Functions {

    /**
	 * Setup class.
	 *
	 * @since v.1.0.0
	 */
	public function __construct() {

		if ( ! isset( $GLOBALS['wholesalex_settings'] ) ) {
			$GLOBALS['wholesalex_settings'] = get_option( 'wholesalex_settings' );
		}

	}

	/**
	 * Check Is Free Plugin Ready.
	 *
	 * @return boolean
	 */
	public function is_free_ready() {
		$active_plugins = get_option( 'active_plugins', array() );
		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, array_keys( get_site_option( 'active_sitewide_plugins', array() ) ) );
		}
		if ( file_exists( WP_PLUGIN_DIR . '/wholesalex/wholesalex.php' ) && in_array( 'wholesalex/wholesalex.php', $active_plugins, true ) &&  file_exists( WP_PLUGIN_DIR . '/woocommerce/woocommerce.php') && in_array( 'woocommerce/woocommerce.php', $active_plugins, true ) ) {
			return true;
		} else {
			return false;
		}
	}


    public function get_license_status() {
        $status= get_option( 'edd_wholesalex_license_status', '' );
        if('invalid'===$status) {
            $license_data = get_option( 'edd_wholesalex_license_data',array() );
            if(isset($license_data['error'])) {
                $status = $license_data['error'];
            }
        }

        return $status;
    }
	/**
	 * Get Eligible Addon List
	 *
	 * @return array
	 * @since 1.1.1
	 */
	public function get_eligible_addon_list() {
		$__price_id = wholesalex_pro()->get_price_id();
		$response   = array();
		switch ( $__price_id ) {
			case '1':
			case '4':
			case '2':
			case '5':
			case '3':
			case '7':
			case '6':
				$response = array( 'raq', 'conversation', 'wallet', 'bulkorder', 'subaccount');
			default:
				break;
		}

		if('3'==$__price_id || '6'==$__price_id || '7'==$__price_id) {
			$response[]='whitelabel';
		}

		return $response;
	}

	/**
	 * Get Price ID
	 *
	 * @return int|string|false
	 * @since 1.1.1
	 */
	public function get_price_id() {
		$__id = get_option( '__wholesalex_price_id', false );
		if($this->is_license_valid()) {
			return $__id;
		}
		return '';
	}

	/**
	 * Check Is License Valid
	 *
	 * @return boolean
	 * @since 1.1.1
	 */
	public function is_license_valid() {
		$__status = get_option( 'edd_wholesalex_license_status', true );
		return 'valid' === $__status;
	}

	/**
	 * Get Pro Activation Status
	 */
	public function is_active() {
		$__status = get_option( 'edd_wholesalex_license_status', true );
		return 'valid' === $__status;
	}

	/**
	 * Price After Currency Changed
	 * For Any Currency Switcher Compability Issue, Add Necessary Codes In This Function.
	 *
	 * @param float $price Price.
	 * @return float $price.
	 * @since 1.0.3
	 */
	public function price_after_currency_changed( $price ) {
		// ProductX Currency Switcher Compatibility.
		if ( defined( 'WOPB_VER' ) && defined( 'WOPB_PRO_VER' ) && class_exists( 'WOPB_PRO\Currency_Switcher_Action' ) ) {
			$current_currency_code = wopb_function()->get_setting( 'wopb_current_currency' );
			$default_currency      = wopb_function()->get_setting( 'wopb_default_currency' );
			$current_currency      = Currency_Switcher_Action::get_currency( $current_currency_code );
			if ( ! $current_currency ) {
				$current_currency = $default_currency;
			}

			if ( $current_currency_code !== $default_currency ) {

				$wopb_current_currency_rate = ( isset( $current_currency['wopb_currency_rate'] ) && $current_currency['wopb_currency_rate'] > 0 && ! ( $current_currency['wopb_currency_rate'] == '' ) ) ? $current_currency['wopb_currency_rate'] : 1;
				$wopb_current_exchange_fee  = ( isset( $current_currency['wopb_currency_exchange_fee'] ) && $current_currency['wopb_currency_exchange_fee'] >= 0 && ! ( $current_currency['wopb_currency_exchange_fee'] == '' ) ) ? $current_currency['wopb_currency_exchange_fee'] : 0;
				$total_rate                 = ( $wopb_current_currency_rate + $wopb_current_exchange_fee );
				return $price / $total_rate;
			}
		}
		return $price;
	}

	/**
	 * Get WholesaleX License Type
	 *
	 * @return string
	 */
	public function get_license_type() {
		$__status = get_option( 'edd_wholesalex_license_status', true );
		if ( 'valid' !== $__status ) {
			return '';
		}
		return get_option( '__wholesalex_license_type', '' );
	}
    
	/**
	 * Unlock Options that are locked for different pricing plans.
	 *
	 * @param array $locked_options Locked Options.
	 * @return array Unlocked Options
	 * @since 1.0.6
	 */
	public function unlock_options( $locked_options ) {

		$unlocked_options = array();
		if ( wholesalex_pro()->is_active() ) {
			foreach ( $locked_options as $key => $value ) {
				$temp       = $key;
				$search_key = '#^pro_(.*)$#i';
				if ( preg_match( $search_key, $key ) ) {
					$option_key                      = str_replace( 'pro_', '', $key );
					$option_name                     = str_replace( '(Pro)', '', $value );
					$unlocked_options[ $option_key ] = $option_name;
					$temp                            = $option_key;
				}
				if ( $key === $temp ) {
					$unlocked_options[ $key ] = $value;
				}
			}
		}
		
		return $unlocked_options;
	}
	/**
	 * Unlock Tier Layouts that are locked for different pricing plans.
	 *
	 * @param array $locked_options Locked Options.
	 * @return array Unlocked Options
	 * @since 1.0.6
	 */
	public function unlock_layouts( $locked_options ) {
		$unlocked_options = array();
		if ( wholesalex_pro()->is_active() ) {
			foreach ( $locked_options as $key => $value ) {
				$temp       = $key;
				$search_key = '#^pro_(.*)$#i';
				if ( preg_match( $search_key, $key ) ) {
					$option_key                      = str_replace( 'pro_', '', $key );
					$unlocked_options[ $option_key ] = $value;
					$temp                            = $option_key;
				}
				if ( $key === $temp ) {
					$unlocked_options[ $key ] = $value;
				}
			}
		}
		
		return $unlocked_options;
	}


	/**
	 * Check current user is a b2b user or has admin access
	 *
	 * @return boolean
	 * @since 1.1.3
	 */
	public function is_active_b2b_user() {
		$account_role      = wholesalex()->get_current_user_role();
		$__current_user_id = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
		$__account_status  = get_user_meta( $__current_user_id, '__wholesalex_status', true );
		return ( ! ( ( 'wholesalex_guest' === $account_role ) || ( 'wholesalex_b2c_users' === $account_role ) ) && 'active' === $__account_status ) || current_user_can( 'manage_options' );
	}

	/**
	 * Get Product Count at cart
	 *
	 * @param int $product_id Product or Variation ID.
	 * @return int Product count at cart.
	 * @since 1.1.6
	 */
	public function product_cart_count( $product_id ) {
		$__quantity = 0;
		$product = wc_get_product($product_id);
		$parent_id = 0;
		if($product) {
			$parent_id = $product->get_parent_id();
		}
		
		if ( ! is_null( WC()->cart ) ) {
			foreach ( WC()->cart->get_cart() as $cart_item ) {
				if ( ! empty( $cart_item['data'] ) && in_array( $parent_id?$parent_id:$parent_id, array( $cart_item['product_id'], $cart_item['variation_id'] ), true ) ) {
					$__quantity += $cart_item['quantity'];
				}
			}
		}

		
		return $__quantity;
	}

	public function starts_with( $haystack, $needle ) {
		return substr( $haystack, 0, strlen( $needle ) ) === $needle;
	}

    /**
	 * Get Global Plugin Settings
	 *
	 * @since 1.0.0
	 * @param STRING $key Key of the Option.
	 * @return ARRAY | STRING
	 * @since 1.2.4 Add Default Option
	 */
	public function get_setting( $key = '', $default = '' ) {
		$data = $GLOBALS['wholesalex_settings'];
		if ( '' !== $key ) {
			return isset( $data[ $key ] ) ? $data[ $key ] : $default;
		} else {
			return $data;
		}
	}


	/**
	 * Set Option Settings
	 *
	 * @since 1.0.0
	 * @param STRING $key Key of the Option .
	 * @param STRING $val Value of the Option .
	 */
	public function set_setting( $key = '', $val = '' ) {
		if ( '' !== $key ) {
			$data         = $GLOBALS['wholesalex_settings'];
			$data[ $key ] = $val;
			update_option( 'wholesalex_settings', $data );
			$GLOBALS['wholesalex_settings'] = $data;
		}
	}
}
