<?php //phpcs:ignore
namespace WHOLESALEX_PRO;

defined( 'ABSPATH' ) || exit;

/**
 * WholesaleX Pro Initialization
 */
class WHOLESALEX_PRO_Initialization {
	/**
	 * Constructor
	 */
	public function __construct() {
		$this->requires();
		add_action( 'wp_enqueue_scripts', array( __CLASS__, 'register_scripts' ) );
	}

	/**
	 * Requires
	 *
	 * @return void
	 */
	public function requires() {
		require_once WHOLESALEX_PRO_PATH . 'classes/Notice.php';
		new \WHOLESALEX_PRO\Notice();
		if ( wholesalex_pro()->is_free_ready() ) {
			$this->include_addons();
			require_once WHOLESALEX_PRO_PATH . 'classes/DynamicRules.php';
			require_once WHOLESALEX_PRO_PATH . 'classes/Category.php';
			require_once WHOLESALEX_PRO_PATH . 'classes/Product.php';
			require_once WHOLESALEX_PRO_PATH . 'classes/Settings.php';
			require_once WHOLESALEX_PRO_PATH . 'classes/Email.php';
			require_once WHOLESALEX_PRO_PATH . 'classes/Profile.php';
			new \WHOLESALEX_PRO\DynamicRules();
			new \WHOLESALEX_PRO\Category();
			new \WHOLESALEX_PRO\Product();
			new \WHOLESALEX_PRO\Settings();
			new \WHOLESALEX_PRO\Email();
			new \WHOLESALEX_PRO\Profile();
		}

		// Pro Plugin Updater Class.
		require_once WHOLESALEX_PRO_PATH . 'classes/updater/License.php';
		new \WHOLESALEX_PRO\License();

		add_action( 'activated_plugin', array( $this, 'activation_redirect' ) );
	}



	/**
	 * Activation Redirect
	 *
	 * @param string $plugin Plugin URL.
	 * @return void
	 */
	public function activation_redirect( $plugin ) {
		// If WholesaleX Is Ready Redirect To License Page.
		if ( wholesalex_pro()->is_free_ready() && 'wholesalex-pro/wholesalex-pro.php' === $plugin ) {
			exit( wp_safe_redirect( admin_url( 'admin.php?page=wholesalex-license' ) ) ); //phpcs:ignore
		}
	}

	/**
	 * Include Addons directory.
	 *
	 * @return void
	 */
	public function include_addons() {
		add_filter(
			'wholesalex_frontend_localize_data',
			function ( $data ) {
				$data['settings'] = wholesalex()->get_setting();
				return $data;
			},
			1
		);
		$eligible_addon_list = wholesalex_pro()->get_eligible_addon_list();
		$addons_dir          = array_filter( glob( WHOLESALEX_PRO_PATH . 'addons/*' ), 'is_dir' );
		if ( count( $addons_dir ) > 0 ) {
			foreach ( $addons_dir as $key => $value ) {
				$addon_dir_name = str_replace( dirname( $value ) . '/', '', $value );
				$file_name      = WHOLESALEX_PRO_PATH . 'addons/' . $addon_dir_name . '/init.php';
				$is_eligible    = in_array( $addon_dir_name, $eligible_addon_list, true ) ? true : false;
				$can_use_addon  = apply_filters( 'wholesalex_addon_' . $addon_dir_name . '_status', true, get_current_user_id() );
				if ( file_exists( $file_name ) && $is_eligible && $can_use_addon ) {
					include_once $file_name;
				}
			}
		}

		/**
		 * Hook Added (wholesalex_pro_after_addon_includes)
		 *
		 * @since 1.0.4
		 */
		do_action( 'wholesalex_pro_after_addon_includes' );

	}


	/**
	 * Register Scripts
	 *
	 * @return void
	 */
	public static function register_scripts() {
		$scripts = array(
			'wholesalex_bulkorder'  => array(
				'src'       => WHOLESALEX_PRO_URL . 'assets/js/wholesalex_bulkorder.js',
				'deps'      => array( 'wp-api-fetch', 'wp-i18n','react', 'react-dom','accounting' ),
				'ver'       => WHOLESALEX_PRO_VER,
				'in_footer' => true,
			),
			'wholesalex_subaccount' => array(
				'src'       => WHOLESALEX_PRO_URL . 'assets/js/wholesalex_subaccount.js',
				'deps'      => array( 'wp-api-fetch', 'wp-i18n','react', 'react-dom' ),
				'ver'       => WHOLESALEX_PRO_VER,
				'in_footer' => true,
			),
			'wholesalex_wallet'     => array(
				'src'       => WHOLESALEX_PRO_URL . 'assets/js/wholesalex_wallet.js',
				'deps'      => array( 'wp-api-fetch', 'wp-i18n','react', 'react-dom' ),
				'ver'       => WHOLESALEX_PRO_VER,
				'in_footer' => true,
            ),
            'whx_conversation'               => array(
                'src'       => WHOLESALEX_PRO_URL . 'assets/js/whx_conversation.js', 
                'deps'      => array('react', 'react-dom','wp-polyfill', 'wp-api-fetch'),
                'ver'       => WHOLESALEX_PRO_VER,
                'in_footer' => true,
            ),
            // 'wholesalex_license'               => array(
            //     'src'       => WHOLESALEX_PRO_URL . 'assets/js/wholesalex_license.js', 
            //     'deps'      => array('react', 'react-dom','wp-polyfill', 'wp-api-fetch'),
            //     'ver'       => WHOLESALEX_PRO_VER,
            //     'in_footer' => true,
            // ),
            'wholesalex-pro-public'               => array(
                'src'       => WHOLESALEX_PRO_URL . 'assets/js/wholesalex-pro-public.js', 
                'deps'      => array('jquery'),
                'ver'       => WHOLESALEX_PRO_VER,
                'in_footer' => true,
            ),
            'whx_wallet_admin_order_script'               => array(
                'src'       => WHOLESALEX_PRO_URL . 'assets/js/wholesalex_wallet_admin_order.js', 
                'deps'      => array('jquery'),
                'ver'       => WHOLESALEX_PRO_VER,
                'in_footer' => true,
            ),
			'wholesalex_request_a_quote'               => array(
				'src'       => WHOLESALEX_PRO_URL . 'addons/raq/assets/js/wholesalex_raq.js', 
				'deps'      => array('jquery'),
				'ver'       => WHOLESALEX_PRO_VER,
				'in_footer' => true,
			),
		);

		foreach ( $scripts as $handle => $args ) {
			wp_register_script( $handle, $args['src'], $args['deps'], $args['ver'], $args['in_footer'] );
			wp_set_script_translations( $handle, 'wholesalex-pro', WHOLESALEX_PRO_PATH . 'languages' );
		}

        wp_enqueue_style( 'wholesalex-pro-public',  WHOLESALEX_PRO_URL . 'assets/css/wholesalex-pro-public.css', array(), WHOLESALEX_PRO_VER );

	}


}
