<?php //phpcs:ignore
namespace WHOLESALEX_PRO;

defined( 'ABSPATH' ) || exit;

/**
 * WholesaleX Pro Notice Class
 */
class Notice {
	/**
	 * Constructor
	 */
	public function __construct() {
		add_action( 'admin_init', array( $this, 'admin_init_callback' ) );
		add_action( 'wp_ajax_wholesalex_install', array( $this, 'install_callback' ) );
		add_action( 'admin_action_wholesalex_activate', array( $this, 'activate_callback' ) );
		add_action( 'wp_ajax_wholesalex_dismiss_notice', array( $this, 'set_dismiss_notice_callback' ) );
	}


	/** Dismiss Notice Callback. */
	public function set_dismiss_notice_callback() {
		if ( ! isset( $_REQUEST['wpnonce'] ) || ! wp_verify_nonce( sanitize_key( $_REQUEST['wpnonce'] ), 'wholesalex-nonce' ) ) {
			return;
		}
		update_option( 'wholesalex_dismiss_notice', 'yes' );
	}


	/** Admin Init Callback. */
	public function admin_init_callback() {
		if ( ! file_exists( WP_PLUGIN_DIR . '/wholesalex/wholesalex.php' ) ) {
			add_action( 'admin_notices', array( $this, 'installation_notice_callback' ) );
		} elseif ( file_exists( WP_PLUGIN_DIR . '/wholesalex/wholesalex.php' ) && ! is_plugin_active( 'wholesalex/wholesalex.php' ) ) {
			add_action( 'admin_notices', array( $this, 'activation_notice_callback' ) );
		}
	}


	/** Installation Notice Callback */
	public function installation_notice_callback() {
		if ( ! get_option( 'wholesalex_dismiss_notice' ) ) {
			$this->wholesalex_notice_css();
			$this->wholesalex_notice_js();
			?>
			<div class="wc-install">
				<img width="150" src="<?php echo esc_url( WHOLESALEX_PRO_URL . 'assets/img/logo.svg' ); ?>" alt="logo" />
				<div class="wc-install-body">
					<a class="wc-dismiss-notice" data-security=<?php echo esc_attr( wp_create_nonce( 'wholesalex-nonce' ) ); ?>  data-ajax=<?php echo esc_url( admin_url( 'admin-ajax.php' ) ); ?> href="#"><span class="dashicons dashicons-no-alt"></span> <?php echo esc_html_e( 'Dismiss', 'wholesalex-pro' ); ?></a>
					<h3><?php echo esc_html_e( 'Welcome to WholesaleX Pro.', 'wholesalex-pro' ); ?></h3>
					<div><?php echo esc_html_e( 'WholesaleX Pro is a WholesaleX based plugin. To use this plugins you have to install and activate WholesaleX Free.', 'wholesalex-pro' ); ?></div>
					<a class="wholesalex-install-btn button button-primary button-hero" href="<?php echo esc_url( add_query_arg( array( 'action' => 'wholesalex_install' ), admin_url() ) ); ?>"><span class="dashicons dashicons-image-rotate"></span><?php echo esc_html_e( 'Install WholesaleX', 'wholesalex-pro' ); ?></a>
					<div id="installation-msg"></div>
				</div>
			</div>
			<?php
		}
	}

	/** Activation notice callback */
	public function activation_notice_callback() {
		if ( ! get_option( 'wholesalex_dismiss_notice' ) ) {
			$this->wholesalex_notice_css();
			$this->wholesalex_notice_js();
			?>
			<div class="wc-install">
				<img width="150" src="<?php echo esc_url( WHOLESALEX_PRO_URL . 'assets/img/logo.svg' ); ?>" alt="Logo of WholesaleX" />
				<div class="wc-install-body">
					<a class="wc-dismiss-notice" data-security=<?php echo esc_attr( wp_create_nonce( 'wholesalex-nonce' ) ); ?>  data-ajax=<?php echo esc_url( admin_url( 'admin-ajax.php' ) ); ?> href="#"><span class="dashicons dashicons-no-alt"></span> <?php echo esc_html_e( 'Dismiss', 'wholesalex-pro' ); ?></a>
					<h3><?php echo esc_html_e( 'Welcome to WholesaleX Pro.', 'wholesalex-pro' ); ?></h3>
					<div><?php echo esc_html_e( 'WholesaleX Pro is a WholesaleX based plugin. To use this plugins you have to install and activate WholesaleX Free.', 'wholesalex-pro' ); ?></div>
					<a class="button button-primary button-hero" href="<?php echo esc_url( add_query_arg( array( 'action' => 'wholesalex_activate' ), admin_url() ) ); ?>"><?php echo esc_html_e( 'Activate WholesaleX', 'wholesalex-pro' ); ?></a>
				</div>
			</div>
			<?php
		}
	}


	/** Notice CSS */
	public function wholesalex_notice_css() {
		?>
		<style type="text/css">
			.wc-install {
				display: -ms-flexbox;
				display: flex;
				align-items: center;
				background: #fff;
				margin-top: 40px;
				width: calc(100% - 50px);
				border: 1px solid #ccd0d4;
				padding: 15px;
				border-radius: 4px;
			}   
			.wc-install img {
				margin-right: 20px; 
			}
			.wc-install-body {
				-ms-flex: 1;
				flex: 1;
			}
			.wc-install-body > div {
				max-width: 450px;
				margin-bottom: 20px;
			}
			.wc-install-body h3 {
				margin-top: 0;
				font-size: 24px;
				margin-bottom: 15px;
			}
			.wholesalex-install-btn {
				margin-top: 15px;
				display: inline-block;
			}
			.wc-install .dashicons{
				display: none;
				animation: dashicons-spin 1s infinite;
				animation-timing-function: linear;
			}
			.wc-install.loading .dashicons {
				display: inline-block;
				margin-top: 12px;
				margin-right: 5px;
			}
			@keyframes dashicons-spin {
				0% {
					transform: rotate( 0deg );
				}
				100% {
					transform: rotate( 360deg );
				}
			}
			.wc-dismiss-notice {
				position: relative;
				text-decoration: none;
				float: right;
				right: 26px;
			}
			.wc-dismiss-notice .dashicons{
				display: inline-block;
				text-decoration: none;
				animation: none;
			}
		</style>
		<?php
	}


	/** Notice Script. */
	public function wholesalex_notice_js() {
		?>
		<script type="text/javascript">
			jQuery(document).ready(function($){
				'use strict';
				$(document).on('click', '.wholesalex-install-btn', function(e){
					e.preventDefault();
					const $that = $(this);
					$.ajax({
						type: 'POST',
						url: ajaxurl,
						data: {install_plugin: 'wholesalex', action: 'wholesalex_install'},
						beforeSend: function(){
								$that.parents('.wc-install').addClass('loading');
						},
						success: function (data) {
							$('#installation-msg').html(data);
							$that.parents('.wc-install').remove();
						},
						complete: function () {
							$that.parents('.wc-install').removeClass('loading');
						}
					});
				});

				// Dismiss notice
				$(document).on('click', '.wc-dismiss-notice', function(e){
					e.preventDefault();
					const that = $(this);
					$.ajax({
						url: that.data('ajax'),
						type: 'POST',
						data: { 
							action: 'wholesalex_dismiss_notice', 
							wpnonce: that.data('security')
						},
						success: function (data) {
							that.parents('.wc-install').hide("slow", function() { that.parents('.wc-install').remove(); });
						},
						error: function(xhr) {
							console.log('Error occured. Please try again' + xhr.statusText + xhr.responseText );
						},
					});
				});
			});
		</script>
		<?php
	}


	/** Install Callback */
	public function install_callback() {
		include ABSPATH . 'wp-admin/includes/plugin-install.php';
		include ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';

		if ( ! class_exists( 'Plugin_Upgrader' ) ) {
			include ABSPATH . 'wp-admin/includes/class-plugin-upgrader.php';
		}
		if ( ! class_exists( 'Plugin_Installer_Skin' ) ) {
			include ABSPATH . 'wp-admin/includes/class-plugin-installer-skin.php';
		}

		$plugin = 'wholesalex';

		$api = plugins_api(
			'plugin_information',
			array(
				'slug'   => $plugin,
				'fields' => array(
					'short_description' => false,
					'sections'          => false,
					'requires'          => false,
					'rating'            => false,
					'ratings'           => false,
					'downloaded'        => false,
					'last_updated'      => false,
					'added'             => false,
					'tags'              => false,
					'compatibility'     => false,
					'homepage'          => false,
					'donate_link'       => false,
				),
			)
		);

		if ( is_wp_error( $api ) ) {
			wp_die( $api ); //phpcs:ignore
		}

		/* translators: %s: API Name and Version */
		$title = sprintf( __( 'Installing Plugin: %s', 'wholesalex-pro' ), $api->name . ' ' . $api->version );
		$nonce = 'install-plugin_' . $plugin;
		$url   = 'update.php?action=install-plugin&plugin=' . rawurldecode( $plugin );

		$upgrader = new \Plugin_Upgrader( new \Plugin_Installer_Skin( compact( 'title', 'url', 'nonce', 'plugin', 'api' ) ) );
		$upgrader->install( $api->download_link );
		die();
	}


	/** Activate Callback */
	public function activate_callback() {
		activate_plugin( 'wholesalex/wholesalex.php' );
		wp_safe_redirect( admin_url( 'admin.php?page=wholesalex-settings' ) );
		exit();
	}

}
