<?php //phpcs:ignore
namespace WHOLESALEX_PRO;


defined( 'ABSPATH' ) || exit;
/**
 * Product Class
 */
class Product {

	/**
	 * Constructor
	 */
	public function __construct() {
		if ( ! wholesalex_pro()->is_active() ) {
			return;
		}

		add_filter( 'wholesalex_single_product_fields', array( $this, 'single_product_tier_fields' ), 1 );

		add_action( 'wholesalex_single_product_discount_action', array( $this, 'single_product_discounts' ) );

		add_filter( 'wholesalex_single_product_tier_layout', array( $this, 'product_tier_layout' ), 30 );

	}


	/**
	 * Product Role Based Discount Fields
	 *
	 * @param array $sections Section Field Data.
	 * @return array
	 */
	public function single_product_tier_fields( $sections ) {
		if ( ! ( wholesalex_pro()->is_free_ready() && version_compare( WHOLESALEX_VER, '1.0.10', '<' ) ) ) {
			return $sections;
		}
		
		$b2b_roles   = wholesalex()->get_roles( 'b2b_roles_option' );
		$b2c_roles   = wholesalex()->get_roles( 'b2c_roles_option' );
		$__b2b_roles = array();
		foreach ( $b2b_roles as $role ) {
			if ( ! ( isset( $role['value'] ) && isset( $role['value'] ) ) ) {
				continue;
			}
			$__b2b_roles[ $role['value'] ] = array(
				'label'  => $role['name'],
				'type'   => 'tiers',
				'is_pro' => true,
				'attr'   => array(
					'_prices'               => array(
						'type' => 'prices',
						'attr' => array(
							'wholesalex_base_price' => array(
								'type'    => 'number',
								'label'   => __( 'Base Price', 'wholesalex-pro' ),
								'default' => '',
							),
							'wholesalex_sale_price' => array(
								'type'    => 'number',
								'label'   => __( 'Sale Price', 'wholesalex-pro' ),
								'default' => '',
							),
						),
					),
					$role['value'] . 'tier' => array(
						'type'   => 'tier',
						'_tiers' => array(
							'columns' => array(
								__( 'Discount Type', 'wholesalex-pro' ),
								/* translators: %s: WholesaleX Role Name */
								sprintf( __( ' %s Price', 'wholesalex-pro' ), $role['name'] ),
								__( 'Min Quantity', 'wholesalex-pro' ),
							),
							'data'    => array(
								'_discount_type'   => array(
									'type'    => 'select',
									'options' => array(
										''            => __( 'Choose Discount Type...', 'wholesalex-pro' ),
										'amount'      => __( 'Discount Amount', 'wholesalex-pro' ),
										'percentage'  => __( 'Discount Percentage', 'wholesalex-pro' ),
										'fixed_price' => __( 'Fixed Price', 'wholesalex-pro' ),
									),
									'label' => __( 'Discount Type', 'wholesalex-pro' ),
									'default' => '',
								),
								'_discount_amount' => array(
									'type'        => 'number',
									'placeholder' => '',
									'default'     => '',
									'label' => /* translators: %s: WholesaleX Role Name */
									sprintf( __( ' %s Price', 'wholesalex-pro' ), $role['name'] ),
								),
								'_min_quantity'    => array(
									'type'        => 'number',
									'placeholder' => '',
									'default'     => '',
									'label' => __( 'Min Quantity', 'wholesalex-pro' ),
								),
							),
							'add'     => array(
								'type'  => 'button',
								'label' => __( 'Add Price Tier', 'wholesalex-pro' ),
							),
						),
					),
				),
			);
		}

		$__b2c_roles = array();
		foreach ( $b2c_roles as $role ) {
			if ( ! ( isset( $role['value'] ) && isset( $role['value'] ) ) ) {
				continue;
			}
			$__b2c_roles[ $role['value'] ] = array(
				'label'  => $role['name'],
				'type'   => 'tiers',
				'is_pro' => true,
				'attr'   => array(
					$role['value'] . 'tier' => array(
						'type'   => 'tier',
						'_tiers' => array(
							'columns' => array(
								__( 'Discount Type', 'wholesalex-pro' ),
								/* translators: %s: WholesaleX Role Name */
								sprintf( __( ' %s Price', 'wholesalex-pro' ), $role['name'] ),
								__( 'Min Quantity', 'wholesalex-pro' ),
							),
							'data'    => array(
								'_discount_type'   => array(
									'type'    => 'select',
									'options' => array(
										''            => __( 'Choose Discount Type...', 'wholesalex-pro' ),
										'amount'      => __( 'Discount Amount', 'wholesalex-pro' ),
										'percentage'  => __( 'Discount Percentage', 'wholesalex-pro' ),
										'fixed_price' => __( 'Fixed Price', 'wholesalex-pro' ),
									),
									'default' => '',
									'label' =>__( 'Discount Type', 'wholesalex-pro' ),
								),
								'_discount_amount' => array(
									'type'        => 'number',
									'placeholder' => '',
									'default'     => '',
									'label' => /* translators: %s: WholesaleX Role Name */
									sprintf( __( ' %s Price', 'wholesalex-pro' ), $role['name'] ),
								),
								'_min_quantity'    => array(
									'type'        => 'number',
									'placeholder' => '',
									'default'     => '',
									'label' => __( 'Min Quantity', 'wholesalex-pro' )
								),
							),
							'add'     => array(
								'type'  => 'button',
								'label' => __( 'Add Price Tier', 'wholesalex-pro' ),
							),
						),
					),
				),
			);
		}

		$sections['_b2c_section']['attr'] = $__b2c_roles;
		$sections['_b2b_section']['attr'] = $__b2b_roles;

		return $sections;

	}


	/**
	 * Single Product Discounts
	 *
	 * @param array $data Data. 
	 * @return array
	 * @since 1.0.0
	 * @since 1.0.3 ProductX Currency Switcher Compatibility Fixed
	 */
	public function single_product_discounts( $data ) {
		if ( ! ( wholesalex_pro()->is_free_ready() && version_compare( WHOLESALEX_VER, '1.0.10', '<' ) ) ) {
			return $data;
		}
		/**
		 * Hook Added For Support Bulk Order Addon (wholesalex_single_product_discounts_data)
		 *
		 * @since 1.0.4
		 */
		$data = apply_filters( 'wholesalex_single_product_discounts_data', $data );

		$sale_price = $data['sale_price'];
		$product    = $data['product'];

		$__override_sale_price = false;
		$__product_id          = $product->get_id();
		$__user_id             = apply_filters( 'wholesalex_set_current_user', get_current_user_id() );
		// delete_transient( 'wholesalex_pricing_tiers_single_product_' . $__user_id );

		$__discounts = wholesalex()->get_discounts( 'product', $__product_id );
		if ( empty( $__discounts ) ) {
			return $data;
		}
		$sale_price = ( isset( $__discounts['sale_price'] ) && ! empty( $__discounts['sale_price'] ) ) ? max( 0, $__discounts['sale_price'] ) : $sale_price;
		/**
		 * Rolewise Sale Price ( Single Product ) Not Working Issue Fixed
		 *
		 * @since 1.0.6
		 */
		$data['sale_price'] = $sale_price;
		/**
		 * ProductX Currency Switcher Compatibility Fixed
		 *
		 * @since 1.0.3
		 */
		$__regular_price = ( isset( $__discounts['regular_price'] ) && ! empty( $__discounts['regular_price'] ) ) ? max( 0, $__discounts['regular_price'] ) : wholesalex_pro()->price_after_currency_changed( $product->get_regular_price() );

		$__tiers = isset( $__discounts['tiers'] ) ? $__discounts['tiers'] : '';
		set_transient( 'wholesalex_pricing_tiers_single_product_' . $__user_id, $__tiers );

		if ( ! ( is_array( $__tiers ) && ! empty( $__tiers ) ) ) {
			return $data;
		}

		$__quantity = isset( $data['quantity'] ) ? $data['quantity'] : wholesalex()->cart_count( $__product_id );

		foreach ( $__tiers as $tier ) {
			if ( ! isset( $tier['_min_quantity'] ) ) {
				continue;
			}
			if ( $__quantity >= $tier['_min_quantity'] ) {
				$sale_price             = wholesalex()->calculate_sale_price( $tier, $__regular_price );
				$__override_sale_price  = true;
				$data['active_tier_id'] = isset( $tier['_id'] ) ? $tier['_id'] : '';
			}
		}

		$data['sale_price'] = $sale_price;

		if ( $__override_sale_price ) {
			remove_all_filters( 'woocommerce_sale_flash' );
			$data['discount_src'] = 'single_product';
		}

		return $data;

	}


	/**
	 * Product Tier Layout
	 *
	 * @param array $options Options.
	 * @return array
	 * @since 1.0.0
	 * @since 1.0.1 More Tire Layout Added.
	 * @since 1.0.6 Delete Existing Code and Rewrite for new pricing plan.
	 */
	public function product_tier_layout( $options ) {
		return wholesalex_pro()->unlock_layouts( $options );
	}
}
