<?php //phpcs:ignore
namespace WHOLESALEX_PRO;

defined( 'ABSPATH' ) || exit;

/**
 * Functions Class
 */
class Profile {

	/**
	 * Constructor
	 */
	public function __construct() {
		if ( ! wholesalex_pro()->is_active() ) {
			return;
		}
		add_filter(
			'wholesalex_backend_localize_data',
			function ( $data ) {
				$data['is_pro_activated'] = wholesalex_pro()->is_active();
				$data['pro_nonce']        = wp_create_nonce( 'wholesalex_pro' );
				$data['settings'] = wholesalex()->get_setting();
				return $data;
			},
			1
		);

		add_filter( 'wholesalex_profile_discounts_fields', array( $this, 'profile_discounts_fields' ), 2 );
		add_filter( 'wholesalex_profile_discount_action', array( $this, 'profile_discounts' ), 1 );

	}


	/**
	 * Profile Discounts Fields
	 *
	 * @param array $fields Profile Discount Fields.
	 */
	public function profile_discounts_fields( $fields ) {
		if ( ! ( wholesalex_pro()->is_free_ready() && version_compare( WHOLESALEX_VER, '1.0.10', '<' ) ) ) {
			return $fields;
		}

		// Category Options.
		$categories         = get_terms(
			array(
				'taxonomy'   => 'product_cat',
				'hide_empty' => false,
			)
		);
		$categories_options = array();

		foreach ( $categories as $category ) {
			$categories_options[] = array(
				'value' => $category->term_id,
				'name'  => $category->name,
			);
		}
		// ------------

		// Product Options.
		$product_ids = get_posts(
			array(
				'post_type'   => 'product',
				'post_status' => 'publish',
				'numberposts' => -1,
				'fields'      => 'ids',
			)
		);

		$products = array();
		foreach ( $product_ids as $product_id ) {
			$product    = array(
				'value' => $product_id,
				'name'  => wc_get_product( $product_id )->get_name(),
			);
			$products[] = $product;
		}
		// -----------------

		// Variation products.....
		$variation_products = array();
		// Get all products.
		$product_variations = get_posts(
			array(
				'post_type'   => 'product_variation',
				'post_status' => 'publish',
				'numberposts' => -1,
				'fields'      => 'ids',
			)
		);

		foreach ( $product_variations as $product ) {
			$productobj = wc_get_product( $product );

			$productobjname = $productobj->get_name();

			if ( is_a( $productobj, 'WC_Product_Variation' ) ) {
				$attributes           = $productobj->get_variation_attributes();
				$number_of_attributes = count( $attributes );
				if ( $number_of_attributes > 2 ) {
					$productobjname .= ' - ';
					foreach ( $attributes as $attribute ) {
						$productobjname .= $attribute . ', ';
					}
					$productobjname = substr( $productobjname, 0, -2 );
				}
			}
			$variation_products[] = array(
				'value' => $product,
				'name'  => $productobjname,
			);
		}
		$fields = array(
			'discounts' => array(
				'type'   => 'tier',
				'_tiers' => array(
					'columns' => array(
						__( 'Discount Type', 'wholesalex-pro' ),
						__( 'WholesaleX Profile Price', 'wholesalex-pro' ),
						__( 'Min Quantity', 'wholesalex-pro' ),
						__( 'Product Filter', 'wholesalex-pro' ),
					),
					'data'    => array(
						'_discount_type'        => array(
							'type'    => 'select',
							'options' => array(
								''           => __( 'Choose Discount Type...', 'wholesalex-pro' ),
								'amount'     => __( 'Discount Amount', 'wholesalex-pro' ),
								'percentage' => __( 'Discount Percentage', 'wholesalex-pro' ),
								'fixed'      => __( 'Fixed Price', 'wholesalex-pro' ),
							),
							'default' => '',
						),
						'_discount_amount'      => array(
							'type'        => 'number',
							'placeholder' => '',
							'default'     => '',
						),
						'_min_quantity'         => array(
							'type'        => 'number',
							'placeholder' => '',
							'default'     => '',
						),
						'_product_filter'       => array(
							'type'    => 'select',
							'options' => array(
								''                      => __( 'Choose Filter...', 'wholesalex-pro' ),
								'all_products'          => __( 'All Products', 'wholesalex-pro' ),
								'products_in_list'      => __( 'Product in list', 'wholesalex-pro' ),
								'products_not_in_list'  => __( 'Product not in list', 'wholesalex-pro' ),
								'cat_in_list'           => __( 'Categories in list', 'wholesalex-pro' ),
								'cat_not_in_list'       => __( 'Categories not in list', 'wholesalex-pro' ),
								'attribute_in_list'     => __( 'Attribute in list', 'wholesalex-pro' ),
								'attribute_not_in_list' => __( 'Attribute not in list', 'wholesalex-pro' ),
							),
							'default' => '',
						),
						'products_in_list'      => array(
							'type'        => 'multiselect',
							'depends_on'  => array(
								array(
									'key'   => '_product_filter',
									'value' => 'products_in_list',
								),
							),
							'options'     => $products,
							'placeholder' => __( 'Choose Products to apply discounts', 'wholesalex-pro' ),
							'default'     => array(),
						),
						'products_not_in_list'  => array(
							'type'        => 'multiselect',
							'depends_on'  => array(
								array(
									'key'   => '_product_filter',
									'value' => 'products_not_in_list',
								),
							),
							'options'     => $products,
							'placeholder' => __( 'Choose Products that wont apply discounts', 'wholesalex-pro' ),
							'default'     => array(),
						),
						'cat_in_list'           => array(
							'type'        => 'multiselect',
							'depends_on'  => array(
								array(
									'key'   => '_product_filter',
									'value' => 'cat_in_list',
								),
							),
							'options'     => $categories_options,
							'placeholder' => __( 'Choose Categories to apply discounts', 'wholesalex-pro' ),
							'default'     => array(),
						),
						'cat_not_in_list'       => array(
							'type'        => 'multiselect',
							'depends_on'  => array(
								array(
									'key'   => '_product_filter',
									'value' => 'cat_not_in_list',
								),
							),
							'options'     => $categories_options,
							'placeholder' => __( 'Choose Categories that wont apply discounts', 'wholesalex-pro' ),
							'default'     => array(),
						),
						'attribute_in_list'     => array(
							'type'        => 'multiselect',
							'depends_on'  => array(
								array(
									'key'   => '_product_filter',
									'value' => 'attribute_in_list',
								),
							),
							'options'     => $variation_products,
							'placeholder' => __( 'Choose Product Variations to apply discounts', 'wholesalex-pro' ),
							'default'     => array(),
						),
						'attribute_not_in_list' => array(
							'type'        => 'multiselect',
							'depends_on'  => array(
								array(
									'key'   => '_product_filter',
									'value' => 'attribute_not_in_list',
								),
							),
							'options'     => $variation_products,
							'placeholder' => __( 'Choose Product Variations that wont apply discounts', 'wholesalex-pro' ),
							'default'     => array(),
						),
					),
					'add'     => array(
						'type'  => 'button',
						'label' => __( 'Add Price Tier', 'wholesalex-pro' ),
					),
				),
			),
		);
		return $fields;
	}

	/**
	 * Profile Discounts
	 *
	 * @param array $data Data.
	 * @return array
	 * @since 1.0.0
	 * @since 1.0.3 ProductX Currency Switcher Compatibility Fixed
	 */
	public function profile_discounts( $data ) {

		if ( ! ( wholesalex_pro()->is_free_ready() && version_compare( WHOLESALEX_VER, '1.0.10', '<' ) ) ) {
			return $data;
		}

		if ( ! function_exists( 'wholesalex' ) ) {
			return;
		}


		$__override_sale_price = false;
		$__user_id             = get_current_user_id();
		$sale_price            = $data['sale_price'];
		$product               = $data['product'];

		// delete_transient( 'wholesalex_pricing_tiers_profile_' . $__user_id );

		$__tiers = get_user_meta( $__user_id, '__wholesalex_profile_discounts', true );

		/**
		 * ProductX Currency Switcher Compatibility Fixed
		 *
		 * @since 1.0.3
		 */
		$__regular_price = wholesalex_pro()->price_after_currency_changed( $product->get_regular_price() );

		if ( ! ( isset( $__tiers['_profile_discounts']['tiers'] ) && ! empty( $__tiers['_profile_discounts']['tiers'] ) ) ) {
			return $data;
		}

		$__tiers                 = $__tiers['_profile_discounts']['tiers'];
		$__product_id            = $product->get_id();
		$__tiers_for_cur_product = array();
		$__parent_id             = $product->get_parent_id();
		$__cat_ids               = wc_get_product_term_ids( 0 === $__parent_id ? $__product_id : $__parent_id, 'product_cat' );
		foreach ( $__tiers as $tier ) {
			if ( isset( $tier['_product_filter'] ) ) {
				switch ( $tier['_product_filter'] ) {
					case 'all_products':
						$__temp_tier = $tier;
						unset( $__temp_tier['_product_filter'] );
						unset( $__temp_tier[ $tier['_product_filter'] ] );
						$__temp_tier['for']    = 'all_products';
						$__temp_tier['src_id'] = -1;
						array_push( $__tiers_for_cur_product, $__temp_tier );

						break;
					case 'products_in_list':
						if ( ! isset( $tier['products_in_list'] ) ) {
							break;

						}
						foreach ( $tier['products_in_list'] as $list ) {
							if ( isset( $list['value'] ) && $__product_id == $list['value'] ) {
								$__temp_tier = $tier;
								unset( $__temp_tier['_product_filter'] );
								unset( $__temp_tier[ $tier['_product_filter'] ] );
								$__temp_tier['for']    = 'product';
								$__temp_tier['src_id'] = $__product_id;
								array_push( $__tiers_for_cur_product, $__temp_tier );
								break;
							}
						}
						break;
					case 'products_not_in_list':
						if ( ! isset( $tier['products_not_in_list'] ) ) {
							break;

						}
						$__flag = true;
						foreach ( $tier['products_not_in_list'] as $list ) {
							if ( isset( $list['value'] ) && $__product_id == $list['value'] ) {
								$__flag = false;
							}
						}
						if ( $__flag ) {
							$__temp_tier = $tier;
							unset( $__temp_tier['_product_filter'] );
							unset( $__temp_tier[ $tier['_product_filter'] ] );
							$__temp_tier['for']    = 'product';
							$__temp_tier['src_id'] = $__product_id;
							array_push( $__tiers_for_cur_product, $__temp_tier );
						}
						break;
					case 'cat_in_list':
						if ( ! isset( $tier['cat_in_list'] ) ) {
							break;

						}
						foreach ( $tier['cat_in_list'] as $list ) {
							if ( in_array( $list['value'], $__cat_ids ) ) {
								$__temp_tier = $tier;
								unset( $__temp_tier['_product_filter'] );
								unset( $__temp_tier[ $tier['_product_filter'] ] );
								$__temp_tier['for']    = 'cat';
								$__temp_tier['src_id'] = $list['value'];
								array_push( $__tiers_for_cur_product, $__temp_tier );
								break;
							}
						}

						break;

					case 'cat_not_in_list':
						if ( ! isset( $tier['cat_not_in_list'] ) ) {
							break;

						}
						$__flag = true;
						foreach ( $tier['cat_not_in_list'] as $list ) {
							if ( in_array( $list['value'], $__cat_ids ) ) {
								$__flag = false;
							}
						}
						if ( $__flag ) {
							$__temp_tier = $tier;
							unset( $__temp_tier['_product_filter'] );
							unset( $__temp_tier[ $tier['_product_filter'] ] );
							$__temp_tier['for']    = 'cat';
							$__temp_tier['src_id'] = $__cat_ids[0];
							array_push( $__tiers_for_cur_product, $__temp_tier );
						}
						break;
					case 'attribute_in_list':
						if ( ! isset( $tier['attribute_in_list'] ) ) {
							break;

						}
						if ( 'product_variation' == $product->post_type ) {
							foreach ( $tier['attribute_in_list'] as $list ) {
								if ( isset( $list['value'] ) && $__product_id == $list['value'] ) {
									$__temp_tier = $tier;
									unset( $__temp_tier['_product_filter'] );
									unset( $__temp_tier[ $tier['_product_filter'] ] );
									$__temp_tier['for']    = 'variation';
									$__temp_tier['src_id'] = $__product_id;
									array_push( $__tiers_for_cur_product, $__temp_tier );
									break;
								}
							}
						}
						break;
					case 'attribute_not_in_list':
						if ( ! isset( $tier['attribute_not_in_list'] ) ) {
							break;

						}
						if ( 'product_variation' == $product->post_type ) {
							$__flag = true;
							foreach ( $tier['attribute_not_in_list'] as $list ) {
								if ( isset( $list['value'] ) && $__product_id == $list['value'] ) {
									$__flag = false;
								}
							}
							if ( $__flag ) {
								$__temp_tier = $tier;
								unset( $__temp_tier['_product_filter'] );
								unset( $__temp_tier[ $tier['_product_filter'] ] );
								$__temp_tier['for']    = 'variation';
								$__temp_tier['src_id'] = $__product_id;
								array_push( $__tiers_for_cur_product, $__temp_tier );
							}
						}
						break;
					default:
						// code...
						break;
				}
			}
		}

		foreach ( $__tiers_for_cur_product as $tier ) {
			if ( isset( $tier['for'] ) && isset( $tier['src_id'] ) ) {
				switch ( $tier['for'] ) {
					case 'cat':
						if ( in_array( $tier['src_id'], $__cat_ids ) ) {
							/**
							 * Hook Added For Support Bulk Order Addon (wholesalex_profile_discounts_category_cart_count)
							 *
							 * @since 1.0.4
							 */
							$__quantity     = apply_filters( 'wholesalex_profile_discounts_category_cart_count', 0, $tier['src_id'], $__product_id );
							$__quantity     = $__quantity ? $__quantity : wholesalex()->category_cart_count( $tier['src_id'] );
							$__min_quantity = isset( $tier['_min_quantity'] ) ? $tier['_min_quantity'] : 999999999;
							if ( $__quantity >= $__min_quantity ) {
								$sale_price             = wholesalex()->calculate_sale_price( $tier, $__regular_price );
								$__override_sale_price  = true;
								$data['active_tier_id'] = isset( $tier['_id'] ) ? $tier['_id'] : '';
							}
						}

						break;
					case 'product':
					case 'variation':
					case 'all_products':
						/**
						 * Hook Added For Support Bulk Order Addon (wholesalex_profile_discounts_product_count)
						 *
						 * @since 1.0.4
						 */
						$__quantity     = apply_filters( 'wholesalex_profile_discounts_product_count', 0, $__product_id );
						$__quantity     = $__quantity ? $__quantity : wholesalex()->cart_count( $__product_id );
						$__min_quantity = isset( $tier['_min_quantity'] ) ? $tier['_min_quantity'] : 999999999;
						if ( $__quantity >= $__min_quantity ) {
							$sale_price             = wholesalex()->calculate_sale_price( $tier, $__regular_price );
							$__override_sale_price  = true;
							$data['active_tier_id'] = isset( $tier['_id'] ) ? $tier['_id'] : '';
						}
						break;
				}
			}
		}

		$data['sale_price'] = $sale_price;

		set_transient( 'wholesalex_pricing_tiers_profile_' . $__user_id, $__tiers_for_cur_product );

		if ( $__override_sale_price ) {
			remove_all_filters( 'woocommerce_sale_flash' );
			$data['discount_src'] = 'profile';
		}

		return $data;
	}

}
