<?php //phpcs:ignore
namespace WHOLESALEX_PRO;

defined( 'ABSPATH' ) || exit;

/**
 * License Class
 */
class License {

	/**
	 * Contains page slug.
	 *
	 * @var string
	 */
	private $page_slug = 'wholesalex-license';

	/**
	 * Contains sever url.
	 *
	 * @var string
	 */
	private $server_url = 'https://account.wpxpo.com';

	private $checkout_url = 'https://www.wpxpo.com/checkout/';
	/**
	 * Contains Item ID.
	 *
	 * @var integer
	 */
	private $item_id = 44997;
	/**
	 * Contains Product Name.
	 *
	 * @var string
	 */
	private $name = 'WholesaleX Pro';
	/**
	 * Contains Pro Version.
	 *
	 * @var string
	 */
	private $version = WHOLESALEX_PRO_VER;
	/**
	 * Contains Plugin Slug.
	 *
	 * @var string
	 */
	private $slug = 'wholesalex-pro/wholesalex-pro.php';

	/**
	 * Constuctor
	 */
	public function __construct() {
		add_action( 'admin_init', array( $this, 'edd_license_updater' ) );
		// add_action( 'admin_init', array( $this, 'edd_activate_license' ) );
        add_action( 'wp_ajax_edd_wholesalex_activate_license', array($this, 'edd_activate_license') );
        // add_action( 'wp_ajax_edd_wholesalex_renew_license', array($this, 'edd_renew_license') );
        add_action( 'wp_ajax_edd_wholesalex_deactivate_license', array($this, 'edd_deactivate_license') );
		add_action( 'admin_init', array( $this, 'price_id_update' ) );
		// add_action( 'admin_init', array( $this, 'edd_check_license' ) );
		add_action( 'admin_menu', array( $this, 'menu_page_callback' ), 11 );
	}

	/**
	 * Price Id Update
	 *
	 * @return void
	 */
	public function price_id_update() {
		$price_id    = get_option( '__wholesalex_price_id', false );
        $license_data = get_option( 'edd_wholesalex_license_data', false);
		$license_key = trim( get_option( 'edd_wholesalex_license_key' ) );

		if ( (! $price_id || !$license_data ) && $license_key ) {
			$api_params = array(
				'edd_action' => 'activate_license',
				'license'    => $license_key,
				'item_id'    => $this->item_id,
				'url'        => home_url(),
			);

			$response = wp_remote_post(
				$this->server_url,
				array(
					'timeout'   => 15,
					'sslverify' => false,
					'body'      => $api_params,
				)
			);
			if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {
				$message = ( is_wp_error( $response ) && ! empty( $response->get_error_message() ) ) ? $response->get_error_message() : __( 'An error occurred, please try again.', 'wholesalex-pro' );
			} else {
				$license_data = json_decode( wp_remote_retrieve_body( $response ) );
				if ( isset( $license_data->price_id ) ) {
					update_option( '__wholesalex_price_id', $license_data->price_id );
				}

                update_option( 'edd_wholesalex_license_data', (array) $license_data);

			}
		}
	}

	/**
	 * EDD License Updater.
	 *
	 * @return void
	 */
	public function edd_license_updater() {

		if ( ! class_exists( 'EDD_SL_Plugin_Updater' ) ) {
			require_once WHOLESALEX_PRO_PATH . 'classes/updater/EDD_SL_Plugin_Updater.php';
		}

		$license_key = trim( get_option( 'edd_wholesalex_license_key' ) );

		$edd_updater = new \EDD_SL_Plugin_Updater(
			$this->server_url,
			$this->slug,
			array(
				'version' => $this->version,
				'license' => $license_key,
				'item_id' => $this->item_id,
				'author'  => $this->name,
				'url'     => home_url(),
				'beta'    => false,
			)
		);

	}

	/**
	 * Menu Page Callback.
	 *
	 * @return void
	 * @since 1.0.0
	 * @since 1.2.4 added wholesalex_show_license_page filter
	 */
	public function menu_page_callback() {
		$status = apply_filters('wholesalex_show_license_page',true);
		$menu_slug = apply_filters( 'wholesalex_plugin_menu_slug', 'wholesalex-overview' );

		if($status) {
			add_submenu_page(
				$menu_slug,
				__( 'License', 'wholesalex-pro' ),
				__( 'License', 'wholesalex-pro' ),
				apply_filters( 'wholesalex_capability_access', 'manage_options' ),
				$this->page_slug,
				array( $this, 'edd_license_page' ),
				10
			);
		}
	}

	/**
	 * License Page.
	 *
	 * @return void
	 */
	public function edd_license_page() {
        $license_data = get_option( 'edd_wholesalex_license_data', array() );
		$status = get_option( 'edd_wholesalex_license_status','' );
        $price_id = get_option( '__wholesalex_price_id','' );
        $expire_date = isset($license_data['expires'])?$license_data['expires']:'';
        $license = get_option( 'edd_wholesalex_license_key', '' );
		$encode_license = base64_encode( json_encode(array('key'=> $license,'hash'=> md5( $license))) );
        wp_enqueue_script( 'wholesalex_license',  WHOLESALEX_PRO_URL . 'assets/js/wholesalex_license.js',  array('react', 'react-dom','wp-polyfill', 'wp-api-fetch'), WHOLESALEX_PRO_VER, true );

    
		$localize =  array('nonce'=> wp_create_nonce( 'edd_download_nonce' ),'status'=> wholesalex_pro()->get_license_status(), 'expire_date'=>$expire_date, 'price_tier' => $price_id,'ajax'=>admin_url('admin-ajax.php'),'checkout_url'=>$this->checkout_url );
		if('expired' == $status) {
			$localize['license']=$encode_license;
		}
		$localize['isLifetime'] = 6==$price_id;
		wp_localize_script( 'wholesalex_license', 'wholesalex_license',$localize);
		?>
            <div id='wholesalex_license_root'></div>
        <?php
	}


	/**
	 * Activate License.
	 *
	 * @return void|string
	 */
	public function edd_activate_license() {
        $ajax_response = array('status'=> false, 'data' => 'Failed!');
		if ( isset( $_POST['edd_wholesalex_license_key'] ) && isset($_POST['nonce']) ) {
			if ( ! wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'edd_download_nonce' ) ) {
				wp_send_json($ajax_response );
			}

			$license = trim( sanitize_text_field( $_POST['edd_wholesalex_license_key'] ) );

			if ( '' !== $license ) {
				update_option( 'edd_wholesalex_license_key', $license );
				$api_params = array(
					'edd_action' => 'activate_license',
					'license'    => $license,
					'item_id'    => $this->item_id,
					'url'        => home_url(),
				);

				$response = wp_remote_post(
					$this->server_url,
					array(
						'timeout'   => 50,
						'sslverify' => false,
						'body'      => $api_params,
					)
				);

				if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {
					$ajax_response['data'] = ( is_wp_error( $response ) && ! empty( $response->get_error_message() ) ) ? $response->get_error_message() : __( 'An error occurred, please try again.', 'wholesalex-pro' );
				} else {
                    $ajax_response['status'] = true;
					$license_data = json_decode( wp_remote_retrieve_body( $response ) );
					if ( false === $license_data->success ) {
						$ajax_response['status'] = false;
						switch ( $license_data->error ) {
							case 'expired':
								$ajax_response['data']  = sprintf(
									__( 'Your license key expired on %s.', 'wholesalex-pro' ),
									date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
								);

								break;
							case 'revoked':
								$ajax_response['data']  = __( 'Your license key has been disabled.', 'wholesalex-pro' );
								break;
							case 'missing':
								$ajax_response['data']  = __( 'Invalid license.', 'wholesalex-pro' );
								break;
							case 'invalid':
							case 'site_inactive':
								$ajax_response['data']  = __( 'Your license is not active for this URL.', 'wholesalex-pro' );
								break;
							case 'item_name_mismatch':
								$ajax_response['data']  = __( 'This appears to be an invalid license key.', 'wholesalex-pro' );
								break;
							case 'no_activations_left':
								$ajax_response['data']  = __( 'Your license key has reached its activation limit.', 'wholesalex-pro' );
								break;
							default:
								$ajax_response['data']  = __( 'An error occurred, please try again.', 'wholesalex-pro' );
								break;
						}
					} else {
                        $ajax_response['data'] = 'License Activated!';
                    }
					update_option( 'edd_wholesalex_license_status', $license_data->license );
					update_option( 'edd_wholesalex_license_data', (array) $license_data);
					update_option( '__wholesalex_price_id', $license_data->price_id );
					
                    wp_send_json( $ajax_response );
				}
			}
		}
	}
	/**
	 * Deactive License.
	 *
	 * @return void|string
	 */
	public function edd_deactivate_license() {
        $ajax_response = array('status'=> false, 'data' => 'Failed!');
		if ( isset( $_POST['is_wholesalex_license_deactive'] ) &&  'yes' === $_POST['is_wholesalex_license_deactive'] && isset($_POST['nonce']) ) {
			if ( ! wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'edd_download_nonce' ) ) {
				wp_send_json($ajax_response );
			}

            $license = get_option( 'edd_wholesalex_license_key', '' );

            $api_params = array(
                'edd_action' => 'deactivate_license',
                'license'    => $license,
                'item_id'    => $this->item_id,
                'url'        => home_url(),
            );

            $response = wp_remote_post(
                $this->server_url,
                array(
                    'timeout'   => 50,
                    'sslverify' => false,
                    'body'      => $api_params,
                )
            );

            if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {
                $ajax_response['data'] = ( is_wp_error( $response ) && ! empty( $response->get_error_message() ) ) ? $response->get_error_message() : __( 'An error occurred, please try again.', 'wholesalex-pro' );
            } else {
                $ajax_response['status'] = true;
                $license_data = json_decode( wp_remote_retrieve_body( $response ) );
                if ( false === $license_data->success ) {
                    switch ( $license_data->error ) {
                        case 'expired':
                            $ajax_response['data']  = sprintf(
                                __( 'Your license key expired on %s.', 'wholesalex-pro' ),
                                date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
                            );

                            break;
                        case 'revoked':
                            $ajax_response['data']  = __( 'Your license key has been disabled.', 'wholesalex-pro' );
                            break;
                        case 'missing':
                            $ajax_response['data']  = __( 'Invalid license.', 'wholesalex-pro' );
                            break;
                        case 'invalid':
                        case 'site_inactive':
                            $ajax_response['data']  = __( 'Your license is not active for this URL.', 'wholesalex-pro' );
                            break;
                        case 'item_name_mismatch':
                            $ajax_response['data']  = __( 'This appears to be an invalid license key.', 'wholesalex-pro' );
                            break;
                        case 'no_activations_left':
                            $ajax_response['data']  = __( 'Your license key has reached its activation limit.', 'wholesalex-pro' );
                            break;
                        default:
                            $ajax_response['data']  = __( 'An error occurred, please try again.', 'wholesalex-pro' );
                            break;
                    }
                } else {
                    $ajax_response['data'] = 'License Deactive.';
                    $this->edd_check_license(true);
                }

                update_option( 'edd_wholesalex_license_status', $license_data->license );
                update_option( 'edd_wholesalex_license_data', (array) $license_data);
                
                wp_send_json( $ajax_response );
            }
        
		}
	}
	// /**
	//  * Renew License.
	//  *
	//  * @return void|string
	//  */
	// public function edd_renew_license() {
    //     $ajax_response = array('status'=> false, 'data' => 'Failed!');
	// 	if (isset($_POST['nonce']) ) {
	// 		if ( ! wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'edd_download_nonce' ) ) {
	// 			wp_send_json($ajax_response );
	// 		}

	// 		$license = get_option('edd_wholesalex_license_key',false);
	// 		if(!$license) {
	// 			$ajax_response['data'] = 'License Key not found!';
	// 		} else {
	// 			$api_params = array(
	// 				'edd_action' => 'apply_license_renewal',
	// 				'edd_license_key'    => $license,
	// 			);

	// 			$response = wp_remote_post(
	// 				$this->checkout_url,
	// 				array(
	// 					'timeout'   => 50,
	// 					'sslverify' => false,
	// 					'body'      => $api_params,
	// 				)
	// 			);

	// 			wp_redirect('https://www.wpxpo.com/checkout/');
				
	// 		}


			
	// 	}
	// }

    public function edd_check_license($force=false) {
        $site_hash = md5(home_url( ));
        $checked_trasient_name = 'edd_wholesalex_'.$site_hash;
        $failed_check_interval = 86400; // 1 day
        $is_already_cheked = get_transient( $checked_trasient_name );
		if ( !$is_already_cheked  || $force) {
			
            $license = get_option( 'edd_wholesalex_license_key', '' );

            $api_params = array(
                'edd_action' => 'check_license',
                'license'    => $license,
                'item_id'    => $this->item_id,
                'url'        => home_url(),
            );

            $response = wp_remote_post(
                $this->server_url,
                array(
                    'timeout'   => 50,
                    'sslverify' => false,
                    'body'      => $api_params,
                )
            );

            set_transient($checked_trasient_name,true,$failed_check_interval );         
            if ( !( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) ) {
                $license_data = json_decode( wp_remote_retrieve_body( $response ) );

                update_option( 'edd_wholesalex_license_status', $license_data->license );
                update_option( 'edd_wholesalex_license_data', (array) $license_data);
            } 
        
		}
    }
}
